(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    426499,      10400]
NotebookOptionsPosition[    412625,      10003]
NotebookOutlinePosition[    412989,      10019]
CellTagsIndexPosition[    412946,      10016]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Volume enclosed by Catmull-Clark surfaces with sharp creases\
\>", "Section",
 CellChangeTimes->{{3.604699325951583*^9, 3.604699342005501*^9}, {
  3.605316105939847*^9, 3.6053161113441563`*^9}, {3.6053189977682505`*^9, 
  3.605319007676817*^9}, {3.60569576697237*^9, 3.6056957867405005`*^9}, {
  3.605735736585387*^9, 3.6057357410236406`*^9}, {3.6123193898083057`*^9, 
  3.6123193990398335`*^9}}],

Cell[TextData[{
 "code by jph, 2014\ndownload from ",
 StyleBox["www.hakenberg.de",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.605695943586472*^9, 3.6056959456205883`*^9}, {
   3.6056970059162335`*^9, 3.6056970389631243`*^9}, 3.605735755713481*^9, 
   3.612138146605847*^9, {3.612175969429984*^9, 3.6121759752833195`*^9}, {
   3.6121774860257287`*^9, 3.6121774882888584`*^9}, 3.6123104446486716`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementation", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
  3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
  3.6056228546483173`*^9}, {3.605646909007148*^9, 3.6056469329195156`*^9}, {
  3.60564698352641*^9, 3.6056469954380913`*^9}, {3.605696857837764*^9, 
  3.605696861844993*^9}, {3.605697358177382*^9, 3.605697358385394*^9}, {
  3.6123087117865577`*^9, 3.6123087140776887`*^9}, {3.612310224706092*^9, 
  3.612310236688777*^9}}],

Cell[CellGroupData[{

Cell["Helper functions", "Subsection",
 CellChangeTimes->{{3.606513742476762*^9, 3.606513746957018*^9}, {
   3.6105195711594224`*^9, 3.610519571356434*^9}, {3.6105717506927967`*^9, 
   3.610571750833805*^9}, {3.611813738357498*^9, 3.6118137385235076`*^9}, 
   3.612318059843236*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"NestT", "[", 
   RowBox[{"form_", ",", "S_"}], "]"}], ":=", 
  RowBox[{"Nest", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{
       RowBox[{"#", ".", "S"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "1"}], "}"}]}], "]"}], "&"}], ",", "form",
     ",", "3"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NestP", "[", 
   RowBox[{"form_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Nest", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{
       RowBox[{"#", "\[LeftDoubleBracket]", "ind", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "1"}], "}"}]}], "]"}], "&"}], ",", "form",
     ",", "3"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Skew3", "[", "frm_", "]"}], ":=", 
  RowBox[{"frm", "-", 
   RowBox[{"Transpose", "[", 
    RowBox[{"frm", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "1", ",", "3"}], "}"}]}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NormalizeWeights", "[", "S_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"#", "/", 
     RowBox[{"Total", "[", "#", "]"}]}], "&"}], "/@", 
   "S"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HashGet", "[", 
   RowBox[{"hash_", ",", "key_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"entry", "=", 
      RowBox[{"hash", "[", "key", "]"}]}], "}"}], ",", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "entry", "]"}], "===", "hash"}], ",", "Null", ",",
       "entry"}], "]"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6057378203785734`*^9, 3.6057378252438517`*^9}, {
   3.6057378900585585`*^9, 3.605737907722569*^9}, {3.605738008106311*^9, 
   3.6057380487296343`*^9}, {3.6057381112752113`*^9, 3.605738113972366*^9}, 
   3.605739035228059*^9, {3.6057806819469023`*^9, 3.6057806828949566`*^9}, {
   3.6062491007071095`*^9, 3.6062491011081324`*^9}, {3.606427549459944*^9, 
   3.6064275497079573`*^9}, {3.6064595139310026`*^9, 
   3.6064595413245697`*^9}, {3.606460398258583*^9, 3.6064603984495945`*^9}, {
   3.607249348229387*^9, 3.6072493498804812`*^9}, {3.6097071001335135`*^9, 
   3.609707124113885*^9}, {3.609712653398142*^9, 3.6097126536091537`*^9}, {
   3.6097523114709177`*^9, 3.6097523117609344`*^9}, {3.609752422785285*^9, 
   3.6097524242233667`*^9}, {3.609752622916732*^9, 3.609752623167746*^9}, {
   3.609764163993843*^9, 3.6097641661649675`*^9}, {3.609764199009846*^9, 
   3.609764222016162*^9}, 3.60976425485004*^9, {3.609764297304468*^9, 
   3.6097643039868507`*^9}, {3.6097643519685946`*^9, 3.609764375784957*^9}, {
   3.609764462345908*^9, 3.609764507257477*^9}, {3.6097645492138767`*^9, 
   3.609764552449062*^9}, {3.6097705406445665`*^9, 3.6097705460058737`*^9}, {
   3.609788698628519*^9, 3.6097887343395615`*^9}, {3.6097892104037905`*^9, 
   3.6097892107648115`*^9}, {3.6099732494972534`*^9, 3.609973275087717*^9}, {
   3.611745638390935*^9, 3.61174564670541*^9}, {3.6123189514122305`*^9, 
   3.6123189872262793`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"BuildEssential", "[", 
   RowBox[{"eqs_", ",", "vars_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"lst", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"CoefficientArrays", "[", 
           RowBox[{"#", ",", 
            RowBox[{"Flatten", "[", "vars", "]"}]}], "]"}], "]"}], "&"}], "/@", 
        RowBox[{"Flatten", "[", "eqs", "]"}]}]}], ",", 
      RowBox[{"outM", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"outV", "=", 
       RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"1", "<", 
          RowBox[{"Length", "[", "#", "]"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"AppendTo", "[", 
           RowBox[{"outM", ",", 
            RowBox[{
            "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
           "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"outV", ",", 
            RowBox[{
            "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
           "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], "&"}], "/@", "lst"}],
      ";", 
     RowBox[{"{", 
      RowBox[{"outM", ",", "outV"}], "}"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SkewTensor", "[", "X_", "]"}], ":=", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"TensorRank", "[", "X", "]"}], "!"}]}], " ", 
   RowBox[{"Plus", "@@", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Signature", "[", "#1", "]"}], 
        RowBox[{"Transpose", "[", 
         RowBox[{"X", ",", "#1"}], "]"}]}], "&"}], "/@", 
      RowBox[{"Permutations", "[", 
       RowBox[{"Range", "[", 
        RowBox[{"TensorRank", "[", "X", "]"}], "]"}], "]"}]}], 
     ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubsetQ", "[", 
   RowBox[{"sub_", ",", "set_"}], "]"}], ":=", 
  RowBox[{"And", "@@", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"MemberQ", "[", 
       RowBox[{"set", ",", "#"}], "]"}], "&"}], "/@", "sub"}], 
    ")"}]}]}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NumericMesh", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"N", "[", "V", "]"}], ",", "F", ",", "E"}], "}"}]}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"PlotMesh", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"Polygon", "[", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], "&"}], 
       "/@", "F"}], ",", 
      RowBox[{"Thickness", "[", ".02", "]"}], ",", 
      RowBox[{"RGBColor", "[", 
       RowBox[{
        RowBox[{"106", "/", "255"}], ",", 
        RowBox[{"144", "/", "255"}], ",", 
        RowBox[{"47", "/", "255"}]}], "]"}], ",", "Black", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{
           RowBox[{"#", "\[LeftDoubleBracket]", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"Range", "[", 
               RowBox[{"Length", "[", "#", "]"}], "]"}], ",", 
              RowBox[{"{", "1", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
           ",", "All"}], "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", 
       "E"}]}], "}"}], ",", 
    RowBox[{"Axes", "\[Rule]", "True"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HueValues", "[", "lst_", "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mn", "=", 
       RowBox[{"Min", "[", "lst", "]"}]}], ",", 
      RowBox[{"mx", "=", 
       RowBox[{"Max", "[", "lst", "]"}]}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"lst", "-", "mn"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"mx", "-", "mn"}], ")"}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlotMesh", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], ",", "vol_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hue", "=", 
      RowBox[{"HueValues", "[", "vol", "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Graphics3D", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Hue", "[", 
             RowBox[{
             "hue", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}], ",", 
            RowBox[{"Polygon", "[", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{
               RowBox[{
               "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
               ",", "All"}], "\[RightDoubleBracket]"}], "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Thickness", "[", ".015", "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
             RowBox[{"#", "\[LeftDoubleBracket]", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"Range", "[", 
                 RowBox[{"Length", "[", "#", "]"}], "]"}], ",", 
                RowBox[{"{", "1", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
             ",", "All"}], "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", 
         "E"}]}], "}"}], ",", 
      RowBox[{"AspectRatio", "\[Rule]", "Automatic"}]}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.612318948348056*^9, 3.612318958799653*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Subdivision", "Subsection",
 CellChangeTimes->{{3.6056968693454227`*^9, 3.605696873892682*^9}, {
  3.6075123394543247`*^9, 3.607512339601333*^9}, {3.6081685642126536`*^9, 
  3.6081685644416666`*^9}, {3.6110970299236603`*^9, 3.611097030359685*^9}, {
  3.612318889980717*^9, 3.612318891235789*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuadLinearSubdivide", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Vout", "=", "V"}], ",", 
      RowBox[{"Fout", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"Eout", "=", 
       RowBox[{"{", "}"}]}], ",", "mid", ",", "key", ",", "hash", ",", "edge",
       ",", "cycl", ",", "list"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"Vout", ",", 
          RowBox[{"Mean", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}]}], "]"}], ";", 
        RowBox[{"mid", "=", 
         RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
        RowBox[{"list", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"key", "=", 
            RowBox[{"Sort", "[", 
             RowBox[{
              RowBox[{"RotateLeft", "[", 
               RowBox[{
                RowBox[{
                "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
                ",", 
                RowBox[{"j", "-", "1"}]}], "]"}], "\[LeftDoubleBracket]", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"edge", "=", 
            RowBox[{"HashGet", "[", 
             RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"edge", "===", "Null"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"AppendTo", "[", 
               RowBox[{"Vout", ",", 
                RowBox[{"Mean", "[", 
                 RowBox[{
                 "V", "\[LeftDoubleBracket]", "key", 
                  "\[RightDoubleBracket]"}], "]"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"edge", "=", 
               RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
              RowBox[{
               RowBox[{"hash", "[", "key", "]"}], "=", "edge"}]}]}], "]"}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"list", ",", "edge"}], "]"}], ";"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"AppendTo", "[", 
           RowBox[{"Fout", ",", 
            RowBox[{"{", 
             RowBox[{"mid", ",", 
              RowBox[{"list", "\[LeftDoubleBracket]", 
               RowBox[{
                RowBox[{"Mod", "[", 
                 RowBox[{
                  RowBox[{"j", "-", "2"}], ",", 
                  RowBox[{"Length", "[", 
                   RowBox[{
                   "F", "\[LeftDoubleBracket]", "k", 
                    "\[RightDoubleBracket]"}], "]"}]}], "]"}], "+", "1"}], 
               "\[RightDoubleBracket]"}], ",", 
              RowBox[{"F", "\[LeftDoubleBracket]", 
               RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{
              "list", "\[LeftDoubleBracket]", "j", 
               "\[RightDoubleBracket]"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"len", "=", 
            RowBox[{"Length", "[", 
             RowBox[{
             "E", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"cycl", "=", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"Table", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"AppendTo", "[", 
               RowBox[{"cycl", ",", 
                RowBox[{"E", "\[LeftDoubleBracket]", 
                 RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}]}], "]"}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{"key", "=", 
               RowBox[{"Sort", "[", 
                RowBox[{
                 RowBox[{"RotateLeft", "[", 
                  RowBox[{
                   RowBox[{
                   "E", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], ",", 
                   RowBox[{"k", "-", "1"}]}], "]"}], "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"edge", "=", 
               RowBox[{"HashGet", "[", 
                RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"cycl", ",", "edge"}], "]"}], ";"}], 
             "\[IndentingNewLine]", ",", 
             RowBox[{"{", 
              RowBox[{"k", ",", "len"}], "}"}]}], "]"}]}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"Eout", ",", "cycl"}], "]"}], ";"}], "\[IndentingNewLine]", 
       ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", 
         RowBox[{"Length", "[", "E", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Vout", ",", "Fout", ",", "Eout"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.6056884725371523`*^9, {3.6056916585413814`*^9, 3.6056916603574853`*^9}, {
   3.6056929270129337`*^9, 3.605692927350953*^9}, 3.6057452624192333`*^9, {
   3.6058281527013636`*^9, 3.6058281529953804`*^9}, {3.6062824779937935`*^9, 
   3.6062824782428083`*^9}, {3.606282515535941*^9, 3.6062825631026616`*^9}, {
   3.6062827326523595`*^9, 3.6062827328383703`*^9}, {3.606282812685937*^9, 
   3.6062828148250594`*^9}, 3.6062830149315047`*^9, {3.606283066759469*^9, 
   3.6062830673405027`*^9}, {3.606283124344763*^9, 3.606283131254158*^9}, {
   3.606283213999891*^9, 3.6062832215433226`*^9}, {3.606393542832162*^9, 
   3.6063936132641897`*^9}, {3.6063939414209595`*^9, 3.6063939622931533`*^9}, 
   3.606460394231353*^9, 3.6064739843526835`*^9, {3.6066436549895115`*^9, 
   3.6066436741326065`*^9}, {3.6066443678902864`*^9, 
   3.6066443685493245`*^9}, {3.6066444304368644`*^9, 3.606644455781314*^9}, {
   3.6073265323306036`*^9, 3.607326532685609*^9}, {3.6075033450068717`*^9, 
   3.6075033454728985`*^9}, {3.609701415154351*^9, 3.609701472463629*^9}, {
   3.6097243518372545`*^9, 3.6097244292796836`*^9}, {3.6097244822397127`*^9, 
   3.6097245291253943`*^9}, 3.6097539931361036`*^9, {3.609762372529378*^9, 
   3.609762385299108*^9}, {3.609762417070925*^9, 3.60976242065013*^9}, {
   3.609762706588485*^9, 3.6097627084865932`*^9}, {3.609763534397833*^9, 
   3.6097636640112457`*^9}, {3.609788800764361*^9, 3.6097888555944967`*^9}, {
   3.6097889136738186`*^9, 3.609788961260541*^9}, {3.610517529999675*^9, 
   3.610517581799638*^9}, 3.6108632771829467`*^9, {3.611032790556466*^9, 
   3.6110327913485117`*^9}, 3.611094216808759*^9, 3.611104816239012*^9, 
   3.6123188947579904`*^9, {3.6123190461706505`*^9, 3.612319046316659*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkWeight", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["n", "2"]}], "-", 
   RowBox[{"7", " ", "n"}]}]}]], "Input",
 CellChangeTimes->{3.6123189943656874`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"vind", " ", "is", " ", "in", " ", 
    RowBox[{"{", 
     RowBox[{"1", ",", " ", "2", ",", " ", "...", ",", " ", 
      RowBox[{"Length", "[", "f", "]"}]}], "}"}], "\[IndentingNewLine]", "f", 
    " ", "is", " ", "element", " ", "of", " ", "F"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"VertexRing", "[", 
     RowBox[{"F_", ",", "f_", ",", "vind_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"fn", "=", 
         RowBox[{"RotateLeft", "[", 
          RowBox[{"f", ",", 
           RowBox[{"vind", "-", "1"}]}], "]"}]}], ",", "v1", ",", "v2", ",", 
        "adj", ",", 
        RowBox[{"ring", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"count", "=", "0"}], ",", 
        RowBox[{"rev", "=", 
         RowBox[{"{", "}"}]}], ",", "tmp", ",", "pos"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"adj", "=", "fn"}], ";", "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"ring", ",", "adj"}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"rev", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"Position", "[", 
             RowBox[{"F", ",", "f"}], "]"}], "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
           RowBox[{"vind", "-", "1"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"++", "count"}], "<", "20"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"v1", "=", 
           RowBox[{
           "adj", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"v2", "=", 
           RowBox[{"Last", "[", "adj", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"tmp", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"F", ",", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Union", "[", "#", "]"}], "\[NotEqual]", 
                 RowBox[{"Union", "[", "adj", "]"}]}], "\[And]", 
                RowBox[{"MemberQ", "[", 
                 RowBox[{"#", ",", "v1"}], "]"}], "\[And]", 
                RowBox[{"MemberQ", "[", 
                 RowBox[{"#", ",", "v2"}], "]"}]}], "&"}]}], "]"}], 
            "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"pos", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"Position", "[", 
              RowBox[{"tmp", ",", "v1"}], "]"}], "\[LeftDoubleBracket]", 
             RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "-", "1"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"adj", "=", 
           RowBox[{"RotateLeft", "[", 
            RowBox[{"tmp", ",", "pos"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"adj", "\[Equal]", "fn"}], ",", 
            RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"ring", ",", "adj"}], "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"rev", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"Position", "[", 
                RowBox[{"F", ",", "tmp"}], "]"}], "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              "pos"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
       RowBox[{"{", 
        RowBox[{"ring", ",", "rev"}], "}"}]}]}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CatmullClarkSubdivide", "[", 
     RowBox[{"{", 
      RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Vout", ",", 
        RowBox[{"Fout", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"Eout", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"count", "=", "0"}], ",", 
        RowBox[{"Vind", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"rings", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", 
             RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
        "ring", ",", "rev", ",", "\[IndentingNewLine]", "hash", ",", "key", 
        ",", "edge1", ",", "edge2", ",", "vert", ",", "cycl", ",", "w"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"(*", "Midpoint", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Vout", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Mean", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
          "&"}], "/@", "F"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"AppendTo", "[", 
          RowBox[{
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{"F", "\[LeftDoubleBracket]", 
             RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], ",", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
          "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ring", ",", "rev"}], "}"}], "=", 
             RowBox[{"VertexRing", "[", 
              RowBox[{
               RowBox[{"rings", "\[LeftDoubleBracket]", 
                RowBox[{"F", "\[LeftDoubleBracket]", 
                 RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}], ",", 
               RowBox[{
               "F", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
               ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"Edge", " ", "1"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"Sort", "[", 
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"edge1", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"edge1", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  RowBox[{"1", "/", "16"}], 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    "6", ",", "6", ",", "1", ",", "1", ",", "1", ",", "1"}], 
                    "}"}], ".", 
                   RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{
                    "ring", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{
                    RowBox[{"Last", "[", "ring", "]"}], 
                    "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"2", ",", "3"}], "}"}], 
                    "\[RightDoubleBracket]"}]}], "]"}], 
                    "\[RightDoubleBracket]"}]}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"edge1", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "edge1"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"Edge", " ", "2"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"Sort", "[", 
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", 
                RowBox[{"{", 
                 RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"edge2", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"edge2", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  RowBox[{"1", "/", "16"}], 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    "6", ",", "1", ",", "1", ",", "6", ",", "1", ",", "1"}], 
                    "}"}], ".", 
                   RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{
                    "ring", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{"ring", "\[LeftDoubleBracket]", 
                    RowBox[{"2", ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "4"}], "}"}]}], 
                    "\[RightDoubleBracket]"}]}], "]"}], 
                    "\[RightDoubleBracket]"}]}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"edge2", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "edge2"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", "Vertex", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"{", 
              RowBox[{"F", "\[LeftDoubleBracket]", 
               RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}], "}"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"vert", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"vert", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"w", "=", 
                RowBox[{"CatmullClarkWeight", "[", 
                 RowBox[{"Length", "[", "ring", "]"}], "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  FractionBox["1", 
                   RowBox[{"w", "+", 
                    RowBox[{"7", " ", 
                    RowBox[{"Length", "[", "ring", "]"}]}]}]], 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"w", " ", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"ring", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], 
                    "\[RightDoubleBracket]"}]}], "+", 
                    RowBox[{"Total", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"6", ",", "1"}], "}"}], ".", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"#", "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"2", ",", "3"}], "}"}], "\[RightDoubleBracket]"}],
                     "\[RightDoubleBracket]"}]}], ")"}], "&"}], "/@", 
                    "ring"}], "]"}]}], ")"}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"vert", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "vert"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"Fout", ",", 
              RowBox[{"{", 
               RowBox[{"vert", ",", "edge1", ",", "n", ",", "edge2"}], 
               "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", 
             RowBox[{"Length", "[", 
              RowBox[{
              "F", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
              "]"}]}], "}"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"n", ",", 
           RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"len", "=", 
              RowBox[{"Length", "[", 
               RowBox[{
               "E", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
               "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"cycl", "=", 
              RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"Table", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{
                RowBox[{"key", "=", 
                 RowBox[{"{", 
                  RowBox[{"E", "\[LeftDoubleBracket]", 
                   RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}], 
                  "}"}]}], ";", "\[IndentingNewLine]", 
                RowBox[{"vert", "=", 
                 RowBox[{"HashGet", "[", 
                  RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"cycl", ",", "vert"}], "]"}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{
                 "Vout", "\[LeftDoubleBracket]", "vert", 
                  "\[RightDoubleBracket]"}], "=", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"1", ",", "6", ",", "1"}], "}"}], ".", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{"RotateLeft", "[", 
                    RowBox[{
                    RowBox[{
                    "E", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{"k", "-", "2"}]}], "]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "2", ",", "3"}], "}"}], 
                    "\[RightDoubleBracket]"}], "\[RightDoubleBracket]"}]}], 
                   ")"}], "/", "8"}]}], ";", "\[IndentingNewLine]", 
                RowBox[{"key", "=", 
                 RowBox[{"Sort", "[", 
                  RowBox[{
                   RowBox[{"RotateLeft", "[", 
                    RowBox[{
                    RowBox[{
                    "E", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{"k", "-", "1"}]}], "]"}], "\[LeftDoubleBracket]", 
                   RowBox[{"{", 
                    RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}],
                   "]"}]}], ";", "\[IndentingNewLine]", 
                RowBox[{"edge1", "=", 
                 RowBox[{"HashGet", "[", 
                  RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"cycl", ",", "edge1"}], "]"}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{
                 "Vout", "\[LeftDoubleBracket]", "edge1", 
                  "\[RightDoubleBracket]"}], "=", 
                 RowBox[{"Mean", "[", 
                  RowBox[{
                  "V", "\[LeftDoubleBracket]", "key", 
                   "\[RightDoubleBracket]"}], "]"}]}], ";"}], 
               "\[IndentingNewLine]", ",", 
               RowBox[{"{", 
                RowBox[{"k", ",", "len"}], "}"}]}], "]"}]}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"Eout", ",", "cycl"}], "]"}], ";"}], "\[IndentingNewLine]",
          ",", 
         RowBox[{"{", 
          RowBox[{"n", ",", 
           RowBox[{"Length", "[", "E", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Simplify", "[", "Vout", "]"}], ",", "Fout", ",", "Eout"}], 
        "}"}]}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{
  3.605824101738662*^9, {3.605824849440428*^9, 3.605824863362224*^9}, {
   3.605824937494464*^9, 3.6058249725954723`*^9}, {3.6058255358466883`*^9, 
   3.6058256619469004`*^9}, {3.605825879489343*^9, 3.6058259515214634`*^9}, {
   3.6058259940338945`*^9, 3.6058261939943323`*^9}, {3.605826236075739*^9, 
   3.6058262978052697`*^9}, {3.6058263426738358`*^9, 3.605826460180557*^9}, {
   3.6058265628904314`*^9, 3.605826575329143*^9}, {3.605826665304289*^9, 
   3.605826866350788*^9}, {3.6058268971855516`*^9, 3.6058269197208414`*^9}, {
   3.605827021714675*^9, 3.605827026840968*^9}, {3.605827080081013*^9, 
   3.6058271005811853`*^9}, {3.605827136589245*^9, 3.60582727059591*^9}, {
   3.6058276054710636`*^9, 3.605827670568787*^9}, {3.6058277090779896`*^9, 
   3.6058277515944214`*^9}, {3.6058277869084415`*^9, 3.605827858446533*^9}, {
   3.605827903734123*^9, 3.605828013325392*^9}, {3.6058280580169477`*^9, 
   3.6058280633622537`*^9}, {3.6058460565460243`*^9, 3.605846079864358*^9}, {
   3.60584611260223*^9, 3.605846154114605*^9}, {3.6062826089242826`*^9, 
   3.60628266552252*^9}, {3.606282855290374*^9, 3.6062828850670767`*^9}, {
   3.6062829976145144`*^9, 3.6062830059769926`*^9}, {3.606283150190241*^9, 
   3.6062831573526506`*^9}, {3.606283274002323*^9, 3.606283290958293*^9}, {
   3.606283338433008*^9, 3.6062833451643934`*^9}, {3.6063888767282753`*^9, 
   3.606388910047181*^9}, {3.6063889964631243`*^9, 3.606389008314802*^9}, {
   3.606459661824462*^9, 3.606459672898095*^9}, {3.606643536620741*^9, 
   3.606643544418187*^9}, {3.6066444247035365`*^9, 3.6066444259826097`*^9}, {
   3.6097021803731194`*^9, 3.6097022636168804`*^9}, {3.609702604787394*^9, 
   3.60970271137049*^9}, {3.6097028293872404`*^9, 3.609702906589656*^9}, {
   3.60970298498114*^9, 3.609703066781819*^9}, {3.609703178663218*^9, 
   3.609703179676276*^9}, {3.6097032954959*^9, 3.609703353304207*^9}, {
   3.609703393607512*^9, 3.609703429679575*^9}, {3.60970348396068*^9, 
   3.609703549647437*^9}, {3.6099730945183897`*^9, 3.609973096763518*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkStencil", "[", 
   RowBox[{"F_", ",", "f_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"rings", ",", "pos"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"rings", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VertexRing", "[", 
          RowBox[{"F", ",", "f", ",", "k"}], "]"}], "\[LeftDoubleBracket]", 
         "1", "\[RightDoubleBracket]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "4"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"pos", "=", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{"Append", "[", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Length", "[", "#", "]"}], "\[NotEqual]", "4"}], "&"}],
              "/@", "rings"}], ",", "True"}], "]"}], ",", "3"}], "]"}], "]"}],
        "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rings", "=", 
      RowBox[{"RotateRight", "[", 
       RowBox[{"rings", ",", 
        RowBox[{"3", "-", "pos"}]}], "]"}]}], ";", 
     RowBox[{"(*", 
      RowBox[{
      "our", " ", "subdiv", " ", "alg", " ", "positions", " ", "irreg", " ", 
       "always", " ", "at", " ", "pos", " ", "1"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Flatten", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"2", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "2", ",", 
          RowBox[{"{", 
           RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"2", ",", "4", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"4", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"3", ",", "2", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "4"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"4", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Reverse", "[", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{"3", ",", "k", ",", 
             RowBox[{"{", 
              RowBox[{"4", ",", "3"}], "}"}]}], "\[RightDoubleBracket]"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "3", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{
               "rings", "\[LeftDoubleBracket]", "3", 
                "\[RightDoubleBracket]"}], "]"}], "-", "1"}]}], "}"}]}], 
          "]"}], "]"}]}], "\[IndentingNewLine]", "}"}], "]"}]}]}], 
   "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Load volume forms", "Subsection",
 CellChangeTimes->{{3.6073261432421274`*^9, 3.6073261482547045`*^9}, {
   3.607687403417124*^9, 3.6076874039291525`*^9}, 3.612318056659054*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Lambda]X", "[", 
   RowBox[{"n_", ",", "\[Lambda]_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"d", "=", 
      RowBox[{"Length", "[", "\[Lambda]", "]"}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"Binomial", "[", 
      RowBox[{"n", ",", "d"}], "]"}], "-", 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{"Binomial", "[", 
        RowBox[{
         RowBox[{"n", "-", 
          RowBox[{
          "\[Lambda]", "\[LeftDoubleBracket]", "i", 
           "\[RightDoubleBracket]"}]}], ",", 
         RowBox[{"d", "+", "1", "-", "i"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "d"}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalLambda]set", "[", 
   RowBox[{"n_", ",", "d_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"lst", "=", 
      RowBox[{"{", "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"{", "##", "}"}], "==", 
            RowBox[{"Sort", "[", 
             RowBox[{"{", "##", "}"}], "]"}]}], "&&", 
           RowBox[{
            RowBox[{"Length", "[", 
             RowBox[{"Union", "[", 
              RowBox[{"{", "##", "}"}], "]"}], "]"}], "\[Equal]", "d"}]}], 
          ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"lst", ",", 
            RowBox[{"{", "##", "}"}]}], "]"}]}], "]"}], "&"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{"n", ",", 
         RowBox[{"{", "d", "}"}]}], "]"}]}], "]"}], ";", "lst"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6117137433353386`*^9, 3.611713743561352*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"InflateForm", "[", 
   RowBox[{"lst_", ",", "n_", ",", "d_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"Array", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"p", "=", 
            RowBox[{"{", "##", "}"}]}], ",", 
           RowBox[{"s", "=", 
            RowBox[{"Signature", "[", 
             RowBox[{"{", "##", "}"}], "]"}]}]}], "}"}], ",", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"s", "\[Equal]", "0"}], ",", "0", ",", 
           RowBox[{"s", " ", 
            RowBox[{"lst", "\[LeftDoubleBracket]", 
             RowBox[{"\[Lambda]X", "[", 
              RowBox[{"n", ",", 
               RowBox[{"Sort", "[", "p", "]"}]}], "]"}], 
             "\[RightDoubleBracket]"}]}]}], "]"}]}], "]"}], "&"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{"n", ",", 
        RowBox[{"{", "d", "}"}]}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.611850156341178*^9, 3.6118501617654877`*^9}, {
  3.6121340542537775`*^9, 3.612134064621371*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CompressForm", "[", "frm_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"frm", "\[LeftDoubleBracket]", 
      RowBox[{
       RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
      "\[RightDoubleBracket]"}], "&"}], "/@", 
    RowBox[{"\[CapitalLambda]set", "[", 
     RowBox[{
      RowBox[{"Length", "[", "frm", "]"}], ",", 
      RowBox[{"TensorRank", "[", "frm", "]"}]}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkListAlt", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"num", "=", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"k", "<", "10"}], ",", "\"\<0\>\"", ",", "\"\<\>\""}], 
           "]"}], "<>", 
          RowBox[{"ToString", "[", "k", "]"}]}]}], "}"}], ",", 
       RowBox[{"InflateForm", "[", 
        RowBox[{
         RowBox[{"<<", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
            "\"\<_forms\\\\CCalt\>\"", "<>", "num", "<>", "\"\<vec.mx\>\""}], 
           ")"}]}], ",", 
         RowBox[{"8", "+", 
          RowBox[{"2", "k"}]}], ",", "3"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "3", ",", "7"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.6121754065977926`*^9, {3.612176704741042*^9, 3.6121767048200464`*^9}, 
   3.612177381216734*^9, {3.612316906230253*^9, 3.6123169269734397`*^9}, {
   3.6123170124863305`*^9, 3.6123170184986744`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkVal1Form", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc11vec.mx\>\""}], ")"}]}], ",", "9", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal2Form", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc21vec.mx\>\""}], ")"}]}], ",", "12", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal3bForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc31vec.mx\>\""}], ")"}]}], ",", "15", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal3aForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc32vec.mx\>\""}], ")"}]}], ",", "14", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal4bForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc41vec.mx\>\""}], ")"}]}], ",", "17", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal4aForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc42vec.mx\>\""}], ")"}]}], ",", "16", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal5cForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc51vec.mx\>\""}], ")"}]}], ",", "19", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal5bForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc52vec.mx\>\""}], ")"}]}], ",", "19", ",", "3"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVal5aForm", "=", 
   RowBox[{"InflateForm", "[", 
    RowBox[{
     RowBox[{"<<", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
        "\"\<_forms\\\\CCaltc53vec.mx\>\""}], ")"}]}], ",", "18", ",", "3"}], 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.612318128837182*^9, 3.6123181739327617`*^9}, {
  3.6123185537844877`*^9, 3.612318581914097*^9}, {3.6123186272816916`*^9, 
  3.612318662261692*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Volume formula", "Subsection",
 CellChangeTimes->{{3.609841324515749*^9, 3.6098413313951426`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LinearVolume", "[", 
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
   RowBox[{"With", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"QuadDet", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             FractionBox["1", "12"], ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"], ",", "0"}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", 
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", 
             FractionBox["1", "12"], ",", "0"}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", "0", ",", "0", ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", "0"}], "}"}]}], "}"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"], ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]}], 
        "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Simplify", "[", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"f", "=", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}], 
           "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "f", "]"}], "\[Equal]", "4"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"QuadDet", ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "1"}], "\[RightDoubleBracket]"}], ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "2"}], "\[RightDoubleBracket]"}], ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "3"}], "\[RightDoubleBracket]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Simplify", "[", 
             RowBox[{
              RowBox[{"Sum", "[", 
               RowBox[{
                RowBox[{"Det", "[", 
                 RowBox[{"V", "\[LeftDoubleBracket]", 
                  RowBox[{
                   RowBox[{"f", "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "k", ",", 
                    RowBox[{"k", "+", "1"}]}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                  "\[RightDoubleBracket]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"k", ",", "2", ",", 
                  RowBox[{
                   RowBox[{"Length", "[", "f", "]"}], "-", "1"}]}], "}"}]}], 
               "]"}], "/", "6"}], "]"}]}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], "]"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkVolumeFace", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"rings", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
      "stencil", ",", "val", ",", 
      RowBox[{"edgeind", "=", 
       RowBox[{"Flatten", "[", "E", "]"}]}], ",", "inter", ",", "IndV", ",", 
      "IndF", ",", "EdgS", ",", "EdgF", ",", "spv", ",", "curve"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"stencil", "=", 
         RowBox[{"CatmullClarkStencil", "[", 
          RowBox[{
           RowBox[{"Join", "@@", 
            RowBox[{"rings", "\[LeftDoubleBracket]", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}]}], ",", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"val", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "stencil", "]"}], "/", "2"}], "-", "4"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"inter", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{"edgeind", ",", 
           RowBox[{"stencil", "\[LeftDoubleBracket]", 
            RowBox[{"{", 
             RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], 
            "\[RightDoubleBracket]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"EdgS", ",", "EdgF"}], "}"}], "=", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"inter", "==", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"stencil", ",", 
             RowBox[{"CatmullClarkListAlt", "\[LeftDoubleBracket]", 
              RowBox[{"val", "-", "2"}], "\[RightDoubleBracket]"}]}], "}"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"spv", "=", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "3"}], ",", "12", ",", "15"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Which", "[", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"3", ",", "7", ",", "9", ",", "10", ",", "11"}], 
                  "}"}], "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "5", ",", "6", ",", "7", ",", 
                   "9", ",", "10", ",", "11"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal1Form"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"8", ",", "7", ",", "6", ",", "10", ",", "14"}], 
                   "}"}], "\[RightDoubleBracket]"}], ",", "edgeind"}], 
                "]"}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "  ", "12", ",", "11", ",",
                    "10", ",", "  ", "8", ",", "7", ",", "6"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal1Form"}], 
               "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"9", ",", "10", ",", "11", ",", "12"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal2Form"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"3", ",", "7", ",", "11", ",", "spv"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "13", ",", "9", ",", "5", ",", "1", ",", "  ", "14", ",", 
                   "10", ",", "6", ",", "2", ",", " ", "spv", ",", "11", ",", 
                   "7", ",", "3"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal2Form"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"2", ",", "6", ",", "10", ",", "14"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "4", ",", "8", ",", "12", ",", "16", ",", "  ", "3", ",", 
                   "7", ",", "11", ",", "15", ",", "  ", "2", ",", "6", ",", 
                   "10", ",", "14"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal2Form"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal2Form"}], "}"}], ",", "\[IndentingNewLine]", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "17", ",", "18"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal3aForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal3aForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal3bForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"8", ",", "7", ",", "6", ",", "2"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5", ",", "  ", "2", ",", "1"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal3aForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"2", ",", "5", ",", "6"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5", ",", "  ", "4", ",", "3", ",", "2"}], "}"}],
                  "\[RightDoubleBracket]"}], ",", "CatmullClarkVal3bForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"5", ",", "6", ",", "10", ",", "14"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "12", ",", "8", ",", "4", ",", "  ", "15", ",", 
                   "11", ",", "7", ",", "3", ",", "  ", "14", ",", "10", ",", 
                   "6", ",", "2", ",", "  ", "5", ",", "1"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal3aForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal3aForm"}], 
               "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "15", ",", "16", ",", "17", ",", 
                   "18"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal4aForm"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "17", ",", "18", ",", "19", ",", 
                   "20"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal4aForm"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "12"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "12", ",", "18", ",", "17", 
                   ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal4aForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "19", ",", "18", ",", "17", 
                   ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal4aForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "17", ",", "18"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal4bForm"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "19", ",", "20"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClarkVal4bForm"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "4", ",", "8", ",", "12", ",", 
                   "17", ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal4bForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "15", ",", "16", ",", "17", ",", 
                   "18", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal5aForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "12"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "12", ",", "20", ",", "19", 
                   ",", "18", ",", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal5aForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "17", ",", "18", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal5bForm"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "4", ",", "8", ",", "12", ",", 
                   "19", ",", "18", ",", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", "CatmullClarkVal5bForm"}]}], "}"}], ",", 
              "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "17", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "16", ",", "17", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClarkVal5cForm"}], 
               "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              "True", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", "1", "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"{", 
                  RowBox[{"{", "0", "}"}], "}"}], "}"}]}], "}"}]}], "]"}]}]}],
           "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"EdgS", "===", 
           RowBox[{"{", "1", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"IndV", ",", "IndF"}], "}"}], "=", 
            RowBox[{"CatmullClarkDefault", "[", "val", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"curve", "=", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"MemberQ", "[", 
                RowBox[{"edgeind", ",", 
                 RowBox[{
                 "stencil", "\[LeftDoubleBracket]", "k", 
                  "\[RightDoubleBracket]"}]}], "]"}], ",", 
               RowBox[{"AppendTo", "[", 
                RowBox[{"curve", ",", 
                 RowBox[{
                 "IndV", "\[LeftDoubleBracket]", "k", 
                  "\[RightDoubleBracket]"}]}], "]"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"k", ",", 
               RowBox[{"Length", "[", "stencil", "]"}]}], "}"}]}], "]"}], ";",
            "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{"Show", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"CatmullClarkPlot", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"IndV", ",", "IndF"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"IndV", ",", "IndF"}], "}"}], ",", "Green"}], 
                 "]"}], ",", 
                RowBox[{"Graphics", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"Disk", "[", 
                    RowBox[{"#", ",", ".07"}], "]"}], "&"}], "/@", "curve"}], 
                 "]"}]}], "}"}], ",", 
              RowBox[{"ImageSize", "\[Rule]", "140"}]}], "]"}], "]"}], 
           ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"point", "=", 
            RowBox[{"Transpose", "[", 
             RowBox[{
             "V", "\[LeftDoubleBracket]", "EdgS", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}], ",", 
          RowBox[{"Simplify", "[", 
           RowBox[{"EdgF", ".", 
            RowBox[{
            "point", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
            ".", 
            RowBox[{
            "point", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
            ".", 
            RowBox[{
            "point", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}],
            "]"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"m", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVolume", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Total", "[", 
   RowBox[{"CatmullClarkVolumeFace", "[", 
    RowBox[{"{", 
     RowBox[{"V", ",", "F", ",", "E"}], "}"}], "]"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.605846228480858*^9, 3.6058462398955107`*^9}, {
   3.60584630124402*^9, 3.605846340747279*^9}, {3.6058464169596386`*^9, 
   3.605846420533843*^9}, {3.605846458206998*^9, 3.6058464704146957`*^9}, {
   3.6058513105105333`*^9, 3.6058513204771037`*^9}, {3.605851485700554*^9, 
   3.605851507644809*^9}, {3.6063891641917176`*^9, 3.606389214861616*^9}, {
   3.606393795352605*^9, 3.6063938285245023`*^9}, {3.6064273125263915`*^9, 
   3.6064273157805777`*^9}, {3.6064329361650457`*^9, 
   3.6064329842947984`*^9}, {3.606459691591164*^9, 3.6064597107402596`*^9}, {
   3.6064739922303047`*^9, 3.6064740025579634`*^9}, {3.6064742969749856`*^9, 
   3.606474300172535*^9}, {3.606644115925875*^9, 3.606644118109*^9}, {
   3.606695526082301*^9, 3.606695528989467*^9}, {3.606717935131093*^9, 
   3.6067179502063246`*^9}, {3.6067699996307235`*^9, 
   3.6067700183997965`*^9}, {3.606770049711588*^9, 3.6067701074848924`*^9}, {
   3.606770149964322*^9, 3.6067701848893194`*^9}, 3.6067702701511965`*^9, {
   3.6073263530603504`*^9, 3.607326389460909*^9}, {3.6081676637091475`*^9, 
   3.6081676640841694`*^9}, {3.6097550188717723`*^9, 
   3.6097550299424057`*^9}, {3.6097567719690437`*^9, 3.609756800489675*^9}, {
   3.609757479429508*^9, 3.6097574952134113`*^9}, {3.609757543730186*^9, 
   3.6097575464183397`*^9}, {3.6097585363909626`*^9, 
   3.6097585388361025`*^9}, {3.6097586640692654`*^9, 3.609758671067666*^9}, {
   3.6097587425337534`*^9, 3.6097587932626553`*^9}, {3.6097588946404533`*^9, 
   3.609758903596966*^9}, {3.609758965748521*^9, 3.609758966211547*^9}, {
   3.609759127540775*^9, 3.6097592691368732`*^9}, 3.6097593636612797`*^9, {
   3.609759399869351*^9, 3.6097594017524586`*^9}, {3.609759508751579*^9, 
   3.6097596257922735`*^9}, {3.609759727225075*^9, 3.6097599179989862`*^9}, {
   3.609760074421933*^9, 3.60976010635176*^9}, {3.6097604129582963`*^9, 
   3.6097604151564217`*^9}, {3.6097605044355288`*^9, 
   3.6097605797558365`*^9}, {3.609760735251731*^9, 3.609760745604323*^9}, {
   3.609760801251506*^9, 3.6097608112370768`*^9}, {3.6097610119795585`*^9, 
   3.609761204232555*^9}, {3.6097612427937603`*^9, 3.6097612452939034`*^9}, {
   3.60976129174156*^9, 3.609761293177642*^9}, {3.609761354907173*^9, 
   3.6097613644657197`*^9}, {3.60976143283963*^9, 3.609761454618876*^9}, {
   3.60976153109225*^9, 3.6097615717515755`*^9}, {3.609761670678234*^9, 
   3.609761675658519*^9}, {3.609761735720954*^9, 3.6097618145854645`*^9}, {
   3.6097621393540406`*^9, 3.6097622220837727`*^9}, {3.6097689058590627`*^9, 
   3.6097689093242607`*^9}, {3.609768991494961*^9, 3.6097690204256153`*^9}, {
   3.609769055033595*^9, 3.609769081976136*^9}, {3.6097695965275664`*^9, 
   3.6097696745630302`*^9}, {3.6097697276000633`*^9, 
   3.6097698103667974`*^9}, {3.6097726802209435`*^9, 3.60977269571483*^9}, {
   3.6097895672242*^9, 3.6097895844501853`*^9}, 3.609789783977597*^9, {
   3.6097898591658983`*^9, 3.6097898594879165`*^9}, {3.60978991636817*^9, 
   3.609789971850343*^9}, {3.6097963195314093`*^9, 3.6097963461509323`*^9}, {
   3.609796432026844*^9, 3.609796468868951*^9}, {3.6097969622241697`*^9, 
   3.6097969772430286`*^9}, {3.609797030570079*^9, 3.6097970658230953`*^9}, {
   3.6097971008520985`*^9, 3.6097971671428905`*^9}, {3.609799599055988*^9, 
   3.609799609430581*^9}, {3.609799656318263*^9, 3.609799660031475*^9}, {
   3.609800568084413*^9, 3.609800568414432*^9}, {3.609800608057699*^9, 
   3.6098007802695494`*^9}, {3.609801858226205*^9, 3.6098020063346763`*^9}, {
   3.609802049388139*^9, 3.609802056491545*^9}, {3.609802244684309*^9, 
   3.6098022625063286`*^9}, {3.609802404374443*^9, 3.6098024872931857`*^9}, {
   3.609802586708872*^9, 3.609802630823395*^9}, {3.6098026720977554`*^9, 
   3.609802681164274*^9}, {3.609802779556902*^9, 3.6098027939557257`*^9}, {
   3.6098028311588535`*^9, 3.6098028592384596`*^9}, {3.6098030663103037`*^9, 
   3.609803080841134*^9}, 3.6098031142680464`*^9, {3.6098031716613293`*^9, 
   3.6098031786517286`*^9}, {3.609803224965378*^9, 3.6098032623745174`*^9}, 
   3.609803831088046*^9, 3.6098038677341423`*^9, {3.609812415692233*^9, 
   3.6098124201923018`*^9}, {3.609812483530775*^9, 3.609812496485974*^9}, {
   3.6098125436391983`*^9, 3.6098125474242563`*^9}, {3.609812597742529*^9, 
   3.609812603457617*^9}, {3.609812856341501*^9, 3.6098129040197334`*^9}, {
   3.6098129597680893`*^9, 3.609812999878706*^9}, 3.609813034259234*^9, {
   3.6098130737813425`*^9, 3.6098130989767294`*^9}, {3.6098131366248074`*^9, 
   3.6098131651827464`*^9}, {3.609813597082381*^9, 3.609813667773466*^9}, {
   3.6098137496922245`*^9, 3.609813772287572*^9}, {3.6098138188907876`*^9, 
   3.609813893156928*^9}, {3.609813926934947*^9, 3.6098139553953843`*^9}, {
   3.6098139921884494`*^9, 3.6098139936384716`*^9}, {3.609814110131261*^9, 
   3.609814145559305*^9}, {3.612318710713464*^9, 3.6123188141083775`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkFacets", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"rings", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
      "stencil", ",", "val", ",", 
      RowBox[{"edgeind", "=", 
       RowBox[{"Flatten", "[", "E", "]"}]}], ",", "inter", ",", "IndV", ",", 
      "IndF", ",", "EdgS", ",", "EdgT", ",", "spv", ",", "curve"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"stencil", "=", 
         RowBox[{"CatmullClarkStencil", "[", 
          RowBox[{
           RowBox[{"Join", "@@", 
            RowBox[{"rings", "\[LeftDoubleBracket]", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}]}], ",", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"val", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "stencil", "]"}], "/", "2"}], "-", "4"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"inter", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{"edgeind", ",", 
           RowBox[{"stencil", "\[LeftDoubleBracket]", 
            RowBox[{"{", 
             RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], 
            "\[RightDoubleBracket]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"EdgS", ",", "EdgT"}], "}"}], "=", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"inter", "==", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"stencil", ",", 
             RowBox[{"CatmullClarkDefaultTop", "[", "val", "]"}]}], "}"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"spv", "=", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "3"}], ",", "12", ",", "15"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Which", "[", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"3", ",", "7", ",", "9", ",", "10", ",", "11"}], 
                  "}"}], "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "5", ",", "6", ",", "7", ",", 
                   "9", ",", "10", ",", "11"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark1Top"}], "}"}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"8", ",", "7", ",", "6", ",", "10", ",", "14"}], 
                   "}"}], "\[RightDoubleBracket]"}], ",", "edgeind"}], 
                "]"}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "  ", "12", ",", "11", ",",
                    "10", ",", "  ", "8", ",", "7", ",", "6"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark1Top"}], "}"}], 
              ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"9", ",", "10", ",", "11", ",", "12"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark2Top"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{"SubsetQ", "[", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{"3", ",", "7", ",", "11", ",", "spv"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "13", ",", "9", ",", "5", ",", "1", ",", "  ", "14", ",", 
                   "10", ",", "6", ",", "2", ",", " ", "spv", ",", "11", ",", 
                   "7", ",", "3"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark2Top"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"2", ",", "6", ",", "10", ",", "14"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "4", ",", "8", ",", "12", ",", "16", ",", "  ", "3", ",", 
                   "7", ",", "11", ",", "15", ",", "  ", "2", ",", "6", ",", 
                   "10", ",", "14"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark2Top"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"5", ",", "6", ",", "7", ",", "8"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark2Top"}], "}"}], ",", "\[IndentingNewLine]", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "17", ",", "18"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3bTop"}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3bTop"}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3aTop"}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"8", ",", "7", ",", "6", ",", "2"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5", ",", "  ", "2", ",", "1"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3bTop"}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"2", ",", "5", ",", "6"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "15", ",", "14", ",", "13", ",", "  ", "12", ",",
                    "11", ",", "10", ",", "9", ",", "  ", "8", ",", "7", ",", 
                   "6", ",", "5", ",", "  ", "4", ",", "3", ",", "2"}], "}"}],
                  "\[RightDoubleBracket]"}], ",", "CatmullClark3aTop"}], 
               "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "4"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"5", ",", "6", ",", "10", ",", "14"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "16", ",", "12", ",", "8", ",", "4", ",", "  ", "15", ",", 
                   "11", ",", "7", ",", "3", ",", "  ", "14", ",", "10", ",", 
                   "6", ",", "2", ",", "  ", "5", ",", "1"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3bTop"}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", "CatmullClark3bTop"}], "}"}],
               ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "15", ",", "16", ",", "17", ",", 
                   "18"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4bTop"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "17", ",", "18", ",", "19", ",", 
                   "20"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4bTop"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "12"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "12", ",", "18", ",", "17", 
                   ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4bTop"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "19", ",", "18", ",", "17", 
                   ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4bTop"}], "}"}], ",", "\[IndentingNewLine]", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "17", ",", "18"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4aTop"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "19", ",", "20"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4aTop"}], "}"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[GreaterEqual]", "5"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "4", ",", "8", ",", "12", ",", 
                   "17", ",", "16"}], "}"}], "\[RightDoubleBracket]"}], ",", 
                "CatmullClark4aTop"}], "}"}], ",", "\[IndentingNewLine]", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"9", ",", "10", ",", "11", ",", "15"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "15", ",", "16", ",", "17", ",", 
                   "18", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"Skew3", "[", "CatmullClarkVal5aForm", "]"}]}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"3", ",", "7", ",", "11", ",", "12"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "12", ",", "20", ",", "19", 
                   ",", "18", ",", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", 
                 RowBox[{"Skew3", "[", "CatmullClarkVal5aForm", "]"}]}]}], 
               "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "15", ",", "17"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "17", ",", "18", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"Skew3", "[", "CatmullClarkVal5bForm", "]"}]}], "}"}],
               ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "12", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "5", ",", "9", ",", "13", ",", "  ", "2", ",", 
                   "6", ",", "10", ",", "14", ",", "  ", "3", ",", "7", ",", 
                   "11", ",", "15", ",", "  ", "4", ",", "8", ",", "12", ",", 
                   "19", ",", "18", ",", "17", ",", "16"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"-", 
                 RowBox[{"Skew3", "[", "CatmullClarkVal5bForm", "]"}]}]}], 
               "}"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"val", "\[Equal]", "6"}], "\[And]", 
               RowBox[{"SubsetQ", "[", 
                RowBox[{
                 RowBox[{"stencil", "\[LeftDoubleBracket]", 
                  RowBox[{"{", 
                   RowBox[{"11", ",", "17", ",", "19"}], "}"}], 
                  "\[RightDoubleBracket]"}], ",", "edgeind"}], "]"}], 
               "\[And]", 
               RowBox[{
                RowBox[{"Length", "[", "inter", "]"}], "\[Equal]", "1"}]}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"stencil", "\[LeftDoubleBracket]", 
                 RowBox[{"{", 
                  RowBox[{
                  "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6",
                    ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11", 
                   ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", 
                   "16", ",", "17", ",", "19", ",", "20"}], "}"}], 
                 "\[RightDoubleBracket]"}], ",", 
                RowBox[{"Skew3", "[", "CatmullClarkVal5cForm", "]"}]}], "}"}],
               ",", "\[IndentingNewLine]", "\[IndentingNewLine]", "True", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", "1", "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"{", 
                  RowBox[{"{", "0", "}"}], "}"}], "}"}]}], "}"}]}], "]"}]}]}],
           "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"EdgS", "===", 
           RowBox[{"{", "1", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"IndV", ",", "IndF"}], "}"}], "=", 
            RowBox[{"CatmullClarkDefault", "[", "val", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"curve", "=", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"MemberQ", "[", 
                RowBox[{"edgeind", ",", 
                 RowBox[{
                 "stencil", "\[LeftDoubleBracket]", "k", 
                  "\[RightDoubleBracket]"}]}], "]"}], ",", 
               RowBox[{"AppendTo", "[", 
                RowBox[{"curve", ",", 
                 RowBox[{
                 "IndV", "\[LeftDoubleBracket]", "k", 
                  "\[RightDoubleBracket]"}]}], "]"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"k", ",", 
               RowBox[{"Length", "[", "stencil", "]"}]}], "}"}]}], "]"}], ";",
            "\[IndentingNewLine]", 
           RowBox[{"Print", "[", 
            RowBox[{"Show", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"CatmullClarkPlot", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"IndV", ",", "IndF"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"IndV", ",", "IndF"}], "}"}], ",", "Green"}], 
                 "]"}], ",", 
                RowBox[{"Graphics", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"Disk", "[", 
                    RowBox[{"#", ",", ".07"}], "]"}], "&"}], "/@", "curve"}], 
                 "]"}]}], "}"}], ",", 
              RowBox[{"ImageSize", "\[Rule]", "140"}]}], "]"}], "]"}], 
           ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"EdgS", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
            "V", "\[LeftDoubleBracket]", "EdgS", "\[RightDoubleBracket]"}], 
            ",", "EdgT"}], "}"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"m", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.605846228480858*^9, 3.6058462398955107`*^9}, {
   3.60584630124402*^9, 3.605846340747279*^9}, {3.6058464169596386`*^9, 
   3.605846420533843*^9}, {3.605846458206998*^9, 3.6058464704146957`*^9}, {
   3.6058513105105333`*^9, 3.6058513204771037`*^9}, {3.605851485700554*^9, 
   3.605851507644809*^9}, {3.6063891641917176`*^9, 3.606389214861616*^9}, {
   3.606393795352605*^9, 3.6063938285245023`*^9}, {3.6064273125263915`*^9, 
   3.6064273157805777`*^9}, {3.6064329361650457`*^9, 
   3.6064329842947984`*^9}, {3.606459691591164*^9, 3.6064597107402596`*^9}, {
   3.6064739922303047`*^9, 3.6064740025579634`*^9}, {3.6064742969749856`*^9, 
   3.606474300172535*^9}, {3.606644115925875*^9, 3.606644118109*^9}, {
   3.606695526082301*^9, 3.606695528989467*^9}, {3.606717935131093*^9, 
   3.6067179502063246`*^9}, {3.6067699996307235`*^9, 
   3.6067700183997965`*^9}, {3.606770049711588*^9, 3.6067701074848924`*^9}, {
   3.606770149964322*^9, 3.6067701848893194`*^9}, 3.6067702701511965`*^9, {
   3.6073263530603504`*^9, 3.607326389460909*^9}, {3.6081676637091475`*^9, 
   3.6081676640841694`*^9}, {3.6097550188717723`*^9, 
   3.6097550299424057`*^9}, {3.6097567719690437`*^9, 3.609756800489675*^9}, {
   3.609757479429508*^9, 3.6097574952134113`*^9}, {3.609757543730186*^9, 
   3.6097575464183397`*^9}, {3.6097585363909626`*^9, 
   3.6097585388361025`*^9}, {3.6097586640692654`*^9, 3.609758671067666*^9}, {
   3.6097587425337534`*^9, 3.6097587932626553`*^9}, {3.6097588946404533`*^9, 
   3.609758903596966*^9}, {3.609758965748521*^9, 3.609758966211547*^9}, {
   3.609759127540775*^9, 3.6097592691368732`*^9}, 3.6097593636612797`*^9, {
   3.609759399869351*^9, 3.6097594017524586`*^9}, {3.609759508751579*^9, 
   3.6097596257922735`*^9}, {3.609759727225075*^9, 3.6097599179989862`*^9}, {
   3.609760074421933*^9, 3.60976010635176*^9}, {3.6097604129582963`*^9, 
   3.6097604151564217`*^9}, {3.6097605044355288`*^9, 
   3.6097605797558365`*^9}, {3.609760735251731*^9, 3.609760745604323*^9}, {
   3.609760801251506*^9, 3.6097608112370768`*^9}, {3.6097610119795585`*^9, 
   3.609761204232555*^9}, {3.6097612427937603`*^9, 3.6097612452939034`*^9}, {
   3.60976129174156*^9, 3.609761293177642*^9}, {3.609761354907173*^9, 
   3.6097613644657197`*^9}, {3.60976143283963*^9, 3.609761454618876*^9}, {
   3.60976153109225*^9, 3.6097615717515755`*^9}, {3.609761670678234*^9, 
   3.609761675658519*^9}, {3.609761735720954*^9, 3.6097618145854645`*^9}, {
   3.6097621393540406`*^9, 3.6097622220837727`*^9}, {3.6097689058590627`*^9, 
   3.6097689093242607`*^9}, {3.609768991494961*^9, 3.6097690204256153`*^9}, {
   3.609769055033595*^9, 3.609769081976136*^9}, {3.6097695965275664`*^9, 
   3.6097696745630302`*^9}, {3.6097697276000633`*^9, 
   3.6097698103667974`*^9}, {3.6097726802209435`*^9, 3.60977269571483*^9}, {
   3.6097895672242*^9, 3.6097895844501853`*^9}, 3.609789783977597*^9, {
   3.6097898591658983`*^9, 3.6097898594879165`*^9}, {3.60978991636817*^9, 
   3.609789971850343*^9}, {3.6097963195314093`*^9, 3.6097963461509323`*^9}, {
   3.609796432026844*^9, 3.609796468868951*^9}, {3.6097969622241697`*^9, 
   3.6097969772430286`*^9}, {3.609797030570079*^9, 3.6097970658230953`*^9}, {
   3.6097971008520985`*^9, 3.6097971671428905`*^9}, {3.609799599055988*^9, 
   3.609799609430581*^9}, {3.609799656318263*^9, 3.609799660031475*^9}, {
   3.609800568084413*^9, 3.609800568414432*^9}, {3.609800608057699*^9, 
   3.6098007802695494`*^9}, {3.609801858226205*^9, 3.6098020063346763`*^9}, {
   3.609802049388139*^9, 3.609802056491545*^9}, {3.609802244684309*^9, 
   3.6098022625063286`*^9}, {3.609802404374443*^9, 3.6098024872931857`*^9}, {
   3.609802586708872*^9, 3.609802630823395*^9}, {3.6098026720977554`*^9, 
   3.609802681164274*^9}, {3.609802779556902*^9, 3.6098027939557257`*^9}, {
   3.6098028311588535`*^9, 3.6098028592384596`*^9}, {3.6098030663103037`*^9, 
   3.609803080841134*^9}, 3.6098031142680464`*^9, {3.6098031716613293`*^9, 
   3.6098031786517286`*^9}, {3.609803224965378*^9, 3.6098032623745174`*^9}, 
   3.609803831088046*^9, 3.6098038677341423`*^9, {3.609812415692233*^9, 
   3.6098124201923018`*^9}, {3.609812483530775*^9, 3.609812496485974*^9}, {
   3.6098125436391983`*^9, 3.6098125474242563`*^9}, {3.609812597742529*^9, 
   3.609812603457617*^9}, {3.609812856341501*^9, 3.6098129040197334`*^9}, {
   3.6098129597680893`*^9, 3.609812999878706*^9}, 3.609813034259234*^9, {
   3.6098130737813425`*^9, 3.6098130989767294`*^9}, {3.6098131366248074`*^9, 
   3.6098131651827464`*^9}, {3.609813597082381*^9, 3.609813667773466*^9}, {
   3.6098137496922245`*^9, 3.609813772287572*^9}, {3.6098138188907876`*^9, 
   3.609813893156928*^9}, {3.609813926934947*^9, 3.6098139553953843`*^9}, {
   3.6098139921884494`*^9, 3.6098139936384716`*^9}, {3.609814110131261*^9, 
   3.609814145559305*^9}, {3.6107047102802935`*^9, 3.6107047340226517`*^9}, {
   3.6107048065808015`*^9, 3.6107048095539713`*^9}, {3.6107048458510475`*^9, 
   3.610705096414379*^9}, {3.6107521362316923`*^9, 3.6107521407989535`*^9}, 
   3.6123188340085154`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkValences", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"rings", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", 
          RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Union", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"Join", "@@", 
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}], "-", "12"}], ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6090520563808985`*^9, 3.6090521099449625`*^9}, {
  3.6090521764047637`*^9, 3.6090521767737846`*^9}, {3.6090522343920803`*^9, 
  3.6090522945615215`*^9}, {3.609052366820655*^9, 3.60905236953981*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkDemo", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], ",", "depth_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"V1", "=", "V"}], ",", 
      RowBox[{"F1", "=", "F"}], ",", 
      RowBox[{"E1", "=", "E"}], ",", "delta", ",", 
      RowBox[{"lst", "=", 
       RowBox[{"{", "}"}]}], ",", "fcv", ",", 
      RowBox[{"vls", "=", 
       RowBox[{"{", "}"}]}], ",", "inf", ",", "val", ",", 
      RowBox[{"cmp", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"size", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "V", "]"}], "<", "100"}], ",", "120", ",", 
         "180"}], "]"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "V", "]"}], "<", "100"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"TableForm", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            "\"\<Vertices \[DownArrow]\>\"", ",", 
             "\"\<Faces \[DownArrow]\>\"", ",", 
             "\"\<Cycles \[DownArrow]\>\""}], "}"}], ",", 
           RowBox[{"MatrixForm", "/@", 
            RowBox[{"{", 
             RowBox[{"V", ",", "F", ",", "E"}], "}"}]}]}], "}"}], "]"}], 
        "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"vls", ",", 
       RowBox[{"LinearVolume", "[", 
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"lst", ",", 
       RowBox[{"Rasterize", "[", 
        RowBox[{
         RowBox[{"Show", "[", 
          RowBox[{
           RowBox[{"PlotMesh", "[", 
            RowBox[{"{", 
             RowBox[{"V", ",", "F", ",", "E"}], "}"}], "]"}], ",", 
           RowBox[{"Axes", "\[Rule]", "False"}], ",", 
           RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
           RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
           RowBox[{"ImageSize", "\[Rule]", "size"}]}], "]"}], ",", 
         RowBox[{"ImageResolution", "\[Rule]", "size"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V1", ",", "F1", ",", "E1"}], "}"}], "=", 
         RowBox[{"CatmullClarkSubdivide", "[", 
          RowBox[{"{", 
           RowBox[{"V1", ",", "F1", ",", "E1"}], "}"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"k", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"inf", "=", 
            RowBox[{"CatmullClarkVolume", "[", 
             RowBox[{"{", 
              RowBox[{"V1", ",", "F1", ",", "E1"}], "}"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"val", "=", 
            RowBox[{"CatmullClarkValences", "[", 
             RowBox[{"{", 
              RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"fcv", "=", 
         RowBox[{"CatmullClarkVolumeFace", "[", 
          RowBox[{"NumericMesh", "[", 
           RowBox[{"{", 
            RowBox[{"V1", ",", "F1", ",", "E1"}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"cmp", ",", 
          RowBox[{"Total", "[", "fcv", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"vls", ",", 
          RowBox[{"LinearVolume", "[", 
           RowBox[{"{", 
            RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"lst", ",", "\[IndentingNewLine]", 
          RowBox[{"Rasterize", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Show", "[", 
             RowBox[{
              RowBox[{"PlotMesh", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"V1", ",", "F1", ",", "E1"}], "}"}], ",", "fcv"}], 
               "]"}], ",", 
              RowBox[{"Axes", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
              RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
              RowBox[{"ImageSize", "\[Rule]", "size"}]}], "]"}], 
            "\[IndentingNewLine]", ",", 
            RowBox[{"ImageResolution", "\[Rule]", "size"}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "depth"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"delta", "=", 
      RowBox[{"N", "[", 
       RowBox[{"vls", "-", "inf"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "lst", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Required valences \[Tau](f) \[Element] \>\"", "<>", 
       RowBox[{"ToString", "[", "val", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Limit volume \[DownArrow] \>\"", "<>", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"InexactNumberQ", "[", "inf", "]"}], "\[Equal]", "False"}], 
         ",", 
         RowBox[{"\"\<(\[TildeTilde]\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"N", "[", 
            RowBox[{"inf", ",", "30"}], "]"}], "]"}], "<>", "\"\<)\>\""}], 
         ",", "\"\<\>\""}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "inf", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TableForm", "[", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"MatrixForm", "[", 
           RowBox[{"Transpose", "[", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Level\>\"", "}"}], ",", 
                RowBox[{"Range", "[", 
                 RowBox[{"0", ",", "depth"}], "]"}]}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Volume\>\"", "}"}], ",", "vls"}], "]"}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Delta to \[Infinity]\>\"", "}"}], ",", 
                "delta"}], "]"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
           "]"}], ",", 
          RowBox[{"ListLogPlot", "[", 
           RowBox[{
            RowBox[{"Transpose", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Range", "[", 
                RowBox[{"0", ",", "depth"}], "]"}], ",", 
               RowBox[{"delta", "/", 
                RowBox[{
                "delta", "\[LeftDoubleBracket]", "1", 
                 "\[RightDoubleBracket]"}]}]}], "}"}], "]"}], ",", 
            RowBox[{"Joined", "\[Rule]", "True"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "200"}], ",", 
            RowBox[{"AxesLabel", "\[Rule]", 
             RowBox[{"{", 
              RowBox[{"\"\<Level\>\"", ",", "\"\<Rel. error\>\""}], "}"}]}]}],
            "]"}]}], "}"}], "}"}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"cmp", "=", 
      RowBox[{"Union", "[", 
       RowBox[{"Chop", "[", 
        RowBox[{"cmp", "-", "inf"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"1", "<", 
        RowBox[{"Length", "[", "cmp", "]"}]}], ",", 
       RowBox[{"Print", "[", "cmp", "]"}]}], "]"}], ";"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6058475746868567`*^9, 3.6058475863275223`*^9}, {
   3.605847712971766*^9, 3.605847801399824*^9}, {3.6058478714508305`*^9, 
   3.605848060577648*^9}, {3.6058480917024283`*^9, 3.6058481842667227`*^9}, {
   3.605848220830814*^9, 3.605848232770497*^9}, {3.605848430292794*^9, 
   3.6058484888111415`*^9}, {3.6058485310395565`*^9, 
   3.6058485795443316`*^9}, {3.605849087329375*^9, 3.6058492709878798`*^9}, {
   3.6058493011656055`*^9, 3.6058493015046253`*^9}, {3.605849336046601*^9, 
   3.6058493744447966`*^9}, {3.605849408872766*^9, 3.6058494246496687`*^9}, {
   3.605849768383329*^9, 3.6058498087546377`*^9}, {3.6058498835739174`*^9, 
   3.6058499010319157`*^9}, {3.605850027835169*^9, 3.605850032282423*^9}, {
   3.605850349872588*^9, 3.6058503649824524`*^9}, {3.6058506459265213`*^9, 
   3.6058507522946053`*^9}, {3.605850784478446*^9, 3.6058508656410885`*^9}, {
   3.6058510063751383`*^9, 3.605851007900225*^9}, {3.6058512689691577`*^9, 
   3.605851304798207*^9}, {3.605851497158209*^9, 3.60585152689191*^9}, {
   3.6058519900824027`*^9, 3.6058519952636995`*^9}, {3.605852040193269*^9, 
   3.605852134135642*^9}, {3.606390965123725*^9, 3.6063909887820787`*^9}, {
   3.6063910565859566`*^9, 3.606391058659075*^9}, 3.606393871599966*^9, 
   3.606643733415997*^9, {3.6066955813704634`*^9, 3.606695672685686*^9}, {
   3.606695752756266*^9, 3.606695785678149*^9}, {3.606696052527412*^9, 
   3.606696159506531*^9}, {3.6067355134675765`*^9, 3.6067355344277754`*^9}, {
   3.6067362776572857`*^9, 3.606736282335553*^9}, {3.607504947722542*^9, 
   3.607504951046732*^9}, {3.607505159920679*^9, 3.607505161302758*^9}, {
   3.6075052135017433`*^9, 3.6075052179419975`*^9}, {3.608865767131858*^9, 
   3.6088658541768365`*^9}, {3.608865888424795*^9, 3.6088658909159374`*^9}, {
   3.608866064113844*^9, 3.6088660644038606`*^9}, {3.608888725154479*^9, 
   3.608888738365235*^9}, {3.6088888189568443`*^9, 3.6088888281233687`*^9}, {
   3.608888961535*^9, 3.6088889694914546`*^9}, {3.608889061777733*^9, 
   3.6088890929555163`*^9}, {3.6088891549060593`*^9, 3.608889219843774*^9}, 
   3.608889282453355*^9, {3.608889324960786*^9, 3.6088893313791533`*^9}, {
   3.6088894818297586`*^9, 3.6088895153966784`*^9}, {3.6088896071199245`*^9, 
   3.608889620368683*^9}, {3.6088897202953978`*^9, 3.6088897307819977`*^9}, {
   3.608889822528246*^9, 3.6088898321957984`*^9}, 3.608889871724059*^9, {
   3.6088899110333076`*^9, 3.608889925648144*^9}, {3.6088899969352207`*^9, 
   3.6088900037016077`*^9}, {3.6088900766147785`*^9, 3.608890079974971*^9}, {
   3.6088902064802065`*^9, 3.608890219030924*^9}, 3.608890290382005*^9, {
   3.6088955971285334`*^9, 3.6088956194608107`*^9}, 3.6090216522411985`*^9, {
   3.6090216921344805`*^9, 3.6090216983198347`*^9}, {3.609021748109682*^9, 
   3.609021821870901*^9}, {3.609051861852772*^9, 3.6090519150998178`*^9}, {
   3.6090521220836563`*^9, 3.609052129640089*^9}, {3.6090534623493156`*^9, 
   3.609053463361373*^9}, {3.6097550624252634`*^9, 3.609755062961294*^9}, {
   3.609762293430853*^9, 3.609762350742131*^9}, {3.6097628115394874`*^9, 
   3.6097628347178135`*^9}, {3.609763150632882*^9, 3.6097631642116594`*^9}, {
   3.609763687228574*^9, 3.6097636890366774`*^9}, {3.6097648029013867`*^9, 
   3.609764807117628*^9}, {3.609769138800386*^9, 3.6097691485809455`*^9}, {
   3.6097890300034723`*^9, 3.609789034807747*^9}, {3.609808007491923*^9, 
   3.6098080082999687`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Mesh examples", "Subsection",
 CellChangeTimes->{{3.6066937912720757`*^9, 3.6066938001225815`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CubeMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"0", "1", "0"},
        {"1", "1", "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {"0", "1", "1"},
        {"1", "1", "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "6", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", "0", "0"},
         {"1", "0", "0"},
         {"0", "1", "0"},
         {"1", "1", "0"},
         {"0", "0", "1"},
         {"1", "0", "1"},
         {"0", "1", "1"},
         {"1", "1", "1"},
         {"1", "0", "2"},
         {"1", "1", "2"},
         {"2", "0", "0"},
         {"2", "1", "0"},
         {"2", "0", "1"},
         {"2", "1", "1"},
         {"2", "0", "2"},
         {"2", "1", "2"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "11", ",", "13", ",", "6"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "13", ",", "15", ",", "9"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "12", ",", "11", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"12", ",", "14", ",", "13", ",", "11"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"14", ",", "16", ",", "15", ",", "13"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "9", ",", "10", ",", "8"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "14", ",", "12"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"8", ",", "10", ",", "16", ",", "14"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", "0", "0"},
         {"1", "0", "0"},
         {"0", "1", "0"},
         {"1", "1", "0"},
         {"0", "0", "1"},
         {"1", "0", "1"},
         {"0", "1", "1"},
         {"1", "1", "1"},
         {"1", "0", "2"},
         {"1", "1", "2"},
         {"2", "0", "0"},
         {"2", "1", "0"},
         {"2", "0", "1"},
         {"2", "1", "1"},
         {"2", "0", "2"},
         {"2", "1", "2"},
         {"1", 
          RowBox[{"-", "1"}], "0"},
         {"2", 
          RowBox[{"-", "1"}], "0"},
         {"1", 
          RowBox[{"-", "1"}], "1"},
         {"2", 
          RowBox[{"-", "1"}], "1"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "13", ",", "15", ",", "9"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "12", ",", "11", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"12", ",", "14", ",", "13", ",", "11"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"14", ",", "16", ",", "15", ",", "13"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "9", ",", "10", ",", "8"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "14", ",", "12"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"8", ",", "10", ",", "16", ",", "14"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"19", ",", "20", ",", "13", ",", "6"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"11", ",", "13", ",", "20", ",", "18"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"11", ",", "18", ",", "17", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"17", ",", "19", ",", "6", ",", "2"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TetrahedronMesh", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"A", "=", 
       RowBox[{"(", GridBox[{
          {"1", 
           FractionBox["1", "2"], 
           FractionBox["1", "2"]},
          {"0", 
           FractionBox[
            SqrtBox["3"], "2"], 
           FractionBox["1", 
            RowBox[{"2", 
             SqrtBox["3"]}]]},
          {"0", "0", 
           SqrtBox[
            FractionBox["2", "3"]]}
         }], ")"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", GridBox[{
           {"0", "0", "0"},
           {"1", "0", "0"},
           {"0", "1", "0"},
           {"0", "0", "1"}
          }], ")"}], ".", 
        RowBox[{"Transpose", "[", "A", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", "2", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"OctahedronMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"1", "1", "0"},
        {"0", "1", "0"},
        {
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", 
          SqrtBox["2"]}]},
        {
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", "2"}], 
         RowBox[{
          RowBox[{"-", "1"}], "/", 
          SqrtBox["2"]}]}
       }], ")"}], ",", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2", ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "3", ",", "2"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "4", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "1", ",", "4"}], "}"}]}], "\[IndentingNewLine]", 
      "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TriQuadPrismMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {
         RowBox[{"1", "/", "2"}], 
         FractionBox[
          SqrtBox["3"], "2"], "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {
         RowBox[{"1", "/", "2"}], 
         FractionBox[
          SqrtBox["3"], "2"], "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1", ",", "4", ",", "6"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TwistedPrismMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"0", "1", "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {"0", "1", "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "2", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "2", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "4", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.606693175688866*^9, 3.6066931925668316`*^9}, {
  3.6066934092942276`*^9, 3.6066934695606747`*^9}, {3.6066936984677677`*^9, 
  3.6066937331517515`*^9}, {3.606693785456743*^9, 3.6066937858307643`*^9}, {
  3.606694300801219*^9, 3.6066943224624577`*^9}, {3.6066944264494057`*^9, 
  3.6066944405282106`*^9}, {3.6066944879599237`*^9, 3.6066944972164536`*^9}, {
  3.6088434287549133`*^9, 3.6088434886233373`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TorusMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {
          RowBox[{"3", "/", "2"}], "0", "0"},
         {"1.2135254915624212`", "0.8816778784387097`", "0"},
         {"0.4635254915624212`", "1.4265847744427302`", "0"},
         {
          RowBox[{"-", "0.463525491562421`"}], "1.4265847744427305`", "0"},
         {
          RowBox[{"-", "1.213525491562421`"}], "0.8816778784387098`", "0"},
         {
          RowBox[{
           RowBox[{"-", "3"}], "/", "2"}], "0", "0"},
         {
          RowBox[{"-", "1.2135254915624212`"}], 
          RowBox[{"-", "0.8816778784387096`"}], "0"},
         {
          RowBox[{"-", "0.46352549156242134`"}], 
          RowBox[{"-", "1.4265847744427302`"}], "0"},
         {"0.46352549156242084`", 
          RowBox[{"-", "1.4265847744427305`"}], "0"},
         {"1.213525491562421`", 
          RowBox[{"-", "0.88167787843871`"}], "0"},
         {"1", "0", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0.8660254037844386`", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], "0.8660254037844387`", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"-", "1"}], "0", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], 
          RowBox[{"-", "0.8660254037844385`"}], 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], 
          RowBox[{"-", "0.8660254037844386`"}], 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0", "0"},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "4"}], "0.43301270189221935`", "0"},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "4"}], 
          RowBox[{"-", "0.43301270189221924`"}], "0"},
         {"1", "0", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0.8660254037844385`", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], "0.8660254037844386`", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"-", "1"}], "0", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], 
          RowBox[{"-", "0.8660254037844384`"}], 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], 
          RowBox[{"-", "0.8660254037844385`"}], 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"3", "4", "13"},
         {"12", "2", "3"},
         {"18", "13", "14"},
         {"23", "18", "19"},
         {"21", "20", "17"},
         {"22", "4", "3"},
         {"24", "19", "25"},
         {"8", "24", "9"},
         {"7", "6", "23"},
         {"6", "14", "5"},
         {"14", "13", "5"},
         {"11", "1", "2"},
         {"12", "3", "13"},
         {"13", "4", "5"},
         {"18", "12", "13"},
         {"11", "2", "12"},
         {"10", "16", "9"},
         {"19", "18", "14"},
         {"17", "11", "12"},
         {"18", "17", "12"},
         {"2", "20", "21"},
         {"23", "5", "22"},
         {"2", "1", "20"},
         {"3", "2", "21"},
         {"22", "3", "21"},
         {"22", "5", "4"},
         {"24", "23", "19"},
         {"21", "17", "18"},
         {"18", "22", "21"},
         {"23", "6", "5"},
         {"18", "23", "22"},
         {"24", "7", "23"},
         {"7", "24", "8"},
         {"24", "25", "9"},
         {"25", "10", "9"},
         {"25", "19", "17"},
         {"25", "20", "10"},
         {"10", "20", "1"},
         {"25", "17", "20"},
         {"16", "10", "11"},
         {"19", "16", "17"},
         {"16", "15", "9"},
         {"19", "14", "15"},
         {"17", "16", "11"},
         {"19", "15", "16"},
         {"7", "14", "6"},
         {"15", "14", "7"},
         {"15", "8", "9"},
         {"15", "7", "8"},
         {"10", "1", "11"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.606694128983392*^9, 3.60669414801748*^9}, {
  3.6088426029046774`*^9, 3.6088426832282715`*^9}, {3.608842745425829*^9, 
  3.6088428057522793`*^9}, {3.60884346279786*^9, 3.6088434648269763`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Torus", "[", 
    RowBox[{"R_", ",", "r_", ",", "\[Theta]_", ",", "\[Phi]_"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{"r", " ", 
      RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"TorusVolume", "[", 
    RowBox[{"R_", ",", "r_"}], "]"}], "=", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Pi]", "2"], "R", " ", 
    SuperscriptBox["r", "2"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TorusSample", "[", 
   RowBox[{"N_", ",", "M_", ",", "i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"Torus", "[", 
   RowBox[{"2", ",", "1", ",", 
    RowBox[{
     FractionBox[
      RowBox[{"2", "\[Pi]", " ", "i"}], "N"], "+", 
     FractionBox["\[Pi]", "4"]}], ",", 
    FractionBox[
     RowBox[{"2", "\[Pi]", " ", "j"}], "M"]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6065531864185305`*^9, 3.6065531923228683`*^9}, {
  3.6065544938033085`*^9, 3.606554505849998*^9}, {3.606554616960353*^9, 
  3.606554618703453*^9}, {3.6067694316322355`*^9, 3.6067694620499754`*^9}, {
  3.608806220050144*^9, 3.608806223829361*^9}, {3.6088425841666055`*^9, 
  3.608842590787984*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"RegularTorusTria", "[", 
   RowBox[{"N_", ",", "M_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "func", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"func", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"i", " ", "M"}], "+", "j"}], ",", 
         RowBox[{"N", " ", "M"}]}], "]"}], "+", "1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"TorusSample", "[", 
            RowBox[{"N", ",", "M", ",", "i", ",", "j"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
             RowBox[{"N", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
             RowBox[{"M", "-", "1"}]}], "}"}]}], "]"}], ",", "1"}], "]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Table", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"func", "[", 
                RowBox[{"i", ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{"i", ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}]}], "}"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{"i", ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}]}], "}"}]}], 
            "\[IndentingNewLine]", "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
             RowBox[{"N", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
             RowBox[{"M", "-", "1"}]}], "}"}]}], "]"}], ",", "2"}], "]"}]}], 
      "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6065532167602663`*^9, 3.6065533077684717`*^9}, {
  3.606553339978314*^9, 3.6065533556702113`*^9}, {3.60655345215273*^9, 
  3.606553453766822*^9}, {3.606553484510581*^9, 3.6065535192295666`*^9}, {
  3.6065536185682487`*^9, 3.6065536493200073`*^9}, {3.606553718464962*^9, 
  3.6065537427133493`*^9}, {3.6065537860728292`*^9, 3.6065538299113364`*^9}, {
  3.6065538753049326`*^9, 3.606553950997262*^9}, {3.6065540157739673`*^9, 
  3.6065540528860903`*^9}, {3.6067688938164744`*^9, 3.606768926741358*^9}, {
  3.6067692563772116`*^9, 3.606769273456188*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Mesh with creases examples", "Subsection",
 CellChangeTimes->{{3.6066937912720757`*^9, 3.6066938001225815`*^9}, {
  3.60975273199197*^9, 3.6097527340790896`*^9}, {3.6104994088206325`*^9, 
  3.610499409024644*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotMeshIndex", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Show", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"PlotMesh", "[", 
      RowBox[{"{", 
       RowBox[{"V", ",", "F", ",", "E"}], "}"}], "]"}], ",", 
     RowBox[{"Graphics3D", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Text", "[", 
         RowBox[{
          RowBox[{"Style", "[", 
           RowBox[{"k", ",", "Bold", ",", "12"}], "]"}], ",", 
          RowBox[{"V", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}],
           ",", 
          RowBox[{"Background", "\[Rule]", "LightRed"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", 
          RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}], "]"}]}], "}"}],
    "]"}]}]], "Input",
 CellChangeTimes->{{3.6097956911824703`*^9, 3.609795795010409*^9}, {
  3.6097958316185026`*^9, 3.6097958512986283`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CubePlainMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CubeMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CylinderMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CubeMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}]}], "}"}], "}"}]}],
     "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TurtleMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CubeMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "4", ",", "8", ",", "7", ",", "5"}], 
        "}"}], "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TennisBallMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CubeMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
        "1", ",", "2", ",", "4", ",", "3", ",", "7", ",", "8", ",", "6", ",", 
         "5"}], "}"}], "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ZuckerHutMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CubeMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "}"}], "}"}], "}"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.60975275506629*^9, 3.6097527636917834`*^9}, {
  3.609752862545438*^9, 3.609752862746449*^9}, {3.6097529174265766`*^9, 
  3.609752931238366*^9}, {3.6097530231126213`*^9, 3.6097530293519783`*^9}, {
  3.609756887080628*^9, 3.6097569001363745`*^9}, {3.6097594158792667`*^9, 
  3.6097594163462934`*^9}}],

Cell[BoxData[
 RowBox[{"CubeCylinerMesh", ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "F", ",", "E"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "F", ",", "E"}], "}"}], "=", 
      RowBox[{"Nest", "[", 
       RowBox[{"QuadLinearSubdivide", ",", "CylinderMesh", ",", "2"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"pnt", "=", 
             RowBox[{"#", "\[LeftDoubleBracket]", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
            "}"}], ",", 
           RowBox[{"Which", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"Norm", "[", 
               RowBox[{"pnt", "-", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}]}], "]"}], "\[LessEqual]", 
              RowBox[{
               SqrtBox["2"], "/", "4"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "0", ",", 
               RowBox[{
               "#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
              "}"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Norm", "[", 
               RowBox[{"pnt", "-", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0"}], "}"}]}], "]"}], "\[LessEqual]", 
              RowBox[{
               SqrtBox["2"], "/", "4"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "0", ",", 
               RowBox[{
               "#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
              "}"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Norm", "[", 
               RowBox[{"pnt", "-", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "]"}], "\[LessEqual]", 
              RowBox[{
               SqrtBox["2"], "/", "4"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "1", ",", 
               RowBox[{
               "#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
              "}"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Norm", "[", 
               RowBox[{"pnt", "-", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "1"}], "}"}]}], "]"}], "\[LessEqual]", 
              RowBox[{
               SqrtBox["2"], "/", "4"}]}], ",", 
             RowBox[{"{", 
              RowBox[{"1", ",", "1", ",", 
               RowBox[{
               "#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
              "}"}], ",", "\[IndentingNewLine]", "True", ",", "#"}], 
            "\[IndentingNewLine]", "]"}]}], "]"}], "&"}], "/@", "V"}], ",", 
       "F", ",", "E"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6097526879224496`*^9, 3.609752698193037*^9}, {
  3.6097532651284637`*^9, 3.609753293639095*^9}, {3.609772552574643*^9, 
  3.609772599167308*^9}, {3.609789005572075*^9, 3.609789011204397*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CornerPlainMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerTubeMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "5", ",", "7", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "10", ",", "16", ",", "15"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerRingMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"6", ",", "13", ",", "14", ",", "8"}], "}"}], "}"}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerRing2Mesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"2", ",", "6", ",", "8", ",", "4"}], "}"}], "}"}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerSplitMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "5", ",", "7", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "10", ",", "16", ",", "15"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "6", ",", "13", ",", "11"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerBraceMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"2", ",", "6", ",", "13", ",", "14", ",", "8", ",", "4"}], 
        "}"}], "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerShoeMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
        "2", ",", "6", ",", "13", ",", "15", ",", "16", ",", "14", ",", "8", 
         ",", "4"}], "}"}], "}"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerCircuitMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
        "2", ",", "6", ",", "13", ",", "15", ",", "16", ",", "10", ",", "8", 
         ",", "7", ",", "3", ",", "4"}], "}"}], "}"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FlatXCornerMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
        "1", ",", "2", ",", "11", ",", "13", ",", "15", ",", "9", ",", "6", 
         ",", "5"}], "}"}], "}"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FlatYCornerMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"CornerMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "11", ",", "12", ",", "14", ",", "16", ",", "15", ",", "13"}], "}"}],
         ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "3", ",", "7", ",", "5"}], "}"}]}], "}"}], "}"}]}],
     "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6097629977451377`*^9, 3.6097630732504563`*^9}, {
  3.609763114537818*^9, 3.609763124001359*^9}, {3.6097631929543033`*^9, 
  3.6097632271132565`*^9}, {3.609763929987459*^9, 3.6097639731169257`*^9}, {
  3.609764028101071*^9, 3.6097640421488743`*^9}, {3.6097958720168133`*^9, 
  3.609795890187853*^9}, {3.6097972794023113`*^9, 3.6097973683313975`*^9}, {
  3.609802141355399*^9, 3.6098021723701725`*^9}, {3.6098039135677633`*^9, 
  3.6098039369491014`*^9}, {3.609804021342928*^9, 3.6098040236620607`*^9}, {
  3.6098042245395503`*^9, 3.609804248709933*^9}, {3.609804332351717*^9, 
  3.609804339645134*^9}, {3.6098045866802635`*^9, 3.60980460358023*^9}, {
  3.609804656803274*^9, 3.609804661758558*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"TripodPlainMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodBraceMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"11", ",", "13", ",", "14", ",", "8", ",", "4", ",", "2"}], 
        "}"}], "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodBoltMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "2", ",", "11", ",", "13", ",", "15", ",", "9", ",", "6", ",", "5", 
          ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodRingMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"2", ",", "11", ",", "13", ",", "6"}], "}"}], "}"}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodPlatesMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"13", ",", "6", ",", "19", ",", "20"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"10", ",", "9", ",", "15", ",", "16"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodTubeMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "5", ",", "7", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "10", ",", "16", ",", "15"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodTube2Mesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"5", ",", "7", ",", "8", ",", "6"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"11", ",", "12", ",", "14", ",", "13"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodTube3Mesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"5", ",", "7", ",", "8", ",", "6"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodSplitMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "5", ",", "7", ",", "3"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"11", ",", "12", ",", "14", ",", "13"}], "}"}]}], "}"}], 
      "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodCircuitMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"TripodMesh", ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{
        "6", ",", "19", ",", "20", ",", "13", ",", "11", ",", "12", ",", "14",
          ",", "16", ",", "15", ",", "9", ",", "10", ",", "8", ",", "4", ",", 
         "3", ",", "1", ",", "5"}], "}"}], "}"}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6098122064815197`*^9, 3.6098122187092075`*^9}, {
   3.6098122708050075`*^9, 3.60981229246034*^9}, {3.609812655570917*^9, 
   3.6098126564909315`*^9}, {3.6098127152193336`*^9, 
   3.6098127376096773`*^9}, {3.6098132782569833`*^9, 
   3.6098132998948154`*^9}, {3.6098133911962175`*^9, 
   3.6098133918687277`*^9}, {3.6098134308648276`*^9, 3.609813431487337*^9}, {
   3.6098134810530987`*^9, 3.6098134812381015`*^9}, {3.6098135133785954`*^9, 
   3.609813525951288*^9}, {3.6098142483633842`*^9, 3.609814259666058*^9}, {
   3.6098143485442953`*^9, 3.609814404011468*^9}, {3.6098144806588516`*^9, 
   3.6098144867542*^9}, {3.609814688493739*^9, 3.609814741698782*^9}, {
   3.609814859007492*^9, 3.6098149071112432`*^9}, {3.609814957491125*^9, 
   3.6098149733460317`*^9}, {3.609816097642338*^9, 3.6098160982343717`*^9}, {
   3.61011468389878*^9, 3.61011469718754*^9}, {3.6101147391499405`*^9, 
   3.6101147552368603`*^9}, 3.61012087969716*^9, {3.610227067628752*^9, 
   3.610227080930513*^9}, {3.6105170970469112`*^9, 3.610517097314926*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print11PlainMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"<<", "\"\<Print11.mx\>\""}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Print11SimpleMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"<<", "\"\<Print11.mx\>\""}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "36", ",", "33", ",", "45", ",", "48", ",", "34", ",", "35"}], "}"}],
         ",", 
        RowBox[{"{", 
         RowBox[{"20", ",", "23", ",", "22", ",", "17"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
         "92", ",", "91", ",", "74", ",", "73", ",", "169", ",", "170", ",", 
          "187", ",", "188"}], "}"}]}], "}"}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.609815139736549*^9, 3.6098151557364635`*^9}, {
  3.609815212200693*^9, 3.6098152469696817`*^9}, {3.6098153266832414`*^9, 
  3.609815331785533*^9}, {3.609815485338316*^9, 3.609815507776599*^9}, {
  3.609815608914384*^9, 3.609815617335866*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Print11PresentMesh", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"<<", "\"\<Print11.mx\>\""}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
         "19", ",", "18", ",", "11", ",", "10", ",", "62", ",", "61", ",", 
          "54", ",", "68", ",", "69", ",", "72", ",", "37", ",", "40", ",", 
          "45", ",", "33", ",", "2", ",", "1", ",", "27", ",", "26"}], "}"}], 
        ",", 
        RowBox[{"{", 
         RowBox[{
         "20", ",", "23", ",", "30", ",", "31", ",", "92", ",", "89", ",", 
          "84", ",", "87", ",", "94", ",", "95", ",", "43", ",", "44", ",", 
          "35", ",", "36", ",", "3", ",", "4", ",", "28", ",", "25"}], "}"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{
         "81", ",", "76", ",", "73", ",", "14", ",", "15", ",", "22", ",", 
          "17", ",", "12", ",", "9", ",", "63", ",", "64", ",", "55", ",", 
          "56", ",", "7", ",", "8", ",", "78", ",", "79", ",", "86"}], "}"}], 
        ",", 
        RowBox[{"{", 
         RowBox[{
         "5", ",", "6", ",", "53", ",", "65", ",", "60", ",", "57", ",", "52",
           ",", "49", ",", "48", ",", "34", ",", "41", ",", "42", ",", "96", 
          ",", "93", ",", "88", ",", "85", ",", "80", ",", "77"}], "}"}]}], 
       "}"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotMeshIndex", "[", "Print11PresentMesh", "]"}]}], "Input",
 CellChangeTimes->{{3.610227839086877*^9, 3.610227870271661*^9}, {
  3.610227922952674*^9, 3.61022798512423*^9}, {3.6102280976246643`*^9, 
  3.6102281739910326`*^9}, {3.610228407462386*^9, 3.610228511957363*^9}, {
  3.6102286171013765`*^9, 3.610228731566924*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"msh", "=", 
    RowBox[{"Nest", "[", 
     RowBox[{"CatmullClarkSubdivide", ",", "TripodCircuitMesh", ",", "4"}], 
     "]"}]}], ";", "\[IndentingNewLine]", 
   RowBox[{"vol", "=", 
    RowBox[{"CatmullClarkVolumeFace", "[", "msh", "]"}]}], ";", 
   "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"PlotMesh", "[", 
      RowBox[{"msh", ",", "vol"}], "]"}], ",", 
     RowBox[{"Boxed", "\[Rule]", "False"}]}], "]"}]}], "*)"}]], "Input",
 CellChangeTimes->{{3.610517171611176*^9, 3.6105172857527046`*^9}, {
  3.610517352621529*^9, 3.6105174091927648`*^9}, {3.610519536456437*^9, 
  3.6105195404966683`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Grid visualization", "Subsection",
 CellChangeTimes->{{3.609881196233368*^9, 3.609881217807602*^9}, {
   3.6101819655132694`*^9, 3.6101819709365797`*^9}, 3.6102372481770535`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkDefault", "[", "n_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"n", "\[Equal]", "3"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}], ",", 
      RowBox[{"With", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"m", "=", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{"n", "-", "3"}], ")"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"OddQ", "[", "k", "]"}], ",", 
             SqrtBox["2"], ",", "1"}], "]"}], 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Sin", "[", 
              FractionBox[
               RowBox[{"k", " ", "\[Pi]"}], 
               RowBox[{"m", " ", "2"}]], "]"}], ",", 
             RowBox[{"Cos", "[", 
              FractionBox[
               RowBox[{"k", " ", "\[Pi]"}], 
               RowBox[{"m", " ", "2"}]], "]"}]}], "}"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "0", ",", 
            RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ring", "=", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"10", ",", "14"}], "}"}], ",", 
          RowBox[{"14", "+", 
           RowBox[{"Range", "[", 
            RowBox[{"2", 
             RowBox[{"(", 
              RowBox[{"n", "-", "3"}], ")"}]}], "]"}]}], ",", 
          RowBox[{"{", "12", "}"}]}], "]"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}]}], "}"}], 
        ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "11", "}"}], ",", 
            RowBox[{"ring", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"2", "k"}], "+", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0", ",", 
                RowBox[{"-", "1"}]}], "}"}]}], "\[RightDoubleBracket]"}]}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"n", "-", "2"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"Vs_", ",", "Fs_"}], "}"}], ",", "col_"}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Gray", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"Vs", "\[LeftDoubleBracket]", 
            RowBox[{"#", "\[LeftDoubleBracket]", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"Range", "[", 
                RowBox[{"Length", "[", "#", "]"}], "]"}], ",", 
               RowBox[{"{", "1", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", "Fs"}]}], "}"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"EdgeForm", "[", 
         RowBox[{"{", "Dashed", "}"}], "]"}], ",", "LightGreen", ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
          "&"}], "/@", "F"}], 
        RowBox[{"(*", 
         RowBox[{"\[LeftDoubleBracket]", 
          RowBox[{"Drop", "[", 
           RowBox[{
            RowBox[{"Range", "[", 
             RowBox[{"Length", "[", "F", "]"}], "]"}], ",", 
            RowBox[{"{", "5", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
         "*)"}], ",", "\[IndentingNewLine]", 
        RowBox[{"EdgeForm", "[", "Dashed", "]"}], ",", "col", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Polygon", "[", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{
           RowBox[{
           "F", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}], ",", 
           "All"}], "\[RightDoubleBracket]"}], "]"}]}], "\[IndentingNewLine]",
        "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "col", "}"}], "]"}], "]"}], "\[Equal]", "0"}], ",", 
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"col", ",", 
          RowBox[{"Opacity", "[", "0.5", "]"}], ",", 
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", "Green", ",", "\[IndentingNewLine]", 
      RowBox[{"EdgeForm", "[", 
       RowBox[{"{", "Blue", "}"}], "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Disk", "[", 
         RowBox[{"#", ",", "0.04"}], "]"}], "&"}], "/@", "Vs"}]}], 
     "\[IndentingNewLine]", "}"}], ",", 
    RowBox[{"Axes", "\[Rule]", "False"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkDecomposition", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"fine", "=", 
       RowBox[{"sub", ".", "V"}]}], ",", 
      RowBox[{"Freg", "=", 
       RowBox[{
        RowBox[{"CatmullClarkDefault", "[", "4", "]"}], 
        "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"CatmullClarkPlot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", "Red"}], "]"}], "}"}], ",", 
      "\"\<=\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "F"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Red"}], "]"}]}], "}"}]}], 
     "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"GraphicsWeights", "[", 
   RowBox[{"V_", ",", "s_", ",", 
    RowBox[{"size_:", "Medium"}]}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
        "\[NotEqual]", "0"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"Style", "[", 
          RowBox[{
           RowBox[{
           "s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], ",", 
           "Bold", ",", "size"}], "]"}], ",", 
         RowBox[{"V", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
         ",", 
         RowBox[{"Background", "\[Rule]", "LightRed"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", "s", "]"}]}], "}"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkStencils", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"pnt", "=", 
      RowBox[{"sub", ".", "V"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", 
          RowBox[{"{", "}"}]}], "]"}], ",", 
        RowBox[{"GraphicsWeights", "[", 
         RowBox[{"V", ",", 
          RowBox[{
          "sub", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Graphics", "[", 
         RowBox[{"{", 
          RowBox[{"Red", ",", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{
             "pnt", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             ",", "0.05"}], "]"}]}], "}"}], "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", "sub", "]"}]}], "}"}]}], "]"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkShow", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"CatmullClarkStencils", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], ",", "sub"}], "]"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"CatmullClarkDecomposition", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], ",", "sub", ",", "ind"}], "]"}], 
      "]"}], ";"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkIndexing", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "F"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "F"}], "}"}], "=", 
      RowBox[{"CatmullClarkDefault", "[", "n", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"CatmullClarkPlot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
       RowBox[{"GraphicsWeights", "[", 
        RowBox[{"V", ",", 
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", "V", "]"}], "]"}], ",", "Large"}], "]"}]}], 
      "]"}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{"GetAt", "[", 
      RowBox[{"frm_", ",", 
       RowBox[{"{", 
        RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}]}], "]"}], ":=", 
     RowBox[{"frm", "\[LeftDoubleBracket]", 
      RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}]}], ";"}], 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.6060114962818995`*^9, 3.606011518097147*^9}, {
   3.6060115842759323`*^9, 3.6060116469145155`*^9}, {3.606011777914008*^9, 
   3.6060118162672014`*^9}, 3.6060118524432707`*^9, {3.6060118846491127`*^9, 
   3.6060119399772778`*^9}, {3.606012010244296*^9, 3.6060120108773327`*^9}, {
   3.6060120671385508`*^9, 3.606012166147214*^9}, {3.6060142069449406`*^9, 
   3.6060142072369576`*^9}, {3.606014237686699*^9, 3.6060142666163535`*^9}, 
   3.606386402256744*^9, {3.6064268688920174`*^9, 3.6064268691570325`*^9}, {
   3.6064649509399815`*^9, 3.6064649511279926`*^9}, {3.606633085667821*^9, 
   3.6066330977625127`*^9}, {3.606947253359164*^9, 3.6069472553662786`*^9}, 
   3.6069473427912793`*^9, 3.6069474110431833`*^9, {3.607026405576552*^9, 
   3.607026413045979*^9}, {3.6098410968617277`*^9, 3.609841115542796*^9}, {
   3.6098412371447515`*^9, 3.609841242625065*^9}, 3.609841668314413*^9, {
   3.609841749609063*^9, 3.609841804379195*^9}, {3.609842561892523*^9, 
   3.6098426096722555`*^9}, {3.6098426884527617`*^9, 
   3.6098426964242177`*^9}, {3.6098427590688004`*^9, 3.6098427602368674`*^9}, 
   3.610240950088791*^9, {3.612318913296051*^9, 3.6123189139250865`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"LoopFlatPlot", "[", 
   RowBox[{"V_", ",", 
    RowBox[{"{", 
     RowBox[{"F_", ",", "col_", ",", "edg_"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"EdgeForm", "[", "Dashed", "]"}], ",", "LightGreen", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
          "&"}], "/@", 
         RowBox[{"Drop", "[", 
          RowBox[{"F", ",", "1"}], "]"}]}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"EdgeForm", "[", "Dashed", "]"}], ",", "col", ",", 
        RowBox[{"Polygon", "[", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
           "\[RightDoubleBracket]"}], "]"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"GrayLevel", "[", "0.3", "]"}], ",", 
        RowBox[{"EdgeForm", "[", "Dashed", "]"}], ",", 
        RowBox[{"Thickness", "[", "0.02", "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{
         "V", "\[LeftDoubleBracket]", "edg", "\[RightDoubleBracket]"}], 
         "]"}]}], "}"}]}], "\[IndentingNewLine]", "}"}], ",", 
    RowBox[{"Axes", "\[Rule]", "False"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"LoopGridPlot", "[", 
   RowBox[{"V_", ",", 
    RowBox[{"{", 
     RowBox[{"F_", ",", "col_", ",", "edg_"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Gray", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", "\[LeftDoubleBracket]", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"Range", "[", 
                RowBox[{"Length", "[", "#", "]"}], "]"}], ",", 
               RowBox[{"{", "1", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", "F"}]}], "}"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"GrayLevel", "[", "0.6", "]"}], ",", 
        RowBox[{"Thickness", "[", "0.02", "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{
         "V", "\[LeftDoubleBracket]", "edg", "\[RightDoubleBracket]"}], 
         "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"EdgeForm", "[", 
         RowBox[{"{", "Blue", "}"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Disk", "[", 
           RowBox[{"#", ",", "0.04"}], "]"}], "&"}], "/@", "V"}]}], "}"}]}], 
     "\[IndentingNewLine]", "}"}], ",", 
    RowBox[{"Axes", "\[Rule]", "False"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.61016122221655*^9, 3.610161314015646*^9}, {
   3.610161345693632*^9, 3.610161351346219*^9}, {3.6101615852373123`*^9, 
   3.6101615926774263`*^9}, {3.610161656450906*^9, 3.6101616570934153`*^9}, {
   3.610161991679555*^9, 3.6101621545945573`*^9}, {3.610162453921916*^9, 
   3.610162484554668*^9}, {3.6101625270751*^9, 3.6101626614317846`*^9}, {
   3.6101666342260323`*^9, 3.61016664058613*^9}, {3.6101821730271387`*^9, 
   3.6101821781144295`*^9}, {3.610182253927766*^9, 3.61018226186722*^9}, {
   3.6101822967672167`*^9, 3.6101823741746435`*^9}, {3.610183773298669*^9, 
   3.6101838116528625`*^9}, {3.6101838967207284`*^9, 
   3.6101839192820187`*^9}, {3.6101839529479446`*^9, 3.610183954805051*^9}, 
   3.6101841098239174`*^9, {3.610184247398786*^9, 3.610184251161001*^9}, 
   3.6101871293386235`*^9, {3.6102302690368624`*^9, 3.6102302839847174`*^9}, {
   3.6102307550046577`*^9, 3.6102307597079268`*^9}, {3.6102307964120264`*^9, 
   3.610230801434314*^9}, {3.6102308400505223`*^9, 3.610230851687188*^9}, {
   3.610314147271871*^9, 3.6103141479229083`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkDecompositionList", "[", 
   RowBox[{"pos_", ",", "top_", ",", "tops_", ",", "sub_", ",", "ind_", ",", 
    RowBox[{"sca_:", "1"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sze", "=", 
      RowBox[{"Round", "[", 
       RowBox[{"110", " ", "sca"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"LoopGridPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}], ",", 
          RowBox[{"LoopFlatPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}]}], "}"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}], ",", "\"\<=\>\"", 
      ",", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"LoopGridPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}], ",", 
          RowBox[{"LoopFlatPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sub", "\[LeftDoubleBracket]", 
              RowBox[{
              "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              "\[RightDoubleBracket]"}], ".", "pos"}], ",", 
            RowBox[{
            "tops", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
           "]"}]}], "}"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}], ",", 
      "\"\<\[Union]\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"LoopGridPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}], ",", 
          RowBox[{"LoopFlatPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sub", "\[LeftDoubleBracket]", 
              RowBox[{
              "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
              "\[RightDoubleBracket]"}], ".", "pos"}], ",", 
            RowBox[{
            "tops", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
           "]"}]}], "}"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}], ",", 
      "\"\<\[Union]\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"LoopGridPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}], ",", 
          RowBox[{"LoopFlatPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sub", "\[LeftDoubleBracket]", 
              RowBox[{
              "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
              "\[RightDoubleBracket]"}], ".", "pos"}], ",", 
            RowBox[{
            "tops", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
           "]"}]}], "}"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}], ",", 
      "\"\<\[Union]\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"LoopGridPlot", "[", 
           RowBox[{"pos", ",", "top"}], "]"}], ",", 
          RowBox[{"LoopFlatPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sub", "\[LeftDoubleBracket]", 
              RowBox[{
              "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
              "\[RightDoubleBracket]"}], ".", "pos"}], ",", 
            RowBox[{
            "tops", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}]}], 
           "]"}]}], "}"}], ",", 
        RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}]}], "}"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.610238408470419*^9, 3.6102384173669276`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkFlatShow", "[", 
   RowBox[{"pos_", ",", "top_", ",", 
    RowBox[{"sca_:", "1"}]}], "]"}], ":=", 
  RowBox[{"With", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sze", "=", 
      RowBox[{"Round", "[", 
       RowBox[{"110", " ", "sca"}], "]"}]}], "}"}], ",", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"LoopFlatPlot", "[", 
        RowBox[{"pos", ",", "top"}], "]"}], "}"}], ",", 
      RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6108586083299036`*^9, 3.6108586382866173`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"GraphicsWeights", "[", 
   RowBox[{"V_", ",", "s_", ",", 
    RowBox[{"size_:", "Medium"}]}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
        "\[NotEqual]", "0"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"Style", "[", 
          RowBox[{
           RowBox[{
           "s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], ",", 
           "Bold", ",", "size"}], "]"}], ",", 
         RowBox[{"V", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
         ",", 
         RowBox[{"Background", "\[Rule]", "LightRed"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", "s", "]"}]}], "}"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkIndexing", "[", 
   RowBox[{"pos_", ",", "top_", ",", 
    RowBox[{"sca_:", "1"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"sze", "=", 
      RowBox[{"Round", "[", 
       RowBox[{"110", " ", "sca"}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Show", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"LoopFlatPlot", "[", 
         RowBox[{"pos", ",", "top"}], "]"}], ",", 
        RowBox[{"GraphicsWeights", "[", 
         RowBox[{"pos", ",", 
          RowBox[{"Range", "[", 
           RowBox[{"Length", "[", "pos", "]"}], "]"}]}], "]"}]}], "}"}], ",", 
      RowBox[{"ImageSize", "\[Rule]", "sze"}]}], "]"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6104747729453745`*^9, 3.610474815655817*^9}, {
  3.6104748568321724`*^9, 3.610474857103188*^9}, {3.610474890447095*^9, 
  3.610474901824746*^9}, {3.6104764427018785`*^9, 3.6104764432269087`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"CatmullClarkDefaultPos", "[", "n_", "]"}], ":=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "2"}], ",", 
         RowBox[{"-", "2"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "2"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "2"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", 
         RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "2"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", 
         RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"n", "\[Equal]", "3"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}], ",", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"m", "=", 
         RowBox[{"2", 
          RowBox[{"(", 
           RowBox[{"n", "-", "3"}], ")"}]}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"OddQ", "[", "k", "]"}], ",", 
            SqrtBox["2"], ",", "1"}], "]"}], 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Sin", "[", 
             FractionBox[
              RowBox[{"k", " ", "\[Pi]"}], 
              RowBox[{"m", " ", "2"}]], "]"}], ",", 
            RowBox[{"Cos", "[", 
             FractionBox[
              RowBox[{"k", " ", "\[Pi]"}], 
              RowBox[{"m", " ", "2"}]], "]"}]}], "}"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "0", ",", 
           RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkDefaultTop", "[", "n_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ring", "=", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"10", ",", "14"}], "}"}], ",", 
          RowBox[{"14", "+", 
           RowBox[{"Range", "[", 
            RowBox[{"2", 
             RowBox[{"(", 
              RowBox[{"n", "-", "3"}], ")"}]}], "]"}]}], ",", 
          RowBox[{"{", "12", "}"}]}], "]"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}]}], "}"}], 
        ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "11", "}"}], ",", 
            RowBox[{"ring", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"2", "k"}], "+", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0", ",", 
                RowBox[{"-", "1"}]}], "}"}]}], "\[RightDoubleBracket]"}]}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"n", "-", "2"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"n", "\[Equal]", "4"}], ",", "Green", ",", "Red"}], "]"}], ",", 
    RowBox[{"{", "}"}]}], "}"}]}]}], "Input",
 CellChangeTimes->{{3.610701792481405*^9, 3.6107018848446875`*^9}, {
  3.6107019772759743`*^9, 3.6107020113389225`*^9}}],

Cell[CellGroupData[{

Cell["reg", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.61047475678345*^9, 3.6104747570804667`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkRegPos", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkRegTop", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "15", ",", "14", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "16", ",", "15"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Green", ",", 
     RowBox[{"{", "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6102373808966446`*^9, 3.610237494068118*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["2", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark2Pos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark2Top", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Yellow", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "11", ",", "12"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, 
   3.6102927311729636`*^9, {3.6107017638017645`*^9, 3.6107017691530704`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["1", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark1Pos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark1Top", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "9", ",", "8"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5", ",", "8", ",", "7"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"7", ",", "8", ",", "9", ",", "6", ",", "3"}], "}"}]}], "}"}]}],
   ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102377138826904`*^9}, 
   3.610292733922121*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.a", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark3aPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark3aTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "11", ",", "15", ",", "14"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"15", ",", "11", ",", "12"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, 3.610292735963238*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.b", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, 
   3.61023801057066*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark3bPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark3bTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "14", ",", "13"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "11", ",", "13"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.61023808627299*^9}, {3.610238117547779*^9, 
   3.6102381179908047`*^9}, 3.610292738739396*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.a", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, {
   3.6102390646899524`*^9, 3.6102390648429613`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark4aPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], ",", 
       FractionBox["1", 
        SqrtBox["2"]]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Cos", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}], ",", 
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Sin", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark4aTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "11", ",", "15", ",", "14"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "17", ",", "16"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"15", ",", "11", ",", "16"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, {3.610239112878709*^9, 3.6102391144798*^9}, {
   3.6102391833337383`*^9, 3.610239239171932*^9}, 3.6102927409865246`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.b", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, {
   3.6102390646899524`*^9, 3.6102390648429613`*^9}, 3.610239506752237*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark4bPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Sin", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}], ",", 
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Cos", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox["1", 
        SqrtBox["2"]], ",", 
       FractionBox["1", 
        SqrtBox["2"]]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Cos", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}], ",", 
       RowBox[{
        SqrtBox["2"], " ", 
        RowBox[{"Sin", "[", 
         FractionBox["\[Pi]", "8"], "]"}]}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark4bTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "15", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "16", ",", "15"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "11", ",", "13"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, {3.610239112878709*^9, 3.6102391144798*^9}, {
   3.6102391833337383`*^9, 3.610239239171932*^9}, {3.6102395170628266`*^9, 
   3.6102396458071904`*^9}, 3.6102927433086576`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.a", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, {
   3.6102390646899524`*^9, 3.6102390648429613`*^9}, 3.6102398703900356`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SqrtBox["3"]}], ")"}]}], "+", "2"}], ")"}], "/", "3"}], "//", 
  "N"}]], "Input",
 CellChangeTimes->{{3.610239324895835*^9, 3.6102393482021685`*^9}, {
  3.610240004672716*^9, 3.610240013931246*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark5aPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["3"]}], ")"}]}], ",", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["3"]}], ")"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox["1", "2"], ",", 
       FractionBox[
        SqrtBox["3"], "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        SqrtBox["3"], "2"], ",", 
       FractionBox["1", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["3"]}], ")"}]}], ",", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["3"]}], ")"}]}]}], "}"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark5aTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "11", ",", "15", ",", "14"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "17", ",", "16", ",", "15"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "19", ",", "18"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"17", ",", "11", ",", "18"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, {3.610239112878709*^9, 3.6102391144798*^9}, {
   3.6102391833337383`*^9, 3.610239239171932*^9}, {3.6102398929933286`*^9, 
   3.6102399878047514`*^9}, 3.6102927458518033`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.b", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, {
   3.6102390646899524`*^9, 3.6102390648429613`*^9}, 3.6102398703900356`*^9, 
   3.6102403210268106`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SqrtBox["3"]}], ")"}]}], "+", "2"}], ")"}], "/", "3"}], "//", 
  "N"}]], "Input",
 CellChangeTimes->{{3.610239324895835*^9, 3.6102393482021685`*^9}, {
  3.610240004672716*^9, 3.610240013931246*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark5bPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       FractionBox["1", "2"], ",", 
       FractionBox[
        SqrtBox["3"], "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        SqrtBox["3"], "2"], ",", 
       FractionBox["1", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["3"]}], ")"}]}], ",", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["3"]}], ")"}]}]}], "}"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark5bTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"10", ",", "11", ",", "15", ",", "14"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "18", ",", "17", ",", "16"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "19", ",", "18"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"15", ",", "11", ",", "16"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, {3.610239112878709*^9, 3.6102391144798*^9}, {
   3.6102391833337383`*^9, 3.610239239171932*^9}, {3.6102398929933286`*^9, 
   3.6102399878047514`*^9}, {3.610240329196278*^9, 3.610240376906007*^9}, 
   3.6102927480439286`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.c", "Subsubsection",
 CellChangeTimes->{{3.610164956566764*^9, 3.6101649597893133`*^9}, 
   3.6102377201560497`*^9, {3.610237844227146*^9, 3.6102378462112594`*^9}, {
   3.6102390646899524`*^9, 3.6102390648429613`*^9}, 3.610239506752237*^9, 
   3.6102420803894405`*^9, 3.610242138645773*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClark5cPos", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["3"]}], ")"}]}], ",", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["3"]}], ")"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox["1", "2"], ",", 
       FractionBox[
        SqrtBox["3"], "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        SqrtBox["3"], "2"], ",", 
       FractionBox["1", "2"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["3"]}], ")"}]}], ",", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["3"]}], ")"}]}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClark5cTop", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "15", ",", "14", ",", "13"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "17", ",", "16", ",", "15"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12", ",", "18", ",", "17"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "Red", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "10", ",", "11", ",", "13"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.610237513935254*^9, 3.6102378745408792`*^9}, {
   3.6102379183193836`*^9, 3.610237992973654*^9}, {3.610238112738504*^9, 
   3.6102381151196404`*^9}, {3.610239112878709*^9, 3.6102391144798*^9}, {
   3.6102391833337383`*^9, 3.610239239171932*^9}, {3.6102395170628266`*^9, 
   3.6102396458071904`*^9}, {3.610242091186058*^9, 3.6102421447671227`*^9}, 
   3.6102927534632387`*^9}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6104735634701967`*^9, 3.610473563696209*^9}, {
  3.6104746799380546`*^9, 3.6104747360422635`*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Duplicate crease vertices", "Subsection",
 CellChangeTimes->{{3.609881196233368*^9, 3.609881217807602*^9}, {
   3.6101882111254983`*^9, 3.610188217756878*^9}, 3.612319436007948*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CreaseDuplicate", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_", ",", "E_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Vout", "=", "V"}], ",", 
      RowBox[{"Fout", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"Eout", "=", 
       RowBox[{"{", "}"}]}], ",", "vert", ",", "key", ",", "cycl", ",", 
      RowBox[{"rings", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", "ring",
       ",", "edge", ",", "side", ",", 
      RowBox[{"set", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"ids", "=", 
       RowBox[{"Complement", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", "V", "]"}], "]"}], ",", 
         RowBox[{"Flatten", "[", "E", "]"}]}], "]"}]}], ",", "value", ",", 
      "hash"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"hash", "[", "#", "]"}], "=", "#"}], ")"}], "&"}], "/@", 
      "ids"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"Print", "[", "ids", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Print", "/@", "rings"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", "\"\<---\>\"", "]"}], ";"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"cycl", "=", 
         RowBox[{
         "E", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Print", "[", "\"\<cycle\>\"", "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Print", "[", "cycl", "]"}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"edge", "=", 
            RowBox[{
             RowBox[{"RotateLeft", "[", 
              RowBox[{"cycl", ",", 
               RowBox[{"j", "-", "2"}]}], "]"}], "\[LeftDoubleBracket]", 
             RowBox[{"{", 
              RowBox[{"1", ",", "2", ",", "3"}], "}"}], 
             "\[RightDoubleBracket]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"vert", "=", 
            RowBox[{
            "edge", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"Vout", ",", 
             RowBox[{
             "Vout", "\[LeftDoubleBracket]", "vert", 
              "\[RightDoubleBracket]"}]}], "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"hash", "[", "vert", "]"}], "=", 
            RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"ring", "=", 
            RowBox[{
             RowBox[{"VertexRing", "[", 
              RowBox[{
               RowBox[{
               "rings", "\[LeftDoubleBracket]", "vert", 
                "\[RightDoubleBracket]"}], ",", 
               RowBox[{"rings", "\[LeftDoubleBracket]", 
                RowBox[{"vert", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
               RowBox[{
                RowBox[{"Position", "[", 
                 RowBox[{
                  RowBox[{"rings", "\[LeftDoubleBracket]", 
                   RowBox[{"vert", ",", "1"}], "\[RightDoubleBracket]"}], ",",
                   "vert"}], "]"}], "\[LeftDoubleBracket]", 
                RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}], "]"}], 
             "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"While", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
              "\[NotEqual]", 
              RowBox[{"edge", "\[LeftDoubleBracket]", 
               RowBox[{"{", 
                RowBox[{"2", ",", "3"}], "}"}], "\[RightDoubleBracket]"}]}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ring", "=", 
               RowBox[{"RotateLeft", "[", 
                RowBox[{"ring", ",", "1"}], "]"}]}], ";"}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"side", "=", 
            RowBox[{"{", 
             RowBox[{
             "ring", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
             "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"While", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "3"}], "\[RightDoubleBracket]"}], 
              "\[NotEqual]", 
              RowBox[{
              "edge", "\[LeftDoubleBracket]", "1", 
               "\[RightDoubleBracket]"}]}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"ring", "=", 
               RowBox[{"RotateLeft", "[", 
                RowBox[{"ring", ",", "1"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"side", ",", 
                RowBox[{
                "ring", "\[LeftDoubleBracket]", "1", 
                 "\[RightDoubleBracket]"}]}], "]"}], ";"}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"Print", "[", "side", "]"}], ";"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"set", ",", "side"}], "]"}], ";"}], "\[IndentingNewLine]",
           ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", "cycl", "]"}]}], "}"}]}], "]"}], ";"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "E", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"set", "=", 
      RowBox[{"Sort", "/@", 
       RowBox[{"Flatten", "[", 
        RowBox[{"set", ",", "1"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"MemberQ", "[", 
           RowBox[{"set", ",", 
            RowBox[{"Sort", "[", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"Fout", ",", 
             RowBox[{"hash", "/@", 
              RowBox[{
              "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}]}],
             "]"}], ";"}], "\[IndentingNewLine]", ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"Fout", ",", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
           "]"}]}], "]"}], ";"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Vout", ",", "Fout", ",", "Eout"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.610189268014949*^9, 3.610189297862656*^9}, {
  3.6101893333926888`*^9, 3.6101893368888884`*^9}, {3.610189440749829*^9, 
  3.610189542902672*^9}, {3.61018998381289*^9, 3.6101900073872385`*^9}, {
  3.6101900606312838`*^9, 3.610190076598197*^9}, {3.6101901517494955`*^9, 
  3.6101902791327815`*^9}, {3.610190360085412*^9, 3.6101903953734303`*^9}, {
  3.610190463706339*^9, 3.610190527321977*^9}, {3.6101906143499546`*^9, 
  3.6101907499427104`*^9}, {3.6101909265408115`*^9, 3.6101909954487524`*^9}, {
  3.610191057855322*^9, 3.6101910874920173`*^9}, {3.6101911178707547`*^9, 
  3.610191167774609*^9}, {3.610191236172521*^9, 3.610191304585434*^9}, {
  3.6101913859020853`*^9, 3.6101914266334147`*^9}, {3.610191549962469*^9, 
  3.610191635985389*^9}, {3.6101917234783936`*^9, 3.6101917278366427`*^9}, {
  3.610191782985797*^9, 3.6101917959645395`*^9}, {3.610191828549403*^9, 
  3.610191873724987*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applications", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
   3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
   3.6056228546483173`*^9}, 3.605696312889595*^9, {3.60570271998906*^9, 
   3.60570272033908*^9}, {3.6057363923638954`*^9, 3.6057364076977725`*^9}, 
   3.6060117523985486`*^9, {3.6067159239480705`*^9, 3.6067159244350986`*^9}, {
   3.6067182744948053`*^9, 3.606718275012313*^9}, {3.612138185056046*^9, 
   3.6121381853740644`*^9}, {3.612175503135314*^9, 3.612175503623342*^9}, {
   3.6121760908519297`*^9, 3.612176092419019*^9}, {3.612308581998134*^9, 
   3.6123085821641436`*^9}}],

Cell[CellGroupData[{

Cell["Simple test", "Subsection",
 CellChangeTimes->{{3.612176085368616*^9, 3.6121760878417573`*^9}, {
  3.6121771037798653`*^9, 3.612177113539424*^9}}],

Cell[BoxData[
 RowBox[{"PlotMesh", "[", 
  RowBox[{"Nest", "[", 
   RowBox[{"CatmullClarkSubdivide", ",", "TripodCircuitMesh", ",", "2"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6123191411730843`*^9, 3.612319149428557*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"msh", "=", 
   RowBox[{"Nest", "[", 
    RowBox[{"CatmullClarkSubdivide", ",", 
     RowBox[{"Nest", "[", 
      RowBox[{"QuadLinearSubdivide", ",", "CylinderMesh", ",", "0"}], "]"}], 
     ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVolumeFace", "[", "msh", "]"}], "//", 
  "Total"}]}], "Input",
 CellChangeTimes->{{3.609756945954995*^9, 3.6097569578746767`*^9}, 
   3.6097588274016075`*^9, 3.6097592775903573`*^9, {3.6097594303010917`*^9, 
   3.6097594553945265`*^9}, 3.6097602558963127`*^9, {3.609760339998123*^9, 
   3.6097603688137712`*^9}, 3.609761318712103*^9, {3.6097620537851467`*^9, 
   3.609762059080449*^9}, 3.6097622495693445`*^9, {3.6097710086573353`*^9, 
   3.6097710184058933`*^9}, {3.6097898824592304`*^9, 
   3.6097898878855405`*^9}, {3.609789982192935*^9, 3.60978998228994*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Demo", "Subsection",
 CellChangeTimes->{{3.612176085368616*^9, 3.6121760878417573`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{"CylinderMesh", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{3.6123191821494284`*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{"TripodBoltMesh", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.612319191378956*^9, 3.6123192035716534`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derivation of coefficients", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
   3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
   3.6056228546483173`*^9}, {3.605646909007148*^9, 3.6056469329195156`*^9}, {
   3.60564698352641*^9, 3.6056469954380913`*^9}, {3.605696857837764*^9, 
   3.605696861844993*^9}, {3.605697358177382*^9, 3.605697358385394*^9}, 
   3.6123191205569053`*^9}],

Cell[CellGroupData[{

Cell["Valence 2 \[Bullet] Form", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
  3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
  3.607244889530364*^9}, {3.6097724939562902`*^9, 3.609772497300481*^9}, {
  3.6097890550409045`*^9, 3.6097890864367003`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark2Pos", ",", "CatmullClark2Top", ",", "1"}], 
  "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC2sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC2ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20"}], 
      "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9, {
   3.6097059288985224`*^9, 3.6097059373440056`*^9}, {3.609705986043791*^9, 
   3.609706151433251*^9}, 3.609707383556724*^9, {3.609749369436643*^9, 
   3.609749369748661*^9}, {3.6097891478292117`*^9, 3.6097891493963013`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark2Pos", ",", "CatmullClark2Top", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", "CatmullClark2Top", 
     ",", "CatmullClark2Top"}], "}"}], ",", "CC2sub", ",", "CC2ind"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6102382184195485`*^9, 3.6102382683014016`*^9}, 
   3.6102384259154167`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{
    RowBox[{"SkewTensor", "[", 
     RowBox[{"<<", "\"\<CatmullClarkForm4.mx\>\""}], "]"}], "-", 
    RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755206201487*^9, 3.6097552191692286`*^9}, 
   3.6097892579665112`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6097505575465994`*^9, 3.6097505621058598`*^9}, {
  3.6103143303753443`*^9, 3.6103143683505163`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"12", ",", "12", ",", "12"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"form", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609705893306487*^9, 3.6097058952665987`*^9}, {3.609707184746353*^9, 
   3.609707189218609*^9}, {3.609749340221972*^9, 3.6097493490584774`*^9}, {
   3.609750538168491*^9, 3.6097505405456266`*^9}, {3.609789281822876*^9, 
   3.609789287215184*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sub", "=", "CC2sub"}], ",", 
       RowBox[{"ind", "=", "CC2ind"}]}], "}"}], ",", 
     RowBox[{
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
   3.6072490656262226`*^9, 3.6072490711795406`*^9}, 3.6097062865219774`*^9, 
   3.609706319720876*^9, {3.609707059148169*^9, 3.609707071435872*^9}, {
   3.6097072012572975`*^9, 3.6097072023443594`*^9}, {3.6097192543796964`*^9, 
   3.6097192547327166`*^9}, {3.609770614932816*^9, 3.609770615125827*^9}, {
   3.609789292884508*^9, 3.6097893622794776`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mir", "=", 
   RowBox[{"NestP", "[", 
    RowBox[{"form", ",", 
     RowBox[{"{", 
      RowBox[{
      "4", ",", "3", ",", "2", ",", "1", ",", "  ", "8", ",", "7", ",", "6", 
       ",", "5", ",", "  ", "12", ",", "11", ",", "10", ",", "9"}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Mir", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Rot", "=", 
     RowBox[{"NestP", "[", 
      RowBox[{"GeneralForm4", ",", 
       RowBox[{"{", 
        RowBox[{
        "4", ",", "8", ",", "12", ",", "16", ",", "  ", "3", ",", "7", ",", 
         "11", ",", "15", ",", "  ", "2", ",", "6", ",", "10", ",", "14", ",",
          "  ", "1", ",", "5", ",", "9", ",", "13"}], "}"}]}], "]"}]}], ";"}],
    "*)"}]}]}], "Input",
 CellChangeTimes->{{3.607245459898987*^9, 3.607245526803814*^9}, {
  3.6072490898826103`*^9, 3.607249145665801*^9}, {3.609707456352888*^9, 
  3.609707458608017*^9}, {3.609770611085596*^9, 3.6097706124996767`*^9}, {
  3.609789366773735*^9, 3.609789367682787*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"affine1", ",", "vector1"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"affine2", ",", "vector2"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "+", "Mir"}], ",", "form"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vector2", "//", "Flatten"}], "//", "Union"}]}], "Input",
 CellChangeTimes->{{3.6097074665034685`*^9, 3.609707466720481*^9}, {
  3.6097495110267415`*^9, 3.609749523401449*^9}, {3.609750018662777*^9, 
  3.609750019021797*^9}, {3.6097706021660857`*^9, 3.609770604246205*^9}, {
  3.6097893775583515`*^9, 3.6097894055829544`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"matrix", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"affine1", ",", "affine2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vector", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"vector1", ",", "vector2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"matrix", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{"vector", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{
  3.6097074768560605`*^9, {3.6097203679463887`*^9, 3.609720370106512*^9}, {
   3.6097204483639884`*^9, 3.609720448580001*^9}, {3.609749943803495*^9, 
   3.609749948882786*^9}, {3.609789397676502*^9, 3.6097894015967264`*^9}}],

Cell[BoxData[
 RowBox[{"affine1", "//", "MatrixRank"}]], "Input",
 CellChangeTimes->{3.6103142212081003`*^9}],

Cell[BoxData[
 RowBox[{"matrix", "//", "MatrixRank"}]], "Input",
 CellChangeTimes->{{3.609707490606847*^9, 3.609707492634963*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.609750068807645*^9, 3.609750089232813*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4Null", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4Null", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607250691259204*^9, 3.6072506923712673`*^9}, {
   3.6072507694596767`*^9, 3.6072507805073085`*^9}, 3.609707599347067*^9, {
   3.6097503767642593`*^9, 3.6097503770592756`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Union", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{"SkewTensor", "[", 
      RowBox[{"Skew3", "[", 
       RowBox[{"ArrayReshape", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}], "]"}], "]"}], 
     "]"}], "]"}], "&"}], "/@", "CatmullClarkForm4Null"}]], "Input",
 CellChangeTimes->{{3.610316249838131*^9, 3.6103162850911474`*^9}, {
  3.610316489012811*^9, 3.6103165063848047`*^9}, {3.6105553808356943`*^9, 
  3.6105553851769423`*^9}, {3.6105560776835513`*^9, 3.6105561248742504`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal2Form", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.610557284986605*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal2Form", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{"sol", "+", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["\[Alpha]", "1"], ",", 
         SubscriptBox["\[Alpha]", "2"], ",", 
         SubscriptBox["\[Alpha]", "3"]}], "}"}], ".", 
       "CatmullClarkForm4Null"}]}], ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607253481136436*^9, 3.6072535127052417`*^9}, {
   3.6072535444205074`*^9, 3.607253563850806*^9}, {3.609707740343131*^9, 
   3.609707749847675*^9}, {3.6097503966533966`*^9, 3.6097504005396185`*^9}, 
   3.610316274202525*^9, {3.61031676627967*^9, 3.610316767206723*^9}, 
   3.6103168783200784`*^9, {3.610557274392999*^9, 3.6105572766151266`*^9}}],

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"0", "\[Equal]", "#"}], "&"}], "/@", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Skew3", "[", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          SubscriptBox["\[Alpha]", "1"], ",", 
          SubscriptBox["\[Alpha]", "2"], ",", 
          SubscriptBox["\[Alpha]", "3"]}], "}"}], ".", 
        "CatmullClarkForm4Null"}], ",", 
       RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.610316890044749*^9, 3.6103169624868927`*^9}, 
   3.610555682950974*^9, {3.610555716887915*^9, 3.6105557265664687`*^9}}],

Cell[BoxData[
 RowBox[{"With", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"frm", "=", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"sol", ",", 
       RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"frm", "-", 
    RowBox[{"Skew3", "[", "frm", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6103174922391925`*^9, 3.610317526440149*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"EdgeRegularForm", "=", 
    RowBox[{
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"sol", ",", 
       RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}], "+", 
     "CatmullClarkForm4Free3"}]}], ";"}], "*)"}]], "Input",
 CellChangeTimes->{{3.6097504461542277`*^9, 3.6097504527856073`*^9}, {
  3.609752376270624*^9, 3.6097523770046663`*^9}, {3.610316849045404*^9, 
  3.6103168547887325`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"CatmullClarkVal2Form", "-", 
      RowBox[{"With", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"sub", "=", "CC2sub"}], ",", 
          RowBox[{"ind", "=", "CC2ind"}]}], "}"}], ",", 
        RowBox[{
         RowBox[{"NestT", "[", 
          RowBox[{"CatmullClarkForm4", ",", 
           RowBox[{"sub", "\[LeftDoubleBracket]", 
            RowBox[{
            "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}], "+", 
         RowBox[{"NestT", "[", 
          RowBox[{"CatmullClarkForm4", ",", 
           RowBox[{"sub", "\[LeftDoubleBracket]", 
            RowBox[{
            "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}], "+", 
         RowBox[{"NestT", "[", 
          RowBox[{"CatmullClarkVal2Form", ",", 
           RowBox[{"sub", "\[LeftDoubleBracket]", 
            RowBox[{
            "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}], "+", 
         RowBox[{"NestT", "[", 
          RowBox[{"CatmullClarkVal2Form", ",", 
           RowBox[{"sub", "\[LeftDoubleBracket]", 
            RowBox[{
            "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], ")"}], "//", 
    "Flatten"}], "//", "Simplify"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.6097504856354856`*^9, 3.609750511947991*^9}, {
  3.6103169848531713`*^9, 3.610317001203107*^9}, {3.6103170328429165`*^9, 
  3.610317042010441*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal2Form", ">>", 
  "\"\<CatmullClarkVal2Alt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.609789600343094*^9, 3.6097896060114183`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "12"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.609719414542857*^9, 3.6097194217562695`*^9}, 
   3.609719465344763*^9, {3.609719712274886*^9, 3.609719713712969*^9}, {
   3.609750225981635*^9, 3.6097502262836523`*^9}, {3.6097502756124735`*^9, 
   3.6097502764695225`*^9}, {3.609750423327922*^9, 3.6097504565168204`*^9}, {
   3.6097522128222756`*^9, 3.60975223493254*^9}, {3.6097524432134533`*^9, 
   3.609752447908722*^9}, {3.609755143546903*^9, 3.609755157353693*^9}, 
   3.6097895267098827`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"10", ",", "6", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.61057276672591*^9, 3.6105727749743824`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"11", ",", "8", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105573080889263`*^9, 3.6105573339534063`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"12", ",", "9", ",", "3"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105727226143875`*^9, 3.610572733415005*^9}, {
  3.610772614638407*^9, 3.6107726157654715`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"9", ",", "5", ",", "4"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105728085663033`*^9, 3.610572816230742*^9}, {
  3.610572853158854*^9, 3.6105728544779296`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"12", ",", "9", ",", "6"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6111292643762207`*^9, 3.6111292949119673`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 2 \[Bullet] Simple Calib", "Subsection",
 CellChangeTimes->{{3.606378196577406*^9, 3.6063782068569937`*^9}, {
  3.6063791489238772`*^9, 3.606379149377903*^9}, {3.6097890679106407`*^9, 
  3.6097890814924173`*^9}, {3.6105555964450264`*^9, 3.6105555980431175`*^9}}],

Cell[BoxData[
 RowBox[{"pos", "=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Append", "[", 
      RowBox[{"#", ",", "1"}], "]"}], "&"}], "/@", "CatmullClark2Pos"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6105558059770107`*^9, 3.6105558491134777`*^9}}],

Cell[BoxData[{
 RowBox[{"CatmullClarkVal2Form", "//", "Variables"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Skew3", "[", "CatmullClarkVal2Form", "]"}], ".", 
   RowBox[{"pos", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], ".", 
   RowBox[{"pos", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], ".", 
   RowBox[{"pos", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], "//",
   "Simplify"}]}], "Input",
 CellChangeTimes->{{3.6105555989921722`*^9, 3.610555629714929*^9}, {
  3.610555856029874*^9, 3.6105558662284565`*^9}, {3.610555904691657*^9, 
  3.610555908257861*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "6"], 
   RowBox[{"Det", "[", 
    RowBox[{"(", GridBox[{
       {"0", "1", "0"},
       {"0", "0", "1"},
       {"1", "1", "1"}
      }], ")"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "6"], 
   RowBox[{"Det", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0"},
       {"0", "1", "1"},
       {"1", "1", "1"}
      }], ")"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.610555976336755*^9, 3.610556025346558*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Valence 2 \[Bullet] Investigation of DOF\
\>", "Subsection",
 CellChangeTimes->{{3.606378196577406*^9, 3.6063782068569937`*^9}, {
  3.6063791489238772`*^9, 3.606379149377903*^9}, {3.6097890679106407`*^9, 
  3.6097890814924173`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVolume", "[", 
  RowBox[{"CatmullClarkSubdivide", "[", "CylinderMesh", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6097726242927446`*^9, 3.609772633566275*^9}, {
  3.60977282317212*^9, 3.6097728545309134`*^9}}],

Cell[BoxData[
 RowBox[{"PlotMesh", "[", 
  RowBox[{"CatmullClarkSubdivide", "[", "CubeCylinerMesh", "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6097526856283183`*^9, 3.6097527066515207`*^9}, {
   3.6097532866716957`*^9, 3.6097532983493643`*^9}, {3.6097534619577217`*^9, 
   3.609753479020698*^9}, {3.6097535705739346`*^9, 3.6097535744201546`*^9}, {
   3.609753775364648*^9, 3.6097537911325493`*^9}, 3.609772614696196*^9, {
   3.6097728892128973`*^9, 3.609772892996114*^9}}],

Cell[BoxData[{
 RowBox[{"val1", "=", 
  RowBox[{
  "CatmullClarkVolume", "[", "CubeCylinerMesh", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"val2", "=", 
  RowBox[{"CatmullClarkVolume", "[", 
   RowBox[{"CatmullClarkSubdivide", "[", "CubeCylinerMesh", "]"}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.610317152571765*^9, 3.610317195202203*^9}}],

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"val1", "\[Equal]", "1"}], ",", 
    RowBox[{"val2", "\[Equal]", "1"}]}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.610317197144314*^9, 3.6103172151463437`*^9}, {
  3.6103172477322073`*^9, 3.6103172634901085`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    SubscriptBox["\[Alpha]", "1"], "\[Rule]", 
    RowBox[{
     FractionBox[
      SubscriptBox["\[Alpha]", "2"], "12"], "+", 
     FractionBox[
      SubscriptBox["\[Alpha]", "3"], "6"]}]}], "}"}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"LinearVolume", "[", 
  RowBox[{"CubeCylinerMesh", "\[LeftDoubleBracket]", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6097540203766613`*^9, 3.6097540357765427`*^9}, {
   3.6097729148853655`*^9, 3.6097729337074423`*^9}, 3.6103171013148327`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Rasterize", "[", 
    RowBox[{
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"PlotMesh", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", "V"}], "+", 
            RowBox[{"\[Lambda]", " ", "Vn"}]}], ",", "F", ",", "Ed"}], "}"}], 
         "/.", 
         RowBox[{"\[Lambda]", "\[Rule]", "#"}]}], "]"}], ",", 
       RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
       RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"ImageSize", "\[Rule]", "80"}]}], "]"}], ",", 
     RowBox[{"ImageResolution", "\[Rule]", "200"}]}], "]"}], "&"}], "/@", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0.4", ",", "0.7", ",", "1"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.607383589899951*^9, 3.60738360219764*^9}, {
  3.6075499124854856`*^9, 3.6075499169737425`*^9}, {3.6097538658528233`*^9, 
  3.6097538889331436`*^9}}],

Cell[BoxData[
 RowBox[{"Rasterize", "[", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"PlotMesh", "[", 
      RowBox[{"Nest", "[", 
       RowBox[{"CatmullClarkSubdivide", ",", 
        RowBox[{"{", 
         RowBox[{"Vn", ",", "F", ",", "Ed"}], "}"}], ",", "1"}], "]"}], "]"}],
      ",", 
     RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
     RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
     RowBox[{"Axes", "\[Rule]", "False"}], ",", 
     RowBox[{"ImageSize", "\[Rule]", "80"}]}], "]"}], ",", 
   RowBox[{"ImageResolution", "\[Rule]", "200"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6073380451999454`*^9, 3.607338049897517*^9}, {
   3.607338080058481*^9, 3.6073380922261677`*^9}, 3.6073381353918314`*^9, {
   3.607383652265909*^9, 3.6073837626176043`*^9}, {3.607549938298962*^9, 
   3.6075499669406004`*^9}, {3.609753899850768*^9, 3.609753915931688*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", "CatmullClarkForm4Free3"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm3", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["b", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"14", ",", "14", ",", "14"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6072538042939587`*^9, 3.607253827278273*^9}}],

Cell[BoxData[
 RowBox[{"LinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"volumeEqs", "=", 
  RowBox[{
   RowBox[{"CatmullClarkVolume", "[", 
    RowBox[{"{", 
     RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}], "\[Equal]", "1"}]}]], "Input",
 CellChangeTimes->{{3.607253784645835*^9, 3.607253790588175*^9}}],

Cell[BoxData[{
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{"volumeEqs", ",", 
     SubscriptBox["\[Alpha]", "3"]}], "]"}], "//", 
   "Simplify"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4Free2", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"CatmullClarkForm4Free3", "/.", 
     RowBox[{"sol", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6072538744809732`*^9, 3.6072538809433427`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{
    RowBox[{"CatmullClarkForm4Free2", "/.", 
     RowBox[{
      SubscriptBox["\[Alpha]", "1"], "\[Rule]", "0"}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Alpha]", "2"], "\[Rule]", "0"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607325799716851*^9, 3.60732582308471*^9}, {
  3.6073259680669365`*^9, 3.6073259767620707`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm4", ">>", "\"\<CatmullClarkAlt4.mx\>\""}]], "Input",
 CellChangeTimes->{{3.607326092018841*^9, 3.6073260935688643`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkForm4", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6072538925210047`*^9, 3.607253909231961*^9}, {
  3.607325831994847*^9, 3.6073258518601522`*^9}, {3.607325950349165*^9, 
  3.6073259815946445`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Valence 1 \[Bullet] Form", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
  3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
  3.607244889530364*^9}, {3.609713622841591*^9, 3.6097136244086804`*^9}, {
  3.609713673929513*^9, 3.609713677272704*^9}, {3.6097725004526615`*^9, 
  3.609772502803796*^9}, {3.609789096878298*^9, 3.6097891031006536`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark1Pos", ",", "CatmullClark1Top", ",", 
   RowBox[{"2", "/", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.610474982337351*^9, 3.6104749916648846`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC1sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "1", "1", "0", "0", "0", "0"},
       {"1", "6", "1", "1", "6", "1", "0", "0", "0"},
       {"0", "1", "1", "0", "1", "1", "0", "0", "0"},
       {"0", "0", "1", "0", "0", "1", "0", "0", "0"},
       {"1", "1", "0", "6", "6", "0", "1", "1", "0"},
       {"1", "6", "1", "6", "36", "6", "1", "6", "1"},
       {"0", "1", "1", "0", "6", "6", "0", "1", "1"},
       {"0", "0", "1", "0", "0", "6", "0", "0", "1"},
       {"0", "0", "0", "1", "1", "0", "1", "1", "0"},
       {"0", "0", "0", "1", "6", "1", "1", "6", "1"},
       {"0", "0", "0", "0", "1", "1", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "1", "0", "0", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "1", "0", "1", "6"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC1ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "5", ",", "6", ",", "7", 
       ",", "8", ",", "  ", "9", ",", "10", ",", "11", ",", "12", ",", "  ", 
       "13", ",", "14", ",", "15", ",", "16"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "14", ",", "10", ",", "6", ",", "2", ",", "  ", "15", ",", "11", ",", 
       "7", ",", "3", ",", "  ", "16", ",", "12", ",", "8", ",", "4"}], "}"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "5", ",", "6", ",", "7", ",", "8", ",", "  ", "9", ",", "10", ",", "11",
        ",", "12", ",", "  ", "13", ",", "14", ",", "15", ",", "16"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "  ", "10", ",", "11", ",", "12", ",", 
       "  ", "14", ",", "15", ",", "16"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9, {
   3.609713698685929*^9, 3.609713749219819*^9}, {3.609713812485438*^9, 
   3.6097138571109905`*^9}, {3.609713895973213*^9, 3.609713904018673*^9}, {
   3.6097139373315783`*^9, 3.6097139568506947`*^9}, 3.6097139953518972`*^9, {
   3.6097140289448185`*^9, 3.6097140867901273`*^9}, 3.609714121829131*^9, {
   3.609714350825229*^9, 3.6097143666111317`*^9}, {3.609714397151879*^9, 
   3.6097144649727583`*^9}, {3.609714505777092*^9, 3.609714505972103*^9}, {
   3.609755327970452*^9, 3.609755333392762*^9}, {3.6097896216103106`*^9, 
   3.6097896235404205`*^9}}],

Cell[BoxData[
 RowBox[{"16", " ", "4", " "}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"16", " ", "4", " ", 
   RowBox[{"CC1sub", "\[LeftDoubleBracket]", 
    RowBox[{"CC1ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
    "\[RightDoubleBracket]"}]}], "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.6106597289144325`*^9, 3.6106599145710516`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark1Pos", ",", "CatmullClark1Top", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClark2Top", ",", "CatmullClark2Top", 
     ",", "CatmullClark1Top"}], "}"}], ",", "CC1sub", ",", "CC1ind", ",", 
   RowBox[{"2", "/", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.61023851601857*^9, 3.610238532306502*^9}, {
  3.6102385898197913`*^9, 3.6102385901208086`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}, {
   3.6097157985010314`*^9, 3.609715800474144*^9}, 3.609755337043971*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "9", ",", "9"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609720799394066*^9, 3.6097208041863403`*^9}, {3.6097553759271946`*^9, 
   3.6097553762562137`*^9}, {3.6097896314018707`*^9, 3.609789633643999*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sub", "=", "CC1sub"}], ",", 
       RowBox[{"ind", "=", "CC1ind"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkVal2Form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkVal2Form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
  3.6072490656262226`*^9, 3.6072490711795406`*^9}, {3.609720709872946*^9, 
  3.60972080610745*^9}, {3.609755344501397*^9, 3.6097553563380747`*^9}, {
  3.6097896427265186`*^9, 3.60978966985507*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{
    RowBox[{"BuildEssential", "[", 
     RowBox[{
      RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}], "//", 
    "Simplify"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097208905382795`*^9, 3.6097208984627323`*^9}, {
  3.609720932384673*^9, 3.6097209381130004`*^9}, {3.6097209971443768`*^9, 
  3.6097210092810707`*^9}, {3.6097238276862745`*^9, 3.609723827979291*^9}, {
  3.6097506804256277`*^9, 3.6097506873960257`*^9}, {3.6097896732522645`*^9, 
  3.6097896793816147`*^9}, {3.6097898081309786`*^9, 3.609789808636008*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097507016308403`*^9, 3.6097507116884155`*^9}, {
  3.609789683128829*^9, 3.6097896856279716`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.609723625978738*^9, 3.6097236338431873`*^9}, {
  3.6097554099441404`*^9, 3.6097554122032695`*^9}, {3.6097898188815937`*^9, 
  3.609789819236614*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Skew3", "[", 
    RowBox[{"ArrayReshape", "[", 
     RowBox[{"nul", ",", 
      RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}], "]"}], "//", 
   "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.609723640055543*^9, 3.609723671948367*^9}, {
  3.6097896980726843`*^9, 3.609789698848728*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal1Form", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097554529335995`*^9, 3.6097554724867177`*^9}, {
  3.6097897054331045`*^9, 3.60978971181647*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal1Form", ">>", 
  "\"\<CatmullClarkVal1Alt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6098064994446673`*^9, 3.609806501353777*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkVal1Form", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal1Form", "]"}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755493974947*^9, 3.609755553022324*^9}, {
  3.609789718408847*^9, 3.609789724440192*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal1Form", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "9"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6097554387977905`*^9, 3.6097554748638535`*^9}, {
  3.609755519447404*^9, 3.6097555333421984`*^9}, {3.609789764392477*^9, 
  3.6097897737200108`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal1Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"8", ",", "4", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105574239215517`*^9, 3.6105574643048615`*^9}, {
  3.610573376993816*^9, 3.6105733780408754`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal1Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"7", ",", "3", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105734304328723`*^9, 3.610573437633284*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal1Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"6", ",", "5", ",", "4"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105733912176294`*^9, 3.6105733968089495`*^9}, {
  3.6111294685829*^9, 3.6111294771193886`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal1Form", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"9", ",", "6", ",", "5"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105733912176294`*^9, 3.6105733968089495`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 3 \[Bullet] Form A (3.2)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.6097715042516823`*^9, 3.609771507394862*^9}, {
   3.6097725070930414`*^9, 3.609772507356056*^9}, {3.6097891201496286`*^9, 
   3.6097891271010265`*^9}, 3.60979941797163*^9, {3.610238735878145*^9, 
   3.610238739548355*^9}, {3.612319290858646*^9, 3.6123192911126604`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark3bPos", ",", "CatmullClark3bTop"}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750122250605`*^9, 3.610475020801551*^9}, {
  3.6104750548174963`*^9, 3.610475057112628*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC3Asub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC3Aind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "21", ",", "22"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9, {
   3.609771529166107*^9, 3.6097715448560047`*^9}, {3.609771586823405*^9, 
   3.6097716697821503`*^9}, {3.6097717071742887`*^9, 3.609771714693719*^9}, {
   3.6097718005546293`*^9, 3.6097718361276646`*^9}, {3.6097718981602125`*^9, 
   3.6097718991392684`*^9}, 3.6097721202319145`*^9, {3.609796611769125*^9, 
   3.60979661465629*^9}, {3.609799470357627*^9, 3.609799472029722*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark3bPos", ",", "CatmullClark3bTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", "CatmullClark2Top", 
     ",", "CatmullClark3bTop"}], "}"}], ",", "CC3Asub", ",", "CC3Aind"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6102386320422063`*^9, 3.6102386734265738`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CC3Asub", "\[LeftDoubleBracket]", 
   RowBox[{"CC3Aind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
   "\[RightDoubleBracket]"}], "//", "Eigenvalues"}]], "Input",
 CellChangeTimes->{{3.6112938826633673`*^9, 3.611293905894696*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}, {
   3.6097157985010314`*^9, 3.609715800474144*^9}, 3.609755337043971*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"14", ",", "14", ",", "14"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609720799394066*^9, 3.6097208041863403`*^9}, {3.6097553759271946`*^9, 
   3.6097553762562137`*^9}, {3.6097720065234103`*^9, 3.609772007972493*^9}, {
   3.6097966310972304`*^9, 3.6097966314292493`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sub", "=", "CC3Asub"}], ",", 
       RowBox[{"ind", "=", "CC3Aind"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkVal2Form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
  3.6072490656262226`*^9, 3.6072490711795406`*^9}, {3.609720709872946*^9, 
  3.60972080610745*^9}, {3.609755344501397*^9, 3.6097553563380747`*^9}, {
  3.6097720153829174`*^9, 3.6097720367191377`*^9}, {3.609796644436993*^9, 
  3.609796667332303*^9}, {3.6097994894547186`*^9, 3.6097994959670916`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097208905382795`*^9, 3.6097208984627323`*^9}, {
   3.609720932384673*^9, 3.6097209381130004`*^9}, {3.6097209971443768`*^9, 
   3.6097210092810707`*^9}, {3.6097238276862745`*^9, 3.609723827979291*^9}, {
   3.6097506804256277`*^9, 3.6097506873960257`*^9}, {3.6097720432345104`*^9, 
   3.609772049114846*^9}, {3.6097966873074455`*^9, 3.609796688507514*^9}, 
   3.609796896769426*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097507016308403`*^9, 3.6097507116884155`*^9}, {
  3.609772135615794*^9, 3.609772139218*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.609723625978738*^9, 3.6097236338431873`*^9}, {
   3.6097554099441404`*^9, 3.6097554122032695`*^9}, 3.60979669631196*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal3aForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097554529335995`*^9, 3.6097554724867177`*^9}, {
   3.6097721510796785`*^9, 3.609772151719715*^9}, {3.609796708387651*^9, 
   3.6097967208973665`*^9}, 3.6097967922664485`*^9, 3.609799507099728*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal3aForm", ">>", 
  "\"\<CatmullClarkVal3aAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.609806523021016*^9, 3.6098065259801855`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkVal3aForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal3aForm", "]"}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755493974947*^9, 3.609755553022324*^9}, {
   3.6097721893868694`*^9, 3.6097721906569424`*^9}, {3.609796709736728*^9, 
   3.609796714104978*^9}, {3.6097967937865353`*^9, 3.609796795801651*^9}, 
   3.609799511187962*^9, 3.609800431749615*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal3aForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "14"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6097554387977905`*^9, 3.6097554748638535`*^9}, {
   3.609755519447404*^9, 3.6097555333421984`*^9}, {3.609772194058137*^9, 
   3.609772202929644*^9}, {3.609796729327849*^9, 3.6097967341841264`*^9}, 
   3.609796798633813*^9, 3.609799513460092*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Skew3", "[", "CatmullClarkVal3aForm", "]"}], 
   "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//", 
  "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.611129586190627*^9, 3.611129589473815*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal3aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"11", ",", "3", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6105575367260036`*^9, 3.6105575426163406`*^9}, {
  3.6105576507595263`*^9, 3.6105576652043524`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal3aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"5", ",", "4", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.611129558287031*^9, 3.6111296084859023`*^9}, 
   3.6111296481101685`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal3aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"9", ",", "5", ",", "3"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.611129669395386*^9, 3.611129695068855*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 3 \[Bullet] Form B (3.1)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.609769897702793*^9, 3.609769904774197*^9}, {
   3.6097725048049107`*^9, 3.609772505075926*^9}, {3.6097891062768354`*^9, 
   3.6097891153483543`*^9}, 3.609799419819736*^9, {3.610238743916605*^9, 
   3.6102387456847067`*^9}, {3.6123193162811003`*^9, 3.612319317768185*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark3aPos", ",", "CatmullClark3aTop"}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750441368856`*^9, 3.6104750491761737`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC3Bsub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC3Bind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9, {
   3.609769918412977*^9, 3.609769923284256*^9}, {3.6097700548027782`*^9, 
   3.609770088553709*^9}, {3.609770143265838*^9, 3.6097701612718678`*^9}, {
   3.6097702062814426`*^9, 3.6097702088145876`*^9}, 3.6097706487647514`*^9, {
   3.6097961106204605`*^9, 3.60979611602977*^9}, {3.6097995260378113`*^9, 
   3.609799527990923*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark3aPos", ",", "CatmullClark3aTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", 
     "CatmullClarkRegTop", ",", "CatmullClark3aTop"}], "}"}], ",", "CC3Bsub", 
   ",", "CC3Bind"}], "]"}]], "Input",
 CellChangeTimes->{{3.6102387507619967`*^9, 3.610238764512783*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CC3Bsub", "\[LeftDoubleBracket]", 
   RowBox[{"CC3Bind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
   "\[RightDoubleBracket]"}], "//", "Eigenvalues"}]], "Input",
 CellChangeTimes->{{3.6112938826633673`*^9, 3.611293905894696*^9}, {
  3.611294164173469*^9, 3.611294167444656*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6097505575465994`*^9, 3.6097505621058598`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"15", ",", "15", ",", "15"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"form", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609705893306487*^9, 3.6097058952665987`*^9}, {3.609707184746353*^9, 
   3.609707189218609*^9}, {3.609749340221972*^9, 3.6097493490584774`*^9}, {
   3.609750538168491*^9, 3.6097505405456266`*^9}, {3.609770331047579*^9, 
   3.609770333326709*^9}, {3.609796133807787*^9, 3.6097961381180334`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"RHS", "[", 
   RowBox[{"reg_", ",", "gen_", ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"gen", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"RHS", "[", 
    RowBox[{
    "CatmullClarkForm4", ",", "form", ",", "CC3Bsub", ",", "CC3Bind"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
   3.6072490656262226`*^9, 3.6072490711795406`*^9}, 3.6097062865219774`*^9, 
   3.609706319720876*^9, {3.609707059148169*^9, 3.609707071435872*^9}, {
   3.6097072012572975`*^9, 3.6097072023443594`*^9}, {3.6097192543796964`*^9, 
   3.6097192547327166`*^9}, {3.6097703430262637`*^9, 3.60977034872859*^9}, {
   3.609770663796611*^9, 3.609770665937733*^9}, {3.609796158145179*^9, 
   3.6097961645855474`*^9}, {3.6097995396845922`*^9, 3.609799543080786*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097074665034685`*^9, 3.609707466720481*^9}, {
  3.6097495110267415`*^9, 3.609749523401449*^9}, {3.609750018662777*^9, 
  3.609750019021797*^9}, {3.6097703574950914`*^9, 3.609770361027293*^9}, {
  3.609770668940905*^9, 3.609770683097715*^9}, {3.609796173678067*^9, 
  3.6097961749741416`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6097706935843143`*^9, 3.6097706967724967`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "MatrixRank"}]], "Input",
 CellChangeTimes->{{3.609707490606847*^9, 3.609707492634963*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.609750068807645*^9, 3.609750089232813*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal3bForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097504461542277`*^9, 3.6097504527856073`*^9}, {
   3.609752376270624*^9, 3.6097523770046663`*^9}, {3.6097708610898952`*^9, 
   3.6097708899135437`*^9}, {3.609796195803333*^9, 3.609796210331164*^9}, 
   3.60979956010276*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal3bForm", ">>", 
  "\"\<CatmullClarkVal3bAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.609806543685198*^9, 3.609806546204342*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CatmullClarkVal3bForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal3bForm", "]"}]}], "//", 
   "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.609796263957231*^9, 3.609796272628727*^9}, 
   3.609799562492896*^9, 3.6097998547326117`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal3bForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "15"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.609719414542857*^9, 3.6097194217562695`*^9}, 
   3.609719465344763*^9, {3.609719712274886*^9, 3.609719713712969*^9}, {
   3.609750225981635*^9, 3.6097502262836523`*^9}, {3.6097502756124735`*^9, 
   3.6097502764695225`*^9}, {3.609750423327922*^9, 3.6097504565168204`*^9}, {
   3.6097522128222756`*^9, 3.60975223493254*^9}, {3.6097524432134533`*^9, 
   3.609752447908722*^9}, {3.609755143546903*^9, 3.609755157353693*^9}, {
   3.6097708978119955`*^9, 3.6097709462907686`*^9}, 3.6097962195586915`*^9, 
   3.6097962790070915`*^9, 3.6097995643120003`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal3bForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"10", ",", "2", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.610557556328125*^9, 3.6105576051129155`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 4 \[Bullet] Form A (4.2)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.6097715042516823`*^9, 3.609771507394862*^9}, {
   3.6097725070930414`*^9, 3.609772507356056*^9}, {3.6097891201496286`*^9, 
   3.6097891271010265`*^9}, 3.60979941797163*^9, 3.6097997577860665`*^9, {
   3.6105709778325915`*^9, 3.610570979463685*^9}, {3.612319321993427*^9, 
   3.6123193249285946`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark4aPos", ",", "CatmullClark4aTop", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750838401566`*^9, 3.610475093030682*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC4Asub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "1", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "1", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "1", "1", 
        "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "1", "1", 
        "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC4Aind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9, {
   3.609771529166107*^9, 3.6097715448560047`*^9}, {3.609771586823405*^9, 
   3.6097716697821503`*^9}, {3.6097717071742887`*^9, 3.609771714693719*^9}, {
   3.6097718005546293`*^9, 3.6097718361276646`*^9}, {3.6097718981602125`*^9, 
   3.6097718991392684`*^9}, 3.6097721202319145`*^9, {3.609796611769125*^9, 
   3.60979661465629*^9}, {3.609799470357627*^9, 3.609799472029722*^9}, {
   3.6097997997604675`*^9, 3.609799836439565*^9}, {3.6097998713065596`*^9, 
   3.6097999160981216`*^9}, {3.609799949171013*^9, 3.609800025242364*^9}, 
   3.609800172569791*^9, {3.609800278557853*^9, 3.609800343646576*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark4bPos", ",", "CatmullClark4bTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", "CatmullClark2Top", 
     ",", "CatmullClark4bTop"}], "}"}], ",", "CC4Asub", ",", "CC4Aind", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6102394726522865`*^9, 3.6102394816267996`*^9}, 
   3.610239675797906*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}, {
   3.6097157985010314`*^9, 3.609715800474144*^9}, 3.609755337043971*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"16", ",", "16", ",", "16"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609720799394066*^9, 3.6097208041863403`*^9}, {3.6097553759271946`*^9, 
   3.6097553762562137`*^9}, {3.6097720065234103`*^9, 3.609772007972493*^9}, {
   3.6097966310972304`*^9, 3.6097966314292493`*^9}, {3.609800376654464*^9, 
   3.6098003780065413`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sub", "=", "CC4Asub"}], ",", 
       RowBox[{"ind", "=", "CC4Aind"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkVal2Form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
  3.6072490656262226`*^9, 3.6072490711795406`*^9}, {3.609720709872946*^9, 
  3.60972080610745*^9}, {3.609755344501397*^9, 3.6097553563380747`*^9}, {
  3.6097720153829174`*^9, 3.6097720367191377`*^9}, {3.609796644436993*^9, 
  3.609796667332303*^9}, {3.6097994894547186`*^9, 3.6097994959670916`*^9}, {
  3.60980038637002*^9, 3.6098003880961185`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097208905382795`*^9, 3.6097208984627323`*^9}, {
   3.609720932384673*^9, 3.6097209381130004`*^9}, {3.6097209971443768`*^9, 
   3.6097210092810707`*^9}, {3.6097238276862745`*^9, 3.609723827979291*^9}, {
   3.6097506804256277`*^9, 3.6097506873960257`*^9}, {3.6097720432345104`*^9, 
   3.609772049114846*^9}, {3.6097966873074455`*^9, 3.609796688507514*^9}, 
   3.609796896769426*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097507016308403`*^9, 3.6097507116884155`*^9}, {
  3.609772135615794*^9, 3.609772139218*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.609723625978738*^9, 3.6097236338431873`*^9}, {
   3.6097554099441404`*^9, 3.6097554122032695`*^9}, 3.60979669631196*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal4aForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097554529335995`*^9, 3.6097554724867177`*^9}, {
   3.6097721510796785`*^9, 3.609772151719715*^9}, {3.609796708387651*^9, 
   3.6097967208973665`*^9}, 3.6097967922664485`*^9, 3.609799507099728*^9, {
   3.609800401853905*^9, 3.609800402036916*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal4aForm", ">>", 
  "\"\<CatmullClarkVal4aAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6098065628062916`*^9, 3.6098065659324703`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkVal4aForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal4aForm", "]"}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755493974947*^9, 3.609755553022324*^9}, {
   3.6097721893868694`*^9, 3.6097721906569424`*^9}, {3.609796709736728*^9, 
   3.609796714104978*^9}, {3.6097967937865353`*^9, 3.609796795801651*^9}, 
   3.609799511187962*^9, {3.609800406926195*^9, 3.609800418421853*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal4aForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "14"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6097554387977905`*^9, 3.6097554748638535`*^9}, {
   3.609755519447404*^9, 3.6097555333421984`*^9}, {3.609772194058137*^9, 
   3.609772202929644*^9}, {3.609796729327849*^9, 3.6097967341841264`*^9}, 
   3.609796798633813*^9, 3.609799513460092*^9, {3.609800423246129*^9, 
   3.6098004234131384`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal4aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"14", ",", "3", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.610557776685729*^9, 3.6105578190941544`*^9}, {
  3.6105578495108943`*^9, 3.6105578753263707`*^9}, {3.6107721012510433`*^9, 
  3.6107721075144014`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal4aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"9", ",", "4", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6111299129483166`*^9, 3.611129926557095*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 4 \[Bullet] Form B (4.1)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.609769897702793*^9, 3.609769904774197*^9}, {
   3.6097725048049107`*^9, 3.609772505075926*^9}, {3.6097891062768354`*^9, 
   3.6097891153483543`*^9}, 3.609799419819736*^9, 3.60980115845518*^9, {
   3.6105709727363*^9, 3.6105709757924747`*^9}, {3.6123193294488535`*^9, 
   3.6123193308569336`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark4bPos", ",", "CatmullClark4bTop", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750838401566`*^9, 3.6104751151519475`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC4Bsub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC4Bind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26"}], "}"}]}], 
    "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.606012291208367*^9, 3.6060123076683083`*^9}, {
   3.609801173957067*^9, 3.6098011923521194`*^9}, {3.6098012279881573`*^9, 
   3.609801257878867*^9}, {3.6098013098968425`*^9, 3.609801342776723*^9}, 
   3.609801462750585*^9, 3.6098014956564674`*^9, 3.609801612864171*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark4aPos", ",", "CatmullClark4aTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", 
     "CatmullClarkRegTop", ",", "CatmullClark4aTop"}], "}"}], ",", "CC4Bsub", 
   ",", "CC4Bind", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6102392680945864`*^9, 3.6102392970862446`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6097505575465994`*^9, 3.6097505621058598`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"17", ",", "17", ",", "17"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"form", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609705893306487*^9, 3.6097058952665987`*^9}, {3.609707184746353*^9, 
   3.609707189218609*^9}, {3.609749340221972*^9, 3.6097493490584774`*^9}, {
   3.609750538168491*^9, 3.6097505405456266`*^9}, {3.609770331047579*^9, 
   3.609770333326709*^9}, {3.609796133807787*^9, 3.6097961381180334`*^9}, {
   3.60980163680554*^9, 3.609801639198677*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"RHS", "[", 
   RowBox[{"reg_", ",", "gen_", ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"gen", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"RHS", "[", 
    RowBox[{
    "CatmullClarkForm4", ",", "form", ",", "CC4Bsub", ",", "CC4Bind"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
   3.6072490656262226`*^9, 3.6072490711795406`*^9}, 3.6097062865219774`*^9, 
   3.609706319720876*^9, {3.609707059148169*^9, 3.609707071435872*^9}, {
   3.6097072012572975`*^9, 3.6097072023443594`*^9}, {3.6097192543796964`*^9, 
   3.6097192547327166`*^9}, {3.6097703430262637`*^9, 3.60977034872859*^9}, {
   3.609770663796611*^9, 3.609770665937733*^9}, {3.609796158145179*^9, 
   3.6097961645855474`*^9}, {3.6097995396845922`*^9, 3.609799543080786*^9}, {
   3.609801650537326*^9, 3.60980165464056*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097074665034685`*^9, 3.609707466720481*^9}, {
  3.6097495110267415`*^9, 3.609749523401449*^9}, {3.609750018662777*^9, 
  3.609750019021797*^9}, {3.6097703574950914`*^9, 3.609770361027293*^9}, {
  3.609770668940905*^9, 3.609770683097715*^9}, {3.609796173678067*^9, 
  3.6097961749741416`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6097706935843143`*^9, 3.6097706967724967`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "MatrixRank"}]], "Input",
 CellChangeTimes->{{3.609707490606847*^9, 3.609707492634963*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.609750068807645*^9, 3.609750089232813*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal4bForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097504461542277`*^9, 3.6097504527856073`*^9}, {
   3.609752376270624*^9, 3.6097523770046663`*^9}, {3.6097708610898952`*^9, 
   3.6097708899135437`*^9}, {3.609796195803333*^9, 3.609796210331164*^9}, 
   3.60979956010276*^9, {3.6098017108287745`*^9, 3.609801711019785*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal4bForm", ">>", 
  "\"\<CatmullClarkVal4bAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.609806579412241*^9, 3.6098065819233847`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CatmullClarkVal4bForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal4bForm", "]"}]}], "//", 
   "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.609796263957231*^9, 3.609796272628727*^9}, 
   3.609799562492896*^9, 3.6097998547326117`*^9, {3.6098017238125167`*^9, 
   3.6098017270297008`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal4bForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "17"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.609719414542857*^9, 3.6097194217562695`*^9}, 
   3.609719465344763*^9, {3.609719712274886*^9, 3.609719713712969*^9}, {
   3.609750225981635*^9, 3.6097502262836523`*^9}, {3.6097502756124735`*^9, 
   3.6097502764695225`*^9}, {3.609750423327922*^9, 3.6097504565168204`*^9}, {
   3.6097522128222756`*^9, 3.60975223493254*^9}, {3.6097524432134533`*^9, 
   3.609752447908722*^9}, {3.609755143546903*^9, 3.609755157353693*^9}, {
   3.6097708978119955`*^9, 3.6097709462907686`*^9}, 3.6097962195586915`*^9, 
   3.6097962790070915`*^9, 3.6097995643120003`*^9, {3.6098017338770924`*^9, 
   3.6098017363802357`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal4bForm", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6105709586174927`*^9, 3.6105709594335394`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal4bForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"15", ",", "3", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6111297647398396`*^9, 3.6111297809797688`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal4bForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"13", ",", "4", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.610557776685729*^9, 3.6105578190941544`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 5 \[Bullet] Form A (5.3)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.6097715042516823`*^9, 3.609771507394862*^9}, {
   3.6097725070930414`*^9, 3.609772507356056*^9}, {3.6097891201496286`*^9, 
   3.6097891271010265`*^9}, 3.60979941797163*^9, 3.6097997577860665`*^9, 
   3.609806838177042*^9, {3.6105709379683113`*^9, 3.610570941112491*^9}, {
   3.6123193350891757`*^9, 3.612319337296302*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark5aPos", ",", "CatmullClark5aTop", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750838401566`*^9, 3.610475093030682*^9}, {
  3.610475127728667*^9, 3.610475129271755*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC5Asub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "1", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "1", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "1", "1", 
        "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "1", "1", 
        "6", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC5Aind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24", ",", "25", ",", "26"}], 
      "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6060143588756304`*^9, 3.606014372372402*^9}, 
   3.6065132271372857`*^9, {3.609806865686615*^9, 3.6098068794574027`*^9}, {
   3.609807853390109*^9, 3.6098079455933824`*^9}, 3.6098080994011793`*^9, 
   3.6098081363172913`*^9, {3.609808243257408*^9, 3.609808271564027*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark5cPos", ",", "CatmullClark5cTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", "CatmullClark2Top", 
     ",", "CatmullClark5cTop"}], "}"}], ",", "CC5Asub", ",", "CC5Aind", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6102394726522865`*^9, 3.6102394816267996`*^9}, 
   3.610239675797906*^9, {3.6102421847124076`*^9, 3.6102421971191173`*^9}}],

Cell[BoxData[
 RowBox[{
  GraphicsBox[{{
     {GrayLevel[0.5], LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}],
       LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
      LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
      LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
      LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
      LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
      LineBox[NCache[{{0, 0}, {
         Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
         Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}, {0, 0}}, {{0, 0}, {
         0.5, 0.8660254037844386}, {0.3660254037844386, 
         1.3660254037844386`}, {0, 1}, {0, 0}}]], 
      LineBox[NCache[{{0, 0}, {
         Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 1}, {
         Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0, 0}}, {{0, 0}, {
         0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}, {0, 
         0}}]], LineBox[
       NCache[{{0, 0}, {1, 0}, {
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
         Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {0, 0}}, {{0, 0}, {
         1, 0}, {1.3660254037844386`, 0.3660254037844386}, {
         0.8660254037844386, 0.5}, {0, 0}}]]}, 
     {GrayLevel[0.6], Thickness[0.02], 
      LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}, 
     {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], DiskBox[{-2, -2}, 0.04],
       DiskBox[{-1, -2}, 0.04], DiskBox[{0, -2}, 0.04], 
      DiskBox[{1, -2}, 0.04], DiskBox[{-2, -1}, 0.04], 
      DiskBox[{-1, -1}, 0.04], DiskBox[{0, -1}, 0.04], DiskBox[{1, -1}, 0.04],
       DiskBox[{-2, 0}, 0.04], DiskBox[{-1, 0}, 0.04], DiskBox[{0, 0}, 0.04], 
      DiskBox[{1, 0}, 0.04], DiskBox[{0, 1}, 0.04], 
      DiskBox[NCache[{
        Rational[1, 2] (-1 + 3^Rational[1, 2]), 
         Rational[1, 2] (1 + 3^Rational[1, 2])}, {0.3660254037844386, 
        1.3660254037844386`}], 0.04], 
      DiskBox[NCache[{Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0.5, 
        0.8660254037844386}], 0.04], DiskBox[{1, 1}, 0.04], 
      DiskBox[
       NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {
        0.8660254037844386, 0.5}], 0.04], 
      DiskBox[NCache[{
        Rational[1, 2] (1 + 3^Rational[1, 2]), 
         Rational[1, 2] (-1 + 3^Rational[1, 2])}, {1.3660254037844386`, 
        0.3660254037844386}], 0.04]}}, {
     {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}}], 
      PolygonBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}}], 
      PolygonBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}}], 
      PolygonBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}}], 
      PolygonBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}}], 
      PolygonBox[
       NCache[{{0, 0}, {Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
         Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}}, {{0, 0}, {0.5, 
         0.8660254037844386}, {0.3660254037844386, 1.3660254037844386`}, {0, 
         1}}]], PolygonBox[
       NCache[{{0, 0}, {Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 
         1}, {Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}}, {{0, 0}, {
         0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}}]], 
      PolygonBox[
       NCache[{{0, 0}, {1, 0}, {
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
         Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}}, {{0, 0}, {1, 0}, {
         1.3660254037844386`, 0.3660254037844386}, {0.8660254037844386, 
         0.5}}]]}, 
     {RGBColor[1, 0, 0], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}]}, 
     {GrayLevel[0.3], Thickness[0.02], EdgeForm[Dashing[{Small, Small}]], 
      LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}}},
   Axes->False,
   ImageSize->121], "=", 
  RowBox[{
   GraphicsBox[{{
      {GrayLevel[0.5], 
       LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
       LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
       LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
       LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
       LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
       LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] (-1 + 3^Rational[1, 2]), 
           Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}, {0, 0}}, {{0, 0}, {
          0.5, 0.8660254037844386}, {0.3660254037844386, 
          1.3660254037844386`}, {0, 1}, {0, 0}}]], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 1}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}, {
          0, 0}}]], 
       LineBox[NCache[{{0, 0}, {1, 0}, {
          Rational[1, 2] (1 + 3^Rational[1, 2]), 
           Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {1, 0}, {1.3660254037844386`, 0.3660254037844386}, {
          0.8660254037844386, 0.5}, {0, 0}}]]}, 
      {GrayLevel[0.6], Thickness[0.02], 
       LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}, 
      {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], 
       DiskBox[{-2, -2}, 0.04], DiskBox[{-1, -2}, 0.04], 
       DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
       DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], 
       DiskBox[{0, -1}, 0.04], DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04],
        DiskBox[{-1, 0}, 0.04], DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], 
       DiskBox[{0, 1}, 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0.3660254037844386, 
         1.3660254037844386`}], 0.04], 
       DiskBox[NCache[{Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0.5,
          0.8660254037844386}], 0.04], DiskBox[{1, 1}, 0.04], 
       DiskBox[NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {
         0.8660254037844386, 0.5}], 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {1.3660254037844386`, 
         0.3660254037844386}], 0.04]}}, {
      {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{
          Rational[-3, 2], Rational[-3, 2]}, {-1, 
           Rational[-3, 2]}, {-1, -1}, {
          Rational[-3, 
            2], -1}}, {{-1.5, -1.5}, {-1, -1.5}, {-1, -1}, {-1.5, -1}}]], 
       PolygonBox[
        NCache[{{-1, Rational[-3, 2]}, {Rational[-1, 2], Rational[-3, 2]}, {
          Rational[-1, 
            2], -1}, {-1, -1}}, {{-1, -1.5}, {-0.5, -1.5}, {-0.5, -1}, {-1, \
-1}}]], PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-3, 2]}, {0, Rational[-3, 2]}, {
          0, -1}, {Rational[-1, 2], -1}}, {{-0.5, -1.5}, {0, -1.5}, {
          0, -1}, {-0.5, -1}}]], 
       PolygonBox[
        NCache[{{Rational[-3, 2], -1}, {-1, -1}, {-1, Rational[-1, 2]}, {
          Rational[-3, 2], 
           Rational[-1, 
            2]}}, {{-1.5, -1}, {-1, -1}, {-1, -0.5}, {-1.5, -0.5}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], -1}, {0, -1}, {0, Rational[-1, 2]}, {
          Rational[-1, 2], Rational[-1, 2]}}, {{-0.5, -1}, {0, -1}, {
          0, -0.5}, {-0.5, -0.5}}]], 
       PolygonBox[
        NCache[{{
          Rational[-3, 2], Rational[-1, 2]}, {-1, Rational[-1, 2]}, {-1, 0}, {
          Rational[-3, 2], 0}}, {{-1.5, -0.5}, {-1, -0.5}, {-1, 
          0}, {-1.5, 0}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-1, 2]}, {Rational[-1, 2], 0}, {-1,
           0}, {-1, Rational[-1, 2]}}, {{-0.5, -0.5}, {-0.5, 0}, {-1, 
          0}, {-1, -0.5}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-1, 2]}, {0, Rational[-1, 2]}, {
          Rational[-1, 8], Rational[1, 8]}, {
          Rational[-1, 2], 0}}, {{-0.5, -0.5}, {0, -0.5}, {-0.125, 
          0.125}, {-0.5, 0}}]]}, 
      {RGBColor[0, 1, 0], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{-1, -1}, {Rational[-1, 2], -1}, {
          Rational[-1, 2], Rational[-1, 2]}, {-1, 
           Rational[-1, 
            2]}}, {{-1, -1}, {-0.5, -1}, {-0.5, -0.5}, {-1, -0.5}}]]}, 
      {GrayLevel[0.3], Thickness[0.02], EdgeForm[Dashing[{Small, Small}]], 
       LineBox[{}]}}},
    Axes->False,
    ImageSize->121], "\[Union]", 
   GraphicsBox[{{
      {GrayLevel[0.5], 
       LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
       LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
       LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
       LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
       LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
       LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] (-1 + 3^Rational[1, 2]), 
           Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}, {0, 0}}, {{0, 0}, {
          0.5, 0.8660254037844386}, {0.3660254037844386, 
          1.3660254037844386`}, {0, 1}, {0, 0}}]], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 1}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}, {
          0, 0}}]], 
       LineBox[NCache[{{0, 0}, {1, 0}, {
          Rational[1, 2] (1 + 3^Rational[1, 2]), 
           Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {1, 0}, {1.3660254037844386`, 0.3660254037844386}, {
          0.8660254037844386, 0.5}, {0, 0}}]]}, 
      {GrayLevel[0.6], Thickness[0.02], 
       LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}, 
      {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], 
       DiskBox[{-2, -2}, 0.04], DiskBox[{-1, -2}, 0.04], 
       DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
       DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], 
       DiskBox[{0, -1}, 0.04], DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04],
        DiskBox[{-1, 0}, 0.04], DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], 
       DiskBox[{0, 1}, 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0.3660254037844386, 
         1.3660254037844386`}], 0.04], 
       DiskBox[NCache[{Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0.5,
          0.8660254037844386}], 0.04], DiskBox[{1, 1}, 0.04], 
       DiskBox[NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {
         0.8660254037844386, 0.5}], 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {1.3660254037844386`, 
         0.3660254037844386}], 0.04]}}, {
      {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{-1, Rational[-3, 2]}, {Rational[-1, 2], Rational[-3, 2]}, {
          Rational[-1, 
            2], -1}, {-1, -1}}, {{-1, -1.5}, {-0.5, -1.5}, {-0.5, -1}, {-1, \
-1}}]], PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-3, 2]}, {0, Rational[-3, 2]}, {
          0, -1}, {Rational[-1, 2], -1}}, {{-0.5, -1.5}, {0, -1.5}, {
          0, -1}, {-0.5, -1}}]], 
       PolygonBox[
        NCache[{{0, Rational[-3, 2]}, {Rational[1, 2], Rational[-3, 2]}, {
          Rational[1, 2], -1}, {0, -1}}, {{0, -1.5}, {0.5, -1.5}, {0.5, -1}, {
          0, -1}}]], 
       PolygonBox[
        NCache[{{-1, -1}, {Rational[-1, 2], -1}, {
          Rational[-1, 2], Rational[-1, 2]}, {-1, 
           Rational[-1, 
            2]}}, {{-1, -1}, {-0.5, -1}, {-0.5, -0.5}, {-1, -0.5}}]], 
       PolygonBox[
        NCache[{{0, -1}, {Rational[1, 2], -1}, {
          Rational[1, 2], Rational[-1, 2]}, {0, Rational[-1, 2]}}, {{0, -1}, {
          0.5, -1}, {0.5, -0.5}, {0, -0.5}}]], 
       PolygonBox[
        NCache[{{-1, Rational[-1, 2]}, {Rational[-1, 2], Rational[-1, 2]}, {
          Rational[-1, 2], 0}, {-1, 
          0}}, {{-1, -0.5}, {-0.5, -0.5}, {-0.5, 0}, {-1, 0}}]], 
       PolygonBox[
        NCache[{{0, Rational[-1, 2]}, {Rational[-1, 8], Rational[1, 8]}, {
          Rational[-1, 2], 0}, {Rational[-1, 2], Rational[-1, 2]}}, {{
          0, -0.5}, {-0.125, 0.125}, {-0.5, 0}, {-0.5, -0.5}}]], 
       PolygonBox[
        NCache[{{0, Rational[-1, 2]}, {Rational[1, 2], Rational[-1, 2]}, {
          Rational[7, 16] + Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2]), Rational[-3, 32] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2])}, {
          Rational[-1, 8], Rational[1, 8]}}, {{0, -0.5}, {0.5, -0.5}, {
          0.5770031754730548, -0.07087341226347259}, {-0.125, 0.125}}]]}, 
      {RGBColor[0, 1, 0], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], -1}, {0, -1}, {0, Rational[-1, 2]}, {
          Rational[-1, 2], Rational[-1, 2]}}, {{-0.5, -1}, {0, -1}, {
          0, -0.5}, {-0.5, -0.5}}]]}, 
      {GrayLevel[0.3], Thickness[0.02], EdgeForm[Dashing[{Small, Small}]], 
       LineBox[{}]}}},
    Axes->False,
    ImageSize->121], "\[Union]", 
   GraphicsBox[{{
      {GrayLevel[0.5], 
       LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
       LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
       LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
       LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
       LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
       LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] (-1 + 3^Rational[1, 2]), 
           Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}, {0, 0}}, {{0, 0}, {
          0.5, 0.8660254037844386}, {0.3660254037844386, 
          1.3660254037844386`}, {0, 1}, {0, 0}}]], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 1}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}, {
          0, 0}}]], 
       LineBox[NCache[{{0, 0}, {1, 0}, {
          Rational[1, 2] (1 + 3^Rational[1, 2]), 
           Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {1, 0}, {1.3660254037844386`, 0.3660254037844386}, {
          0.8660254037844386, 0.5}, {0, 0}}]]}, 
      {GrayLevel[0.6], Thickness[0.02], 
       LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}, 
      {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], 
       DiskBox[{-2, -2}, 0.04], DiskBox[{-1, -2}, 0.04], 
       DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
       DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], 
       DiskBox[{0, -1}, 0.04], DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04],
        DiskBox[{-1, 0}, 0.04], DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], 
       DiskBox[{0, 1}, 0.04], 
       DiskBox[
        NCache[{Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0.3660254037844386, 
         1.3660254037844386`}], 0.04], 
       DiskBox[NCache[{Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0.5,
          0.8660254037844386}], 0.04], DiskBox[{1, 1}, 0.04], 
       DiskBox[NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {
         0.8660254037844386, 0.5}], 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {1.3660254037844386`, 
         0.3660254037844386}], 0.04]}}, {
      {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{Rational[-3, 2], -1}, {-1, -1}, {-1, Rational[-1, 2]}, {
          Rational[-3, 2], 
           Rational[-1, 
            2]}}, {{-1.5, -1}, {-1, -1}, {-1, -0.5}, {-1.5, -0.5}}]], 
       PolygonBox[
        NCache[{{-1, -1}, {Rational[-1, 2], -1}, {
          Rational[-1, 2], Rational[-1, 2]}, {-1, 
           Rational[-1, 
            2]}}, {{-1, -1}, {-0.5, -1}, {-0.5, -0.5}, {-1, -0.5}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], -1}, {0, -1}, {0, Rational[-1, 2]}, {
          Rational[-1, 2], Rational[-1, 2]}}, {{-0.5, -1}, {0, -1}, {
          0, -0.5}, {-0.5, -0.5}}]], 
       PolygonBox[
        NCache[{{
          Rational[-3, 2], Rational[-1, 2]}, {-1, Rational[-1, 2]}, {-1, 0}, {
          Rational[-3, 2], 0}}, {{-1.5, -0.5}, {-1, -0.5}, {-1, 
          0}, {-1.5, 0}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-1, 2]}, {0, Rational[-1, 2]}, {
          Rational[-1, 8], Rational[1, 8]}, {
          Rational[-1, 2], 0}}, {{-0.5, -0.5}, {0, -0.5}, {-0.125, 
          0.125}, {-0.5, 0}}]]}, 
      {RGBColor[1, 1, 0], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{-1, Rational[-1, 2]}, {Rational[-1, 2], Rational[-1, 2]}, {
          Rational[-1, 2], 0}, {-1, 
          0}}, {{-1, -0.5}, {-0.5, -0.5}, {-0.5, 0}, {-1, 0}}]]}, 
      {GrayLevel[0.3], Thickness[0.02], EdgeForm[Dashing[{Small, Small}]], 
       LineBox[NCache[{{Rational[-3, 2], 0}, {-1, 0}, {Rational[-1, 2], 0}, {
          Rational[-1, 8], Rational[1, 8]}}, {{-1.5, 0}, {-1, 
          0}, {-0.5, 0}, {-0.125, 0.125}}]]}}},
    Axes->False,
    ImageSize->121], "\[Union]", 
   GraphicsBox[{{
      {GrayLevel[0.5], 
       LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
       LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
       LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
       LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
       LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
       LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] (-1 + 3^Rational[1, 2]), 
           Rational[1, 2] (1 + 3^Rational[1, 2])}, {0, 1}, {0, 0}}, {{0, 0}, {
          0.5, 0.8660254037844386}, {0.3660254037844386, 
          1.3660254037844386`}, {0, 1}, {0, 0}}]], 
       LineBox[NCache[{{0, 0}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {1, 1}, {
          Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {0.8660254037844386, 0.5}, {1, 1}, {0.5, 0.8660254037844386}, {
          0, 0}}]], 
       LineBox[NCache[{{0, 0}, {1, 0}, {
          Rational[1, 2] (1 + 3^Rational[1, 2]), 
           Rational[1, 2] (-1 + 3^Rational[1, 2])}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {0, 0}}, {{0, 
          0}, {1, 0}, {1.3660254037844386`, 0.3660254037844386}, {
          0.8660254037844386, 0.5}, {0, 0}}]]}, 
      {GrayLevel[0.6], Thickness[0.02], 
       LineBox[{{-2, 0}, {-1, 0}, {0, 0}, {0, 1}}]}, 
      {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], 
       DiskBox[{-2, -2}, 0.04], DiskBox[{-1, -2}, 0.04], 
       DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
       DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], 
       DiskBox[{0, -1}, 0.04], DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04],
        DiskBox[{-1, 0}, 0.04], DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], 
       DiskBox[{0, 1}, 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (-1 + 3^Rational[1, 2]), 
          Rational[1, 2] (1 + 3^Rational[1, 2])}, {0.3660254037844386, 
         1.3660254037844386`}], 0.04], 
       DiskBox[
        NCache[{Rational[1, 2], Rational[1, 2] 3^Rational[1, 2]}, {0.5, 
         0.8660254037844386}], 0.04], DiskBox[{1, 1}, 0.04], 
       DiskBox[NCache[{Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}, {
         0.8660254037844386, 0.5}], 0.04], 
       DiskBox[NCache[{
         Rational[1, 2] (1 + 3^Rational[1, 2]), 
          Rational[1, 2] (-1 + 3^Rational[1, 2])}, {1.3660254037844386`, 
         0.3660254037844386}], 0.04]}}, {
      {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{-1, -1}, {Rational[-1, 2], -1}, {
          Rational[-1, 2], Rational[-1, 2]}, {-1, 
           Rational[-1, 
            2]}}, {{-1, -1}, {-0.5, -1}, {-0.5, -0.5}, {-1, -0.5}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], -1}, {0, -1}, {0, Rational[-1, 2]}, {
          Rational[-1, 2], Rational[-1, 2]}}, {{-0.5, -1}, {0, -1}, {
          0, -0.5}, {-0.5, -0.5}}]], 
       PolygonBox[
        NCache[{{0, -1}, {Rational[1, 2], -1}, {
          Rational[1, 2], Rational[-1, 2]}, {0, Rational[-1, 2]}}, {{0, -1}, {
          0.5, -1}, {0.5, -0.5}, {0, -0.5}}]], 
       PolygonBox[
        NCache[{{-1, Rational[-1, 2]}, {Rational[-1, 2], Rational[-1, 2]}, {
          Rational[-1, 2], 0}, {-1, 
          0}}, {{-1, -0.5}, {-0.5, -0.5}, {-0.5, 0}, {-1, 0}}]], 
       PolygonBox[
        NCache[{{0, Rational[-1, 2]}, {Rational[1, 2], Rational[-1, 2]}, {
          Rational[7, 16] + Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2]), Rational[-3, 32] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2])}, {
          Rational[-1, 8], Rational[1, 8]}}, {{0, -0.5}, {0.5, -0.5}, {
          0.5770031754730548, -0.07087341226347259}, {-0.125, 0.125}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 8], Rational[1, 8]}, {
          Rational[1, 4] + Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2]), Rational[5, 32] + 
           Rational[3, 16] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2])}, {
          Rational[1, 8] + Rational[1, 8] (-1 + 3^Rational[1, 2]), 
           Rational[1, 4] + Rational[1, 8] 3^Rational[1, 2] + 
           Rational[1, 8] (1 + 3^Rational[1, 2])}, {
          0, Rational[1, 2]}}, {{-0.125, 0.125}, {0.3270031754730548, 
          0.566386114155692}, {0.21650635094610965`, 0.8080127018922193}, {
          0, 0.5}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 8], Rational[1, 8]}, {
          Rational[5, 32] + Rational[3, 16] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2]), Rational[1, 4] + 
           Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2])}, {
          Rational[3, 8] + Rational[1, 8] 3^Rational[1, 2], Rational[3, 8] + 
           Rational[1, 8] 3^Rational[1, 2]}, {
          Rational[1, 4] + Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2]), Rational[5, 32] + 
           Rational[3, 16] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2])}}, {{-0.125, 0.125}, {
         0.566386114155692, 0.3270031754730548}, {0.5915063509461096, 
         0.5915063509461096}, {0.3270031754730548, 0.566386114155692}}]], 
       PolygonBox[
        NCache[{{Rational[-1, 8], Rational[1, 8]}, {
          Rational[7, 16] + Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2]), Rational[-3, 32] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2])}, {
          Rational[1, 4] + Rational[1, 8] 3^Rational[1, 2] + 
           Rational[1, 8] (1 + 3^Rational[1, 2]), Rational[1, 8] + 
           Rational[1, 8] (-1 + 3^Rational[1, 2])}, {
          Rational[5, 32] + Rational[3, 16] 3^Rational[1, 2] + 
           Rational[1, 32] (1 + 3^Rational[1, 2]), Rational[1, 4] + 
           Rational[1, 32] 3^Rational[1, 2] + 
           Rational[1, 32] (-1 + 3^Rational[1, 2])}}, {{-0.125, 0.125}, {
         0.5770031754730548, -0.07087341226347259}, {0.8080127018922193, 
         0.21650635094610965`}, {0.566386114155692, 0.3270031754730548}}]]}, 
      {RGBColor[1, 0, 0], EdgeForm[Dashing[{Small, Small}]], 
       PolygonBox[
        NCache[{{Rational[-1, 2], Rational[-1, 2]}, {0, Rational[-1, 2]}, {
          Rational[-1, 8], Rational[1, 8]}, {
          Rational[-1, 2], 0}}, {{-0.5, -0.5}, {0, -0.5}, {-0.125, 
          0.125}, {-0.5, 0}}]]}, 
      {GrayLevel[0.3], Thickness[0.02], EdgeForm[Dashing[{Small, Small}]], 
       LineBox[NCache[{{-1, 0}, {Rational[-1, 2], 0}, {
          Rational[-1, 8], Rational[1, 8]}, {0, Rational[1, 2]}}, {{-1, 
          0}, {-0.5, 0}, {-0.125, 0.125}, {0, 0.5}}]]}}},
    Axes->False,
    ImageSize->121]}]}]], "Input",
 CellChangeTimes->{{3.6102422151021457`*^9, 3.610242223547629*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}, {
   3.6097157985010314`*^9, 3.609715800474144*^9}, 3.609755337043971*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"18", ",", "18", ",", "18"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609720799394066*^9, 3.6097208041863403`*^9}, {3.6097553759271946`*^9, 
   3.6097553762562137`*^9}, {3.6097720065234103`*^9, 3.609772007972493*^9}, {
   3.6097966310972304`*^9, 3.6097966314292493`*^9}, {3.609800376654464*^9, 
   3.6098003780065413`*^9}, {3.609808329649349*^9, 3.609808331150435*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"sub", "=", "CC5Asub"}], ",", 
       RowBox[{"ind", "=", "CC5Aind"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkForm4", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"CatmullClarkVal2Form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}], "+", 
      RowBox[{"NestT", "[", 
       RowBox[{"form", ",", 
        RowBox[{"sub", "\[LeftDoubleBracket]", 
         RowBox[{
         "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
         "\[RightDoubleBracket]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
  3.6072490656262226`*^9, 3.6072490711795406`*^9}, {3.609720709872946*^9, 
  3.60972080610745*^9}, {3.609755344501397*^9, 3.6097553563380747`*^9}, {
  3.6097720153829174`*^9, 3.6097720367191377`*^9}, {3.609796644436993*^9, 
  3.609796667332303*^9}, {3.6097994894547186`*^9, 3.6097994959670916`*^9}, {
  3.60980038637002*^9, 3.6098003880961185`*^9}, {3.6098083400279427`*^9, 
  3.6098083418160453`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097208905382795`*^9, 3.6097208984627323`*^9}, {
   3.609720932384673*^9, 3.6097209381130004`*^9}, {3.6097209971443768`*^9, 
   3.6097210092810707`*^9}, {3.6097238276862745`*^9, 3.609723827979291*^9}, {
   3.6097506804256277`*^9, 3.6097506873960257`*^9}, {3.6097720432345104`*^9, 
   3.609772049114846*^9}, {3.6097966873074455`*^9, 3.609796688507514*^9}, 
   3.609796896769426*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097507016308403`*^9, 3.6097507116884155`*^9}, {
  3.609772135615794*^9, 3.609772139218*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.609723625978738*^9, 3.6097236338431873`*^9}, {
   3.6097554099441404`*^9, 3.6097554122032695`*^9}, 3.60979669631196*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal5aForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097554529335995`*^9, 3.6097554724867177`*^9}, {
   3.6097721510796785`*^9, 3.609772151719715*^9}, {3.609796708387651*^9, 
   3.6097967208973665`*^9}, 3.6097967922664485`*^9, 3.609799507099728*^9, {
   3.609800401853905*^9, 3.609800402036916*^9}, {3.6098083638373046`*^9, 
   3.6098083639413104`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal5aForm", ">>", 
  "\"\<CatmullClarkVal5aAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6098065628062916`*^9, 3.6098065659324703`*^9}, {
  3.6098083683495626`*^9, 3.6098083704366817`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal5aForm", "=", 
   RowBox[{"<<", "\"\<CatmullClarkVal5aAlt.mx\>\""}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6098106238482676`*^9, 3.6098106264174137`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkVal5aForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal5aForm", "]"}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755493974947*^9, 3.609755553022324*^9}, {
   3.6097721893868694`*^9, 3.6097721906569424`*^9}, {3.609796709736728*^9, 
   3.609796714104978*^9}, {3.6097967937865353`*^9, 3.609796795801651*^9}, 
   3.609799511187962*^9, {3.609800406926195*^9, 3.609800418421853*^9}, {
   3.6098083744459114`*^9, 3.6098083769410543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal5aForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "18"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6097554387977905`*^9, 3.6097554748638535`*^9}, {
   3.609755519447404*^9, 3.6097555333421984`*^9}, {3.609772194058137*^9, 
   3.609772202929644*^9}, {3.609796729327849*^9, 3.6097967341841264`*^9}, 
   3.609796798633813*^9, 3.609799513460092*^9, {3.609800423246129*^9, 
   3.6098004234131384`*^9}, {3.6098083809012804`*^9, 3.6098083842054696`*^9}, 
   3.609810669131857*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal5cForm", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6105707168206625`*^9, 3.610570717716714*^9}, 
   3.6105711731457624`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal5aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"18", ",", "5", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.610571327912615*^9, 3.610571328895671*^9}, {
  3.6107727620228367`*^9, 3.6107727718153973`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal5aForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"18", ",", "4", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.611130131169798*^9, 3.6111302340736837`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 5 \[Bullet] Form B (5.2)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.6097715042516823`*^9, 3.609771507394862*^9}, {
   3.6097725070930414`*^9, 3.609772507356056*^9}, {3.6097891201496286`*^9, 
   3.6097891271010265`*^9}, 3.60979941797163*^9, 3.6097997577860665`*^9, 
   3.609806838177042*^9, 3.609807806744441*^9, {3.61231933935242*^9, 
   3.6123193419205666`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}, {
   3.609810678759408*^9, 3.609810681837584*^9}, {3.6098108688212786`*^9, 
   3.609810872620496*^9}, 3.610242153339613*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark5bPos", ",", "CatmullClark5bTop", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750838401566`*^9, 3.610475093030682*^9}, {
  3.610475142457509*^9, 3.61047514823884*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC5Bsub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "0", "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC5Bind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
       ",", "28"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CC5Bsub", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606012291208367*^9, 3.6060123076683083`*^9}, {
   3.610241087622658*^9, 3.610241201781187*^9}, {3.610241234425054*^9, 
   3.610241239994373*^9}, 3.6102412861150103`*^9, 3.610241336853913*^9, 
   3.6102414572768*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark5bPos", ",", "CatmullClark5bTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", 
     "CatmullClarkRegTop", ",", "CatmullClark5bTop"}], "}"}], ",", "CC5Bsub", 
   ",", "CC5Bind", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.610240296414403*^9, 3.6102403071740184`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}, {
   3.6097157985010314`*^9, 3.609715800474144*^9}, 3.609755337043971*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal5bForm", "=", 
   RowBox[{"<<", "\"\<CatmullClarkVal5bAlt.mx\>\""}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6098106238482676`*^9, 3.6098106264174137`*^9}, {
  3.609810718221665*^9, 3.6098107201917777`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkVal5bForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal5bForm", "]"}]}], "]"}], "//", 
  "Union"}]], "Input",
 CellChangeTimes->{{3.609755493974947*^9, 3.609755553022324*^9}, {
   3.6097721893868694`*^9, 3.6097721906569424`*^9}, {3.609796709736728*^9, 
   3.609796714104978*^9}, {3.6097967937865353`*^9, 3.609796795801651*^9}, 
   3.609799511187962*^9, {3.609800406926195*^9, 3.609800418421853*^9}, {
   3.6098083744459114`*^9, 3.6098083769410543`*^9}, {3.6098107226729193`*^9, 
   3.609810724496024*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal5bForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "19"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6097554387977905`*^9, 3.6097554748638535`*^9}, {
   3.609755519447404*^9, 3.6097555333421984`*^9}, {3.609772194058137*^9, 
   3.609772202929644*^9}, {3.609796729327849*^9, 3.6097967341841264`*^9}, 
   3.609796798633813*^9, 3.609799513460092*^9, {3.609800423246129*^9, 
   3.6098004234131384`*^9}, {3.6098083809012804`*^9, 3.6098083842054696`*^9}, 
   3.609810669131857*^9, {3.6098107447931848`*^9, 3.6098107529356503`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal5bForm", "\[LeftDoubleBracket]", 
  RowBox[{"8", ",", "2", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.610571116081499*^9, 3.6105711276331596`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 5 \[Bullet] Form C (5.1)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
   3.607244889530364*^9}, {3.6097715042516823`*^9, 3.609771507394862*^9}, {
   3.6097725070930414`*^9, 3.609772507356056*^9}, {3.6097891201496286`*^9, 
   3.6097891271010265`*^9}, 3.60979941797163*^9, 3.6097997577860665`*^9, 
   3.609806838177042*^9, 3.609807805405364*^9, {3.6105709307769003`*^9, 
   3.6105709335120564`*^9}, {3.612319345639779*^9, 3.612319346984856*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", 
  RowBox[{"CatmullClark5cPos", ",", "CatmullClark5cTop", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6104750838401566`*^9, 3.610475093030682*^9}, {
  3.610475142457509*^9, 3.610475162943681*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CC5Csub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "0", "0", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "0", "1", 
        "6", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "0", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC5Cind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
       ",", "28"}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CC5Csub", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6060143588756304`*^9, 3.606014372372402*^9}, 
   3.6065132271372857`*^9, {3.609806865686615*^9, 3.6098068794574027`*^9}, {
   3.6098108936687*^9, 3.609810896339853*^9}, {3.6098109789135756`*^9, 
   3.6098110277353683`*^9}, 3.6098112493810453`*^9, {3.609811295039657*^9, 
   3.6098113041771793`*^9}, {3.609811356653181*^9, 3.609811357054204*^9}, {
   3.610240827649788*^9, 3.610240828962863*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDecompositionList", "[", 
  RowBox[{"CatmullClark5aPos", ",", "CatmullClark5aTop", ",", 
   RowBox[{"{", 
    RowBox[{
    "CatmullClarkRegTop", ",", "CatmullClarkRegTop", ",", 
     "CatmullClarkRegTop", ",", "CatmullClark5aTop"}], "}"}], ",", "CC5Csub", 
   ",", "CC5Cind", ",", 
   RowBox[{"11", "/", "10"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.610240060519911*^9, 3.610240072659605*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6097505575465994`*^9, 3.6097505621058598`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"form", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"19", ",", "19", ",", "19"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"form", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}, {
   3.609705893306487*^9, 3.6097058952665987`*^9}, {3.609707184746353*^9, 
   3.609707189218609*^9}, {3.609749340221972*^9, 3.6097493490584774`*^9}, {
   3.609750538168491*^9, 3.6097505405456266`*^9}, {3.609770331047579*^9, 
   3.609770333326709*^9}, {3.609796133807787*^9, 3.6097961381180334`*^9}, {
   3.60980163680554*^9, 3.609801639198677*^9}, {3.6098113377080975`*^9, 
   3.6098113696909266`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"RHS", "[", 
   RowBox[{"reg_", ",", "gen_", ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"gen", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"RHS", "[", 
    RowBox[{
    "CatmullClarkForm4", ",", "form", ",", "CC5Csub", ",", "CC5Cind"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Rhs", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
   3.6072490656262226`*^9, 3.6072490711795406`*^9}, 3.6097062865219774`*^9, 
   3.609706319720876*^9, {3.609707059148169*^9, 3.609707071435872*^9}, {
   3.6097072012572975`*^9, 3.6097072023443594`*^9}, {3.6097192543796964`*^9, 
   3.6097192547327166`*^9}, {3.6097703430262637`*^9, 3.60977034872859*^9}, {
   3.609770663796611*^9, 3.609770665937733*^9}, {3.609796158145179*^9, 
   3.6097961645855474`*^9}, {3.6097995396845922`*^9, 3.609799543080786*^9}, {
   3.609801650537326*^9, 3.60980165464056*^9}, {3.609811378422426*^9, 
   3.609811381970629*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{
     RowBox[{"form", "-", "Rhs"}], ",", "form"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097074665034685`*^9, 3.609707466720481*^9}, {
  3.6097495110267415`*^9, 3.609749523401449*^9}, {3.609750018662777*^9, 
  3.609750019021797*^9}, {3.6097703574950914`*^9, 3.609770361027293*^9}, {
  3.609770668940905*^9, 3.609770683097715*^9}, {3.609796173678067*^9, 
  3.6097961749741416`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6097706935843143`*^9, 3.6097706967724967`*^9}}],

Cell[BoxData[
 RowBox[{"matrix", "//", "MatrixRank"}]], "Input",
 CellChangeTimes->{{3.609707490606847*^9, 3.609707492634963*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", 
     RowBox[{"-", "vector"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.609750068807645*^9, 3.609750089232813*^9}, {
  3.609811469385688*^9, 3.609811470385703*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkVal5cForm", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{"sol", ",", 
     RowBox[{"Dimensions", "[", "form", "]"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6097504461542277`*^9, 3.6097504527856073`*^9}, {
   3.609752376270624*^9, 3.6097523770046663`*^9}, {3.6097708610898952`*^9, 
   3.6097708899135437`*^9}, {3.609796195803333*^9, 3.609796210331164*^9}, 
   3.60979956010276*^9, {3.6098017108287745`*^9, 3.609801711019785*^9}, {
   3.6098114753682795`*^9, 3.6098114889909887`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal5cForm", ">>", 
  "\"\<CatmullClarkVal5cAlt.mx\>\""}]], "Input",
 CellChangeTimes->{{3.609806579412241*^9, 3.6098065819233847`*^9}, {
  3.609811478488327*^9, 3.6098115006636677`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CatmullClarkVal5cForm", "-", 
    RowBox[{"SkewTensor", "[", "CatmullClarkVal5cForm", "]"}]}], "//", 
   "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.609796263957231*^9, 3.609796272628727*^9}, 
   3.609799562492896*^9, 3.6097998547326117`*^9, {3.6098017238125167`*^9, 
   3.6098017270297008`*^9}, {3.6098114827283926`*^9, 3.609811502231192*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkVal5cForm", "]"}], 
    "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "19"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.609719414542857*^9, 3.6097194217562695`*^9}, 
   3.609719465344763*^9, {3.609719712274886*^9, 3.609719713712969*^9}, {
   3.609750225981635*^9, 3.6097502262836523`*^9}, {3.6097502756124735`*^9, 
   3.6097502764695225`*^9}, {3.609750423327922*^9, 3.6097504565168204`*^9}, {
   3.6097522128222756`*^9, 3.60975223493254*^9}, {3.6097524432134533`*^9, 
   3.609752447908722*^9}, {3.609755143546903*^9, 3.609755157353693*^9}, {
   3.6097708978119955`*^9, 3.6097709462907686`*^9}, 3.6097962195586915`*^9, 
   3.6097962790070915`*^9, 3.6097995643120003`*^9, {3.6098017338770924`*^9, 
   3.6098017363802357`*^9}, {3.6098115071587677`*^9, 3.609811511016327*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal5cForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"6", ",", "2", ",", "1"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6107721778734255`*^9, 3.610772278809199*^9}, {
  3.6107723157283106`*^9, 3.6107723494342384`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkVal5cForm", "]"}], 
  "\[LeftDoubleBracket]", 
  RowBox[{"13", ",", "3", ",", "2"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6111300331461916`*^9, 3.611130039642563*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkVal5cForm", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6105706923802648`*^9, 3.610570693386322*^9}}]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{928, 985},
WindowMargins->{{93, Automatic}, {Automatic, 8}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 406, 7, 79, "Section"],
Cell[988, 31, 414, 8, 49, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1439, 44, 502, 7, 79, "Section"],
Cell[CellGroupData[{
Cell[1966, 55, 282, 4, 43, "Subsection"],
Cell[2251, 61, 3092, 73, 112, "Input"],
Cell[5346, 136, 2262, 69, 192, "Input"],
Cell[7611, 207, 237, 7, 31, "Input"],
Cell[7851, 216, 3587, 99, 172, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[11475, 320, 302, 4, 35, "Subsection"],
Cell[11780, 326, 8094, 181, 472, "Input"],
Cell[19877, 509, 237, 7, 31, "Input"],
Cell[20117, 518, 19703, 441, 1397, "Input"],
Cell[39823, 961, 3649, 93, 212, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[43509, 1059, 183, 2, 35, "Subsection"],
Cell[43695, 1063, 1842, 56, 92, "Input"],
Cell[45540, 1121, 1176, 32, 52, "Input"],
Cell[46719, 1155, 660, 17, 31, "Input"],
Cell[47382, 1174, 1124, 30, 72, "Input"],
Cell[48509, 1206, 3147, 93, 192, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[51693, 1304, 104, 1, 35, "Subsection"],
Cell[51800, 1307, 5119, 134, 271, "Input"],
Cell[56922, 1443, 36348, 727, 1592, "Input"],
Cell[93273, 2172, 35689, 710, 1652, "Input"],
Cell[128965, 2884, 1825, 50, 72, "Input"],
Cell[130793, 2936, 11373, 243, 732, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[142203, 3184, 105, 1, 35, "Subsection"],
Cell[142311, 3187, 12510, 356, 2379, "Input"],
Cell[154824, 3545, 5566, 165, 1183, "Input"],
Cell[160393, 3712, 1609, 44, 87, "Input"],
Cell[162005, 3758, 3085, 77, 152, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[165127, 3840, 219, 3, 35, "Subsection"],
Cell[165349, 3845, 966, 26, 15, "Input"],
Cell[166318, 3873, 1814, 51, 27, "Input"],
Cell[168135, 3926, 3202, 79, 66, "Input"],
Cell[171340, 4007, 4166, 116, 56, "Input"],
Cell[175509, 4125, 5297, 137, 56, "Input"],
Cell[180809, 4264, 1126, 30, 22, "Input"],
Cell[181938, 4296, 1756, 37, 39, "Input"],
Cell[183697, 4335, 686, 17, 15, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[184420, 4357, 184, 2, 35, "Subsection"],
Cell[184607, 4361, 16060, 429, 953, "Input"],
Cell[200670, 4792, 4121, 95, 212, "Input"],
Cell[204794, 4889, 3750, 95, 132, "Input"],
Cell[208547, 4986, 636, 17, 52, "Input"],
Cell[209186, 5005, 1910, 51, 92, "Input"],
Cell[211099, 5058, 5230, 151, 385, "Input"],
Cell[CellGroupData[{
Cell[216354, 5213, 171, 2, 34, "Subsubsection"],
Cell[216528, 5217, 2589, 79, 132, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219154, 5301, 122, 2, 34, "Subsubsection"],
Cell[219279, 5305, 2275, 67, 172, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[221591, 5377, 94, 1, 34, "Subsubsection"],
Cell[221688, 5380, 1762, 55, 132, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[223487, 5440, 172, 2, 34, "Subsubsection"],
Cell[223662, 5444, 2748, 79, 192, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226447, 5528, 197, 3, 34, "Subsubsection"],
Cell[226647, 5533, 2492, 74, 152, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229176, 5612, 226, 3, 34, "Subsubsection"],
Cell[229405, 5617, 3382, 99, 103, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[232824, 5721, 248, 3, 11, "Subsubsection"],
Cell[233075, 5726, 3478, 103, 103, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[236590, 5834, 250, 3, 34, "Subsubsection"],
Cell[236843, 5839, 366, 12, 46, "Input"],
Cell[237212, 5853, 4111, 122, 271, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[241360, 5980, 278, 4, 34, "Subsubsection"],
Cell[241641, 5986, 366, 12, 46, "Input"],
Cell[242010, 6000, 3867, 112, 271, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[245914, 6117, 298, 4, 34, "Subsubsection"],
Cell[246215, 6123, 3931, 118, 223, "Input"],
Cell[250149, 6243, 147, 2, 31, "Input"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[250345, 6251, 187, 2, 35, "Subsection"],
Cell[250535, 6255, 9133, 218, 752, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[259717, 6479, 659, 9, 79, "Section"],
Cell[CellGroupData[{
Cell[260401, 6492, 152, 2, 43, "Subsection"],
Cell[260556, 6496, 234, 5, 31, "Input"],
Cell[260793, 6503, 874, 17, 52, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[261704, 6525, 94, 1, 35, "Subsection"],
Cell[261801, 6528, 147, 3, 31, "Input"],
Cell[261951, 6533, 173, 3, 31, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[262173, 6542, 444, 6, 79, "Section"],
Cell[CellGroupData[{
Cell[262642, 6552, 314, 4, 43, "Subsection"],
Cell[CellGroupData[{
Cell[262981, 6560, 104, 1, 34, "Subsubsection"],
Cell[263088, 6563, 139, 3, 5, "Input"],
Cell[263230, 6568, 3612, 64, 116, "Input"],
Cell[266845, 6634, 420, 9, 15, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[267302, 6648, 177, 2, 28, "Subsubsection"],
Cell[267482, 6652, 346, 9, 26, "Input"],
Cell[267831, 6663, 333, 7, 44, "Input"],
Cell[268167, 6672, 697, 17, 44, "Input"],
Cell[268867, 6691, 1764, 41, 80, "Input"],
Cell[270634, 6734, 1084, 26, 61, "Input"],
Cell[271721, 6762, 827, 22, 61, "Input"],
Cell[272551, 6786, 666, 16, 80, "Input"],
Cell[273220, 6804, 109, 2, 26, "Input"],
Cell[273332, 6808, 130, 2, 26, "Input"],
Cell[273465, 6812, 230, 6, 26, "Input"],
Cell[273698, 6820, 409, 7, 44, "Input"],
Cell[274110, 6829, 581, 13, 26, "Input"],
Cell[274694, 6844, 238, 6, 26, "Input"],
Cell[274935, 6852, 819, 18, 26, "Input"],
Cell[275757, 6872, 698, 19, 26, "Input"],
Cell[276458, 6893, 377, 10, 26, "Input"],
Cell[276838, 6905, 447, 11, 26, "Input"],
Cell[277288, 6918, 1690, 40, 80, "Input"],
Cell[278981, 6960, 169, 3, 26, "Input"],
Cell[279153, 6965, 813, 18, 26, "Input"],
Cell[279969, 6985, 245, 4, 26, "Input"],
Cell[280217, 6991, 248, 4, 26, "Input"],
Cell[280468, 6997, 297, 5, 26, "Input"],
Cell[280768, 7004, 296, 5, 26, "Input"],
Cell[281067, 7011, 248, 4, 26, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[281364, 7021, 276, 3, 43, "Subsection"],
Cell[281643, 7026, 284, 8, 5, "Input"],
Cell[281930, 7036, 613, 11, 11, "Input"],
Cell[282546, 7049, 479, 18, 13, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[283062, 7072, 242, 5, 35, "Subsection"],
Cell[283307, 7079, 248, 4, 31, "Input"],
Cell[283558, 7085, 478, 8, 31, "Input"],
Cell[284039, 7095, 351, 9, 52, "Input"],
Cell[284393, 7106, 296, 7, 31, "Input"],
Cell[284692, 7115, 273, 9, 44, "Input"],
Cell[284968, 7126, 343, 6, 31, "Input"],
Cell[285314, 7134, 1069, 27, 52, "Input"],
Cell[286386, 7163, 890, 19, 52, "Input"],
Cell[287279, 7184, 473, 13, 52, "Input"],
Cell[287755, 7199, 114, 3, 31, "Input"],
Cell[287872, 7204, 252, 6, 31, "Input"],
Cell[288127, 7212, 505, 13, 52, "Input"],
Cell[288635, 7227, 401, 10, 31, "Input"],
Cell[289039, 7239, 160, 2, 31, "Input"],
Cell[289202, 7243, 503, 13, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[289742, 7261, 409, 5, 35, "Subsection"],
Cell[CellGroupData[{
Cell[290176, 7270, 104, 1, 34, "Subsubsection"],
Cell[290283, 7273, 228, 4, 5, "Input"],
Cell[290514, 7279, 3325, 63, 100, "Input"],
Cell[293842, 7344, 55, 1, 5, "Input"],
Cell[293900, 7347, 315, 6, 5, "Input"],
Cell[294218, 7355, 469, 9, 15, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[294724, 7369, 251, 3, 28, "Subsubsection"],
Cell[294978, 7374, 739, 18, 61, "Input"],
Cell[295720, 7394, 1649, 39, 80, "Input"],
Cell[297372, 7435, 643, 14, 26, "Input"],
Cell[298018, 7451, 285, 7, 26, "Input"],
Cell[298306, 7460, 347, 7, 44, "Input"],
Cell[298656, 7469, 351, 9, 26, "Input"],
Cell[299010, 7480, 316, 7, 26, "Input"],
Cell[299329, 7489, 169, 3, 26, "Input"],
Cell[299501, 7494, 307, 7, 26, "Input"],
Cell[299811, 7503, 513, 14, 26, "Input"],
Cell[300327, 7519, 298, 5, 26, "Input"],
Cell[300628, 7526, 245, 4, 26, "Input"],
Cell[300876, 7532, 296, 5, 26, "Input"],
Cell[301175, 7539, 247, 4, 26, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[301471, 7549, 496, 6, 43, "Subsection"],
Cell[CellGroupData[{
Cell[301992, 7559, 104, 1, 34, "Subsubsection"],
Cell[302099, 7562, 247, 4, 31, "Input"],
Cell[302349, 7568, 4154, 68, 456, "Input"],
Cell[306506, 7638, 397, 8, 52, "Input"],
Cell[306906, 7648, 279, 5, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[307222, 7658, 251, 3, 28, "Subsubsection"],
Cell[307476, 7663, 796, 19, 72, "Input"],
Cell[308275, 7684, 1750, 40, 92, "Input"],
Cell[310028, 7726, 631, 13, 31, "Input"],
Cell[310662, 7741, 280, 7, 31, "Input"],
Cell[310945, 7750, 318, 6, 52, "Input"],
Cell[311266, 7758, 417, 8, 31, "Input"],
Cell[311686, 7768, 171, 3, 31, "Input"],
Cell[311860, 7773, 460, 9, 31, "Input"],
Cell[312323, 7784, 611, 15, 31, "Input"],
Cell[312937, 7801, 252, 6, 31, "Input"],
Cell[313192, 7809, 305, 6, 31, "Input"],
Cell[313500, 7817, 277, 6, 31, "Input"],
Cell[313780, 7825, 247, 5, 31, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[314076, 7836, 497, 6, 43, "Subsection"],
Cell[CellGroupData[{
Cell[314598, 7846, 104, 1, 34, "Subsubsection"],
Cell[314705, 7849, 198, 3, 31, "Input"],
Cell[314906, 7854, 4669, 95, 486, "Input"],
Cell[319578, 7951, 398, 8, 72, "Input"],
Cell[319979, 7961, 328, 6, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[320344, 7972, 177, 2, 28, "Subsubsection"],
Cell[320524, 7976, 280, 6, 11, "Input"],
Cell[320807, 7984, 745, 17, 11, "Input"],
Cell[321555, 8003, 1801, 38, 22, "Input"],
Cell[323359, 8043, 543, 12, 5, "Input"],
Cell[323905, 8057, 134, 2, 5, "Input"],
Cell[324042, 8061, 130, 2, 5, "Input"],
Cell[324175, 8065, 230, 6, 5, "Input"],
Cell[324408, 8073, 444, 9, 5, "Input"],
Cell[324855, 8084, 169, 3, 5, "Input"],
Cell[325027, 8089, 307, 7, 5, "Input"],
Cell[325337, 8098, 916, 19, 5, "Input"],
Cell[326256, 8119, 250, 5, 5, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[326555, 8130, 526, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[327106, 8141, 104, 1, 34, "Subsubsection"],
Cell[327213, 8144, 232, 4, 7, "Input"],
Cell[327448, 8150, 5017, 98, 165, "Input"],
Cell[332468, 8250, 456, 9, 14, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[332961, 8264, 251, 3, 28, "Subsubsection"],
Cell[333215, 8269, 848, 20, 61, "Input"],
Cell[334066, 8291, 1800, 41, 80, "Input"],
Cell[335869, 8334, 631, 13, 26, "Input"],
Cell[336503, 8349, 280, 7, 26, "Input"],
Cell[336786, 8358, 318, 6, 44, "Input"],
Cell[337107, 8366, 467, 9, 26, "Input"],
Cell[337577, 8377, 173, 3, 26, "Input"],
Cell[337753, 8382, 484, 9, 26, "Input"],
Cell[338240, 8393, 663, 16, 26, "Input"],
Cell[338906, 8411, 356, 7, 26, "Input"],
Cell[339265, 8420, 249, 5, 26, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[339563, 8431, 522, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[340110, 8442, 104, 1, 34, "Subsubsection"],
Cell[340217, 8445, 234, 4, 14, "Input"],
Cell[340454, 8451, 4378, 89, 301, "Input"],
Cell[344835, 8542, 436, 9, 39, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[345308, 8556, 177, 2, 28, "Subsubsection"],
Cell[345488, 8560, 280, 6, 44, "Input"],
Cell[345771, 8568, 794, 18, 44, "Input"],
Cell[346568, 8588, 1850, 39, 80, "Input"],
Cell[348421, 8629, 543, 12, 26, "Input"],
Cell[348967, 8643, 134, 2, 26, "Input"],
Cell[349104, 8647, 130, 2, 26, "Input"],
Cell[349237, 8651, 230, 6, 26, "Input"],
Cell[349470, 8659, 492, 9, 26, "Input"],
Cell[349965, 8670, 171, 3, 26, "Input"],
Cell[350139, 8675, 361, 8, 26, "Input"],
Cell[350503, 8685, 970, 20, 26, "Input"],
Cell[351476, 8707, 149, 2, 26, "Input"],
Cell[351628, 8711, 252, 5, 26, "Input"],
Cell[351883, 8718, 250, 5, 26, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[352182, 8729, 548, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[352755, 8740, 104, 1, 34, "Subsubsection"],
Cell[352862, 8743, 281, 5, 14, "Input"],
Cell[353146, 8750, 4465, 88, 301, "Input"],
Cell[357614, 8840, 506, 9, 39, "Input"],
Cell[358123, 8851, 24789, 468, 71, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[382949, 9324, 251, 3, 28, "Subsubsection"],
Cell[383203, 9329, 894, 20, 72, "Input"],
Cell[384100, 9351, 1853, 42, 92, "Input"],
Cell[385956, 9395, 631, 13, 31, "Input"],
Cell[386590, 9410, 280, 7, 31, "Input"],
Cell[386873, 9419, 318, 6, 52, "Input"],
Cell[387194, 9427, 521, 10, 31, "Input"],
Cell[387718, 9439, 226, 4, 31, "Input"],
Cell[387947, 9445, 207, 4, 31, "Input"],
Cell[388157, 9451, 538, 10, 31, "Input"],
Cell[388698, 9463, 739, 17, 31, "Input"],
Cell[389440, 9482, 175, 3, 31, "Input"],
Cell[389618, 9487, 301, 6, 31, "Input"],
Cell[389922, 9495, 250, 5, 31, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[390221, 9506, 521, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[390767, 9517, 228, 3, 34, "Subsubsection"],
Cell[390998, 9522, 280, 5, 5, "Input"],
Cell[391281, 9529, 4901, 94, 156, "Input"],
Cell[396185, 9625, 434, 9, 15, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[396656, 9639, 251, 3, 28, "Subsubsection"],
Cell[396910, 9644, 258, 5, 5, "Input"],
Cell[397171, 9651, 590, 11, 5, "Input"],
Cell[397764, 9664, 789, 17, 5, "Input"],
Cell[398556, 9683, 214, 3, 5, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[398819, 9692, 570, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[399414, 9703, 104, 1, 34, "Subsubsection"],
Cell[399521, 9706, 281, 5, 5, "Input"],
Cell[399805, 9713, 5081, 96, 156, "Input"],
Cell[404889, 9811, 432, 9, 15, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[405358, 9825, 177, 2, 28, "Subsubsection"],
Cell[405538, 9829, 280, 6, 44, "Input"],
Cell[405821, 9837, 848, 19, 44, "Input"],
Cell[406672, 9858, 1900, 40, 80, "Input"],
Cell[408575, 9900, 543, 12, 26, "Input"],
Cell[409121, 9914, 134, 2, 26, "Input"],
Cell[409258, 9918, 130, 2, 26, "Input"],
Cell[409391, 9922, 279, 7, 26, "Input"],
Cell[409673, 9931, 546, 10, 26, "Input"],
Cell[410222, 9943, 222, 4, 26, "Input"],
Cell[410447, 9949, 409, 8, 26, "Input"],
Cell[410859, 9959, 1018, 20, 26, "Input"],
Cell[411880, 9981, 302, 6, 26, "Input"],
Cell[412185, 9989, 250, 5, 26, "Input"],
Cell[412438, 9996, 147, 2, 26, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
