(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    316539,       8015]
NotebookOptionsPosition[    304357,       7679]
NotebookOutlinePosition[    304722,       7695]
CellTagsIndexPosition[    304679,       7692]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Volume enclosed by Catmull-Clark surfaces", "Section",
 CellChangeTimes->{{3.604699325951583*^9, 3.604699342005501*^9}, {
  3.605316105939847*^9, 3.6053161113441563`*^9}, {3.6053189977682505`*^9, 
  3.605319007676817*^9}, {3.60569576697237*^9, 3.6056957867405005`*^9}, {
  3.605735736585387*^9, 3.6057357410236406`*^9}, {3.6121381529952126`*^9, 
  3.6121381538782625`*^9}, {3.612177494337204*^9, 3.612177497288373*^9}}],

Cell[TextData[{
 "code by jph, 2014\ndownload from ",
 StyleBox["www.hakenberg.de",
  FontSlant->"Italic"]
}], "Text",
 CellChangeTimes->{{3.605695943586472*^9, 3.6056959456205883`*^9}, {
   3.6056970059162335`*^9, 3.6056970389631243`*^9}, 3.605735755713481*^9, 
   3.612138146605847*^9, {3.612175969429984*^9, 3.6121759752833195`*^9}, {
   3.6121774860257287`*^9, 3.6121774882888584`*^9}, 3.6123104446486716`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementation", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
  3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
  3.6056228546483173`*^9}, {3.605646909007148*^9, 3.6056469329195156`*^9}, {
  3.60564698352641*^9, 3.6056469954380913`*^9}, {3.605696857837764*^9, 
  3.605696861844993*^9}, {3.605697358177382*^9, 3.605697358385394*^9}, {
  3.6123087117865577`*^9, 3.6123087140776887`*^9}, {3.612310224706092*^9, 
  3.612310236688777*^9}}],

Cell[CellGroupData[{

Cell["Helper functions", "Subsection",
 CellChangeTimes->{{3.606513742476762*^9, 3.606513746957018*^9}, 
   3.6122811832721615`*^9, {3.612310550720739*^9, 3.6123105508867483`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"NestT", "[", 
   RowBox[{"form_", ",", "S_"}], "]"}], ":=", 
  RowBox[{"Nest", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{
       RowBox[{"#", ".", "S"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "1"}], "}"}]}], "]"}], "&"}], ",", "form",
     ",", "3"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NestP", "[", 
   RowBox[{"form_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Nest", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{
       RowBox[{"#", "\[LeftDoubleBracket]", "ind", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "1"}], "}"}]}], "]"}], "&"}], ",", "form",
     ",", "3"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"RHS", "[", 
   RowBox[{"reg_", ",", "gen_", ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"reg", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}], "+", 
   RowBox[{"NestT", "[", 
    RowBox[{"gen", ",", 
     RowBox[{"sub", "\[LeftDoubleBracket]", 
      RowBox[{"ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Skew3", "[", "frm_", "]"}], ":=", 
  RowBox[{"frm", "-", 
   RowBox[{"Transpose", "[", 
    RowBox[{"frm", ",", 
     RowBox[{"{", 
      RowBox[{"2", ",", "1", ",", "3"}], "}"}]}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NormalizeWeights", "[", "S_", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"#", "/", 
     RowBox[{"Total", "[", "#", "]"}]}], "&"}], "/@", 
   "S"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HashGet", "[", 
   RowBox[{"hash_", ",", "key_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"entry", "=", 
      RowBox[{"hash", "[", "key", "]"}]}], "}"}], ",", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Head", "[", "entry", "]"}], "===", "hash"}], ",", "Null", ",",
       "entry"}], "]"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6057378203785734`*^9, 3.6057378252438517`*^9}, {
   3.6057378900585585`*^9, 3.605737907722569*^9}, {3.605738008106311*^9, 
   3.6057380487296343`*^9}, {3.6057381112752113`*^9, 3.605738113972366*^9}, 
   3.605739035228059*^9, {3.6057806819469023`*^9, 3.6057806828949566`*^9}, {
   3.6062491007071095`*^9, 3.6062491011081324`*^9}, {3.606427549459944*^9, 
   3.6064275497079573`*^9}, {3.6064595139310026`*^9, 
   3.6064595413245697`*^9}, {3.606460398258583*^9, 3.6064603984495945`*^9}, {
   3.607249348229387*^9, 3.6072493498804812`*^9}, {3.612176371915005*^9, 
   3.6121763745901585`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"NumericMesh", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"N", "[", "V", "]"}], ",", "F"}], "}"}]}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"PlotMesh", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Polygon", "[", 
       RowBox[{"V", "\[LeftDoubleBracket]", 
        RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], "&"}], "/@",
      "F"}], ",", 
    RowBox[{"Axes", "\[Rule]", "True"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HueValues", "[", "lst_", "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mn", "=", 
       RowBox[{"Min", "[", "lst", "]"}]}], ",", 
      RowBox[{"mx", "=", 
       RowBox[{"Max", "[", "lst", "]"}]}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"lst", "-", "mn"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"mx", "-", "mn"}], ")"}]}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlotMesh", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "vol_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hue", "=", 
      RowBox[{"HueValues", "[", "vol", "]"}]}], "}"}], ",", 
    RowBox[{"Graphics3D", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Hue", "[", 
           RowBox[{
           "hue", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
           "]"}], ",", 
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], ",",
              "All"}], "\[RightDoubleBracket]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"AspectRatio", "\[Rule]", "Automatic"}]}], "]"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{
  3.6056884725371523`*^9, {3.6056916585413814`*^9, 3.6056916603574853`*^9}, {
   3.6056929270129337`*^9, 3.605692927350953*^9}, 3.6057452624192333`*^9, {
   3.6058281527013636`*^9, 3.6058281529953804`*^9}, {3.6062824779937935`*^9, 
   3.6062824782428083`*^9}, {3.606282515535941*^9, 3.6062825631026616`*^9}, {
   3.6062827326523595`*^9, 3.6062827328383703`*^9}, {3.606282812685937*^9, 
   3.6062828148250594`*^9}, 3.6062830149315047`*^9, {3.606283066759469*^9, 
   3.6062830673405027`*^9}, {3.606283124344763*^9, 3.606283131254158*^9}, {
   3.606283213999891*^9, 3.6062832215433226`*^9}, {3.606393542832162*^9, 
   3.6063936132641897`*^9}, {3.6063939414209595`*^9, 3.6063939622931533`*^9}, 
   3.606460394231353*^9, 3.6064739843526835`*^9, {3.6066436549895115`*^9, 
   3.6066436741326065`*^9}, {3.6066443678902864`*^9, 
   3.6066443685493245`*^9}, {3.6066444304368644`*^9, 3.606644455781314*^9}, {
   3.6073265323306036`*^9, 3.607326532685609*^9}, {3.6075033450068717`*^9, 
   3.6075033454728985`*^9}, {3.612176259998604*^9, 3.612176314437718*^9}, {
   3.6121764994082975`*^9, 3.612176500015332*^9}, 3.6121766812827*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Subdivision", "Subsection",
 CellChangeTimes->{{3.6121763837136803`*^9, 3.6121763858338013`*^9}, {
  3.612176446705283*^9, 3.6121764479763556`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuadLinearSubdivide", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Vout", "=", "V"}], ",", 
      RowBox[{"Fout", "=", 
       RowBox[{"{", "}"}]}], ",", "mid", ",", "key", ",", "hash", ",", "edge",
       ",", "list"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"Vout", ",", 
          RowBox[{"Mean", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}]}], "]"}], ";", 
        RowBox[{"mid", "=", 
         RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
        RowBox[{"list", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"key", "=", 
            RowBox[{"Sort", "[", 
             RowBox[{
              RowBox[{"RotateLeft", "[", 
               RowBox[{
                RowBox[{
                "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
                ",", 
                RowBox[{"j", "-", "1"}]}], "]"}], "\[LeftDoubleBracket]", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"edge", "=", 
            RowBox[{"HashGet", "[", 
             RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"edge", "===", "Null"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"AppendTo", "[", 
               RowBox[{"Vout", ",", 
                RowBox[{"Mean", "[", 
                 RowBox[{
                 "V", "\[LeftDoubleBracket]", "key", 
                  "\[RightDoubleBracket]"}], "]"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"edge", "=", 
               RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
              RowBox[{
               RowBox[{"hash", "[", "key", "]"}], "=", "edge"}]}]}], "]"}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"list", ",", "edge"}], "]"}], ";"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"AppendTo", "[", 
           RowBox[{"Fout", ",", 
            RowBox[{"{", 
             RowBox[{"mid", ",", 
              RowBox[{"list", "\[LeftDoubleBracket]", 
               RowBox[{
                RowBox[{"Mod", "[", 
                 RowBox[{
                  RowBox[{"j", "-", "2"}], ",", 
                  RowBox[{"Length", "[", 
                   RowBox[{
                   "F", "\[LeftDoubleBracket]", "k", 
                    "\[RightDoubleBracket]"}], "]"}]}], "]"}], "+", "1"}], 
               "\[RightDoubleBracket]"}], ",", 
              RowBox[{"F", "\[LeftDoubleBracket]", 
               RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], ",", 
              RowBox[{
              "list", "\[LeftDoubleBracket]", "j", 
               "\[RightDoubleBracket]"}]}], "}"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", 
             RowBox[{
             "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             "]"}]}], "}"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     RowBox[{"{", 
      RowBox[{"Vout", ",", "Fout"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.6121765101389112`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkWeight", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["n", "2"]}], "-", 
   RowBox[{"7", " ", "n"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"vind", " ", "is", " ", "in", " ", 
    RowBox[{"{", 
     RowBox[{"1", ",", " ", "2", ",", " ", "...", ",", " ", 
      RowBox[{"Length", "[", "f", "]"}]}], "}"}], "\[IndentingNewLine]", "f", 
    " ", "is", " ", "element", " ", "of", " ", "F"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"VertexRing", "[", 
     RowBox[{"F_", ",", "f_", ",", "vind_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"fn", "=", 
         RowBox[{"RotateLeft", "[", 
          RowBox[{"f", ",", 
           RowBox[{"vind", "-", "1"}]}], "]"}]}], ",", "v1", ",", "v2", ",", 
        "adj", ",", 
        RowBox[{"ring", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"count", "=", "0"}], ",", 
        RowBox[{"rev", "=", 
         RowBox[{"{", "}"}]}], ",", "tmp", ",", "pos"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"adj", "=", "fn"}], ";", "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"ring", ",", "adj"}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"rev", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"Position", "[", 
             RowBox[{"F", ",", "f"}], "]"}], "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
           RowBox[{"vind", "-", "1"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"++", "count"}], "<", "20"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"v1", "=", 
           RowBox[{
           "adj", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"v2", "=", 
           RowBox[{"Last", "[", "adj", "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"tmp", "=", 
           RowBox[{
            RowBox[{"Select", "[", 
             RowBox[{"F", ",", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"Union", "[", "#", "]"}], "\[NotEqual]", 
                 RowBox[{"Union", "[", "adj", "]"}]}], "\[And]", 
                RowBox[{"MemberQ", "[", 
                 RowBox[{"#", ",", "v1"}], "]"}], "\[And]", 
                RowBox[{"MemberQ", "[", 
                 RowBox[{"#", ",", "v2"}], "]"}]}], "&"}]}], "]"}], 
            "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"pos", "=", 
           RowBox[{
            RowBox[{
             RowBox[{"Position", "[", 
              RowBox[{"tmp", ",", "v1"}], "]"}], "\[LeftDoubleBracket]", 
             RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "-", "1"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"adj", "=", 
           RowBox[{"RotateLeft", "[", 
            RowBox[{"tmp", ",", "pos"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"adj", "\[Equal]", "fn"}], ",", 
            RowBox[{"Break", "[", "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"ring", ",", "adj"}], "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"rev", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"Position", "[", 
                RowBox[{"F", ",", "tmp"}], "]"}], "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], ",", 
              "pos"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
       RowBox[{"{", 
        RowBox[{"ring", ",", "rev"}], "}"}]}]}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CatmullClarkSubdivide", "[", 
     RowBox[{"{", 
      RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Vout", ",", 
        RowBox[{"Fout", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"count", "=", "0"}], ",", 
        RowBox[{"Vind", "=", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"rings", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", 
             RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
        "ring", ",", "rev", ",", "\[IndentingNewLine]", "hash", ",", "key", 
        ",", "edge1", ",", "edge2", ",", "vert", ",", "w"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", "Midpoint", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Vout", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Mean", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
          "&"}], "/@", "F"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"AppendTo", "[", 
          RowBox[{
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{"F", "\[LeftDoubleBracket]", 
             RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], ",", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
          "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"{", 
              RowBox[{"ring", ",", "rev"}], "}"}], "=", 
             RowBox[{"VertexRing", "[", 
              RowBox[{
               RowBox[{"rings", "\[LeftDoubleBracket]", 
                RowBox[{"F", "\[LeftDoubleBracket]", 
                 RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}], ",", 
               RowBox[{
               "F", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
               ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"Edge", " ", "1"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"Sort", "[", 
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"edge1", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"edge1", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  RowBox[{"1", "/", "16"}], 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    "6", ",", "6", ",", "1", ",", "1", ",", "1", ",", "1"}], 
                    "}"}], ".", 
                   RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{
                    "ring", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{
                    RowBox[{"Last", "[", "ring", "]"}], 
                    "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"2", ",", "3"}], "}"}], 
                    "\[RightDoubleBracket]"}]}], "]"}], 
                    "\[RightDoubleBracket]"}]}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"edge1", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "edge1"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"Edge", " ", "2"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"Sort", "[", 
              RowBox[{"ring", "\[LeftDoubleBracket]", 
               RowBox[{"1", ",", 
                RowBox[{"{", 
                 RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"edge2", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"edge2", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  RowBox[{"1", "/", "16"}], 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    "6", ",", "1", ",", "1", ",", "6", ",", "1", ",", "1"}], 
                    "}"}], ".", 
                   RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{
                    "ring", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", 
                    RowBox[{"ring", "\[LeftDoubleBracket]", 
                    RowBox[{"2", ",", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "4"}], "}"}]}], 
                    "\[RightDoubleBracket]"}]}], "]"}], 
                    "\[RightDoubleBracket]"}]}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"edge2", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "edge2"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", "Vertex", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"key", "=", 
             RowBox[{"{", 
              RowBox[{"F", "\[LeftDoubleBracket]", 
               RowBox[{"n", ",", "k"}], "\[RightDoubleBracket]"}], "}"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"vert", "=", 
             RowBox[{"HashGet", "[", 
              RowBox[{"hash", ",", "key"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"vert", "===", "Null"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"w", "=", 
                RowBox[{"CatmullClarkWeight", "[", 
                 RowBox[{"Length", "[", "ring", "]"}], "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"AppendTo", "[", 
                RowBox[{"Vout", ",", 
                 RowBox[{
                  FractionBox["1", 
                   RowBox[{"w", "+", 
                    RowBox[{"7", " ", 
                    RowBox[{"Length", "[", "ring", "]"}]}]}]], 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"w", " ", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"ring", "\[LeftDoubleBracket]", 
                    RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], 
                    "\[RightDoubleBracket]"}]}], "+", 
                    RowBox[{"Total", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"6", ",", "1"}], "}"}], ".", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{"#", "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"2", ",", "3"}], "}"}], "\[RightDoubleBracket]"}],
                     "\[RightDoubleBracket]"}]}], ")"}], "&"}], "/@", 
                    "ring"}], "]"}]}], ")"}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"vert", "=", 
                RowBox[{"Length", "[", "Vout", "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"hash", "[", "key", "]"}], "=", "vert"}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"Fout", ",", 
              RowBox[{"{", 
               RowBox[{"vert", ",", "edge1", ",", "n", ",", "edge2"}], 
               "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", 
             RowBox[{"Length", "[", 
              RowBox[{
              "F", "\[LeftDoubleBracket]", "n", "\[RightDoubleBracket]"}], 
              "]"}]}], "}"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"n", ",", 
           RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"Vout", ",", "Fout"}], "}"}]}]}], "]"}]}]}]}]], "Input",
 CellChangeTimes->{
  3.605824101738662*^9, {3.605824849440428*^9, 3.605824863362224*^9}, {
   3.605824937494464*^9, 3.6058249725954723`*^9}, {3.6058255358466883`*^9, 
   3.6058256619469004`*^9}, {3.605825879489343*^9, 3.6058259515214634`*^9}, {
   3.6058259940338945`*^9, 3.6058261939943323`*^9}, {3.605826236075739*^9, 
   3.6058262978052697`*^9}, {3.6058263426738358`*^9, 3.605826460180557*^9}, {
   3.6058265628904314`*^9, 3.605826575329143*^9}, {3.605826665304289*^9, 
   3.605826866350788*^9}, {3.6058268971855516`*^9, 3.6058269197208414`*^9}, {
   3.605827021714675*^9, 3.605827026840968*^9}, {3.605827080081013*^9, 
   3.6058271005811853`*^9}, {3.605827136589245*^9, 3.60582727059591*^9}, {
   3.6058276054710636`*^9, 3.605827670568787*^9}, {3.6058277090779896`*^9, 
   3.6058277515944214`*^9}, {3.6058277869084415`*^9, 3.605827858446533*^9}, {
   3.605827903734123*^9, 3.605828013325392*^9}, {3.6058280580169477`*^9, 
   3.6058280633622537`*^9}, {3.6058460565460243`*^9, 3.605846079864358*^9}, {
   3.60584611260223*^9, 3.605846154114605*^9}, {3.6062826089242826`*^9, 
   3.60628266552252*^9}, {3.606282855290374*^9, 3.6062828850670767`*^9}, {
   3.6062829976145144`*^9, 3.6062830059769926`*^9}, {3.606283150190241*^9, 
   3.6062831573526506`*^9}, {3.606283274002323*^9, 3.606283290958293*^9}, {
   3.606283338433008*^9, 3.6062833451643934`*^9}, {3.6063888767282753`*^9, 
   3.606388910047181*^9}, {3.6063889964631243`*^9, 3.606389008314802*^9}, {
   3.606459661824462*^9, 3.606459672898095*^9}, {3.606643536620741*^9, 
   3.606643544418187*^9}, {3.6066444247035365`*^9, 3.6066444259826097`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Load volume forms", "Subsection",
 CellChangeTimes->{{3.6073261432421274`*^9, 3.6073261482547045`*^9}, {
  3.607687403417124*^9, 3.6076874039291525`*^9}, {3.6121764832003703`*^9, 
  3.6121764841274233`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Lambda]X", "[", 
   RowBox[{"n_", ",", "\[Lambda]_"}], "]"}], ":=", 
  RowBox[{"With", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"d", "=", 
      RowBox[{"Length", "[", "\[Lambda]", "]"}]}], "}"}], ",", 
    RowBox[{
     RowBox[{"Binomial", "[", 
      RowBox[{"n", ",", "d"}], "]"}], "-", 
     RowBox[{"Sum", "[", 
      RowBox[{
       RowBox[{"Binomial", "[", 
        RowBox[{
         RowBox[{"n", "-", 
          RowBox[{
          "\[Lambda]", "\[LeftDoubleBracket]", "i", 
           "\[RightDoubleBracket]"}]}], ",", 
         RowBox[{"d", "+", "1", "-", "i"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "d"}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalLambda]set", "[", 
   RowBox[{"n_", ",", "d_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"lst", "=", 
      RowBox[{"{", "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"{", "##", "}"}], "==", 
            RowBox[{"Sort", "[", 
             RowBox[{"{", "##", "}"}], "]"}]}], "&&", 
           RowBox[{
            RowBox[{"Length", "[", 
             RowBox[{"Union", "[", 
              RowBox[{"{", "##", "}"}], "]"}], "]"}], "\[Equal]", "d"}]}], 
          ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"lst", ",", 
            RowBox[{"{", "##", "}"}]}], "]"}]}], "]"}], "&"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{"n", ",", 
         RowBox[{"{", "d", "}"}]}], "]"}]}], "]"}], ";", "lst"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6117137433353386`*^9, 3.611713743561352*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"InflateForm", "[", 
   RowBox[{"lst_", ",", "n_", ",", "d_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{"Array", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"p", "=", 
            RowBox[{"{", "##", "}"}]}], ",", 
           RowBox[{"s", "=", 
            RowBox[{"Signature", "[", 
             RowBox[{"{", "##", "}"}], "]"}]}]}], "}"}], ",", " ", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"s", "\[Equal]", "0"}], ",", "0", ",", 
           RowBox[{"s", " ", 
            RowBox[{"lst", "\[LeftDoubleBracket]", 
             RowBox[{"\[Lambda]X", "[", 
              RowBox[{"n", ",", 
               RowBox[{"Sort", "[", "p", "]"}]}], "]"}], 
             "\[RightDoubleBracket]"}]}]}], "]"}]}], "]"}], "&"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{"n", ",", 
        RowBox[{"{", "d", "}"}]}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.611850156341178*^9, 3.6118501617654877`*^9}, {
  3.6121340542537775`*^9, 3.612134064621371*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CompressForm", "[", "frm_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"frm", "\[LeftDoubleBracket]", 
      RowBox[{
       RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       ",", 
       RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}], 
      "\[RightDoubleBracket]"}], "&"}], "/@", 
    RowBox[{"\[CapitalLambda]set", "[", 
     RowBox[{
      RowBox[{"Length", "[", "frm", "]"}], ",", 
      RowBox[{"TensorRank", "[", "frm", "]"}]}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkListAlt", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"num", "=", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"k", "<", "10"}], ",", "\"\<0\>\"", ",", "\"\<\>\""}], 
           "]"}], "<>", 
          RowBox[{"ToString", "[", "k", "]"}]}]}], "}"}], ",", 
       RowBox[{"InflateForm", "[", 
        RowBox[{
         RowBox[{"<<", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"NotebookDirectory", "[", "]"}], "<>", 
            "\"\<_forms\\\\CCalt\>\"", "<>", "num", "<>", "\"\<vec.mx\>\""}], 
           ")"}]}], ",", 
         RowBox[{"8", "+", 
          RowBox[{"2", "k"}]}], ",", "3"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "3", ",", "7"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.6121754065977926`*^9, {3.612176704741042*^9, 3.6121767048200464`*^9}, 
   3.612177381216734*^9, {3.612316906230253*^9, 3.6123169269734397`*^9}, {
   3.6123170124863305`*^9, 3.6123170184986744`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Volume", "Subsection",
 CellChangeTimes->{{3.6121763837136803`*^9, 3.6121763858338013`*^9}, {
  3.612176446705283*^9, 3.612176453687682*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LinearVolume", "[", 
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
   RowBox[{"With", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"QuadDet", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             FractionBox["1", "12"], ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"], ",", "0"}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", 
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", 
             FractionBox["1", "12"], ",", "0"}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", "0", ",", "0", ",", 
             FractionBox["1", "12"]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0", ",", "0"}], "}"}]}], "}"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", "0", ",", 
             RowBox[{"-", 
              FractionBox["1", "12"]}], ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
             FractionBox["1", "12"], ",", 
             FractionBox["1", "12"], ",", "0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]}], 
        "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Simplify", "[", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"f", "=", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}], 
           "}"}], ",", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "f", "]"}], "\[Equal]", "4"}], ",", 
            RowBox[{"QuadDet", ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "1"}], "\[RightDoubleBracket]"}], ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "2"}], "\[RightDoubleBracket]"}], ".", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"f", ",", "3"}], "\[RightDoubleBracket]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Simplify", "[", 
             RowBox[{
              RowBox[{"Sum", "[", 
               RowBox[{
                RowBox[{"Det", "[", 
                 RowBox[{"V", "\[LeftDoubleBracket]", 
                  RowBox[{
                   RowBox[{"f", "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "k", ",", 
                    RowBox[{"k", "+", "1"}]}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                  "\[RightDoubleBracket]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"k", ",", "2", ",", 
                  RowBox[{
                   RowBox[{"Length", "[", "f", "]"}], "-", "1"}]}], "}"}]}], 
               "]"}], "/", "6"}], "]"}]}], "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], "]"}]}], 
    "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkStencil", "[", 
   RowBox[{"F_", ",", "f_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"rings", ",", "pos"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"rings", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"VertexRing", "[", 
          RowBox[{"F", ",", "f", ",", "k"}], "]"}], "\[LeftDoubleBracket]", 
         "1", "\[RightDoubleBracket]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "4"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"pos", "=", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{"Append", "[", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Length", "[", "#", "]"}], "\[NotEqual]", "4"}], "&"}],
              "/@", "rings"}], ",", "True"}], "]"}], ",", "3"}], "]"}], "]"}],
        "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rings", "=", 
      RowBox[{"RotateRight", "[", 
       RowBox[{"rings", ",", 
        RowBox[{"3", "-", "pos"}]}], "]"}]}], ";", 
     RowBox[{"(*", 
      RowBox[{
      "our", " ", "subdiv", " ", "alg", " ", "positions", " ", "irreg", " ", 
       "always", " ", "at", " ", "pos", " ", "1"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Flatten", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"2", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "2", ",", 
          RowBox[{"{", 
           RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"2", ",", "4", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"4", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"4", ",", "1"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"3", ",", "2", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "4"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rings", "\[LeftDoubleBracket]", 
         RowBox[{"4", ",", "3", ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "2"}], "}"}]}], "\[RightDoubleBracket]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Reverse", "[", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{"3", ",", "k", ",", 
             RowBox[{"{", 
              RowBox[{"4", ",", "3"}], "}"}]}], "\[RightDoubleBracket]"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "3", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{
               "rings", "\[LeftDoubleBracket]", "3", 
                "\[RightDoubleBracket]"}], "]"}], "-", "1"}]}], "}"}]}], 
          "]"}], "]"}]}], "\[IndentingNewLine]", "}"}], "]"}]}]}], 
   "]"}]}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"VolumeForm", "::", "valence"}], "=", 
   "\"\<form for valence is not available\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVolumeFace", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rings", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
      "local"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"local", "=", 
         RowBox[{"Join", "@@", 
          RowBox[{"rings", "\[LeftDoubleBracket]", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
           "\[RightDoubleBracket]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"point", "=", 
            RowBox[{"Transpose", "[", 
             RowBox[{"V", "\[LeftDoubleBracket]", 
              RowBox[{"CatmullClarkStencil", "[", 
               RowBox[{"local", ",", 
                RowBox[{
                "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}],
                "]"}], "\[RightDoubleBracket]"}], "]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"val", "=", 
              RowBox[{
               RowBox[{
                RowBox[{"Length", "[", 
                 RowBox[{
                 "point", "\[LeftDoubleBracket]", "1", 
                  "\[RightDoubleBracket]"}], "]"}], "/", "2"}], "-", "4"}]}], 
             "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"Simplify", "[", 
             RowBox[{
              RowBox[{"CatmullClarkListAlt", "\[LeftDoubleBracket]", 
               RowBox[{"val", "-", "2"}], "\[RightDoubleBracket]"}], ".", 
              RowBox[{
              "point", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               ".", 
              RowBox[{
              "point", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               ".", 
              RowBox[{
              "point", "\[LeftDoubleBracket]", "3", 
               "\[RightDoubleBracket]"}]}], "]"}]}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"m", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkVolume", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Total", "[", 
   RowBox[{"CatmullClarkVolumeFace", "[", 
    RowBox[{"{", 
     RowBox[{"V", ",", "F"}], "}"}], "]"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.605846228480858*^9, 3.6058462398955107`*^9}, {
   3.60584630124402*^9, 3.605846340747279*^9}, {3.6058464169596386`*^9, 
   3.605846420533843*^9}, {3.605846458206998*^9, 3.6058464704146957`*^9}, {
   3.6058513105105333`*^9, 3.6058513204771037`*^9}, {3.605851485700554*^9, 
   3.605851507644809*^9}, {3.6063891641917176`*^9, 3.606389214861616*^9}, {
   3.606393795352605*^9, 3.6063938285245023`*^9}, {3.6064273125263915`*^9, 
   3.6064273157805777`*^9}, {3.6064329361650457`*^9, 
   3.6064329842947984`*^9}, {3.606459691591164*^9, 3.6064597107402596`*^9}, {
   3.6064739922303047`*^9, 3.6064740025579634`*^9}, {3.6064742969749856`*^9, 
   3.606474300172535*^9}, {3.606644115925875*^9, 3.606644118109*^9}, {
   3.606695526082301*^9, 3.606695528989467*^9}, {3.606717935131093*^9, 
   3.6067179502063246`*^9}, {3.6067699996307235`*^9, 
   3.6067700183997965`*^9}, {3.606770049711588*^9, 3.6067701074848924`*^9}, {
   3.606770149964322*^9, 3.6067701848893194`*^9}, 3.6067702701511965`*^9, {
   3.6073263530603504`*^9, 3.607326389460909*^9}, {3.6081676637091475`*^9, 
   3.6081676640841694`*^9}, {3.6118573856946735`*^9, 
   3.6118574011365566`*^9}, {3.6118577816223187`*^9, 
   3.6118577865135984`*^9}, {3.6118880986793556`*^9, 3.611888099599408*^9}, {
   3.611907483049078*^9, 3.6119074846991725`*^9}, {3.612009217514154*^9, 
   3.6120092181901927`*^9}, {3.61217543663251*^9, 3.612175446440071*^9}, {
   3.612176338919118*^9, 3.612176353148932*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkValences", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"rings", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", 
          RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Union", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"Join", "@@", 
           RowBox[{"rings", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}]}], "]"}], "-", "12"}], ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6090520563808985`*^9, 3.6090521099449625`*^9}, {
  3.6090521764047637`*^9, 3.6090521767737846`*^9}, {3.6090522343920803`*^9, 
  3.6090522945615215`*^9}, {3.609052366820655*^9, 3.60905236953981*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkDemo", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "depth_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"V1", "=", "V"}], ",", 
      RowBox[{"F1", "=", "F"}], ",", "delta", ",", 
      RowBox[{"lst", "=", 
       RowBox[{"{", "}"}]}], ",", "fcv", ",", 
      RowBox[{"vls", "=", 
       RowBox[{"{", "}"}]}], ",", "inf", ",", "val", ",", 
      RowBox[{"cmp", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"size", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "V", "]"}], "<", "100"}], ",", "120", ",", 
         "180"}], "]"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "V", "]"}], "<", "20"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"TableForm", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            "\"\<Vertices \[DownArrow]\>\"", ",", 
             "\"\<Faces \[DownArrow]\>\""}], "}"}], ",", 
           RowBox[{"MatrixForm", "/@", 
            RowBox[{"{", 
             RowBox[{"V", ",", "F"}], "}"}]}]}], "}"}], "]"}], "]"}]}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"vls", ",", 
       RowBox[{"LinearVolume", "[", 
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"lst", ",", 
       RowBox[{"Rasterize", "[", 
        RowBox[{
         RowBox[{"Show", "[", 
          RowBox[{
           RowBox[{"PlotMesh", "[", 
            RowBox[{"{", 
             RowBox[{"V", ",", "F"}], "}"}], "]"}], ",", 
           RowBox[{"Axes", "\[Rule]", "False"}], ",", 
           RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
           RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
           RowBox[{"ImageSize", "\[Rule]", "size"}]}], "]"}], ",", 
         RowBox[{"ImageResolution", "\[Rule]", "size"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V1", ",", "F1"}], "}"}], "=", 
         RowBox[{"CatmullClarkSubdivide", "[", 
          RowBox[{"{", 
           RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"k", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"inf", "=", 
            RowBox[{"CatmullClarkVolume", "[", 
             RowBox[{"{", 
              RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"val", "=", 
            RowBox[{"CatmullClarkValences", "[", 
             RowBox[{"{", 
              RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}]}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"fcv", "=", 
         RowBox[{"CatmullClarkVolumeFace", "[", 
          RowBox[{"NumericMesh", "[", 
           RowBox[{"{", 
            RowBox[{"V1", ",", "F1"}], "}"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"cmp", ",", 
          RowBox[{"Total", "[", "fcv", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"vls", ",", 
          RowBox[{"LinearVolume", "[", 
           RowBox[{"{", 
            RowBox[{"V1", ",", "F1"}], "}"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"lst", ",", 
          RowBox[{"Rasterize", "[", 
           RowBox[{
            RowBox[{"Show", "[", 
             RowBox[{
              RowBox[{"PlotMesh", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"V1", ",", "F1"}], "}"}], ",", "fcv"}], "]"}], ",", 
              RowBox[{"Axes", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
              RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
              RowBox[{"ImageSize", "\[Rule]", "size"}]}], "]"}], ",", 
            RowBox[{"ImageResolution", "\[Rule]", "size"}]}], "]"}]}], "]"}], 
        ";"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "depth"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"delta", "=", 
      RowBox[{"N", "[", 
       RowBox[{"vls", "-", "inf"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "lst", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Required valences \[Tau](f) \[Element] \>\"", "<>", 
       RowBox[{"ToString", "[", "val", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Limit volume \[DownArrow] \>\"", "<>", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"InexactNumberQ", "[", "inf", "]"}], "\[Equal]", "False"}], 
         ",", 
         RowBox[{"\"\<(\[TildeTilde]\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"N", "[", 
            RowBox[{"inf", ",", "30"}], "]"}], "]"}], "<>", "\"\<)\>\""}], 
         ",", "\"\<\>\""}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "inf", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TableForm", "[", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"MatrixForm", "[", 
           RowBox[{"Transpose", "[", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Level\>\"", "}"}], ",", 
                RowBox[{"Range", "[", 
                 RowBox[{"0", ",", "depth"}], "]"}]}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Volume\>\"", "}"}], ",", "vls"}], "]"}], 
              ",", "\[IndentingNewLine]", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", "\"\<Delta to \[Infinity]\>\"", "}"}], ",", 
                "delta"}], "]"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
           "]"}], ",", 
          RowBox[{"ListLogPlot", "[", 
           RowBox[{
            RowBox[{"Transpose", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Range", "[", 
                RowBox[{"0", ",", "depth"}], "]"}], ",", 
               RowBox[{"delta", "/", 
                RowBox[{
                "delta", "\[LeftDoubleBracket]", "1", 
                 "\[RightDoubleBracket]"}]}]}], "}"}], "]"}], ",", 
            RowBox[{"Joined", "\[Rule]", "True"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "200"}], ",", 
            RowBox[{"AxesLabel", "\[Rule]", 
             RowBox[{"{", 
              RowBox[{"\"\<Level\>\"", ",", "\"\<Rel. error\>\""}], "}"}]}]}],
            "]"}]}], "}"}], "}"}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"cmp", "=", 
      RowBox[{"Union", "[", 
       RowBox[{"Chop", "[", 
        RowBox[{"cmp", "-", "inf"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"1", "<", 
        RowBox[{"Length", "[", "cmp", "]"}]}], ",", 
       RowBox[{"Print", "[", "cmp", "]"}]}], "]"}], ";"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6058475746868567`*^9, 3.6058475863275223`*^9}, {
   3.605847712971766*^9, 3.605847801399824*^9}, {3.6058478714508305`*^9, 
   3.605848060577648*^9}, {3.6058480917024283`*^9, 3.6058481842667227`*^9}, {
   3.605848220830814*^9, 3.605848232770497*^9}, {3.605848430292794*^9, 
   3.6058484888111415`*^9}, {3.6058485310395565`*^9, 
   3.6058485795443316`*^9}, {3.605849087329375*^9, 3.6058492709878798`*^9}, {
   3.6058493011656055`*^9, 3.6058493015046253`*^9}, {3.605849336046601*^9, 
   3.6058493744447966`*^9}, {3.605849408872766*^9, 3.6058494246496687`*^9}, {
   3.605849768383329*^9, 3.6058498087546377`*^9}, {3.6058498835739174`*^9, 
   3.6058499010319157`*^9}, {3.605850027835169*^9, 3.605850032282423*^9}, {
   3.605850349872588*^9, 3.6058503649824524`*^9}, {3.6058506459265213`*^9, 
   3.6058507522946053`*^9}, {3.605850784478446*^9, 3.6058508656410885`*^9}, {
   3.6058510063751383`*^9, 3.605851007900225*^9}, {3.6058512689691577`*^9, 
   3.605851304798207*^9}, {3.605851497158209*^9, 3.60585152689191*^9}, {
   3.6058519900824027`*^9, 3.6058519952636995`*^9}, {3.605852040193269*^9, 
   3.605852134135642*^9}, {3.606390965123725*^9, 3.6063909887820787`*^9}, {
   3.6063910565859566`*^9, 3.606391058659075*^9}, 3.606393871599966*^9, 
   3.606643733415997*^9, {3.6066955813704634`*^9, 3.606695672685686*^9}, {
   3.606695752756266*^9, 3.606695785678149*^9}, {3.606696052527412*^9, 
   3.606696159506531*^9}, {3.6067355134675765`*^9, 3.6067355344277754`*^9}, {
   3.6067362776572857`*^9, 3.606736282335553*^9}, {3.607504947722542*^9, 
   3.607504951046732*^9}, {3.607505159920679*^9, 3.607505161302758*^9}, {
   3.6075052135017433`*^9, 3.6075052179419975`*^9}, {3.608865767131858*^9, 
   3.6088658541768365`*^9}, {3.608865888424795*^9, 3.6088658909159374`*^9}, {
   3.608866064113844*^9, 3.6088660644038606`*^9}, {3.608888725154479*^9, 
   3.608888738365235*^9}, {3.6088888189568443`*^9, 3.6088888281233687`*^9}, {
   3.608888961535*^9, 3.6088889694914546`*^9}, {3.608889061777733*^9, 
   3.6088890929555163`*^9}, {3.6088891549060593`*^9, 3.608889219843774*^9}, 
   3.608889282453355*^9, {3.608889324960786*^9, 3.6088893313791533`*^9}, {
   3.6088894818297586`*^9, 3.6088895153966784`*^9}, {3.6088896071199245`*^9, 
   3.608889620368683*^9}, {3.6088897202953978`*^9, 3.6088897307819977`*^9}, {
   3.608889822528246*^9, 3.6088898321957984`*^9}, 3.608889871724059*^9, {
   3.6088899110333076`*^9, 3.608889925648144*^9}, {3.6088899969352207`*^9, 
   3.6088900037016077`*^9}, {3.6088900766147785`*^9, 3.608890079974971*^9}, {
   3.6088902064802065`*^9, 3.608890219030924*^9}, 3.608890290382005*^9, {
   3.6088955971285334`*^9, 3.6088956194608107`*^9}, 3.6090216522411985`*^9, {
   3.6090216921344805`*^9, 3.6090216983198347`*^9}, {3.609021748109682*^9, 
   3.609021821870901*^9}, {3.609051861852772*^9, 3.6090519150998178`*^9}, {
   3.6090521220836563`*^9, 3.609052129640089*^9}, {3.6090534623493156`*^9, 
   3.609053463361373*^9}, {3.612176995720685*^9, 3.6121770164498706`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Mesh examples", "Subsection",
 CellChangeTimes->{{3.6066937912720757`*^9, 3.6066938001225815`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CubeMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"0", "1", "0"},
        {"1", "1", "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {"0", "1", "1"},
        {"1", "1", "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "6", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CornerMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", "0", "0"},
         {"1", "0", "0"},
         {"0", "1", "0"},
         {"1", "1", "0"},
         {"0", "0", "1"},
         {"1", "0", "1"},
         {"0", "1", "1"},
         {"1", "1", "1"},
         {"1", "0", "2"},
         {"1", "1", "2"},
         {"2", "0", "0"},
         {"2", "1", "0"},
         {"2", "0", "1"},
         {"2", "1", "1"},
         {"2", "0", "2"},
         {"2", "1", "2"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "11", ",", "13", ",", "6"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "13", ",", "15", ",", "9"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "12", ",", "11", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"12", ",", "14", ",", "13", ",", "11"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"14", ",", "16", ",", "15", ",", "13"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "9", ",", "10", ",", "8"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "14", ",", "12"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"8", ",", "10", ",", "16", ",", "14"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TripodMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", "0", "0"},
         {"1", "0", "0"},
         {"0", "1", "0"},
         {"1", "1", "0"},
         {"0", "0", "1"},
         {"1", "0", "1"},
         {"0", "1", "1"},
         {"1", "1", "1"},
         {"1", "0", "2"},
         {"1", "1", "2"},
         {"2", "0", "0"},
         {"2", "1", "0"},
         {"2", "0", "1"},
         {"2", "1", "1"},
         {"2", "0", "2"},
         {"2", "1", "2"},
         {"1", 
          RowBox[{"-", "1"}], "0"},
         {"2", 
          RowBox[{"-", "1"}], "0"},
         {"1", 
          RowBox[{"-", "1"}], "1"},
         {"2", 
          RowBox[{"-", "1"}], "1"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6", ",", "8", ",", "7"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "8", ",", "4", ",", "3"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3", ",", "1", ",", "5"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "13", ",", "15", ",", "9"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "12", ",", "11", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"12", ",", "14", ",", "13", ",", "11"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"14", ",", "16", ",", "15", ",", "13"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"9", ",", "15", ",", "16", ",", "10"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "9", ",", "10", ",", "8"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "8", ",", "14", ",", "12"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"8", ",", "10", ",", "16", ",", "14"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"17", ",", "18", ",", "20", ",", "19"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"19", ",", "20", ",", "13", ",", "6"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"11", ",", "13", ",", "20", ",", "18"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"11", ",", "18", ",", "17", ",", "2"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"17", ",", "19", ",", "6", ",", "2"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TetrahedronMesh", "=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"A", "=", 
       RowBox[{"(", GridBox[{
          {"1", 
           FractionBox["1", "2"], 
           FractionBox["1", "2"]},
          {"0", 
           FractionBox[
            SqrtBox["3"], "2"], 
           FractionBox["1", 
            RowBox[{"2", 
             SqrtBox["3"]}]]},
          {"0", "0", 
           SqrtBox[
            FractionBox["2", "3"]]}
         }], ")"}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", GridBox[{
           {"0", "0", "0"},
           {"1", "0", "0"},
           {"0", "1", "0"},
           {"0", "0", "1"}
          }], ")"}], ".", 
        RowBox[{"Transpose", "[", "A", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"4", ",", "2", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"OctahedronMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"1", "1", "0"},
        {"0", "1", "0"},
        {
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", 
          SqrtBox["2"]}]},
        {
         RowBox[{"1", "/", "2"}], 
         RowBox[{"1", "/", "2"}], 
         RowBox[{
          RowBox[{"-", "1"}], "/", 
          SqrtBox["2"]}]}
       }], ")"}], ",", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2", ",", "1"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "3", ",", "2"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "4", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "1", ",", "4"}], "}"}]}], "\[IndentingNewLine]", 
      "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TriQuadPrismMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {
         RowBox[{"1", "/", "2"}], 
         FractionBox[
          SqrtBox["3"], "2"], "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {
         RowBox[{"1", "/", "2"}], 
         FractionBox[
          SqrtBox["3"], "2"], "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "5", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "6", ",", "5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1", ",", "4", ",", "6"}], "}"}]}], "}"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TwistedPrismMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"0", "0", "0"},
        {"1", "0", "0"},
        {"0", "1", "0"},
        {"0", "0", "1"},
        {"1", "0", "1"},
        {"0", "1", "1"}
       }], ")"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "2", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"5", ",", "2", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2", ",", "3"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "4", ",", "6"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6", ",", "3"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.606693175688866*^9, 3.6066931925668316`*^9}, {
  3.6066934092942276`*^9, 3.6066934695606747`*^9}, {3.6066936984677677`*^9, 
  3.6066937331517515`*^9}, {3.606693785456743*^9, 3.6066937858307643`*^9}, {
  3.606694300801219*^9, 3.6066943224624577`*^9}, {3.6066944264494057`*^9, 
  3.6066944405282106`*^9}, {3.6066944879599237`*^9, 3.6066944972164536`*^9}, {
  3.6088434287549133`*^9, 3.6088434886233373`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TorusMesh", "=", 
   RowBox[{"{", 
    RowBox[{
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {
          RowBox[{"3", "/", "2"}], "0", "0"},
         {"1.2135254915624212`", "0.8816778784387097`", "0"},
         {"0.4635254915624212`", "1.4265847744427302`", "0"},
         {
          RowBox[{"-", "0.463525491562421`"}], "1.4265847744427305`", "0"},
         {
          RowBox[{"-", "1.213525491562421`"}], "0.8816778784387098`", "0"},
         {
          RowBox[{
           RowBox[{"-", "3"}], "/", "2"}], "0", "0"},
         {
          RowBox[{"-", "1.2135254915624212`"}], 
          RowBox[{"-", "0.8816778784387096`"}], "0"},
         {
          RowBox[{"-", "0.46352549156242134`"}], 
          RowBox[{"-", "1.4265847744427302`"}], "0"},
         {"0.46352549156242084`", 
          RowBox[{"-", "1.4265847744427305`"}], "0"},
         {"1.213525491562421`", 
          RowBox[{"-", "0.88167787843871`"}], "0"},
         {"1", "0", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0.8660254037844386`", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], "0.8660254037844387`", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"-", "1"}], "0", 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], 
          RowBox[{"-", "0.8660254037844385`"}], 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], 
          RowBox[{"-", "0.8660254037844386`"}], 
          RowBox[{"1", "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0", "0"},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "4"}], "0.43301270189221935`", "0"},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "4"}], 
          RowBox[{"-", "0.43301270189221924`"}], "0"},
         {"1", "0", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], "0.8660254037844385`", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], "0.8660254037844386`", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"-", "1"}], "0", 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}], 
          RowBox[{"-", "0.8660254037844384`"}], 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]},
         {
          RowBox[{"1", "/", "2"}], 
          RowBox[{"-", "0.8660254037844385`"}], 
          RowBox[{
           RowBox[{"-", "1"}], "/", "2"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]], ",", "\[IndentingNewLine]", 
     TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"3", "4", "13"},
         {"12", "2", "3"},
         {"18", "13", "14"},
         {"23", "18", "19"},
         {"21", "20", "17"},
         {"22", "4", "3"},
         {"24", "19", "25"},
         {"8", "24", "9"},
         {"7", "6", "23"},
         {"6", "14", "5"},
         {"14", "13", "5"},
         {"11", "1", "2"},
         {"12", "3", "13"},
         {"13", "4", "5"},
         {"18", "12", "13"},
         {"11", "2", "12"},
         {"10", "16", "9"},
         {"19", "18", "14"},
         {"17", "11", "12"},
         {"18", "17", "12"},
         {"2", "20", "21"},
         {"23", "5", "22"},
         {"2", "1", "20"},
         {"3", "2", "21"},
         {"22", "3", "21"},
         {"22", "5", "4"},
         {"24", "23", "19"},
         {"21", "17", "18"},
         {"18", "22", "21"},
         {"23", "6", "5"},
         {"18", "23", "22"},
         {"24", "7", "23"},
         {"7", "24", "8"},
         {"24", "25", "9"},
         {"25", "10", "9"},
         {"25", "19", "17"},
         {"25", "20", "10"},
         {"10", "20", "1"},
         {"25", "17", "20"},
         {"16", "10", "11"},
         {"19", "16", "17"},
         {"16", "15", "9"},
         {"19", "14", "15"},
         {"17", "16", "11"},
         {"19", "15", "16"},
         {"7", "14", "6"},
         {"15", "14", "7"},
         {"15", "8", "9"},
         {"15", "7", "8"},
         {"10", "1", "11"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.606694128983392*^9, 3.60669414801748*^9}, {
  3.6088426029046774`*^9, 3.6088426832282715`*^9}, {3.608842745425829*^9, 
  3.6088428057522793`*^9}, {3.60884346279786*^9, 3.6088434648269763`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Torus", "[", 
    RowBox[{"R_", ",", "r_", ",", "\[Theta]_", ",", "\[Phi]_"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{"r", " ", 
      RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"TorusVolume", "[", 
    RowBox[{"R_", ",", "r_"}], "]"}], "=", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Pi]", "2"], "R", " ", 
    SuperscriptBox["r", "2"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TorusSample", "[", 
   RowBox[{"N_", ",", "M_", ",", "i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"Torus", "[", 
   RowBox[{"2", ",", "1", ",", 
    RowBox[{
     FractionBox[
      RowBox[{"2", "\[Pi]", " ", "i"}], "N"], "+", 
     FractionBox["\[Pi]", "4"]}], ",", 
    FractionBox[
     RowBox[{"2", "\[Pi]", " ", "j"}], "M"]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6065531864185305`*^9, 3.6065531923228683`*^9}, {
  3.6065544938033085`*^9, 3.606554505849998*^9}, {3.606554616960353*^9, 
  3.606554618703453*^9}, {3.6067694316322355`*^9, 3.6067694620499754`*^9}, {
  3.608806220050144*^9, 3.608806223829361*^9}, {3.6088425841666055`*^9, 
  3.608842590787984*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"RegularTorusTria", "[", 
   RowBox[{"N_", ",", "M_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "func", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"func", "[", 
       RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"i", " ", "M"}], "+", "j"}], ",", 
         RowBox[{"N", " ", "M"}]}], "]"}], "+", "1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"TorusSample", "[", 
            RowBox[{"N", ",", "M", ",", "i", ",", "j"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
             RowBox[{"N", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
             RowBox[{"M", "-", "1"}]}], "}"}]}], "]"}], ",", "1"}], "]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"Flatten", "[", 
        RowBox[{
         RowBox[{"Table", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"func", "[", 
                RowBox[{"i", ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{"i", ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}]}], "}"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", "j"}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{
                 RowBox[{"i", "+", "1"}], ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}], ",", 
               RowBox[{"func", "[", 
                RowBox[{"i", ",", 
                 RowBox[{"j", "+", "1"}]}], "]"}]}], "}"}]}], 
            "\[IndentingNewLine]", "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "0", ",", 
             RowBox[{"N", "-", "1"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", 
             RowBox[{"M", "-", "1"}]}], "}"}]}], "]"}], ",", "2"}], "]"}]}], 
      "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6065532167602663`*^9, 3.6065533077684717`*^9}, {
  3.606553339978314*^9, 3.6065533556702113`*^9}, {3.60655345215273*^9, 
  3.606553453766822*^9}, {3.606553484510581*^9, 3.6065535192295666`*^9}, {
  3.6065536185682487`*^9, 3.6065536493200073`*^9}, {3.606553718464962*^9, 
  3.6065537427133493`*^9}, {3.6065537860728292`*^9, 3.6065538299113364`*^9}, {
  3.6065538753049326`*^9, 3.606553950997262*^9}, {3.6065540157739673`*^9, 
  3.6065540528860903`*^9}, {3.6067688938164744`*^9, 3.606768926741358*^9}, {
  3.6067692563772116`*^9, 3.606769273456188*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Special meshes", "Subsection",
 CellChangeTimes->{{3.6066937912720757`*^9, 3.6066938001225815`*^9}, {
  3.612176759095151*^9, 3.6121767627573605`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ProjectQ", "[", "num_", "]"}], ":=", 
  RowBox[{"Rationalize", "[", 
   RowBox[{"num", ",", 
    RowBox[{"1", "/", "1000"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"ProjectQ", ",", "Listable"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6064270002485304`*^9, 3.6064271431847057`*^9}, {
  3.606428907467617*^9, 3.606428912845925*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuadDiamongMesh", "[", "n_", "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ProjectQ", "[", 
     RowBox[{"Join", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"ang", "=", 
             RowBox[{"\[Pi]", " ", 
              RowBox[{"k", "/", "n"}]}]}], "}"}], ",", 
           RowBox[{"{", " ", 
            RowBox[{
             RowBox[{"Cos", "[", "ang", "]"}], ",", " ", 
             RowBox[{"Sin", "[", "ang", "]"}], ",", "1"}], "}"}]}], "]"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "0", ",", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "1"}]}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"ang", "=", 
             RowBox[{"\[Pi]", " ", 
              RowBox[{"k", "/", "n"}]}]}], "}"}], ",", 
           RowBox[{"{", " ", 
            RowBox[{
             RowBox[{"Cos", "[", "ang", "]"}], ",", " ", 
             RowBox[{"Sin", "[", "ang", "]"}], ",", "0"}], "}"}]}], "]"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "0", ",", 
           RowBox[{
            RowBox[{"2", "n"}], "-", "1"}]}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]}], 
      "\[IndentingNewLine]", "]"}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Join", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"k", "+", 
            RowBox[{"{", 
             RowBox[{"1", ",", "1", ",", "2", ",", "2"}], "}"}]}], ",", 
           RowBox[{"2", " ", "n"}]}], "]"}], "+", "1", "+", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{"2", "n"}], ",", 
           RowBox[{"2", "n"}], ",", "0"}], "}"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "0", ",", 
          RowBox[{
           RowBox[{"2", "n"}], "-", "1"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"2", "k"}], "+", "1"}], ",", 
          RowBox[{
           RowBox[{"2", "k"}], "+", "2"}], ",", 
          RowBox[{
           RowBox[{"Mod", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"2", "k"}], "+", "3", "-", "1"}], ",", 
             RowBox[{"2", "n"}]}], "]"}], "+", "1"}], ",", 
          RowBox[{
           RowBox[{"4", "n"}], "+", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "0", ",", 
          RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Reverse", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"2", "n"}], "+", 
            RowBox[{"2", "k"}], "+", "1"}], ",", 
           RowBox[{
            RowBox[{"2", "n"}], "+", 
            RowBox[{"2", "k"}], "+", "2"}], ",", 
           RowBox[{
            RowBox[{"2", "n"}], "+", 
            RowBox[{"Mod", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"2", "k"}], "+", "3", "-", "1"}], ",", 
              RowBox[{"2", "n"}]}], "]"}], "+", "1"}], ",", 
           RowBox[{
            RowBox[{"4", "n"}], "+", "1"}]}], "}"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "0", ",", 
          RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
     "]"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.6063919539452825`*^9, 3.6063919876042075`*^9}, {
  3.6063920286355543`*^9, 3.6063920328687963`*^9}, {3.6063921224279194`*^9, 
  3.606392295616825*^9}, {3.6063923437195764`*^9, 3.6063924627363834`*^9}, {
  3.6063925197126427`*^9, 3.60639256863144*^9}, {3.6063926009942913`*^9, 
  3.606392655006381*^9}, {3.6063926922555113`*^9, 3.6063928981632886`*^9}, {
  3.6063929612788987`*^9, 3.60639296637119*^9}, {3.6064271514421782`*^9, 
  3.6064271582075653`*^9}, {3.606427937414133*^9, 3.6064280242681007`*^9}, {
  3.612009262567731*^9, 3.6120092629587536`*^9}, {3.6121767942941637`*^9, 
  3.6121767980273776`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CubedMesh", "[", "m_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"pnt", ",", 
      RowBox[{"n", "=", 
       RowBox[{"2", " ", "m"}]}], ",", "lne"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"lne", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{"2", 
            RowBox[{"k", "/", "m"}]}]}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "m"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"pnt", "=", 
      RowBox[{"Flatten", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"lne", ".", 
           RowBox[{"RotationMatrix", "[", "#", "]"}]}], "&"}], "/@", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{
            RowBox[{"-", "\[Pi]"}], "/", "2"}], ",", 
           RowBox[{"-", "\[Pi]"}], ",", 
           RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], ",", "1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Join", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Join", "[", 
             RowBox[{"#", ",", 
              RowBox[{"{", "1", "}"}]}], "]"}], "&"}], "/@", "pnt"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"Join", "[", 
             RowBox[{"#", ",", 
              RowBox[{"{", 
               RowBox[{"-", "1"}], "}"}]}], "]"}], "&"}], "/@", "pnt"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", 
              RowBox[{"-", "1"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}]}], 
         "\[IndentingNewLine]", "]"}], "/", "2"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Join", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Mod", "[", 
             RowBox[{
              RowBox[{"k", "+", 
               RowBox[{"{", 
                RowBox[{"1", ",", "1", ",", "2", ",", "2"}], "}"}]}], ",", 
              RowBox[{"2", " ", "n"}]}], "]"}], "+", "1", "+", 
            RowBox[{"{", 
             RowBox[{"0", ",", 
              RowBox[{"2", "n"}], ",", 
              RowBox[{"2", "n"}], ",", "0"}], "}"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "0", ",", 
             RowBox[{
              RowBox[{"2", "n"}], "-", "1"}]}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"2", "k"}], "+", "1"}], ",", 
             RowBox[{
              RowBox[{"2", "k"}], "+", "2"}], ",", 
             RowBox[{
              RowBox[{"Mod", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"2", "k"}], "+", "3", "-", "1"}], ",", 
                RowBox[{"2", "n"}]}], "]"}], "+", "1"}], ",", 
             RowBox[{
              RowBox[{"4", "n"}], "+", "2"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "0", ",", 
             RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"2", "n"}], "+", 
               RowBox[{"2", "k"}], "+", "1"}], ",", 
              RowBox[{
               RowBox[{"2", "n"}], "+", 
               RowBox[{"2", "k"}], "+", "2"}], ",", 
              RowBox[{
               RowBox[{"2", "n"}], "+", 
               RowBox[{"Mod", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"2", "k"}], "+", "3", "-", "1"}], ",", 
                 RowBox[{"2", "n"}]}], "]"}], "+", "1"}], ",", 
              RowBox[{
               RowBox[{"4", "n"}], "+", "1"}]}], "}"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "0", ",", 
             RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.612176801885598*^9, 3.612176911132847*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Facet topology", "Subsection",
 CellChangeTimes->{{3.612175287980008*^9, 3.612175296248481*^9}, {
  3.612176390496068*^9, 3.6121764020637293`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkDefault", "[", "n_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"-", "1"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"n", "\[Equal]", "3"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0"}], "}"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "2"}], ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "}"}], ",", 
      RowBox[{"With", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"m", "=", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{"n", "-", "3"}], ")"}]}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"OddQ", "[", "k", "]"}], ",", 
             SqrtBox["2"], ",", "1"}], "]"}], 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Sin", "[", 
              FractionBox[
               RowBox[{"k", " ", "\[Pi]"}], 
               RowBox[{"m", " ", "2"}]], "]"}], ",", 
             RowBox[{"Cos", "[", 
              FractionBox[
               RowBox[{"k", " ", "\[Pi]"}], 
               RowBox[{"m", " ", "2"}]], "]"}]}], "}"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "0", ",", 
            RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"ring", "=", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"10", ",", "14"}], "}"}], ",", 
          RowBox[{"14", "+", 
           RowBox[{"Range", "[", 
            RowBox[{"2", 
             RowBox[{"(", 
              RowBox[{"n", "-", "3"}], ")"}]}], "]"}]}], ",", 
          RowBox[{"{", "12", "}"}]}], "]"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}]}], "}"}], 
        ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Join", "[", 
           RowBox[{
            RowBox[{"{", "11", "}"}], ",", 
            RowBox[{"ring", "\[LeftDoubleBracket]", 
             RowBox[{
              RowBox[{"2", "k"}], "+", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0", ",", 
                RowBox[{"-", "1"}]}], "}"}]}], "\[RightDoubleBracket]"}]}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", 
            RowBox[{"n", "-", "2"}]}], "}"}]}], "]"}]}], "]"}]}], "]"}]}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"Vs_", ",", "Fs_"}], "}"}], ",", "col_"}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Gray", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"Vs", "\[LeftDoubleBracket]", 
            RowBox[{"#", "\[LeftDoubleBracket]", 
             RowBox[{"Join", "[", 
              RowBox[{
               RowBox[{"Range", "[", 
                RowBox[{"Length", "[", "#", "]"}], "]"}], ",", 
               RowBox[{"{", "1", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", "Fs"}]}], "}"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"EdgeForm", "[", 
         RowBox[{"{", "Dashed", "}"}], "]"}], ",", "LightGreen", ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
          "&"}], "/@", "F"}], 
        RowBox[{"(*", 
         RowBox[{"\[LeftDoubleBracket]", 
          RowBox[{"Drop", "[", 
           RowBox[{
            RowBox[{"Range", "[", 
             RowBox[{"Length", "[", "F", "]"}], "]"}], ",", 
            RowBox[{"{", "5", "}"}]}], "]"}], "\[RightDoubleBracket]"}], 
         "*)"}], ",", "\[IndentingNewLine]", 
        RowBox[{"EdgeForm", "[", "Dashed", "]"}], ",", "col", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Polygon", "[", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{
           RowBox[{
           "F", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}], ",", 
           "All"}], "\[RightDoubleBracket]"}], "]"}]}], "\[IndentingNewLine]",
        "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"Flatten", "[", 
           RowBox[{"{", "col", "}"}], "]"}], "]"}], "\[Equal]", "0"}], ",", 
        RowBox[{"{", "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"col", ",", 
          RowBox[{"Opacity", "[", "0.5", "]"}], ",", 
          RowBox[{"Polygon", "[", 
           RowBox[{"V", "\[LeftDoubleBracket]", 
            RowBox[{
            "F", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}], 
            "\[RightDoubleBracket]"}], "]"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", "Green", ",", "\[IndentingNewLine]", 
      RowBox[{"EdgeForm", "[", 
       RowBox[{"{", "Blue", "}"}], "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Disk", "[", 
         RowBox[{"#", ",", "0.04"}], "]"}], "&"}], "/@", "Vs"}]}], 
     "\[IndentingNewLine]", "}"}], ",", 
    RowBox[{"Axes", "\[Rule]", "False"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkDecomposition", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"fine", "=", 
       RowBox[{"sub", ".", "V"}]}], ",", 
      RowBox[{"Freg", "=", 
       RowBox[{
        RowBox[{"CatmullClarkDefault", "[", "4", "]"}], 
        "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"CatmullClarkPlot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", "Red"}], "]"}], "}"}], ",", 
      "\"\<=\>\"", ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "Freg"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
        "\"\<+\>\"", ",", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"fine", "\[LeftDoubleBracket]", 
             RowBox[{
             "ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
             "\[RightDoubleBracket]"}], ",", "F"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", "Red"}], "]"}]}], "}"}]}], 
     "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"GraphicsWeights", "[", 
   RowBox[{"V_", ",", "s_", ",", 
    RowBox[{"size_:", "Medium"}]}], "]"}], ":=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
        "\[NotEqual]", "0"}], ",", 
       RowBox[{"Text", "[", 
        RowBox[{
         RowBox[{"Style", "[", 
          RowBox[{
           RowBox[{
           "s", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], ",", 
           "Bold", ",", "size"}], "]"}], ",", 
         RowBox[{"V", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
         ",", 
         RowBox[{"Background", "\[Rule]", "LightRed"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", "s", "]"}]}], "}"}]}], "]"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkStencils", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"pnt", "=", 
      RowBox[{"sub", ".", "V"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Show", "[", 
       RowBox[{
        RowBox[{"CatmullClarkPlot", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"V", ",", "F"}], "}"}], ",", 
          RowBox[{"{", "}"}]}], "]"}], ",", 
        RowBox[{"GraphicsWeights", "[", 
         RowBox[{"V", ",", 
          RowBox[{
          "sub", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
         "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Graphics", "[", 
         RowBox[{"{", 
          RowBox[{"Red", ",", 
           RowBox[{"Disk", "[", 
            RowBox[{
             RowBox[{
             "pnt", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
             ",", "0.05"}], "]"}]}], "}"}], "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", 
        RowBox[{"Length", "[", "sub", "]"}]}], "}"}]}], "]"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkShow", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "sub_", ",", "ind_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{"CatmullClarkStencils", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], ",", "sub"}], "]"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"CatmullClarkDecomposition", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"V", ",", "F"}], "}"}], ",", "sub", ",", "ind"}], "]"}], 
      "]"}], ";"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkIndexing", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "F"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "F"}], "}"}], "=", 
      RowBox[{"CatmullClarkDefault", "[", "n", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"CatmullClarkPlot", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"V", ",", "F"}], "}"}], ",", "Green"}], "]"}], ",", 
       RowBox[{"GraphicsWeights", "[", 
        RowBox[{"V", ",", 
         RowBox[{"Range", "[", 
          RowBox[{"Length", "[", "V", "]"}], "]"}], ",", "Large"}], "]"}]}], 
      "]"}]}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6060114962818995`*^9, 3.606011518097147*^9}, {
   3.6060115842759323`*^9, 3.6060116469145155`*^9}, {3.606011777914008*^9, 
   3.6060118162672014`*^9}, 3.6060118524432707`*^9, {3.6060118846491127`*^9, 
   3.6060119399772778`*^9}, {3.606012010244296*^9, 3.6060120108773327`*^9}, {
   3.6060120671385508`*^9, 3.606012166147214*^9}, {3.6060142069449406`*^9, 
   3.6060142072369576`*^9}, {3.606014237686699*^9, 3.6060142666163535`*^9}, 
   3.606386402256744*^9, {3.6064268688920174`*^9, 3.6064268691570325`*^9}, {
   3.6064649509399815`*^9, 3.6064649511279926`*^9}, {3.606633085667821*^9, 
   3.6066330977625127`*^9}, {3.606947253359164*^9, 3.6069472553662786`*^9}, 
   3.6069473427912793`*^9, 3.6069474110431833`*^9, {3.607026405576552*^9, 
   3.607026413045979*^9}, {3.6121760071741433`*^9, 3.612176007749176*^9}, {
   3.6121760459933634`*^9, 3.612176048672517*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Animation utilities", "Subsection",
 CellChangeTimes->{{3.6121765636889744`*^9, 3.6121765698473263`*^9}, {
   3.6121773544582033`*^9, 3.612177355481262*^9}, 3.612281197503976*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkStencilList", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rings", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}]}], ",", 
      "local"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{
         RowBox[{"rings", "\[LeftDoubleBracket]", 
          RowBox[{"F", "\[LeftDoubleBracket]", 
           RowBox[{"k", ",", "j"}], "\[RightDoubleBracket]"}], 
          "\[RightDoubleBracket]"}], ",", 
         RowBox[{
         "F", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], "]"}],
        ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"local", "=", 
         RowBox[{"Join", "@@", 
          RowBox[{"rings", "\[LeftDoubleBracket]", 
           RowBox[{
           "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
           "\[RightDoubleBracket]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"CatmullClarkStencil", "[", 
         RowBox[{"local", ",", 
          RowBox[{
          "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}]}], 
         "]"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"m", ",", 
         RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CCRingIndex", "[", "n_", "]"}], ":=", 
  RowBox[{"ArrayReshape", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"14", ",", "10"}], "}"}], ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"11", ",", 
           RowBox[{"8", "+", 
            RowBox[{"2", "k"}], "+", "5"}], ",", 
           RowBox[{"8", "+", 
            RowBox[{"2", "k"}], "+", "6"}], ",", 
           RowBox[{"8", "+", 
            RowBox[{"2", "k"}], "+", "5"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", 
           RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"11", ",", "12"}], "}"}]}], "]"}], "//", "Flatten"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "4"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkStencilPlot", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"lst", "=", 
      RowBox[{"CatmullClarkStencilList", "[", 
       RowBox[{"{", 
        RowBox[{"V", ",", "F"}], "}"}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Rasterize", "[", 
       RowBox[{
        RowBox[{"Show", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Graphics3D", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Opacity", "[", "0.1", "]"}], ",", 
             RowBox[{
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"V", "\[LeftDoubleBracket]", 
                 RowBox[{"#", ",", "All"}], "\[RightDoubleBracket]"}], "]"}], 
               "&"}], "/@", "F"}]}], "}"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"Graphics3D", "[", 
           RowBox[{"{", "\[IndentingNewLine]", 
            RowBox[{"With", "[", 
             RowBox[{
              RowBox[{"{", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"mid", "=", 
                 RowBox[{"V", "\[LeftDoubleBracket]", 
                  RowBox[{
                   RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"6", ",", "7", ",", "11", ",", "10"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                  "\[RightDoubleBracket]"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{"ver", "=", 
                 RowBox[{"Join", "[", "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"1", ",", "2", ",", "6", ",", "5"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"2", ",", "3", ",", "7", ",", "6"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"3", ",", "4", ",", "8", ",", "7"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"5", ",", "6", ",", "10", ",", "9"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"7", ",", "8", ",", "12", ",", "11"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                    RowBox[{"{", 
                    RowBox[{"9", ",", "10", ",", "14", ",", "13"}], "}"}], 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", "}"}],
                    ",", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"V", "\[LeftDoubleBracket]", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ",", "All"}], 
                    "\[RightDoubleBracket]"}], "&"}], "/@", 
                    RowBox[{"CCRingIndex", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Length", "[", 
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "]"}], "/", "2"}], "-", "6"}], 
                    "]"}]}]}], "]"}]}]}], "\[IndentingNewLine]", "}"}], ",", 
              RowBox[{"{", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"Opacity", "[", "1", "]"}], ",", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"Length", "[", 
                    RowBox[{
                    "lst", "\[LeftDoubleBracket]", "n", 
                    "\[RightDoubleBracket]"}], "]"}], "\[Equal]", "16"}], ",",
                   "Green", ",", "Red"}], "]"}], ",", 
                RowBox[{"Polygon", "[", "mid", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Opacity", "[", "0.7", "]"}], ",", "Gray", ",", 
                RowBox[{"Polygon", "/@", "ver"}]}], "}"}]}], 
             "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", "}"}], 
           "]"}], ",", 
          RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
          RowBox[{"PlotRangePadding", "\[Rule]", "0"}], ",", 
          RowBox[{"ViewPoint", "\[Rule]", 
           RowBox[{"5", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "2.2"}], ",", 
              RowBox[{"-", "2.9"}], ",", "1.5"}], "}"}]}]}], ",", 
          RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"ImageResolution", "\[Rule]", "200"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"n", ",", 
        RowBox[{"Min", "[", 
         RowBox[{
          RowBox[{"Length", "[", "lst", "]"}], ",", "300"}], "]"}]}], "}"}]}],
      "]"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6081101870555944`*^9, 3.6081101873946133`*^9}, {
  3.6121754747126884`*^9, 3.6121754786639147`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applications", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
   3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
   3.6056228546483173`*^9}, 3.605696312889595*^9, {3.60570271998906*^9, 
   3.60570272033908*^9}, {3.6057363923638954`*^9, 3.6057364076977725`*^9}, 
   3.6060117523985486`*^9, {3.6067159239480705`*^9, 3.6067159244350986`*^9}, {
   3.6067182744948053`*^9, 3.606718275012313*^9}, {3.612138185056046*^9, 
   3.6121381853740644`*^9}, {3.612175503135314*^9, 3.612175503623342*^9}, {
   3.6121760908519297`*^9, 3.612176092419019*^9}, {3.612308581998134*^9, 
   3.6123085821641436`*^9}}],

Cell[CellGroupData[{

Cell["Simple test", "Subsection",
 CellChangeTimes->{{3.612176085368616*^9, 3.6121760878417573`*^9}, {
  3.6121771037798653`*^9, 3.612177113539424*^9}}],

Cell[BoxData[
 RowBox[{"PlotMesh", "[", 
  RowBox[{"Nest", "[", 
   RowBox[{"CatmullClarkSubdivide", ",", "CubeMesh", ",", "2"}], "]"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6123066099993424`*^9, 3.6123066125514884`*^9}}],

Cell[BoxData[{
 RowBox[{"CatmullClarkVolume", "[", 
  RowBox[{"Nest", "[", 
   RowBox[{"CatmullClarkSubdivide", ",", "CubeMesh", ",", "1"}], "]"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkVolume", "[", 
  RowBox[{"Nest", "[", 
   RowBox[{"CatmullClarkSubdivide", ",", "CubeMesh", ",", "2"}], "]"}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.6121770739191575`*^9, 3.612177094359327*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Demo", "Subsection",
 CellChangeTimes->{{3.612176085368616*^9, 3.6121760878417573`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{"CubeMesh", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.6121758752936*^9, 3.6121758824540095`*^9}, 
   3.612176750057634*^9, 3.6121771197347784`*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{"TripodMesh", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.6121759043662634`*^9, 3.6121759052313128`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{
   RowBox[{"CubedMesh", "[", "6", "]"}], ",", "2"}], "]"}]], "Input",
 CellChangeTimes->{{3.6121768068868847`*^9, 3.612176868632416*^9}, 
   3.6121769220484715`*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Article", "Subsection",
 CellChangeTimes->{{3.6069472766574965`*^9, 3.606947283201871*^9}, {
  3.612176098402361*^9, 3.6121761035386553`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "5", "]"}]], "Input",
 CellChangeTimes->{{3.612177253796446*^9, 3.612177254669496*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkTopology", "[", "n_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "F"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "F"}], "}"}], "=", 
      RowBox[{"CatmullClarkDefault", "[", "n", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"CatmullClarkPlot", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"V", ",", "F"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"V", ",", "F"}], "}"}], ",", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"n", "\[Equal]", "4"}], ",", "Green", ",", "Red"}], "]"}]}], 
      "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6069472964436283`*^9, 3.606947314898684*^9}, {
  3.606947363767479*^9, 3.606947369951833*^9}, {3.6070265610234427`*^9, 
  3.6070265613664627`*^9}}],

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{
   RowBox[{"CatmullClarkTopology", "[", "7", "]"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "100"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.606947318955916*^9, 3.6069473198269663`*^9}, 
   3.6069473723319693`*^9, {3.6069474340685005`*^9, 3.6069474599479804`*^9}, {
   3.606948344022547*^9, 3.6069483556052094`*^9}, {3.606948397549608*^9, 
   3.606948441315111*^9}, {3.607026431127013*^9, 3.607026540958295*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derivation of coefficients", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
   3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
   3.6056228546483173`*^9}, 3.605696312889595*^9, {3.60570271998906*^9, 
   3.60570272033908*^9}, {3.6057363923638954`*^9, 3.6057364076977725`*^9}, 
   3.6060117523985486`*^9, {3.6067159239480705`*^9, 3.6067159244350986`*^9}, {
   3.6067182744948053`*^9, 3.606718275012313*^9}, {3.612138185056046*^9, 
   3.6121381853740644`*^9}, 3.6122814883176093`*^9}],

Cell[CellGroupData[{

Cell["Valence 4 (general solution space)", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
  3.6056963600892944`*^9, 3.605696366944687*^9}, {3.607244876780635*^9, 
  3.607244889530364*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "4", "]"}]], "Input",
 CellChangeTimes->{3.6063887160510855`*^9, 3.6072482609651985`*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "4", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC4sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", 
        "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC4ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 
   3.6060116738330545`*^9}, {3.607248269427683*^9, 3.6072483625000067`*^9}, {
   3.6072484241485324`*^9, 3.607248539987158*^9}, {3.607248578192343*^9, 
   3.607248673744808*^9}, {3.60724874405583*^9, 3.6072487516682653`*^9}, {
   3.6072487986419525`*^9, 3.607248799635009*^9}, 3.6072490029476376`*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "4", "]"}], ",", "CC4sub", ",", 
   "CC4ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.6060117000325537`*^9, 3.6060117098881173`*^9}, {
   3.6060118342312293`*^9, 3.6060118374784145`*^9}, {3.60724829001186*^9, 
   3.607248291331936*^9}, 3.607248331109211*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.606507015219984*^9, 3.606507020668296*^9}, 
   3.6065108234578032`*^9, {3.6072490345094433`*^9, 3.6072490357255125`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm4", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"16", ",", "16", ",", "16"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.607248193969367*^9, {3.6072490370015855`*^9, 3.6072490438699784`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Rhs", "=", 
   RowBox[{"RHS", "[", 
    RowBox[{
    "GeneralForm4", ",", "GeneralForm4", ",", "CC4sub", ",", "CC4ind"}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607245545248869*^9, 3.6072455552644415`*^9}, {
  3.6072490656262226`*^9, 3.6072490711795406`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mir", "=", 
   RowBox[{"NestP", "[", 
    RowBox[{"GeneralForm4", ",", 
     RowBox[{"{", 
      RowBox[{
      "4", ",", "3", ",", "2", ",", "1", ",", "  ", "8", ",", "7", ",", "6", 
       ",", "5", ",", "  ", "12", ",", "11", ",", "10", ",", "9", ",", "  ", 
       "16", ",", "15", ",", "14", ",", "13"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rot", "=", 
   RowBox[{"NestP", "[", 
    RowBox[{"GeneralForm4", ",", 
     RowBox[{"{", 
      RowBox[{
      "4", ",", "8", ",", "12", ",", "16", ",", "  ", "3", ",", "7", ",", 
       "11", ",", "15", ",", "  ", "2", ",", "6", ",", "10", ",", "14", ",", 
       "  ", "1", ",", "5", ",", "9", ",", "13"}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.607245459898987*^9, 3.607245526803814*^9}, {
  3.6072490898826103`*^9, 3.607249145665801*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rot", "=", 
   RowBox[{"{", 
    RowBox[{
    "4", ",", "8", ",", "12", ",", "16", ",", "  ", "3", ",", "7", ",", "11", 
     ",", "15", ",", "  ", "2", ",", "6", ",", "10", ",", "14", ",", "  ", 
     "1", ",", "5", ",", "9", ",", "13"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"rng", "=", 
  RowBox[{"Range", "[", "16", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"rng", "=", 
  RowBox[{
  "rng", "\[LeftDoubleBracket]", "rot", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"rng", "=", 
  RowBox[{
  "rng", "\[LeftDoubleBracket]", "rot", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"rng", "=", 
  RowBox[{
  "rng", "\[LeftDoubleBracket]", "rot", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"rng", "=", 
  RowBox[{
  "rng", "\[LeftDoubleBracket]", "rot", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"rng", "\[Equal]", 
  RowBox[{"Range", "[", "16", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6072455744265375`*^9, 3.6072455847301273`*^9}, {
  3.6072491573074665`*^9, 3.60724916453788*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"BuildEssential", "[", "eqs_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"lst", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Normal", "[", 
          RowBox[{"CoefficientArrays", "[", 
           RowBox[{"#", ",", 
            RowBox[{"Flatten", "[", "GeneralForm4", "]"}]}], "]"}], "]"}], 
         "&"}], "/@", 
        RowBox[{"Flatten", "[", "eqs", "]"}]}]}], ",", 
      RowBox[{"out", "=", 
       RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"1", "<", 
          RowBox[{"Length", "[", "#", "]"}]}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"out", ",", 
           RowBox[{
           "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
          "]"}]}], "]"}], "&"}], "/@", "lst"}], ";", "out"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.607249253106946*^9, 3.6072492559211073`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{"GeneralForm4", "-", "Rhs"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine2", "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{"GeneralForm4", "+", "Mir"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine3", "=", 
   RowBox[{"BuildEssential", "[", 
    RowBox[{"GeneralForm4", "-", "Rot"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6072457915999594`*^9, 3.607245895920926*^9}, 
   3.6072460094634204`*^9, {3.607246105253899*^9, 3.607246113294359*^9}, {
   3.6072461772930193`*^9, 3.607246194550007*^9}}],

Cell[BoxData[
 RowBox[{"affine1", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.610298607629078*^9, 3.6102986084151225`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"NullSpace", "[", 
   RowBox[{"affine1", "//", "N"}], "]"}], "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6102986003506613`*^9, 3.6102986137834296`*^9}, {
  3.6102989419812016`*^9, 3.6102989424042253`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"NullSpace", "[", 
   RowBox[{
    RowBox[{"Join", "[", 
     RowBox[{"affine1", ",", "affine2"}], "]"}], "//", "N"}], "]"}], "//", 
  "Dimensions"}]], "Input",
 CellChangeTimes->{{3.610299756565793*^9, 3.6102997574448433`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"NullSpace", "[", 
   RowBox[{
    RowBox[{"Join", "[", 
     RowBox[{"affine1", ",", "affine3"}], "]"}], "//", "N"}], "]"}], "//", 
  "Dimensions"}]], "Input",
 CellChangeTimes->{{3.610299764749261*^9, 3.6102997647652617`*^9}}],

Cell[BoxData[
 RowBox[{"Export", "[", 
  RowBox[{"\"\<d://catmullclark_reg.mat\>\"", ",", 
   RowBox[{"affine1", "//", "N"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.610299787156543*^9, 3.6102997913727837`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"matrix", "=", 
   RowBox[{"Join", "[", 
    RowBox[{"affine1", ",", "affine2", ",", "affine3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"matrix", "//", "Dimensions"}]}], "Input"],

Cell[BoxData[
 RowBox[{"matrix", ">>", "\"\<findMyNullspace.mx\>\""}]], "Input",
 CellChangeTimes->{{3.607250438318736*^9, 3.6072504451721287`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sparseMatrix", "=", 
   RowBox[{"SparseArray", "[", "matrix", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6072505133080254`*^9, 3.607250538805484*^9}}],

Cell[BoxData[
 RowBox[{"sparseMatrix", ">>", "\"\<sparseMatrix.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6072507199478445`*^9, 3.6072507279563026`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"sparseMatrix", "=", 
   RowBox[{"<<", "\"\<sparseMatrix.mx\>\""}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6072507208868985`*^9, 3.6072507505135927`*^9}}],

Cell[BoxData[
 RowBox[{"MatrixPlot", "[", "sparseMatrix", "]"}]], "Input",
 CellChangeTimes->{{3.607250550944178*^9, 3.607250555572443*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm4Null", "=", 
   RowBox[{"NullSpace", "[", "sparseMatrix", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607250691259204*^9, 3.6072506923712673`*^9}, {
  3.6072507694596767`*^9, 3.6072507805073085`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm4Null", ">>", 
  "\"\<CatmullClarkForm4Null.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6072534211672564`*^9, 3.607253424377305*^9}}],

Cell[BoxData[{
 RowBox[{"Clear", "[", "CatmullClarkForm4", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4Null", "=", 
   RowBox[{"<<", "\"\<CatmullClarkForm4Null.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4Null", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6073256151590166`*^9, 3.6073256227991333`*^9}, {
  3.607325742573473*^9, 3.6073257429734793`*^9}, {3.6075464858914957`*^9, 
  3.607546486179512*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm4Free3", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        SubscriptBox["\[Alpha]", "1"], ",", 
        SubscriptBox["\[Alpha]", "2"], ",", 
        SubscriptBox["\[Alpha]", "3"]}], "}"}], ".", 
      RowBox[{"Normal", "[", "CatmullClarkForm4Null", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"16", ",", "16", ",", "16"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607253481136436*^9, 3.6072535127052417`*^9}, {
  3.6072535444205074`*^9, 3.607253563850806*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4Null", "=", 
   RowBox[{"NullSpace", "[", "matrix", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkForm4Null", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6072492878359327`*^9, 3.6072492894360237`*^9}}],

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"0", "\[Equal]", "#"}], "&"}], "/@", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Skew3", "[", 
     RowBox[{"CatmullClarkForm4Free3", "-", 
      RowBox[{"<<", "\"\<CatmullClarkForm4.mx\>\""}]}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6072536469609594`*^9, 3.607253651897242*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calibration of 1st DOF", "Subsection",
 CellChangeTimes->{{3.606378196577406*^9, 3.6063782068569937`*^9}, {
  3.6063791489238772`*^9, 3.606379149377903*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}], "=", "CubeMesh"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Vs", ",", "Fs"}], "}"}], "=", 
   RowBox[{"Nest", "[", 
    RowBox[{"QuadLinearSubdivide", ",", 
     RowBox[{"{", 
      RowBox[{"V", ",", "F"}], "}"}], ",", "3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Vn", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"V", "\[LeftDoubleBracket]", 
      RowBox[{
       RowBox[{"Sort", "[", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"dif", "=", 
              RowBox[{
               RowBox[{
               "Vs", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
               "-", 
               RowBox[{
               "V", "\[LeftDoubleBracket]", "j", 
                "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"N", "[", 
               RowBox[{"dif", ".", "dif"}], "]"}], ",", "j"}], "}"}]}], "]"}],
           ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", 
            RowBox[{"Length", "[", "V", "]"}]}], "}"}]}], "]"}], "]"}], 
       "\[LeftDoubleBracket]", 
       RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], 
      "\[RightDoubleBracket]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", "Vs", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Rasterize", "[", 
    RowBox[{
     RowBox[{"Show", "[", 
      RowBox[{
       RowBox[{"PlotMesh", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", "Vs"}], "+", 
            RowBox[{"\[Lambda]", " ", "Vn"}]}], ",", "Fs"}], "}"}], "/.", 
         RowBox[{"\[Lambda]", "\[Rule]", "#"}]}], "]"}], ",", 
       RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
       RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
       RowBox[{"Axes", "\[Rule]", "False"}], ",", 
       RowBox[{"ImageSize", "\[Rule]", "80"}]}], "]"}], ",", 
     RowBox[{"ImageResolution", "\[Rule]", "200"}]}], "]"}], "&"}], "/@", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0.4", ",", "0.7", ",", "1"}], "}"}]}]], "Input",
 CellChangeTimes->{{3.607383589899951*^9, 3.60738360219764*^9}, {
  3.6075499124854856`*^9, 3.6075499169737425`*^9}}],

Cell[BoxData[
 RowBox[{"Rasterize", "[", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"PlotMesh", "[", 
      RowBox[{"Nest", "[", 
       RowBox[{"CatmullClarkSubdivide", ",", 
        RowBox[{"{", 
         RowBox[{"Vn", ",", "Fs"}], "}"}], ",", "3"}], "]"}], "]"}], ",", 
     RowBox[{"Boxed", "\[Rule]", "False"}], ",", 
     RowBox[{"PlotRangePadding", "\[Rule]", "None"}], ",", 
     RowBox[{"Axes", "\[Rule]", "False"}], ",", 
     RowBox[{"ImageSize", "\[Rule]", "80"}]}], "]"}], ",", 
   RowBox[{"ImageResolution", "\[Rule]", "200"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6073380451999454`*^9, 3.607338049897517*^9}, {
   3.607338080058481*^9, 3.6073380922261677`*^9}, 3.6073381353918314`*^9, {
   3.607383652265909*^9, 3.6073837626176043`*^9}, {3.607549938298962*^9, 
   3.6075499669406004`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", "CatmullClarkForm4Free3"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm3", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["b", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"14", ",", "14", ",", "14"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6072538042939587`*^9, 3.607253827278273*^9}}],

Cell[BoxData[
 RowBox[{"LinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"volumeEqs", "=", 
  RowBox[{
   RowBox[{"CatmullClarkVolume", "[", 
    RowBox[{"{", 
     RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}], "\[Equal]", "1"}]}]], "Input",
 CellChangeTimes->{{3.607253784645835*^9, 3.607253790588175*^9}}],

Cell[BoxData[{
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{"volumeEqs", ",", 
     SubscriptBox["\[Alpha]", "3"]}], "]"}], "//", 
   "Simplify"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4Free2", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"CatmullClarkForm4Free3", "/.", 
     RowBox[{"sol", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6072538744809732`*^9, 3.6072538809433427`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{
    RowBox[{"CatmullClarkForm4Free2", "/.", 
     RowBox[{
      SubscriptBox["\[Alpha]", "1"], "\[Rule]", "0"}]}], "/.", 
    RowBox[{
     SubscriptBox["\[Alpha]", "2"], "\[Rule]", "0"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.607325799716851*^9, 3.60732582308471*^9}, {
  3.6073259680669365`*^9, 3.6073259767620707`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm4", ">>", "\"\<CatmullClarkAlt4.mx\>\""}]], "Input",
 CellChangeTimes->{{3.607326092018841*^9, 3.6073260935688643`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkForm4", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6072538925210047`*^9, 3.607253909231961*^9}, {
  3.607325831994847*^9, 3.6073258518601522`*^9}, {3.607325950349165*^9, 
  3.6073259815946445`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Valence 3", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
  3.6056963600892944`*^9, 3.605696366944687*^9}, {3.60732669615062*^9, 
  3.6073267037482367`*^9}, {3.6121771683555593`*^9, 3.6121771686265745`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513664109279*^9, 3.6065136676054792`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "3", "]"}]], "Input",
 CellChangeTimes->{3.6063887160510855`*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "3", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC3sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "1", "1", "6"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "1", "0", "1"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "1", "6", "6", "0", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "1", "1", "6"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "1", "0", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC3ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605739358486548*^9, 3.6057393808738284`*^9}, {
   3.605739542501073*^9, 3.6057396306311135`*^9}, {3.6057793945372667`*^9, 
   3.605779399264537*^9}, 3.6059089051377163`*^9, {3.6059089731156044`*^9, 
   3.605908973995655*^9}, {3.605949314763294*^9, 3.605949315019309*^9}, {
   3.605949354872588*^9, 3.6059494330120573`*^9}, {3.605949614851458*^9, 
   3.6059496837633996`*^9}, {3.6060116631534443`*^9, 3.6060116738330545`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "3", "]"}], ",", "CC3sub", ",", 
   "CC3ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.6060117000325537`*^9, 3.6060117098881173`*^9}, {
  3.6060118342312293`*^9, 3.6060118374784145`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SUB", "=", 
   RowBox[{"CC3sub", "\[LeftDoubleBracket]", 
    RowBox[{"CC3ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
    "\[RightDoubleBracket]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SUB", "//", "Transpose"}], "//", "MatrixForm"}]}], "Input",
 CellChangeTimes->{{3.6076218663303285`*^9, 3.6076218683934464`*^9}}],

Cell[BoxData[
 RowBox[{"Eigensystem", "[", 
  RowBox[{"SUB", "//", "Transpose"}], "]"}]], "Input",
 CellChangeTimes->{{3.60762187299771*^9, 3.6076218795150824`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.6056459833952055`*^9, 3.6056459893175445`*^9}, {
  3.6056964913278008`*^9, 3.6056965130550437`*^9}, {3.6065136729657855`*^9, 
  3.606513674477872*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm3", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"14", ",", "14", ",", "14"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6056233521867743`*^9, 3.6056233902269506`*^9}, 
   3.605623495750986*^9, {3.6056799017582197`*^9, 3.605679914208932*^9}, 
   3.605679976213478*^9, {3.605680025240282*^9, 3.605680040530157*^9}, {
   3.6056800958383207`*^9, 3.605680185222433*^9}, 3.605680249966136*^9, {
   3.605680280892905*^9, 3.6056803189730835`*^9}, {3.6056863919401493`*^9, 
   3.605686399854602*^9}, {3.605686817407484*^9, 3.605686819028577*^9}, {
   3.605737935670168*^9, 3.6057379758284645`*^9}, {3.605738388115046*^9, 
   3.6057383908132*^9}, {3.6062492501646576`*^9, 3.6062492552029457`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Flatten", "[", "GeneralForm3", "]"}]}], "]"}], "]"}], "&"}],
      "/@", 
     RowBox[{"Flatten", "[", 
      RowBox[{"GeneralForm3", "-", 
       RowBox[{"RHS", "[", 
        RowBox[{
        "CatmullClarkForm4", ",", "GeneralForm3", ",", "CC3sub", ",", 
         "CC3ind"}], "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//",
   "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6062493238898745`*^9, 3.6062493511444335`*^9}, 
   3.6064276017059317`*^9, {3.6064625607022676`*^9, 3.6064625627183833`*^9}, {
   3.607326011290101*^9, 3.6073260218277626`*^9}, {3.6073271116500025`*^9, 
   3.6073271131975265`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"NullSpace", "[", 
       RowBox[{
       "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       "]"}], "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], ",", 
     RowBox[{"{", 
      RowBox[{"14", ",", "14", ",", "14"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Print", "[", 
    RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
  "nul"}]}], "Input",
 CellChangeTimes->{3.607622267097251*^9}],

Cell[BoxData[
 RowBox[{"ArrayPlot", "[", 
  RowBox[{"Unitize", "[", 
   RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6064654675505304`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mirror3", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "12", ",", "4", ",", "8"}], 
    "}"}], "\[LeftDoubleBracket]", "n", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"Mirror3", ",", "Listable"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MirrorIndex3", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{"Mirror3", "/@", 
   RowBox[{"{", 
    RowBox[{"i", ",", "k", ",", "j"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MapMulti3", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "14", " ", "14"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"j", "-", "1"}], ")"}], "14"}], "+", 
   "i"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mirror3", "[", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "12", ",", "4", ",", "8"}], 
    "}"}], "]"}], "\[Equal]", 
  RowBox[{"Range", "[", "14", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064597960761404`*^9, 3.606459981197729*^9}, {
   3.6064600503186827`*^9, 3.606460053797881*^9}, {3.6064648600587835`*^9, 
   3.6064648602527947`*^9}, 3.606464912318773*^9, {3.606465074482048*^9, 
   3.6064650746660585`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"compress3", ",", "inflate3"}], "}"}], "=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"hash", ",", 
       RowBox[{"n", "=", "14"}], ",", 
       RowBox[{"list", "=", 
        RowBox[{"{", "}"}]}], ",", "key", ",", "value", ",", "mir", ",", 
       RowBox[{"index", "=", 
        RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"key", "=", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j", ",", "k"}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"value", "=", 
          RowBox[{"HashGet", "[", 
           RowBox[{"hash", ",", "key"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"value", "===", "Null"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mir", "=", 
             RowBox[{"MirrorIndex3", "[", "key", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"list", ",", 
              RowBox[{"MapMulti3", "/@", 
               RowBox[{"Union", "[", 
                RowBox[{"{", 
                 RowBox[{"key", ",", "mir"}], "}"}], "]"}]}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "key", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "mir", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"index", ",", 
           RowBox[{"hash", "[", "key", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"list", ",", "index"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compress3", "[", "mat_", "]"}], ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"Transpose", "[", 
       RowBox[{"mat", "\[LeftDoubleBracket]", 
        RowBox[{"All", ",", "#"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
     "&"}], "/@", "compress3"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064607678477225`*^9, 3.606460818710632*^9}, {
  3.606464024319982*^9, 3.606464039159831*^9}, {3.606464975580391*^9, 
  3.6064650084722724`*^9}, {3.6064650852146616`*^9, 3.606465085394672*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
     "#", "\[LeftDoubleBracket]", "inflate3", "\[RightDoubleBracket]"}], 
     "&"}], "/@", 
    RowBox[{"NullSpace", "[", 
     RowBox[{"Compress3", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606462536417879*^9, 3.60646255024967*^9}, 
   3.606464157033573*^9, {3.6064652563384495`*^9, 3.6064652608327065`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Skew3", "[", 
    RowBox[{"ArrayReshape", "[", 
     RowBox[{"nul", ",", 
      RowBox[{"Dimensions", "[", "GeneralForm3", "]"}]}], "]"}], "]"}], "//", 
   "Flatten"}], "//", "Union"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{
     RowBox[{"Compress3", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], ",", 
     RowBox[{"-", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6064653638155966`*^9, 3.606465412204365*^9}, {
  3.607326051828223*^9, 3.6073260611558666`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm3", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
     "sol", "\[LeftDoubleBracket]", "inflate3", "\[RightDoubleBracket]"}], 
     ",", 
     RowBox[{"Dimensions", "[", "GeneralForm3", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6064654316644773`*^9, 3.606465438503869*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm3", ">>", "\"\<CatmullClarkAlt3.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6057402733758764`*^9, 3.6057402754249935`*^9}, {
  3.6073260791786437`*^9, 3.60732608026616*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CatmullClarkForm3", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt3.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm3Null", "=", 
   RowBox[{"<<", "\"\<CatmullClarkForm3Null.mx\>\""}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm3", "=", 
   RowBox[{"CatmullClarkForm3", "+", 
    RowBox[{
     SubscriptBox["\[Alpha]", "1"], " ", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"CatmullClarkForm3Null", ",", 
       RowBox[{"Dimensions", "[", "GeneralForm3", "]"}]}], "]"}]}]}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6064277082970285`*^9, 3.606427750544445*^9}, {
  3.6064328283458786`*^9, 3.606432841709643*^9}, {3.6073261071965737`*^9, 
  3.607326108109088*^9}}],

Cell[BoxData[
 RowBox[{"Variables", "[", "CatmullClarkForm3", "]"}]], "Input",
 CellChangeTimes->{{3.6064277380887327`*^9, 3.6064277562977743`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{"CatmullClarkForm3", "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6057402829214225`*^9, {3.6059498206562295`*^9, 3.6059498230243645`*^9}, {
   3.605949889776183*^9, 3.605949893626403*^9}, {3.6064277639782133`*^9, 
   3.606427784345378*^9}, {3.606465580035964*^9, 3.606465580717003*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 5", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, 3.605780216264267*^9, {
   3.607327099719819*^9, 3.607327100142326*^9}, {3.612177169346616*^9, 
   3.6121771695146255`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513637606763*^9, 3.6065136422620296`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "5", "]"}]], "Input",
 CellChangeTimes->{3.6063887372823*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "5", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC5sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1", "6", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "0", "1", 
        "6", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC5ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27"}], 
      "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.606012291208367*^9, 3.6060123076683083`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "5", "]"}], ",", "CC5sub", ",", 
   "CC5ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.6060117000325537`*^9, 3.6060117098881173`*^9}, {
  3.6060118342312293`*^9, 3.6060118374784145`*^9}, {3.606012345337463*^9, 
  3.606012349153681*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sys", "=", 
   RowBox[{"Eigensystem", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"CC5sub", "\[LeftDoubleBracket]", 
       RowBox[{
       "CC5ind", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}], 
       "\[RightDoubleBracket]"}], "//", "N"}], "//", "Transpose"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"sys", "\[LeftDoubleBracket]", "1", 
  "\[RightDoubleBracket]"}]}], "Input",
 CellChangeTimes->{{3.607681425975233*^9, 3.607681499958465*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.6056459833952055`*^9, 3.6056459893175445`*^9}, {
  3.6056964913278008`*^9, 3.6056965130550437`*^9}, {3.6065136548307486`*^9, 
  3.6065136567018557`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm5", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"18", ",", "18", ",", "18"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6056233521867743`*^9, 3.6056233902269506`*^9}, 
   3.605623495750986*^9, {3.6056799017582197`*^9, 3.605679914208932*^9}, 
   3.605679976213478*^9, {3.605680025240282*^9, 3.605680040530157*^9}, {
   3.6056800958383207`*^9, 3.605680185222433*^9}, 3.605680249966136*^9, {
   3.605680280892905*^9, 3.6056803189730835`*^9}, {3.6056863919401493`*^9, 
   3.605686399854602*^9}, {3.605686817407484*^9, 3.605686819028577*^9}, {
   3.605737935670168*^9, 3.6057379758284645`*^9}, {3.605738388115046*^9, 
   3.6057383908132*^9}, {3.6062492501646576`*^9, 3.6062492552029457`*^9}, {
   3.606465618518165*^9, 3.60646562734867*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Flatten", "[", "GeneralForm5", "]"}]}], "]"}], "]"}], "&"}],
      "/@", 
     RowBox[{"Flatten", "[", 
      RowBox[{"GeneralForm5", "-", 
       RowBox[{"RHS", "[", 
        RowBox[{
        "CatmullClarkForm4", ",", "GeneralForm5", ",", "CC5sub", ",", 
         "CC5ind"}], "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//",
   "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6062493238898745`*^9, 3.6062493511444335`*^9}, 
   3.6064276017059317`*^9, {3.6064625607022676`*^9, 3.6064625627183833`*^9}, {
   3.606465632122943*^9, 3.6064656463857594`*^9}, {3.6073267247785597`*^9, 
   3.607326728828622*^9}, {3.607327124812705*^9, 3.6073271259127216`*^9}}],

Cell[BoxData[
 RowBox[{"ArrayPlot", "[", 
  RowBox[{"Unitize", "[", 
   RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6064654675505304`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mirror5", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "18", ",", "17", ",", "16"}], "}"}], "\[LeftDoubleBracket]", 
   "n", "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"Mirror5", ",", "Listable"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MirrorIndex5", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{"Mirror5", "/@", 
   RowBox[{"{", 
    RowBox[{"i", ",", "k", ",", "j"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MapMulti5", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "18", " ", "18"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"j", "-", "1"}], ")"}], "18"}], "+", 
   "i"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mirror5", "[", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "18", ",", "17", ",", "16"}], "}"}], "]"}], "\[Equal]", 
  RowBox[{"Range", "[", "18", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064597960761404`*^9, 3.606459981197729*^9}, {
   3.6064600503186827`*^9, 3.606460053797881*^9}, {3.6064648600587835`*^9, 
   3.6064648602527947`*^9}, 3.606464912318773*^9, {3.606465074482048*^9, 
   3.6064650746660585`*^9}, {3.6064657433613057`*^9, 3.606465765479571*^9}, {
   3.606465797968429*^9, 3.6064659030854416`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"compress5", ",", "inflate5"}], "}"}], "=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"hash", ",", 
       RowBox[{"n", "=", "18"}], ",", 
       RowBox[{"list", "=", 
        RowBox[{"{", "}"}]}], ",", "key", ",", "value", ",", "mir", ",", 
       RowBox[{"index", "=", 
        RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"key", "=", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j", ",", "k"}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"value", "=", 
          RowBox[{"HashGet", "[", 
           RowBox[{"hash", ",", "key"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"value", "===", "Null"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mir", "=", 
             RowBox[{"MirrorIndex5", "[", "key", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"list", ",", 
              RowBox[{"MapMulti5", "/@", 
               RowBox[{"Union", "[", 
                RowBox[{"{", 
                 RowBox[{"key", ",", "mir"}], "}"}], "]"}]}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "key", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "mir", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"index", ",", 
           RowBox[{"hash", "[", "key", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"list", ",", "index"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compress5", "[", "mat_", "]"}], ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"Transpose", "[", 
       RowBox[{"mat", "\[LeftDoubleBracket]", 
        RowBox[{"All", ",", "#"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
     "&"}], "/@", "compress5"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064607678477225`*^9, 3.606460818710632*^9}, {
  3.606464024319982*^9, 3.606464039159831*^9}, {3.606464975580391*^9, 
  3.6064650084722724`*^9}, {3.6064650852146616`*^9, 3.606465085394672*^9}, {
  3.606465910809883*^9, 3.60646593453024*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
     "#", "\[LeftDoubleBracket]", "inflate5", "\[RightDoubleBracket]"}], 
     "&"}], "/@", 
    RowBox[{"NullSpace", "[", 
     RowBox[{"Compress5", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606462536417879*^9, 3.60646255024967*^9}, 
   3.606464157033573*^9, {3.6064652563384495`*^9, 3.6064652608327065`*^9}, {
   3.6064659418686595`*^9, 3.6064659444858093`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nul", "//", "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.606466086045906*^9, 3.6064660901051383`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Skew3", "[", 
    RowBox[{"ArrayReshape", "[", 
     RowBox[{"nul", ",", 
      RowBox[{"Dimensions", "[", "GeneralForm5", "]"}]}], "]"}], "]"}], "//", 
   "Flatten"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.606465964967981*^9, 3.606465965175993*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{
     RowBox[{"Compress5", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], ",", 
     RowBox[{"-", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6064653638155966`*^9, 3.606465412204365*^9}, 
   3.606466099523677*^9, {3.607326749698942*^9, 3.6073267594440923`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm5", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
     "sol", "\[LeftDoubleBracket]", "inflate5", "\[RightDoubleBracket]"}], 
     ",", 
     RowBox[{"Dimensions", "[", "GeneralForm5", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6064654316644773`*^9, 3.606465438503869*^9}, {
  3.606466122622998*^9, 3.6064661267432337`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm5", ">>", "\"\<CatmullClarkAlt5.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6057402733758764`*^9, 3.6057402754249935`*^9}, {
  3.6064661464733624`*^9, 3.606466148232463*^9}, {3.607326777474369*^9, 
  3.6073267782043805`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{"CatmullClarkForm5", "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6057402829214225`*^9, {3.6059498206562295`*^9, 3.6059498230243645`*^9}, {
   3.605949889776183*^9, 3.605949893626403*^9}, {3.6064277639782133`*^9, 
   3.606427784345378*^9}, {3.606465580035964*^9, 3.606465580717003*^9}, 
   3.6064661574499903`*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 6", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, 3.605780216264267*^9, 
   3.6057832882509747`*^9, {3.605783521695327*^9, 3.6057835245754914`*^9}, {
   3.605823463641165*^9, 3.605823494663939*^9}, {3.605852564805275*^9, 
   3.605852569810561*^9}, 3.6064734561420703`*^9, {3.607327102519862*^9, 
   3.6073271029998693`*^9}, {3.6121771703306723`*^9, 3.612177170482681*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513637606763*^9, 3.6065136422620296`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "6", "]"}]], "Input",
 CellChangeTimes->{3.6063887648108745`*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "6", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC6sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1", "6", "1", "6", "1", "6",
         "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "0", "1", 
        "6", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "1", "1", "6", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "0", "0", "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC6ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
       ",", "28", ",", "29"}], "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6060143588756304`*^9, 3.606014372372402*^9}, 
   3.6065132271372857`*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "6", "]"}], ",", "CC6sub", ",", 
   "CC6ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.606014398075873*^9, 3.6060144003800044`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.6056459833952055`*^9, 3.6056459893175445`*^9}, {
  3.6056964913278008`*^9, 3.6056965130550437`*^9}, {3.6065132359387894`*^9, 
  3.606513237200861*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm6", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"20", ",", "20", ",", "20"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6056233521867743`*^9, 3.6056233902269506`*^9}, 
   3.605623495750986*^9, {3.6056799017582197`*^9, 3.605679914208932*^9}, 
   3.605679976213478*^9, {3.605680025240282*^9, 3.605680040530157*^9}, {
   3.6056800958383207`*^9, 3.605680185222433*^9}, 3.605680249966136*^9, {
   3.605680280892905*^9, 3.6056803189730835`*^9}, {3.6056863919401493`*^9, 
   3.605686399854602*^9}, {3.605686817407484*^9, 3.605686819028577*^9}, {
   3.605737935670168*^9, 3.6057379758284645`*^9}, {3.605738388115046*^9, 
   3.6057383908132*^9}, {3.6062492501646576`*^9, 3.6062492552029457`*^9}, {
   3.606465618518165*^9, 3.60646562734867*^9}, {3.6064734862825336`*^9, 
   3.6064734921776237`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Flatten", "[", "GeneralForm6", "]"}]}], "]"}], "]"}], "&"}],
      "/@", 
     RowBox[{"Flatten", "[", 
      RowBox[{"GeneralForm6", "-", 
       RowBox[{"RHS", "[", 
        RowBox[{
        "CatmullClarkForm4", ",", "GeneralForm6", ",", "CC6sub", ",", 
         "CC6ind"}], "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//",
   "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6062493238898745`*^9, 3.6062493511444335`*^9}, 
   3.6064276017059317`*^9, {3.6064625607022676`*^9, 3.6064625627183833`*^9}, {
   3.606465632122943*^9, 3.6064656463857594`*^9}, {3.6064734982377167`*^9, 
   3.6064735066553464`*^9}, {3.6065131372851467`*^9, 3.606513140180312*^9}}],

Cell[BoxData[
 RowBox[{"ArrayPlot", "[", 
  RowBox[{"Unitize", "[", 
   RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6064654675505304`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mirror6", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "20", ",", "19", ",", "18", ",", "17", ",", "16"}], "}"}], 
   "\[LeftDoubleBracket]", "n", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"Mirror6", ",", "Listable"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MirrorIndex6", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{"Mirror6", "/@", 
   RowBox[{"{", 
    RowBox[{"i", ",", "k", ",", "j"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MapMulti6", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "20", " ", "20"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"j", "-", "1"}], ")"}], "20"}], "+", 
   "i"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mirror6", "[", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "20", ",", "19", ",", "18", ",", "17", ",", "16"}], "}"}], 
   "]"}], "\[Equal]", 
  RowBox[{"Range", "[", "20", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064597960761404`*^9, 3.606459981197729*^9}, {
   3.6064600503186827`*^9, 3.606460053797881*^9}, {3.6064648600587835`*^9, 
   3.6064648602527947`*^9}, 3.606464912318773*^9, {3.606465074482048*^9, 
   3.6064650746660585`*^9}, {3.6064657433613057`*^9, 3.606465765479571*^9}, {
   3.606465797968429*^9, 3.6064659030854416`*^9}, 3.60647352255309*^9, {
   3.606473589631621*^9, 3.6064735968167315`*^9}, 3.6064736384523706`*^9, {
   3.606473675910446*^9, 3.606473676885461*^9}, {3.606473716861075*^9, 
   3.606473757814204*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"compress6", ",", "inflate6"}], "}"}], "=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"hash", ",", 
       RowBox[{"n", "=", "20"}], ",", 
       RowBox[{"list", "=", 
        RowBox[{"{", "}"}]}], ",", "key", ",", "value", ",", "mir", ",", 
       RowBox[{"index", "=", 
        RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"key", "=", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j", ",", "k"}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"value", "=", 
          RowBox[{"HashGet", "[", 
           RowBox[{"hash", ",", "key"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"value", "===", "Null"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mir", "=", 
             RowBox[{"MirrorIndex6", "[", "key", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"list", ",", 
              RowBox[{"MapMulti6", "/@", 
               RowBox[{"Union", "[", 
                RowBox[{"{", 
                 RowBox[{"key", ",", "mir"}], "}"}], "]"}]}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "key", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "mir", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"index", ",", 
           RowBox[{"hash", "[", "key", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"list", ",", "index"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compress6", "[", "mat_", "]"}], ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"Transpose", "[", 
       RowBox[{"mat", "\[LeftDoubleBracket]", 
        RowBox[{"All", ",", "#"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
     "&"}], "/@", "compress6"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064607678477225`*^9, 3.606460818710632*^9}, {
  3.606464024319982*^9, 3.606464039159831*^9}, {3.606464975580391*^9, 
  3.6064650084722724`*^9}, {3.6064650852146616`*^9, 3.606465085394672*^9}, {
  3.606465910809883*^9, 3.60646593453024*^9}, {3.6064737675993547`*^9, 
  3.606473802734894*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
     "#", "\[LeftDoubleBracket]", "inflate6", "\[RightDoubleBracket]"}], 
     "&"}], "/@", 
    RowBox[{"NullSpace", "[", 
     RowBox[{
      RowBox[{"Compress6", "[", 
       RowBox[{
       "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       "]"}], "//", "N"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606462536417879*^9, 3.60646255024967*^9}, 
   3.606464157033573*^9, {3.6064652563384495`*^9, 3.6064652608327065`*^9}, {
   3.6064659418686595`*^9, 3.6064659444858093`*^9}, {3.6065132923130136`*^9, 
   3.606513294313128*^9}, {3.6065133447960157`*^9, 3.6065133455550585`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Skew3", "[", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"nul", ",", 
       RowBox[{"Dimensions", "[", "GeneralForm6", "]"}]}], "]"}], "]"}], "//",
     "Flatten"}], "//", "Chop"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.606465964967981*^9, 3.606465965175993*^9}, 
   3.6065132971052876`*^9, {3.6065134460108047`*^9, 3.606513446755847*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{
     RowBox[{"Compress6", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], ",", 
     RowBox[{"-", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6064653638155966`*^9, 3.606465412204365*^9}, 
   3.606466099523677*^9, 3.6064738231527076`*^9, {3.606513149773861*^9, 
   3.6065131642766905`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm6", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
     "sol", "\[LeftDoubleBracket]", "inflate6", "\[RightDoubleBracket]"}], 
     ",", 
     RowBox[{"Dimensions", "[", "GeneralForm6", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6064654316644773`*^9, 3.606465438503869*^9}, {
  3.606466122622998*^9, 3.6064661267432337`*^9}, {3.6064738756685143`*^9, 
  3.6064738814936037`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm6", ">>", "\"\<CatmullClarkAlt6.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6057402733758764`*^9, 3.6057402754249935`*^9}, {
  3.6064661464733624`*^9, 3.606466148232463*^9}, {3.6064738839661417`*^9, 
  3.606473886253677*^9}, {3.6073269067138543`*^9, 3.607326907401365*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkForm6", "-", 
    RowBox[{"RHS", "[", 
     RowBox[{
     "CatmullClarkForm4", ",", "CatmullClarkForm6", ",", "CC6sub", ",", 
      "CC6ind"}], "]"}]}], "]"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.6064740505512004`*^9, 3.606474056083785*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkForm6", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6057402829214225`*^9, {3.6059498206562295`*^9, 3.6059498230243645`*^9}, {
   3.605949889776183*^9, 3.605949893626403*^9}, {3.6064277639782133`*^9, 
   3.606427784345378*^9}, {3.606465580035964*^9, 3.606465580717003*^9}, 
   3.6064661574499903`*^9, 3.60647389100375*^9, {3.6065135495307255`*^9, 
   3.6065135534979525`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 7", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, 3.605780216264267*^9, 
   3.6057832882509747`*^9, {3.605783521695327*^9, 3.6057835245754914`*^9}, {
   3.605823463641165*^9, 3.605823494663939*^9}, {3.605852564805275*^9, 
   3.605852569810561*^9}, 3.6064734561420703`*^9, {3.606513815116916*^9, 
   3.6065138165960007`*^9}, 3.6066342120892487`*^9, {3.6073279282310452`*^9, 
   3.6073279285860505`*^9}, {3.6121771713707314`*^9, 3.6121771715307407`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513637606763*^9, 3.6065136422620296`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "7", "]"}]], "Input",
 CellChangeTimes->{3.6063887648108745`*^9, 3.6065138233523874`*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "7", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC7sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1", "6", "1", "6", "1", "6",
         "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "0", "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "0", "1", 
        "6", "1", "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "1", "1", "6", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "1", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "0", "0", "1", "1", "6", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "0", "0", "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "0", "0", "0", "0", "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC7ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
       ",", "28", ",", "29", ",", "30", ",", "31"}], "}"}]}], "}"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6060143588756304`*^9, 3.606014372372402*^9}, 
   3.6065132271372857`*^9, {3.6065138266415753`*^9, 3.606513899600748*^9}, {
   3.6065139325216312`*^9, 3.60651394752549*^9}, {3.6065140091940165`*^9, 
   3.6065140745837564`*^9}, 3.6065141796917686`*^9, 3.6065142207361164`*^9, 
   3.6065142609524164`*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "7", "]"}], ",", "CC7sub", ",", 
   "CC7ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.606014398075873*^9, 3.6060144003800044`*^9}, {
  3.6065140818141704`*^9, 3.606514086165419*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Subsubsection",
 CellChangeTimes->{{3.6056459833952055`*^9, 3.6056459893175445`*^9}, {
  3.6056964913278008`*^9, 3.6056965130550437`*^9}, {3.6065132359387894`*^9, 
  3.606513237200861*^9}}],

Cell[BoxData[{
 RowBox[{"Clear", "[", "CatmullClarkForm4", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.607327358746298*^9, 3.6073273713614917`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm7", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"22", ",", "22", ",", "22"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6056233521867743`*^9, 3.6056233902269506`*^9}, 
   3.605623495750986*^9, {3.6056799017582197`*^9, 3.605679914208932*^9}, 
   3.605679976213478*^9, {3.605680025240282*^9, 3.605680040530157*^9}, {
   3.6056800958383207`*^9, 3.605680185222433*^9}, 3.605680249966136*^9, {
   3.605680280892905*^9, 3.6056803189730835`*^9}, {3.6056863919401493`*^9, 
   3.605686399854602*^9}, {3.605686817407484*^9, 3.605686819028577*^9}, {
   3.605737935670168*^9, 3.6057379758284645`*^9}, {3.605738388115046*^9, 
   3.6057383908132*^9}, {3.6062492501646576`*^9, 3.6062492552029457`*^9}, {
   3.606465618518165*^9, 3.60646562734867*^9}, {3.6064734862825336`*^9, 
   3.6064734921776237`*^9}, {3.6065143291333165`*^9, 3.6065143349556494`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Flatten", "[", "GeneralForm7", "]"}]}], "]"}], "]"}], "&"}],
      "/@", 
     RowBox[{"Flatten", "[", 
      RowBox[{"GeneralForm7", "-", 
       RowBox[{"RHS", "[", 
        RowBox[{
        "CatmullClarkForm4", ",", "GeneralForm7", ",", "CC7sub", ",", 
         "CC7ind"}], "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//",
   "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6062493238898745`*^9, 3.6062493511444335`*^9}, 
   3.6064276017059317`*^9, {3.6064625607022676`*^9, 3.6064625627183833`*^9}, {
   3.606465632122943*^9, 3.6064656463857594`*^9}, {3.6064734982377167`*^9, 
   3.6064735066553464`*^9}, {3.6065131372851467`*^9, 3.606513140180312*^9}, {
   3.606514339110887*^9, 3.6065143461652904`*^9}}],

Cell[BoxData[
 RowBox[{"ArrayPlot", "[", 
  RowBox[{"Unitize", "[", 
   RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6064654675505304`*^9}],

Cell[BoxData[
 GraphicsBox[{{
    {GrayLevel[0.5], 
     LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
     LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
     LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
     LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
     LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
     LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
     LineBox[{{-2, 0}, {-1, 0}, {-1, 1}, {-2, 1}, {-2, 0}}], 
     LineBox[{{0, 0}, {0, 1}, {-1, 1}, {-1, 0}, {0, 0}}], 
     LineBox[NCache[{{0, 0}, {
        Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}, {
        2^Rational[1, 2] Sin[Rational[1, 16] Pi], 2^Rational[1, 2] 
         Cos[Rational[1, 16] Pi]}, {0, 1}, {0, 0}}, {{0, 0}, {
        0.3826834323650898, 0.9238795325112867}, {0.275899379282943, 
        1.3870398453221475`}, {0, 1}, {0, 0}}]], 
     LineBox[NCache[{{0, 0}, {2^Rational[-1, 2], 2^Rational[-1, 2]}, {
        2^Rational[1, 2] Sin[Rational[3, 16] Pi], 2^Rational[1, 2] 
         Cos[Rational[3, 16] Pi]}, {
        Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}, {0, 0}}, {{0, 0}, {
        0.7071067811865475, 0.7071067811865475}, {0.7856949583871021, 
        1.1758756024193588`}, {0.3826834323650898, 0.9238795325112867}, {0, 
        0}}]], LineBox[
      NCache[{{0, 0}, {Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}, {
        2^Rational[1, 2] Cos[Rational[3, 16] Pi], 2^Rational[1, 2] 
         Sin[Rational[3, 16] Pi]}, {2^Rational[-1, 2], 2^Rational[-1, 2]}, {0,
         0}}, {{0, 0}, {0.9238795325112867, 0.3826834323650898}, {
        1.1758756024193588`, 0.7856949583871021}, {0.7071067811865475, 
        0.7071067811865475}, {0, 0}}]], 
     LineBox[NCache[{{0, 0}, {1, 0}, {
        2^Rational[1, 2] Cos[Rational[1, 16] Pi], 2^Rational[1, 2] 
         Sin[Rational[1, 16] Pi]}, {
        Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}, {0, 0}}, {{0, 0}, {1,
         0}, {1.3870398453221475`, 0.275899379282943}, {0.9238795325112867, 
        0.3826834323650898}, {0, 0}}]]}, {
     {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}}], 
      PolygonBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}}], 
      PolygonBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}}], 
      PolygonBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}}], 
      PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}], 
      PolygonBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}}], 
      PolygonBox[{{-2, 0}, {-1, 0}, {-1, 1}, {-2, 1}}], 
      PolygonBox[{{0, 0}, {0, 1}, {-1, 1}, {-1, 0}}], 
      PolygonBox[
       NCache[{{0, 0}, {Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}, {
         2^Rational[1, 2] Sin[Rational[1, 16] Pi], 2^Rational[1, 2] 
          Cos[Rational[1, 16] Pi]}, {0, 1}}, {{0, 0}, {0.3826834323650898, 
         0.9238795325112867}, {0.275899379282943, 1.3870398453221475`}, {0, 
         1}}]], PolygonBox[
       NCache[{{0, 0}, {2^Rational[-1, 2], 2^Rational[-1, 2]}, {
         2^Rational[1, 2] Sin[Rational[3, 16] Pi], 2^Rational[1, 2] 
          Cos[Rational[3, 16] Pi]}, {
         Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}}, {{0, 0}, {
         0.7071067811865475, 0.7071067811865475}, {0.7856949583871021, 
         1.1758756024193588`}, {0.3826834323650898, 0.9238795325112867}}]], 
      PolygonBox[
       NCache[{{0, 0}, {Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}, {
         2^Rational[1, 2] Cos[Rational[3, 16] Pi], 2^Rational[1, 2] 
          Sin[Rational[3, 16] Pi]}, {
         2^Rational[-1, 2], 2^Rational[-1, 2]}}, {{0, 0}, {0.9238795325112867,
          0.3826834323650898}, {1.1758756024193588`, 0.7856949583871021}, {
         0.7071067811865475, 0.7071067811865475}}]], 
      PolygonBox[
       NCache[{{0, 0}, {1, 0}, {
         2^Rational[1, 2] Cos[Rational[1, 16] Pi], 2^Rational[1, 2] 
          Sin[Rational[1, 16] Pi]}, {
         Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}}, {{0, 0}, {1, 0}, {
         1.3870398453221475`, 0.275899379282943}, {0.9238795325112867, 
         0.3826834323650898}}]]}, 
     {RGBColor[0, 1, 0], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}]}}, 
    {RGBColor[0, 1, 0], Opacity[0.5], 
     PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}]}, 
    {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], DiskBox[{-2, -2}, 0.04], 
     DiskBox[{-1, -2}, 0.04], DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
     DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], DiskBox[{0, -1}, 0.04],
      DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04], DiskBox[{-1, 0}, 0.04], 
     DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], DiskBox[{-2, 1}, 0.04], 
     DiskBox[{-1, 1}, 0.04], DiskBox[{0, 1}, 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Sin[Rational[1, 16] Pi], 2^Rational[1, 2] 
        Cos[Rational[1, 16] Pi]}, {0.275899379282943, 1.3870398453221475`}], 
      0.04], DiskBox[
      NCache[{Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}, {
       0.3826834323650898, 0.9238795325112867}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Sin[Rational[3, 16] Pi], 2^Rational[1, 2] 
        Cos[Rational[3, 16] Pi]}, {0.7856949583871021, 1.1758756024193588`}], 
      0.04], DiskBox[
      NCache[{2^Rational[-1, 2], 2^Rational[-1, 2]}, {0.7071067811865475, 
       0.7071067811865475}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Cos[Rational[3, 16] Pi], 2^Rational[1, 2] 
        Sin[Rational[3, 16] Pi]}, {1.1758756024193588`, 0.7856949583871021}], 
      0.04], DiskBox[
      NCache[{Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}, {
       0.9238795325112867, 0.3826834323650898}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Cos[Rational[1, 16] Pi], 2^Rational[1, 2] 
        Sin[Rational[1, 16] Pi]}, {1.3870398453221475`, 0.275899379282943}], 
      0.04]}}, {InsetBox[
     StyleBox["1",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["2",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["3",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["4",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["5",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["6",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["7",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["8",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["9",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["10",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["11",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["12",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["13",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["14",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["15",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["16",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Sin[Rational[1, 16] Pi], 2^Rational[1, 2] 
       Cos[Rational[1, 16] Pi]}, {0.275899379282943, 1.3870398453221475`}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["17",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{Sin[Rational[1, 8] Pi], Cos[Rational[1, 8] Pi]}, {
      0.3826834323650898, 0.9238795325112867}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["18",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Sin[Rational[3, 16] Pi], 2^Rational[1, 2] 
       Cos[Rational[3, 16] Pi]}, {0.7856949583871021, 1.1758756024193588`}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["19",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[-1, 2], 2^Rational[-1, 2]}, {0.7071067811865475, 
      0.7071067811865475}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["20",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Cos[Rational[3, 16] Pi], 2^Rational[1, 2] 
       Sin[Rational[3, 16] Pi]}, {1.1758756024193588`, 0.7856949583871021}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["21",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{Cos[Rational[1, 8] Pi], Sin[Rational[1, 8] Pi]}, {
      0.9238795325112867, 0.3826834323650898}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["22",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Cos[Rational[1, 16] Pi], 2^Rational[1, 2] 
       Sin[Rational[1, 16] Pi]}, {1.3870398453221475`, 0.275899379282943}],
     Background->RGBColor[1, 0.85, 0.85]]}},
  Axes->False]], "Input",
 CellChangeTimes->{{3.6066333253305287`*^9, 3.6066333255825434`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mirror7", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "\[IndentingNewLine]", "2", ",", 
     "6", ",", "10", ",", "14", ",", "\[IndentingNewLine]", "3", ",", "7", 
     ",", "11", ",", "15", ",", "\[IndentingNewLine]", "4", ",", "8", ",", 
     "12", ",", "22", ",", "\[IndentingNewLine]", "21", ",", "20", ",", "19", 
     ",", "18", ",", "17", ",", "16"}], "}"}], "\[LeftDoubleBracket]", "n", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"Mirror7", ",", "Listable"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MirrorIndex7", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{"Mirror7", "/@", 
   RowBox[{"{", 
    RowBox[{"i", ",", "k", ",", "j"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MapMulti7", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "22", " ", "22"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"j", "-", "1"}], ")"}], "22"}], "+", 
   "i"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mirror7", "[", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "22", ",", "21", ",", "20", ",", "19", ",", "18", ",", "17", 
     ",", "16"}], "}"}], "]"}], "\[Equal]", 
  RowBox[{"Range", "[", "22", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064597960761404`*^9, 3.606459981197729*^9}, {
   3.6064600503186827`*^9, 3.606460053797881*^9}, {3.6064648600587835`*^9, 
   3.6064648602527947`*^9}, 3.606464912318773*^9, {3.606465074482048*^9, 
   3.6064650746660585`*^9}, {3.6064657433613057`*^9, 3.606465765479571*^9}, {
   3.606465797968429*^9, 3.6064659030854416`*^9}, 3.60647352255309*^9, {
   3.606473589631621*^9, 3.6064735968167315`*^9}, 3.6064736384523706`*^9, {
   3.606473675910446*^9, 3.606473676885461*^9}, {3.606473716861075*^9, 
   3.606473757814204*^9}, {3.606514371978767*^9, 3.60651444389088*^9}, {
   3.606633332777955*^9, 3.606633344185607*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"compress7", ",", "inflate7"}], "}"}], "=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"hash", ",", 
       RowBox[{"n", "=", "22"}], ",", 
       RowBox[{"list", "=", 
        RowBox[{"{", "}"}]}], ",", "key", ",", "value", ",", "mir", ",", 
       RowBox[{"index", "=", 
        RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"key", "=", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j", ",", "k"}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"value", "=", 
          RowBox[{"HashGet", "[", 
           RowBox[{"hash", ",", "key"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"value", "===", "Null"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mir", "=", 
             RowBox[{"MirrorIndex7", "[", "key", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"list", ",", 
              RowBox[{"MapMulti7", "/@", 
               RowBox[{"Union", "[", 
                RowBox[{"{", 
                 RowBox[{"key", ",", "mir"}], "}"}], "]"}]}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "key", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "mir", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"index", ",", 
           RowBox[{"hash", "[", "key", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"list", ",", "index"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compress7", "[", "mat_", "]"}], ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"Transpose", "[", 
       RowBox[{"mat", "\[LeftDoubleBracket]", 
        RowBox[{"All", ",", "#"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
     "&"}], "/@", "compress7"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064607678477225`*^9, 3.606460818710632*^9}, {
  3.606464024319982*^9, 3.606464039159831*^9}, {3.606464975580391*^9, 
  3.6064650084722724`*^9}, {3.6064650852146616`*^9, 3.606465085394672*^9}, {
  3.606465910809883*^9, 3.60646593453024*^9}, {3.6064737675993547`*^9, 
  3.606473802734894*^9}, {3.6065144492671876`*^9, 3.606514465116094*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
     "#", "\[LeftDoubleBracket]", "inflate7", "\[RightDoubleBracket]"}], 
     "&"}], "/@", 
    RowBox[{"NullSpace", "[", 
     RowBox[{
      RowBox[{"Compress7", "[", 
       RowBox[{
       "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       "]"}], "//", "N"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606462536417879*^9, 3.60646255024967*^9}, 
   3.606464157033573*^9, {3.6064652563384495`*^9, 3.6064652608327065`*^9}, {
   3.6064659418686595`*^9, 3.6064659444858093`*^9}, {3.6065132923130136`*^9, 
   3.606513294313128*^9}, {3.6065133447960157`*^9, 3.6065133455550585`*^9}, {
   3.6065144809369993`*^9, 3.606514482641096*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Skew3", "[", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"nul", ",", 
       RowBox[{"Dimensions", "[", "GeneralForm7", "]"}]}], "]"}], "]"}], "//",
     "Flatten"}], "//", "Chop"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.606465964967981*^9, 3.606465965175993*^9}, 
   3.6065132971052876`*^9, {3.6065134460108047`*^9, 3.606513446755847*^9}, 
   3.6065144861292963`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{
     RowBox[{"Compress7", "[", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
      "]"}], ",", 
     RowBox[{"-", 
      RowBox[{
      "affine1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6064653638155966`*^9, 3.606465412204365*^9}, 
   3.606466099523677*^9, 3.6064738231527076`*^9, {3.606513149773861*^9, 
   3.6065131642766905`*^9}, 3.6065144900405197`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm7", "=", 
   RowBox[{"ArrayReshape", "[", 
    RowBox[{
     RowBox[{
     "sol", "\[LeftDoubleBracket]", "inflate7", "\[RightDoubleBracket]"}], 
     ",", 
     RowBox[{"Dimensions", "[", "GeneralForm7", "]"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6064654316644773`*^9, 3.606465438503869*^9}, {
  3.606466122622998*^9, 3.6064661267432337`*^9}, {3.6064738756685143`*^9, 
  3.6064738814936037`*^9}, {3.606514493595723*^9, 3.606514496898912*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm7", ">>", "\"\<CatmullClarkAlt7.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6057402733758764`*^9, 3.6057402754249935`*^9}, {
  3.6064661464733624`*^9, 3.606466148232463*^9}, {3.6064738839661417`*^9, 
  3.606473886253677*^9}, {3.6065145331199837`*^9, 3.606514535944145*^9}, {
  3.60732739728689*^9, 3.607327397751897*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{"CatmullClarkForm7", "-", 
    RowBox[{"RHS", "[", 
     RowBox[{
     "CatmullClarkForm4", ",", "CatmullClarkForm7", ",", "CC7sub", ",", 
      "CC7ind"}], "]"}]}], "]"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.6064740505512004`*^9, 3.606474056083785*^9}, {
  3.606514540536408*^9, 3.6065145458487115`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkForm7", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6057402829214225`*^9, {3.6059498206562295`*^9, 3.6059498230243645`*^9}, {
   3.605949889776183*^9, 3.605949893626403*^9}, {3.6064277639782133`*^9, 
   3.606427784345378*^9}, {3.606465580035964*^9, 3.606465580717003*^9}, 
   3.6064661574499903`*^9, 3.60647389100375*^9, {3.6065135495307255`*^9, 
   3.6065135534979525`*^9}, 3.606514981592635*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valence 8", "Subsection",
 CellChangeTimes->{{3.60562336381744*^9, 3.6056233784292755`*^9}, {
   3.6056963600892944`*^9, 3.605696366944687*^9}, 3.605780216264267*^9, 
   3.6057832882509747`*^9, {3.605783521695327*^9, 3.6057835245754914`*^9}, {
   3.605823463641165*^9, 3.605823494663939*^9}, {3.605852564805275*^9, 
   3.605852569810561*^9}, 3.6064734561420703`*^9, {3.606513815116916*^9, 
   3.6065138165960007`*^9}, 3.6066342120892487`*^9, {3.6067159384409*^9, 
   3.606715943149169*^9}, 3.606718272324772*^9, {3.607329582623956*^9, 
   3.607329583071463*^9}, {3.6121771723227863`*^9, 3.6121771724667945`*^9}}],

Cell[CellGroupData[{

Cell["Subdivision", "Subsubsection",
 CellChangeTimes->{{3.606513637606763*^9, 3.6065136422620296`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkIndexing", "[", "8", "]"}]], "Input",
 CellChangeTimes->{3.6063887648108745`*^9, 3.6065138233523874`*^9, 
  3.606715945654312*^9}],

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Omega]", "1"], "=", 
  RowBox[{"CatmullClarkWeight", "[", "8", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC8sub", "=", 
   RowBox[{"NormalizeWeights", "[", 
    RowBox[{"(", GridBox[{
       {"1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0", "0",
         "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6", "1", "0", "0",
         "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "1", "6", "1", "0", "6", "36", "6", "0", "1", "6",
         "1", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "1", "6", "1", "0", "6", 
        SubscriptBox["\[Omega]", "1"], "6", "0", "1", "6", "1", "6", "1", "6",
         "1", "6", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "6", "6", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "1", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", "1", 
        "0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "1", "0", "1", "6", 
        "1", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", "1", 
        "1", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "6", "0", "0", "1", 
        "6", "1", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "1", "1", "1", "0", "0", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "1", "1", "6", "1", "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "1", "1", "1", "0", "0", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "0", "0", "1", "1", "6", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "0", "0", "1", "1", "1", "0", "0", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "0", "0", "0", 
        "0", "0", "0", "0", "1", "1", "6", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", 
        "0", "0", "0", "0", "0", "0", "1", "1", "1", "0"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "6", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "1", "1", "6", "1"},
       {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "1", "1", "0", "0", 
        "0", "0", "0", "0", "0", "0", "0", "0", "1", "1"}
      }], ")"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CC8ind", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "1", ",", "2", ",", "3", ",", "4", ",", "  ", "6", ",", "7", ",", "8", 
       ",", "9", ",", "  ", "11", ",", "12", ",", "13", ",", "14", ",", "  ", 
       "16", ",", "17", ",", "18", ",", "19"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "4", ",", "5", ",", "  ", "7", ",", "8", ",", "9", 
       ",", "10", ",", "  ", "12", ",", "13", ",", "14", ",", "15", ",", "  ",
        "17", ",", "18", ",", "19", ",", "20"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "6", ",", "7", ",", "8", ",", "9", ",", "  ", "11", ",", "12", ",", 
       "13", ",", "14", ",", "  ", "16", ",", "17", ",", "18", ",", "19", ",",
        "  ", "21", ",", "22", ",", "23", ",", "24"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "7", ",", "8", ",", "9", ",", "10", ",", "  ", "12", ",", "13", ",", 
       "14", ",", "15", ",", "  ", "17", ",", "18", ",", "19", ",", "20", ",",
        "  ", "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", 
       ",", "28", ",", "29", ",", "30", ",", "31", ",", "32", ",", "33"}], 
      "}"}]}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6060143588756304`*^9, 3.606014372372402*^9}, 
   3.6065132271372857`*^9, {3.6065138266415753`*^9, 3.606513899600748*^9}, {
   3.6065139325216312`*^9, 3.60651394752549*^9}, {3.6065140091940165`*^9, 
   3.6065140745837564`*^9}, 3.6065141796917686`*^9, 3.6065142207361164`*^9, 
   3.6065142609524164`*^9, {3.6067159510516214`*^9, 3.606716118592204*^9}, {
   3.6067162628774567`*^9, 3.606716287905888*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkShow", "[", 
  RowBox[{
   RowBox[{"CatmullClarkDefault", "[", "8", "]"}], ",", "CC8sub", ",", 
   "CC8ind"}], "]"}]], "Input",
 CellChangeTimes->{{3.606014398075873*^9, 3.6060144003800044`*^9}, {
  3.6065140818141704`*^9, 3.606514086165419*^9}, {3.606716130799902*^9, 
  3.606716133862077*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution (numerical)", "Subsubsection",
 CellChangeTimes->{{3.6056459833952055`*^9, 3.6056459893175445`*^9}, {
  3.6056964913278008`*^9, 3.6056965130550437`*^9}, {3.6065132359387894`*^9, 
  3.606513237200861*^9}, {3.6067182646421537`*^9, 3.6067182674621973`*^9}}],

Cell[BoxData[{
 RowBox[{"Clear", "[", "CatmullClarkForm4", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{"<<", "\"\<CatmullClarkAlt4.mx\>\""}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeneralForm8", "=", 
   RowBox[{"Array", "[", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"#3", ",", "#2", ",", "#1"}]], "&"}], ",", 
     RowBox[{"{", 
      RowBox[{"24", ",", "24", ",", "24"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6056233521867743`*^9, 3.6056233902269506`*^9}, 
   3.605623495750986*^9, {3.6056799017582197`*^9, 3.605679914208932*^9}, 
   3.605679976213478*^9, {3.605680025240282*^9, 3.605680040530157*^9}, {
   3.6056800958383207`*^9, 3.605680185222433*^9}, 3.605680249966136*^9, {
   3.605680280892905*^9, 3.6056803189730835`*^9}, {3.6056863919401493`*^9, 
   3.605686399854602*^9}, {3.605686817407484*^9, 3.605686819028577*^9}, {
   3.605737935670168*^9, 3.6057379758284645`*^9}, {3.605738388115046*^9, 
   3.6057383908132*^9}, {3.6062492501646576`*^9, 3.6062492552029457`*^9}, {
   3.606465618518165*^9, 3.60646562734867*^9}, {3.6064734862825336`*^9, 
   3.6064734921776237`*^9}, {3.6065143291333165`*^9, 
   3.6065143349556494`*^9}, {3.606716343596073*^9, 3.606716347283284*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"affine1", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Normal", "[", 
       RowBox[{"CoefficientArrays", "[", 
        RowBox[{"#", ",", 
         RowBox[{"Flatten", "[", "GeneralForm8", "]"}]}], "]"}], "]"}], "&"}],
      "/@", 
     RowBox[{"Flatten", "[", 
      RowBox[{"GeneralForm8", "-", 
       RowBox[{"RHS", "[", 
        RowBox[{
        "CatmullClarkForm4", ",", "GeneralForm8", ",", "CC8sub", ",", 
         "CC8ind"}], "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//",
   "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6062493238898745`*^9, 3.6062493511444335`*^9}, 
   3.6064276017059317`*^9, {3.6064625607022676`*^9, 3.6064625627183833`*^9}, {
   3.606465632122943*^9, 3.6064656463857594`*^9}, {3.6064734982377167`*^9, 
   3.6064735066553464`*^9}, {3.6065131372851467`*^9, 3.606513140180312*^9}, {
   3.606514339110887*^9, 3.6065143461652904`*^9}, {3.606716350777484*^9, 
   3.606716358265912*^9}}],

Cell[BoxData[
 RowBox[{"affine1", ">>", "\"\<cc8_affine1\>\""}]], "Input",
 CellChangeTimes->{{3.6067166139425364`*^9, 3.606716641824131*^9}}],

Cell[BoxData[
 RowBox[{"ArrayPlot", "[", 
  RowBox[{"Unitize", "[", 
   RowBox[{"affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{3.6064654675505304`*^9}],

Cell[BoxData[
 GraphicsBox[{{
    {GrayLevel[0.5], 
     LineBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}, {-2, -2}}], 
     LineBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}, {-1, -2}}], 
     LineBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}, {0, -2}}], 
     LineBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}, {-2, -1}}], 
     LineBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}, {-1, -1}}], 
     LineBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}, {0, -1}}], 
     LineBox[{{-2, 0}, {-1, 0}, {-1, 1}, {-2, 1}, {-2, 0}}], 
     LineBox[{{0, 0}, {0, 1}, {-1, 1}, {-1, 0}, {0, 0}}], 
     LineBox[NCache[{{0, 0}, {
        Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
          Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
        2^Rational[1, 2] Sin[Rational[1, 20] Pi], 2^Rational[1, 2] 
         Cos[Rational[1, 20] Pi]}, {0, 1}, {0, 0}}, {{0, 0}, {
        0.30901699437494745`, 0.9510565162951535}, {0.22123174208247431`, 
        1.3968022466674208`}, {0, 1}, {0, 0}}]], 
     LineBox[NCache[{{0, 
        0}, {(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
         Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}, {
        2^Rational[1, 2] Sin[Rational[3, 20] Pi], 2^Rational[1, 2] 
         Cos[Rational[3, 20] Pi]}, {
        Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
          Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {0, 0}}, {{0, 
        0}, {0.5877852522924731, 0.8090169943749475}, {0.6420395219202061, 
        1.260073510670101}, {0.30901699437494745`, 0.9510565162951535}, {0, 
        0}}]], LineBox[
      NCache[{{0, 0}, {
        Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
          Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {1, 
        1}, {(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
         Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}, {0, 0}}, {{0,
         0}, {0.8090169943749475, 0.5877852522924731}, {1, 1}, {
        0.5877852522924731, 0.8090169943749475}, {0, 0}}]], 
     LineBox[NCache[{{0, 
        0}, {(Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
         Rational[1, 2], Rational[1, 4] (-1 + 5^Rational[1, 2])}, {
        2^Rational[1, 2] Cos[Rational[3, 20] Pi], 2^Rational[1, 2] 
         Sin[Rational[3, 20] Pi]}, {
        Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
          Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {0, 0}}, {{0, 
        0}, {0.9510565162951535, 0.30901699437494745`}, {1.260073510670101, 
        0.6420395219202061}, {0.8090169943749475, 0.5877852522924731}, {0, 
        0}}]], LineBox[
      NCache[{{0, 0}, {1, 0}, {
        2^Rational[1, 2] Cos[Rational[1, 20] Pi], 2^Rational[1, 2] 
         Sin[Rational[1, 20] Pi]}, {(Rational[5, 8] + 
          Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2], 
         Rational[1, 4] (-1 + 5^Rational[1, 2])}, {0, 0}}, {{0, 0}, {1, 0}, {
        1.3968022466674208`, 0.22123174208247431`}, {0.9510565162951535, 
        0.30901699437494745`}, {0, 0}}]]}, {
     {RGBColor[0.88, 1, 0.88], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-2, -2}, {-1, -2}, {-1, -1}, {-2, -1}}], 
      PolygonBox[{{-1, -2}, {0, -2}, {0, -1}, {-1, -1}}], 
      PolygonBox[{{0, -2}, {1, -2}, {1, -1}, {0, -1}}], 
      PolygonBox[{{-2, -1}, {-1, -1}, {-1, 0}, {-2, 0}}], 
      PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}], 
      PolygonBox[{{0, -1}, {1, -1}, {1, 0}, {0, 0}}], 
      PolygonBox[{{-2, 0}, {-1, 0}, {-1, 1}, {-2, 1}}], 
      PolygonBox[{{0, 0}, {0, 1}, {-1, 1}, {-1, 0}}], 
      PolygonBox[
       NCache[{{0, 0}, {
         Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
           Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
         2^Rational[1, 2] Sin[Rational[1, 20] Pi], 2^Rational[1, 2] 
          Cos[Rational[1, 20] Pi]}, {0, 1}}, {{0, 0}, {0.30901699437494745`, 
         0.9510565162951535}, {0.22123174208247431`, 1.3968022466674208`}, {0,
          1}}]], PolygonBox[
       NCache[{{0, 
         0}, {(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
          Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}, {
         2^Rational[1, 2] Sin[Rational[3, 20] Pi], 2^Rational[1, 2] 
          Cos[Rational[3, 20] Pi]}, {
         Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
           Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{0, 0}, {
         0.5877852522924731, 0.8090169943749475}, {0.6420395219202061, 
         1.260073510670101}, {0.30901699437494745`, 0.9510565162951535}}]], 
      PolygonBox[
       NCache[{{0, 0}, {
         Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
           Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {1, 
         1}, {(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
          Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}}, {{0, 0}, {
         0.8090169943749475, 0.5877852522924731}, {1, 1}, {0.5877852522924731,
          0.8090169943749475}}]], 
      PolygonBox[
       NCache[{{0, 
         0}, {(Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
          Rational[1, 2], Rational[1, 4] (-1 + 5^Rational[1, 2])}, {
         2^Rational[1, 2] Cos[Rational[3, 20] Pi], 2^Rational[1, 2] 
          Sin[Rational[3, 20] Pi]}, {
         Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
           Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}}, {{0, 0}, {
         0.9510565162951535, 0.30901699437494745`}, {1.260073510670101, 
         0.6420395219202061}, {0.8090169943749475, 0.5877852522924731}}]], 
      PolygonBox[
       NCache[{{0, 0}, {1, 0}, {
         2^Rational[1, 2] Cos[Rational[1, 20] Pi], 2^Rational[1, 2] 
          Sin[Rational[1, 20] Pi]}, {(Rational[5, 8] + 
           Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2], 
          Rational[1, 4] (-1 + 5^Rational[1, 2])}}, {{0, 0}, {1, 0}, {
         1.3968022466674208`, 0.22123174208247431`}, {0.9510565162951535, 
         0.30901699437494745`}}]]}, 
     {RGBColor[0, 1, 0], EdgeForm[Dashing[{Small, Small}]], 
      PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}]}}, 
    {RGBColor[0, 1, 0], Opacity[0.5], 
     PolygonBox[{{-1, -1}, {0, -1}, {0, 0}, {-1, 0}}]}, 
    {RGBColor[0, 1, 0], EdgeForm[RGBColor[0, 0, 1]], DiskBox[{-2, -2}, 0.04], 
     DiskBox[{-1, -2}, 0.04], DiskBox[{0, -2}, 0.04], DiskBox[{1, -2}, 0.04], 
     DiskBox[{-2, -1}, 0.04], DiskBox[{-1, -1}, 0.04], DiskBox[{0, -1}, 0.04],
      DiskBox[{1, -1}, 0.04], DiskBox[{-2, 0}, 0.04], DiskBox[{-1, 0}, 0.04], 
     DiskBox[{0, 0}, 0.04], DiskBox[{1, 0}, 0.04], DiskBox[{-2, 1}, 0.04], 
     DiskBox[{-1, 1}, 0.04], DiskBox[{0, 1}, 0.04], 
     DiskBox[
      NCache[{2^Rational[1, 2] Sin[Rational[1, 20] Pi], 2^Rational[1, 2] 
        Cos[Rational[1, 20] Pi]}, {0.22123174208247431`, 
       1.3968022466674208`}], 0.04], 
     DiskBox[NCache[{
       Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
         Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
       0.30901699437494745`, 0.9510565162951535}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Sin[Rational[3, 20] Pi], 2^Rational[1, 2] 
        Cos[Rational[3, 20] Pi]}, {0.6420395219202061, 1.260073510670101}], 
      0.04], DiskBox[
      NCache[{(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
        Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}, {
       0.5877852522924731, 0.8090169943749475}], 0.04], DiskBox[{1, 1}, 0.04],
      DiskBox[NCache[{
       Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
         Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
       0.8090169943749475, 0.5877852522924731}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Cos[Rational[3, 20] Pi], 2^Rational[1, 2] 
        Sin[Rational[3, 20] Pi]}, {1.260073510670101, 0.6420395219202061}], 
      0.04], DiskBox[
      NCache[{(Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
        Rational[1, 2], Rational[1, 4] (-1 + 5^Rational[1, 2])}, {
       0.9510565162951535, 0.30901699437494745`}], 0.04], 
     DiskBox[NCache[{
       2^Rational[1, 2] Cos[Rational[1, 20] Pi], 2^Rational[1, 2] 
        Sin[Rational[1, 20] Pi]}, {1.3968022466674208`, 
       0.22123174208247431`}], 0.04]}}, {InsetBox[
     StyleBox["1",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["2",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["3",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["4",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, -2},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["5",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["6",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["7",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["8",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, -1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["9",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["10",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["11",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["12",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, 0},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["13",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-2, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["14",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {-1, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["15",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {0, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["16",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Sin[Rational[1, 20] Pi], 2^Rational[1, 2] 
       Cos[Rational[1, 20] Pi]}, {0.22123174208247431`, 1.3968022466674208`}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["17",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{Rational[1, 4] (-1 + 5^Rational[1, 2]), (Rational[5, 8] + 
        Rational[1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
      0.30901699437494745`, 0.9510565162951535}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["18",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Sin[Rational[3, 20] Pi], 2^Rational[1, 2] 
       Cos[Rational[3, 20] Pi]}, {0.6420395219202061, 1.260073510670101}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["19",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{(Rational[5, 8] + Rational[-1, 8] 5^Rational[1, 2])^
       Rational[1, 2], Rational[1, 4] (1 + 5^Rational[1, 2])}, {
      0.5877852522924731, 0.8090169943749475}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["20",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], {1, 1},
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["21",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{Rational[1, 4] (1 + 5^Rational[1, 2]), (Rational[5, 8] + 
        Rational[-1, 8] 5^Rational[1, 2])^Rational[1, 2]}, {
      0.8090169943749475, 0.5877852522924731}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["22",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Cos[Rational[3, 20] Pi], 2^Rational[1, 2] 
       Sin[Rational[3, 20] Pi]}, {1.260073510670101, 0.6420395219202061}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["23",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{(Rational[5, 8] + Rational[1, 8] 5^Rational[1, 2])^
       Rational[1, 2], Rational[1, 4] (-1 + 5^Rational[1, 2])}, {
      0.9510565162951535, 0.30901699437494745`}],
     Background->RGBColor[1, 0.85, 0.85]], InsetBox[
     StyleBox["24",
      StripOnInput->False,
      FontSize->Large,
      FontWeight->Bold], 
     NCache[{2^Rational[1, 2] Cos[Rational[1, 20] Pi], 2^Rational[1, 2] 
       Sin[Rational[1, 20] Pi]}, {1.3968022466674208`, 0.22123174208247431`}],
     Background->RGBColor[1, 0.85, 0.85]]}},
  Axes->False]], "Input",
 CellChangeTimes->{{3.6066333253305287`*^9, 3.6066333255825434`*^9}, {
  3.6067163988592343`*^9, 3.606716399588276*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Mirror8", "[", "n_", "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "\[IndentingNewLine]", "2", ",", 
     "6", ",", "10", ",", "14", ",", "\[IndentingNewLine]", "3", ",", "7", 
     ",", "11", ",", "15", ",", "\[IndentingNewLine]", "4", ",", "8", ",", 
     "12", ",", "24", ",", "\[IndentingNewLine]", "23", ",", "22", ",", "21", 
     ",", "20", ",", "19", ",", "18", ",", "17", ",", "16"}], "}"}], 
   "\[LeftDoubleBracket]", "n", 
   "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{"Mirror8", ",", "Listable"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MirrorIndex8", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{"Mirror8", "/@", 
   RowBox[{"{", 
    RowBox[{"i", ",", "k", ",", "j"}], "}"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MapMulti8", "[", 
   RowBox[{"{", 
    RowBox[{"i_", ",", "j_", ",", "k_"}], "}"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "24", " ", "24"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"j", "-", "1"}], ")"}], "24"}], "+", 
   "i"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mirror8", "[", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "5", ",", "9", ",", "13", ",", "2", ",", "6", ",", "10", ",", 
     "14", ",", "3", ",", "7", ",", "11", ",", "15", ",", "4", ",", "8", ",", 
     "12", ",", "24", ",", "23", ",", "22", ",", "21", ",", "20", ",", "19", 
     ",", "18", ",", "17", ",", "16"}], "}"}], "]"}], "\[Equal]", 
  RowBox[{"Range", "[", "24", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064597960761404`*^9, 3.606459981197729*^9}, {
   3.6064600503186827`*^9, 3.606460053797881*^9}, {3.6064648600587835`*^9, 
   3.6064648602527947`*^9}, 3.606464912318773*^9, {3.606465074482048*^9, 
   3.6064650746660585`*^9}, {3.6064657433613057`*^9, 3.606465765479571*^9}, {
   3.606465797968429*^9, 3.6064659030854416`*^9}, 3.60647352255309*^9, {
   3.606473589631621*^9, 3.6064735968167315`*^9}, 3.6064736384523706`*^9, {
   3.606473675910446*^9, 3.606473676885461*^9}, {3.606473716861075*^9, 
   3.606473757814204*^9}, {3.606514371978767*^9, 3.60651444389088*^9}, {
   3.606633332777955*^9, 3.606633344185607*^9}, 3.6067163880516157`*^9, {
   3.606716483595081*^9, 3.606716513732805*^9}, {3.606717801236536*^9, 
   3.606717809371661*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"compress8", ",", "inflate8"}], "}"}], "=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"hash", ",", 
       RowBox[{"n", "=", "24"}], ",", 
       RowBox[{"list", "=", 
        RowBox[{"{", "}"}]}], ",", "key", ",", "value", ",", "mir", ",", 
       RowBox[{"index", "=", 
        RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"key", "=", 
          RowBox[{"{", 
           RowBox[{"i", ",", "j", ",", "k"}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"value", "=", 
          RowBox[{"HashGet", "[", 
           RowBox[{"hash", ",", "key"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"value", "===", "Null"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mir", "=", 
             RowBox[{"MirrorIndex8", "[", "key", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"list", ",", 
              RowBox[{"MapMulti8", "/@", 
               RowBox[{"Union", "[", 
                RowBox[{"{", 
                 RowBox[{"key", ",", "mir"}], "}"}], "]"}]}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "key", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"hash", "[", "mir", "]"}], "=", 
             RowBox[{"Length", "[", "list", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"index", ",", 
           RowBox[{"hash", "[", "key", "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "n"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"list", ",", "index"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compress8", "[", "mat_", "]"}], ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Total", "[", 
      RowBox[{"Transpose", "[", 
       RowBox[{"mat", "\[LeftDoubleBracket]", 
        RowBox[{"All", ",", "#"}], "\[RightDoubleBracket]"}], "]"}], "]"}], 
     "&"}], "/@", "compress8"}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.6064607678477225`*^9, 3.606460818710632*^9}, {
  3.606464024319982*^9, 3.606464039159831*^9}, {3.606464975580391*^9, 
  3.6064650084722724`*^9}, {3.6064650852146616`*^9, 3.606465085394672*^9}, {
  3.606465910809883*^9, 3.60646593453024*^9}, {3.6064737675993547`*^9, 
  3.606473802734894*^9}, {3.6065144492671876`*^9, 3.606514465116094*^9}, {
  3.606716542060425*^9, 3.6067165548571568`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"nul", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
     "#", "\[LeftDoubleBracket]", "inflate8", "\[RightDoubleBracket]"}], 
     "&"}], "/@", 
    RowBox[{"NullSpace", "[", 
     RowBox[{
      RowBox[{"Compress8", "[", 
       RowBox[{
       "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
       "]"}], "//", "N"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nul", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.606462536417879*^9, 3.60646255024967*^9}, 
   3.606464157033573*^9, {3.6064652563384495`*^9, 3.6064652608327065`*^9}, {
   3.6064659418686595`*^9, 3.6064659444858093`*^9}, {3.6065132923130136`*^9, 
   3.606513294313128*^9}, {3.6065133447960157`*^9, 3.6065133455550585`*^9}, {
   3.6065144809369993`*^9, 3.606514482641096*^9}, {3.6067165573182974`*^9, 
   3.606716558981393*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Skew3", "[", 
     RowBox[{"ArrayReshape", "[", 
      RowBox[{"nul", ",", 
       RowBox[{"Dimensions", "[", "GeneralForm8", "]"}]}], "]"}], "]"}], "//",
     "Flatten"}], "//", "Chop"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.606465964967981*^9, 3.606465965175993*^9}, 
   3.6065132971052876`*^9, {3.6065134460108047`*^9, 3.606513446755847*^9}, 
   3.6065144861292963`*^9, 3.606717085272495*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"matrix", "=", 
   RowBox[{"Compress8", "[", 
    RowBox[{
     RowBox[{
     "affine1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], "//", 
     "N"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6067167535555215`*^9, 3.6067168125738974`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vector", "=", 
   RowBox[{"-", 
    RowBox[{
    "affine1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.60671693203373*^9, 3.6067169437343993`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"matrix", ",", "vector"}], "}"}], ">>", 
  "\"\<cc8_lgs.mx\>\""}]], "Input",
 CellChangeTimes->{{3.606716946949583*^9, 3.606716959201284*^9}}],

Cell[BoxData[
 RowBox[{"Clear", "[", "affine1", "]"}]], "Input",
 CellChangeTimes->{{3.606717012616339*^9, 3.606717015855524*^9}}],

Cell["\<\
quit kernel here to free memory\
\>", "Text",
 CellChangeTimes->{{3.6067169660276747`*^9, 3.606716975971243*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"matrix", ",", "vector"}], "}"}], "=", 
   RowBox[{"<<", "\"\<cc8_lgs.mx\>\""}]}], ";"}]], "Input",
 CellChangeTimes->{{3.606716982623624*^9, 3.6067169839677005`*^9}, 
   3.6067173620053234`*^9}],

Cell[BoxData[
 RowBox[{"matrix", "//", "Dimensions"}]], "Input",
 CellChangeTimes->{{3.6067175293898606`*^9, 3.6067175314023914`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"sol", "=", 
   RowBox[{"LinearSolve", "[", 
    RowBox[{"matrix", ",", "vector"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "//", "Dimensions"}]}], "Input",
 CellChangeTimes->{{3.6064653638155966`*^9, 3.606465412204365*^9}, 
   3.606466099523677*^9, 3.6064738231527076`*^9, {3.606513149773861*^9, 
   3.6065131642766905`*^9}, 3.6065144900405197`*^9, {3.606716997888497*^9, 
   3.6067170016437116`*^9}}],

Cell[BoxData[
 RowBox[{"sol", ">>", "\"\<cc8_sol.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6067177084976115`*^9, 3.606717713265185*^9}}],

Cell[BoxData[{
 RowBox[{"sol", "\[LeftDoubleBracket]", "3", 
  "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{"sol", "\[LeftDoubleBracket]", "10", 
  "\[RightDoubleBracket]"}]}], "Input",
 CellChangeTimes->{{3.6067178362095733`*^9, 3.606717844689704*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CatmullClarkForm8", "=", 
   RowBox[{
    RowBox[{"ArrayReshape", "[", 
     RowBox[{
      RowBox[{
      "sol", "\[LeftDoubleBracket]", "inflate8", "\[RightDoubleBracket]"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{"24", ",", "24", ",", "24"}], "}"}]}], "]"}], "//", "Chop"}]}],
   ";"}]], "Input",
 CellChangeTimes->{{3.6064654316644773`*^9, 3.606465438503869*^9}, {
   3.606466122622998*^9, 3.6064661267432337`*^9}, {3.6064738756685143`*^9, 
   3.6064738814936037`*^9}, {3.606514493595723*^9, 3.606514496898912*^9}, {
   3.6067175542002416`*^9, 3.6067175711080017`*^9}, 3.607329439229254*^9}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm8", ">>", "\"\<CatmullClarkAlt8.mx\>\""}]], "Input",
 CellChangeTimes->{{3.6057402733758764`*^9, 3.6057402754249935`*^9}, {
  3.6064661464733624`*^9, 3.606466148232463*^9}, {3.6064738839661417`*^9, 
  3.606473886253677*^9}, {3.6065145331199837`*^9, 3.606514535944145*^9}, {
  3.606717871927622*^9, 3.606717873835151*^9}, {3.6073286059539547`*^9, 
  3.6073286064064617`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Flatten", "[", 
    RowBox[{"CatmullClarkForm8", "-", 
     RowBox[{"RHS", "[", 
      RowBox[{
      "CatmullClarkForm4", ",", "CatmullClarkForm8", ",", "CC8sub", ",", 
       "CC8ind"}], "]"}]}], "]"}], "//", "Chop"}], "//", "Union"}]], "Input",
 CellChangeTimes->{{3.6064740505512004`*^9, 3.606474056083785*^9}, {
  3.606514540536408*^9, 3.6065145458487115`*^9}, {3.6067178774477067`*^9, 
  3.606717889075385*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", "#", "]"}], "]"}], "&"}], "/@", 
   RowBox[{
    RowBox[{"Skew3", "[", "CatmullClarkForm8", "]"}], "\[LeftDoubleBracket]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], "\[RightDoubleBracket]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.6057402829214225`*^9, {3.6059498206562295`*^9, 3.6059498230243645`*^9}, {
   3.605949889776183*^9, 3.605949893626403*^9}, {3.6064277639782133`*^9, 
   3.606427784345378*^9}, {3.606465580035964*^9, 3.606465580717003*^9}, 
   3.6064661574499903`*^9, 3.60647389100375*^9, {3.6065135495307255`*^9, 
   3.6065135534979525`*^9}, 3.606514981592635*^9, 3.606717897688018*^9}]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other", "Section",
 CellChangeTimes->{{3.605327924449959*^9, 3.6053279283121796`*^9}, {
  3.6056149079927945`*^9, 3.6056149089858513`*^9}, {3.6056228516001425`*^9, 
  3.6056228546483173`*^9}, {3.605646909007148*^9, 3.6056469329195156`*^9}, {
  3.60564698352641*^9, 3.6056469954380913`*^9}, {3.605696857837764*^9, 
  3.605696861844993*^9}, {3.605697358177382*^9, 3.605697358385394*^9}, {
  3.606474502673145*^9, 3.6064745042006683`*^9}, {3.612177468354718*^9, 
  3.6121774689607525`*^9}}],

Cell[CellGroupData[{

Cell["Deprecated", "Subsection",
 CellChangeTimes->{{3.6056540916429653`*^9, 3.6056540971372795`*^9}, {
  3.606474578729313*^9, 3.606474580536841*^9}, {3.6121383503004975`*^9, 
  3.612138350844529*^9}, {3.612177470241826*^9, 3.612177471568902*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"PlotMeshNormals", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}], ",", "scale_"}], "]"}], ":=", 
  RowBox[{"Graphics3D", "[", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Thick", ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"mid", "=", 
            RowBox[{"Mean", "[", 
             RowBox[{
             "V", "\[LeftDoubleBracket]", "#", "\[RightDoubleBracket]"}], 
             "]"}]}], ",", 
           RowBox[{"nrm", "=", 
            RowBox[{"Cross", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"V", "\[LeftDoubleBracket]", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}], "-", 
               RowBox[{"V", "\[LeftDoubleBracket]", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}]}], ",", 
              RowBox[{
               RowBox[{"V", "\[LeftDoubleBracket]", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}], "-", 
               RowBox[{"V", "\[LeftDoubleBracket]", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "\[RightDoubleBracket]"}]}]}], "]"}]}]}], "}"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"mid", ",", 
            RowBox[{"mid", "+", 
             RowBox[{"scale", " ", 
              RowBox[{"nrm", "/", 
               RowBox[{"Norm", "[", "nrm", "]"}]}]}]}]}], "}"}], "]"}]}], 
        "]"}], "&"}], "/@", "F"}]}], "}"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlanarQuadQ", "[", 
   RowBox[{"{", 
    RowBox[{"V_", ",", "F_"}], "}"}], "]"}], ":=", 
  RowBox[{"And", "@@", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"0", "==", 
       RowBox[{"Det", "[", 
        RowBox[{
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{"#", "\[LeftDoubleBracket]", 
           RowBox[{"{", 
            RowBox[{"2", ",", "3", ",", "4"}], "}"}], 
           "\[RightDoubleBracket]"}], "\[RightDoubleBracket]"}], "-", 
         RowBox[{"V", "\[LeftDoubleBracket]", 
          RowBox[{"#", "\[LeftDoubleBracket]", 
           RowBox[{"{", 
            RowBox[{"1", ",", "1", ",", "1"}], "}"}], 
           "\[RightDoubleBracket]"}], "\[RightDoubleBracket]"}]}], "]"}]}], 
      "&"}], "/@", "F"}], ")"}]}]}]}], "Input",
 CellChangeTimes->{{3.6064294557149754`*^9, 3.6064294560209923`*^9}, {
  3.6064329013770556`*^9, 3.6064329026251273`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}], "=", 
   RowBox[{"QuadDiamongMesh", "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"V", "\[LeftDoubleBracket]", "1", 
  "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{"1", "/", "16"}]}], ",", 
    RowBox[{
     RowBox[{"-", "1"}], "/", "16"}], ",", "1"}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "\[LeftDoubleBracket]", "7", "\[RightDoubleBracket]"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "+", 
     RowBox[{"1", "/", "16"}]}], ",", 
    RowBox[{
     RowBox[{"-", "1"}], "/", "16"}], ",", "0"}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"#", "+", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "&"}], "/@", "V"}], 
    ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlanarQuadQ", "[", 
  RowBox[{"{", 
   RowBox[{"V", ",", "F"}], "}"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6064280587240715`*^9, 3.6064280705737495`*^9}, {
   3.606428624386426*^9, 3.6064286584903765`*^9}, {3.606429032904792*^9, 
   3.606429056041115*^9}, {3.6064290921181784`*^9, 3.606429093280245*^9}, {
   3.6064293442776012`*^9, 3.606429350439954*^9}, {3.6064294613132954`*^9, 
   3.606429461497306*^9}, {3.606429575147806*^9, 3.606429591581746*^9}, 
   3.606432890720446*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Vs", ",", "Fs"}], "}"}], "=", 
   RowBox[{"Nest", "[", 
    RowBox[{"QuadLinearSubdivide", ",", 
     RowBox[{"{", 
      RowBox[{"V", ",", "F"}], "}"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Vn", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Vs", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
        "\[Equal]", 
        RowBox[{"Last", "[", "V", "]"}]}], ",", 
       RowBox[{"Vs", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"V", "\[LeftDoubleBracket]", 
        RowBox[{
         RowBox[{"Sort", "[", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"With", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"dif", "=", 
                RowBox[{
                 RowBox[{
                 "Vs", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}],
                  "-", 
                 RowBox[{
                 "V", "\[LeftDoubleBracket]", "j", 
                  "\[RightDoubleBracket]"}]}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"N", "[", 
                 RowBox[{"dif", ".", "dif"}], "]"}], ",", "j"}], "}"}]}], 
             "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"j", ",", 
              RowBox[{
               RowBox[{"Length", "[", "V", "]"}], "-", "1"}]}], "}"}]}], 
           "]"}], "]"}], "\[LeftDoubleBracket]", 
         RowBox[{"1", ",", "2"}], "\[RightDoubleBracket]"}], 
        "\[RightDoubleBracket]"}]}], "]"}], "\[IndentingNewLine]", ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"Length", "[", "Vs", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.606392973339588*^9, 3.6063929821680937`*^9}, 
   3.6063930171720953`*^9, 3.6063932333464603`*^9, {3.6063936689423747`*^9, 
   3.606393699564126*^9}, 3.6063939062379475`*^9, 3.6064278483420386`*^9, {
   3.606428043190183*^9, 3.606428058037032*^9}, 3.6064286383922267`*^9, {
   3.60642896119669*^9, 3.6064289875191956`*^9}, 3.6064294683146954`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{
     RowBox[{"PlotMesh", "[", 
      RowBox[{"{", 
       RowBox[{"V", ",", "F"}], "}"}], "]"}], ",", 
     RowBox[{"PlotMeshNormals", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"V", ",", "F"}], "}"}], ",", ".3"}], "]"}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotMesh", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", "Vs"}], "+", 
        RowBox[{"\[Lambda]", " ", "Vn"}]}], "/.", 
       RowBox[{"\[Lambda]", "\[Rule]", "0.7"}]}], ",", "Fs"}], "}"}], "]"}]}],
   "}"}]], "Input",
 CellChangeTimes->{{3.606393062140668*^9, 3.6063930712441883`*^9}, {
  3.6063931016879296`*^9, 3.6063931144626603`*^9}, {3.606428991105401*^9, 
  3.6064289942425804`*^9}}],

Cell[BoxData[{
 RowBox[{"QuadLinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"V", ",", "F"}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"QuadLinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"Vs", ",", "Fs"}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"QuadLinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Va", ",", "Fa"}], "}"}], "=", 
   RowBox[{"CatmullClarkSubdivide", "[", 
    RowBox[{"{", 
     RowBox[{"Vn", ",", "Fs"}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkVolume", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Vn", ".", 
     RowBox[{"RotationMatrix", "[", 
      RowBox[{
       RowBox[{"\[Pi]", "/", "2"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "1"}], "}"}]}], "]"}]}], ",", "Fs"}], 
   "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"QuadLinearVolume", "[", 
  RowBox[{"{", 
   RowBox[{"Va", ",", "Fa"}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"PlotMesh", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Va", ".", 
     RowBox[{"RotationMatrix", "[", 
      RowBox[{
       RowBox[{"\[Pi]", "/", "2"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "1"}], "}"}]}], "]"}]}], ",", "Fa"}], 
   "}"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6063931271963882`*^9, 3.606393214475381*^9}, {
   3.6063932797751155`*^9, 3.606393311125909*^9}, 3.606393352439272*^9, {
   3.60639364228685*^9, 3.6063936490142345`*^9}, {3.606393711244794*^9, 
   3.606393765243883*^9}, {3.606394039476568*^9, 3.6063940553374753`*^9}, {
   3.6064271906444206`*^9, 3.6064272014960413`*^9}, 3.606427232111792*^9, {
   3.6064273675185375`*^9, 3.60642736773555*^9}, {3.606428715671647*^9, 
   3.6064287439022617`*^9}, {3.6064294771422005`*^9, 
   3.6064294903449554`*^9}, {3.6064296229485407`*^9, 3.6064296374233685`*^9}, 
   3.6064300082215767`*^9}],

Cell[BoxData[
 FractionBox["637", "240"]], "Input"],

Cell[BoxData[
 RowBox[{"CatmullClarkForm3", "//", "Variables"}]], "Input",
 CellChangeTimes->{{3.6064295122492085`*^9, 3.606429520190663*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Other", "Subsection",
 CellChangeTimes->{{3.6056540916429653`*^9, 3.6056540971372795`*^9}, {
  3.606474578729313*^9, 3.606474580536841*^9}, {3.6121383503004975`*^9, 
  3.612138350844529*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Form3ToMatlabM", "[", 
   RowBox[{"fname_", ",", "func_", ",", "frm_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", "frm", "]"}]}], ",", "str"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"str", "=", 
      RowBox[{"OpenWrite", "[", 
       RowBox[{"fname", ",", 
        RowBox[{"PageWidth", "\[Rule]", "10000"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{"\"\<function [Y,Z]=\>\"", "<>", "func"}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{
        "\"\<% \>\"", "<>", "func", "<>", 
         "\"\< volumeform from www.hakenberg.org\>\""}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{"\"\<Y=zeros(\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         "\[IndentingNewLine]", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<);\>\""}], "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{"\"\<Z=cell(\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         "\[IndentingNewLine]", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<);\>\""}], "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Write", "[", 
         RowBox[{"str", ",", 
          RowBox[{"OutputForm", "[", 
           RowBox[{"\"\<Y(\>\"", "<>", 
            RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<,\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<,\>\"", "<>", 
            RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<)=\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"CForm", "[", 
              RowBox[{
               RowBox[{"frm", "\[LeftDoubleBracket]", 
                RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}], 
               "//", "N"}], "]"}], "]"}], "<>", "\"\<;\>\""}], "]"}]}], "]"}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"Write", "[", 
         RowBox[{"str", ",", 
          RowBox[{"OutputForm", "[", 
           RowBox[{"\"\<Z{\>\"", "<>", 
            RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<,\>\"", "<>", 
            RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<,\>\"", "<>", 
            RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<}='\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{
              RowBox[{"frm", "\[LeftDoubleBracket]", 
               RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}], 
              ",", "InputForm"}], "]"}], "<>", "\"\<';\>\""}], "]"}]}], "]"}],
         ";"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"Write", "[", 
        RowBox[{"str", ",", 
         RowBox[{
         "OutputForm", "[", "\"\<Yn=cellfun(@str2num,Y);\>\"", "]"}]}], "]"}],
        ";"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Close", "[", "str", "]"}], ";"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.606598511390336*^9, 3.6065985262751875`*^9}, {
   3.606598566148468*^9, 3.606598616588353*^9}, {3.6065986611739035`*^9, 
   3.606598703965351*^9}, {3.606598812021531*^9, 3.60659897162066*^9}, {
   3.6065990018533893`*^9, 3.606599008802787*^9}, {3.606599070629323*^9, 
   3.606599124371397*^9}, {3.606599221831971*^9, 3.606599263975382*^9}, {
   3.6065992952711716`*^9, 3.6065993386306515`*^9}, {3.6065996219658575`*^9, 
   3.6065996373877397`*^9}, {3.606599873355236*^9, 3.606599892434327*^9}, {
   3.6065999247491755`*^9, 3.6065999362268324`*^9}, {3.6066001433016763`*^9, 
   3.6066001564254265`*^9}, {3.606600329072302*^9, 3.6066003384898405`*^9}, {
   3.606601714135523*^9, 3.606601736270789*^9}, 3.6066018029046*^9, {
   3.6066216550020247`*^9, 3.6066216554250484`*^9}, {3.6066299670224447`*^9, 
   3.606630014464158*^9}, {3.6066300470220203`*^9, 3.606630056159543*^9}, {
   3.6066300956338005`*^9, 3.6066300970278807`*^9}, {3.6066301865810027`*^9, 
   3.6066302302304993`*^9}, {3.6066306591800337`*^9, 3.606630660580114*^9}, {
   3.6066307397406416`*^9, 3.606630807396511*^9}, {3.6066308603175383`*^9, 
   3.60663091839386*^9}, {3.6066321999161587`*^9, 3.606632203483363*^9}, 
   3.606632873289674*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Form3ToMatlabOnlyNumeric", "[", 
   RowBox[{"fname_", ",", "func_", ",", "frm_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", "frm", "]"}]}], ",", "str"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"str", "=", 
      RowBox[{"OpenWrite", "[", 
       RowBox[{"fname", ",", 
        RowBox[{"PageWidth", "\[Rule]", "10000"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{"\"\<function Y=\>\"", "<>", "func"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{
        "\"\<% \>\"", "<>", "func", "<>", 
         "\"\< volumeform from www.hakenberg.org\>\""}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", "\"\<Y=reshape([\>\"", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"Write", "[", 
        RowBox[{"str", ",", 
         RowBox[{"OutputForm", "[", 
          RowBox[{"\"\<Z=cell(\>\"", "<>", 
           RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
           RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
           "\[IndentingNewLine]", 
           RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<);\>\""}], "]"}]}],
         "]"}], ";"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"Write", "[", 
         RowBox[{"str", ",", 
          RowBox[{"OutputForm", "[", 
           RowBox[{"ToString", "[", 
            RowBox[{"CForm", "[", 
             RowBox[{
              RowBox[{"frm", "\[LeftDoubleBracket]", 
               RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}], "//",
               "N"}], "]"}], "]"}], "]"}]}], "]"}], ";"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Write", "[", 
          RowBox[{"str", ",", 
           RowBox[{"OutputForm", "[", 
            RowBox[{"\"\<Y(\>\"", "<>", 
             RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<,\>\"", "<>", 
             RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<,\>\"", "<>", 
             RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<)=\>\"", "<>", 
             RowBox[{"ToString", "[", 
              RowBox[{"CForm", "[", 
               RowBox[{
                RowBox[{"frm", "\[LeftDoubleBracket]", 
                 RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}],
                 "//", "N"}], "]"}], "]"}], "<>", "\"\<;\>\""}], "]"}]}], 
          "]"}], ";"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Write", "[", 
          RowBox[{"str", ",", 
           RowBox[{"OutputForm", "[", 
            RowBox[{"\"\<Z{\>\"", "<>", 
             RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\<,\>\"", "<>", 
             RowBox[{"ToString", "[", "j", "]"}], "<>", "\"\<,\>\"", "<>", 
             RowBox[{"ToString", "[", "k", "]"}], "<>", "\"\<}='\>\"", "<>", 
             RowBox[{"ToString", "[", 
              RowBox[{
               RowBox[{"frm", "\[LeftDoubleBracket]", 
                RowBox[{"k", ",", "j", ",", "i"}], "\[RightDoubleBracket]"}], 
               ",", "InputForm"}], "]"}], "<>", "\"\<';\>\""}], "]"}]}], 
          "]"}], ";"}], "*)"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", 
        RowBox[{"\"\<],[\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<,\>\"", "<>", 
         "\[IndentingNewLine]", 
         RowBox[{"ToString", "[", "n", "]"}], "<>", "\"\<]);\>\""}], "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"Write", "[", 
        RowBox[{"str", ",", 
         RowBox[{
         "OutputForm", "[", "\"\<Yn=cellfun(@str2num,Y);\>\"", "]"}]}], "]"}],
        ";"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"Close", "[", "str", "]"}], ";"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6066336841350513`*^9, 3.6066337047042274`*^9}, {
  3.6066338182307215`*^9, 3.6066338930129986`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and MATLAB"
}], "Subsection",
 CellChangeTimes->{{3.606631336131753*^9, 3.6066313428541374`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Skew3", "[", "CatmullClarkForm5", "]"}], "\[LeftDoubleBracket]", 
  RowBox[{"1", ",", "13", ",", "18"}], "\[RightDoubleBracket]"}]], "Input",
 CellChangeTimes->{{3.6069897440822926`*^9, 3.606989802328624*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"val", "=", 
         RowBox[{"ToString", "[", 
          RowBox[{"k", "+", "2"}], "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Form3ToMatlabOnlyNumeric", "[", 
         RowBox[{
          RowBox[{
          "\"\<d:/volumeforms/catmullclark/getformcc\>\"", "<>", "val", "<>", 
           "\"\<.m\>\""}], ",", 
          RowBox[{"\"\<getformcc\>\"", "<>", "val"}], ",", 
          RowBox[{
          "forms", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}]}], 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Export", "[", 
         RowBox[{
          RowBox[{
          "\"\<d:/volumeforms/catmullclark/indexcc\>\"", "<>", "val", "<>", 
           "\"\<.gif\>\""}], ",", 
          RowBox[{"CatmullClarkIndexing", "[", 
           RowBox[{"k", "+", "2"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]",
       "]"}], ";"}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"Length", "[", "forms", "]"}]}], "}"}]}], "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.606598981381218*^9, 3.6065990348082743`*^9}, {
   3.6065991418723974`*^9, 3.606599144791565*^9}, {3.606599200407746*^9, 
   3.6065992030218954`*^9}, {3.606599274215967*^9, 3.6065992764870973`*^9}, {
   3.6065995625734606`*^9, 3.6065995633895073`*^9}, 3.606599947930502*^9, {
   3.606600196763734*^9, 3.6066002224232016`*^9}, {3.6066015407366047`*^9, 
   3.606601552051252*^9}, {3.6066216266864047`*^9, 3.60662166628467*^9}, {
   3.606621713775386*^9, 3.6066217194057083`*^9}, {3.606630325857969*^9, 
   3.606630465496956*^9}, 3.606630664058313*^9, {3.6066316643235245`*^9, 
   3.6066316883528996`*^9}, {3.6066318761926427`*^9, 3.606631876450658*^9}, {
   3.6066335404938354`*^9, 3.6066335551756754`*^9}, 3.606633718471015*^9, {
   3.6066339143472185`*^9, 3.606633929668095*^9}, 3.6066340440356364`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ToLongString", "[", "vec_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"str", "=", "\"\<\>\""}], "}"}], ",", 
    RowBox[{
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"str", "=", 
         RowBox[{"str", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{
            RowBox[{
            "vec", "\[LeftDoubleBracket]", "k", "\[RightDoubleBracket]"}], 
            ",", "InputForm"}], "]"}], "<>", "\"\< \>\""}]}], ";"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", 
         RowBox[{"Length", "[", "vec", "]"}]}], "}"}]}], "]"}], ";", 
     "str"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6066011327972717`*^9, 3.6066012396443834`*^9}, {
  3.606601399347518*^9, 3.606601400617591*^9}, {3.606601888550499*^9, 
  3.6066018921987076`*^9}, {3.606601924607561*^9, 3.606601927192709*^9}}],

Cell[BoxData[
 RowBox[{"ToLongString", "[", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3", ",", "4"}], "}"}], "]"}]], "Input",
 CellChangeTimes->{{3.6066012049754004`*^9, 3.6066012068995104`*^9}, {
  3.606601243800621*^9, 3.6066012442106447`*^9}, {3.60660190552547*^9, 
  3.606601905860489*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DooSabinVolumeExp", "[", 
   RowBox[{"fname_", ",", 
    RowBox[{"{", 
     RowBox[{"V_", ",", "F_"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"count", "=", "0"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"str", "=", 
      RowBox[{"OpenWrite", "[", 
       RowBox[{"fname", ",", 
        RowBox[{"PageWidth", "\[Rule]", "10000"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Write", "[", 
      RowBox[{"str", ",", 
       RowBox[{"OutputForm", "[", "\"\<function rings=dsring\>\"", "]"}]}], 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Simplify", "[", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{"With", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"point", "=", 
            RowBox[{"DSRing", "[", 
             RowBox[{"V", ",", "F", ",", 
              RowBox[{
              "F", "\[LeftDoubleBracket]", "m", "\[RightDoubleBracket]"}], 
              ",", "k"}], "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Write", "[", 
            RowBox[{"str", ",", 
             RowBox[{"OutputForm", "[", 
              RowBox[{"\"\<rings{\>\"", "<>", 
               RowBox[{"ToString", "[", 
                RowBox[{"++", "count"}], "]"}], "<>", 
               "\"\<}=reshape(str2num('\>\"", "<>", 
               RowBox[{"ToLongString", "[", 
                RowBox[{"Flatten", "[", 
                 RowBox[{"point", "//", "Transpose"}], "]"}], "]"}], "<>", 
               "\"\<'),[3 \>\"", "<>", 
               RowBox[{"ToString", "[", 
                RowBox[{"Length", "[", 
                 RowBox[{
                 "point", "\[LeftDoubleBracket]", "1", 
                  "\[RightDoubleBracket]"}], "]"}], "]"}], "<>", 
               "\"\<]);\>\""}], "]"}]}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"m", ",", 
          RowBox[{"Length", "[", "F", "]"}]}], "}"}]}], "]"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Close", "[", "str", "]"}], ";"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.606600701034577*^9, 3.6066007222537904`*^9}, {
   3.6066008888353186`*^9, 3.6066011059507365`*^9}, {3.606601261650642*^9, 
   3.606601284015921*^9}, 3.6066018521174145`*^9, 3.606601899075101*^9, {
   3.606601950403036*^9, 3.6066020038110914`*^9}, {3.606602456535986*^9, 
   3.6066024895558743`*^9}}],

Cell[BoxData[
 RowBox[{"DooSabinVolumeExp", "[", 
  RowBox[{"\"\<d:/volumeforms/doosabin/ringscube.m\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.606600728052122*^9, 3.6066007287651625`*^9}, {
  3.606600989611082*^9, 3.60660099657248*^9}, {3.6066312627435555`*^9, 
  3.6066312690439157`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Deformation", "Subsection",
 CellChangeTimes->{{3.6069900464815884`*^9, 3.606990051333866*^9}}],

Cell[BoxData["V"], "Input",
 CellChangeTimes->{3.6069912623531322`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}], "=", "TripodMesh"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"V", "=", 
   RowBox[{"N", "[", "V", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6069900561021385`*^9, 3.6069900625275064`*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}], ",", "1", ",", "True"}], "]"}]], "Input",
 CellChangeTimes->{{3.6069900542250314`*^9, 3.6069900692408905`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Vn", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"#", "+", 
      RowBox[{"0.2", " ", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"RandomReal", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "}"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "3"}], "}"}]}], "]"}]}]}], "&"}], "/@", "V"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ind", "=", 
   RowBox[{
    RowBox[{"Length", "[", "V", "]"}], "-", "1"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Vn", "\[LeftDoubleBracket]", "ind", "\[RightDoubleBracket]"}], 
   "=", 
   RowBox[{
    RowBox[{"Vn", "\[LeftDoubleBracket]", "ind", "\[RightDoubleBracket]"}], 
    "+", 
    RowBox[{"{", 
     RowBox[{"0", ",", "\[Lambda]", ",", "0"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"vol", "=", 
  RowBox[{"CatmullClarkVolume", "[", 
   RowBox[{"CatmullClarkSubdivide", "[", 
    RowBox[{"{", 
     RowBox[{"Vn", ",", "F"}], "}"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"vol", "==", "2.5040054761592043`"}], "//", "Solve"}]}]}], "Input",
 CellChangeTimes->{{3.606990162398218*^9, 3.606990240418681*^9}, {
   3.6069903788796005`*^9, 3.606990425303256*^9}, {3.6069905104091234`*^9, 
   3.606990512744257*^9}, {3.6069911270613937`*^9, 3.6069911288674974`*^9}, 
   3.606991160976334*^9, {3.6069911951482887`*^9, 3.6069911984054747`*^9}, {
   3.6069912759169083`*^9, 3.6069913238616505`*^9}}],

Cell[BoxData[{
 RowBox[{"Vf", "=", 
  RowBox[{"Vn", "/.", 
   RowBox[{
   "sol", "\[LeftDoubleBracket]", "1", 
    "\[RightDoubleBracket]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Vf", ",", "F"}], "}"}], ",", "1", ",", "True"}], "]"}]}], "Input",
 CellChangeTimes->{{3.606990460900292*^9, 3.606990469621791*^9}, {
   3.6069905355655622`*^9, 3.6069905403888383`*^9}, {3.6069911803784437`*^9, 
   3.6069912082110357`*^9}, 3.606991300970341*^9}],

Cell[BoxData[{
 RowBox[{"MeshToMatlab", "[", 
  RowBox[{"\"\<d:/matlab/volume/tripod2hull.m\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"V", ",", "F"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"MeshToMatlab", "[", 
  RowBox[{"\"\<d:/matlab/volume/tridef2hull.m\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"Vf", ",", "F"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"MeshToMatlab", "[", 
  RowBox[{"\"\<d:/matlab/volume/tridef2.m\>\"", ",", 
   RowBox[{"TriangulateMesh", "[", 
    RowBox[{"Nest", "[", 
     RowBox[{"CatmullClarkSubdivide", ",", 
      RowBox[{"{", 
       RowBox[{"Vf", ",", "F"}], "}"}], ",", "4"}], "]"}], "]"}]}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.6069906151341133`*^9, 3.606990661509766*^9}, 
   3.6069912245569706`*^9, 3.606991362635868*^9}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6069906709653068`*^9, 3.606990691182463*^9}, 
   3.606991038244314*^9, {3.606991221858816*^9, 3.6069912236099167`*^9}}],

Cell[BoxData[
 RowBox[{"lin", "=", 
  RowBox[{"LinearVolume", "[", 
   RowBox[{"<<", "\"\<Print11.mx\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6110942490966063`*^9, 3.6110942542068987`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"msh", "=", 
   RowBox[{"<<", "\"\<Print11.mx\>\""}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"msh", "=", 
   RowBox[{"Nest", "[", 
    RowBox[{"CatmullClarkSubdivide", ",", "msh", ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"vol1", "=", 
  RowBox[{"CatmullClarkVolume", "[", "msh", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6110942585811486`*^9, 3.611094301413598*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"msh", "=", 
   RowBox[{"Nest", "[", 
    RowBox[{"CatmullClarkSubdivide", ",", "msh", ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"vol2", "=", 
  RowBox[{"CatmullClarkVolume", "[", "msh", "]"}]}]}], "Input",
 CellChangeTimes->{{3.6110943067639046`*^9, 3.611094313086266*^9}}],

Cell[BoxData[
 RowBox[{"vol1", "/", "lin"}]], "Input",
 CellChangeTimes->{{3.6110943433519974`*^9, 3.611094344322053*^9}}],

Cell[BoxData[
 RowBox[{"vol2", "-", "vol1"}]], "Input",
 CellChangeTimes->{{3.611094317208502*^9, 3.611094319046607*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkDemo", "[", 
  RowBox[{
   RowBox[{"<<", "\"\<Print11.mx\>\""}], ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.6110941811217184`*^9, 3.611094196208581*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Cubic B-Spline", "Subsection",
 CellChangeTimes->{{3.604664346574308*^9, 3.6046643521896296`*^9}, {
  3.6052569012648907`*^9, 3.6052569032320037`*^9}, {3.605318991083868*^9, 
  3.6053189916358995`*^9}, {3.6057358648537235`*^9, 3.6057358656537695`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"bas3", "[", "x_", "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", "6"]}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "x"}], ")"}], "3"]}], ",", 
     RowBox[{
      FractionBox["1", "6"], " ", 
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"6", " ", 
         SuperscriptBox["x", "2"]}], "+", 
        RowBox[{"3", " ", 
         SuperscriptBox["x", "3"]}]}], ")"}]}], ",", 
     RowBox[{
      FractionBox["1", "6"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"3", " ", "x"}], "+", 
        RowBox[{"3", " ", 
         SuperscriptBox["x", "2"]}], "-", 
        RowBox[{"3", " ", 
         SuperscriptBox["x", "3"]}]}], ")"}]}], ",", 
     FractionBox[
      SuperscriptBox["x", "3"], "6"]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.605311971091342*^9, 3.605312017687007*^9, {3.6053178412511015`*^9, 
   3.6053178579680576`*^9}, 3.6055933709862037`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Bas3", "[", 
    RowBox[{"s_", ",", "t_"}], "]"}], "=", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"bas3", "[", "s", "]"}], "\[LeftDoubleBracket]", "k1", 
        "\[RightDoubleBracket]"}], 
       RowBox[{
        RowBox[{"bas3", "[", "t", "]"}], "\[LeftDoubleBracket]", "k2", 
        "\[RightDoubleBracket]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"k2", ",", "4"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k1", ",", "4"}], "}"}]}], "]"}], "//", "Flatten"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Sas3", "[", 
    RowBox[{"s_", ",", "t_"}], "]"}], "=", 
   RowBox[{
    SubscriptBox["\[PartialD]", "s"], 
    RowBox[{"Bas3", "[", 
     RowBox[{"s", ",", "t"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Tas3", "[", 
    RowBox[{"s_", ",", "t_"}], "]"}], "=", 
   RowBox[{
    SubscriptBox["\[PartialD]", "t"], 
    RowBox[{"Bas3", "[", 
     RowBox[{"s", ",", "t"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6052845886434135`*^9, 3.6052846784535503`*^9}, {
  3.605312042805444*^9, 3.6053120729261665`*^9}, {3.6057359556119146`*^9, 
  3.6057359580530543`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", "#", "]"}], "&"}], "/@", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Bas3", "[", 
      RowBox[{"s", ",", "t"}], "]"}], "//", "Simplify"}], ")"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6060870600195527`*^9, 3.606087079416662*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Ten3", "[", 
    RowBox[{"s_", ",", "t_"}], "]"}], "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Bas3", "[", 
        RowBox[{"s", ",", "t"}], "]"}], "\[LeftDoubleBracket]", "a1", 
       "\[RightDoubleBracket]"}], " ", 
      RowBox[{
       RowBox[{"Sas3", "[", 
        RowBox[{"s", ",", "t"}], "]"}], "\[LeftDoubleBracket]", "b1", 
       "\[RightDoubleBracket]"}], 
      RowBox[{
       RowBox[{"Tas3", "[", 
        RowBox[{"s", ",", "t"}], "]"}], "\[LeftDoubleBracket]", "c1", 
       "\[RightDoubleBracket]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"c1", ",", "16"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b1", ",", "16"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"a1", ",", "16"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CatmullClarkForm4", "=", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "1"], 
       RowBox[{
        RowBox[{"Ten3", "[", 
         RowBox[{"s", ",", "t"}], "]"}], 
        RowBox[{"\[DifferentialD]", "t"}]}]}], ")"}], 
     RowBox[{"\[DifferentialD]", "s"}]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.605311415842583*^9, 3.605311425491135*^9}, {
  3.6053121211939273`*^9, 3.6053121374498568`*^9}, {3.6053178643064203`*^9, 
  3.6053178831264963`*^9}, {3.6057359642494087`*^9, 3.605735968555655*^9}}],

Cell[BoxData[
 RowBox[{"CatmullClarkForm4", ">>", "\"\<CatmullClarkForm4.mx\>\""}]], "Input",
 CellChangeTimes->{{3.605736003333644*^9, 3.6057360067888417`*^9}}],

Cell["\<\
Here we ensure that our orientation of vertices yields positive volume \
(instead of negative):\
\>", "Text",
 CellChangeTimes->{{3.6056960408260336`*^9, 3.605696080666312*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"P", "=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"-", "1"}], "0", "1", "2", 
       RowBox[{"-", "1"}], "0", "1", "2", 
       RowBox[{"-", "1"}], "0", "1", "2", 
       RowBox[{"-", "1"}], "0", "1", "2"},
      {
       RowBox[{"-", "1"}], 
       RowBox[{"-", "1"}], 
       RowBox[{"-", "1"}], 
       RowBox[{"-", "1"}], "0", "0", "0", "0", "1", "1", "1", "1", "2", "2", 
       "2", "2"},
      {"1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", "1", 
       "1", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"CatmullClarkForm4", ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}]}], ",", 
   RowBox[{"-", 
    RowBox[{"CatmullClarkForm4", ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}]}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"CatmullClarkForm4", ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}]}], ",", 
   RowBox[{"-", 
    RowBox[{"CatmullClarkForm4", ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}]}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"CatmullClarkForm4", ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
    RowBox[{"P", "\[LeftDoubleBracket]", 
     RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}]}], ",", 
   RowBox[{"-", 
    RowBox[{"CatmullClarkForm4", ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}], ".", 
     RowBox[{"P", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}]}]}]}], 
  "}"}]}], "Input",
 CellChangeTimes->{{3.6056751414849477`*^9, 3.6056752216245317`*^9}, {
  3.605675324382409*^9, 3.605675368213916*^9}, {3.60567916833327*^9, 
  3.605679197614945*^9}, {3.60567929840771*^9, 3.6056793214150257`*^9}, {
  3.6056807076843157`*^9, 3.6056807842326946`*^9}, {3.605736054328561*^9, 
  3.6057360893565645`*^9}, {3.6057361809178014`*^9, 3.6057361868211393`*^9}}],

Cell["Coefficients", "Text",
 CellChangeTimes->{{3.6057360277250395`*^9, 3.6057360326283197`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"MatrixForm", "[", 
      RowBox[{"32920473600", " ", "#1"}], "]"}], "]"}], "&"}], "/@", 
   "CatmullClarkForm4"}], ";"}]], "Input",
 CellChangeTimes->{{3.605736021237668*^9, 3.605736024285843*^9}}],

Cell["We approximate the volume of a torus (only to check):", "Text",
 CellChangeTimes->{{3.605610756768358*^9, 3.605610762173667*^9}, {
  3.6056961984740505`*^9, 3.60569627748057*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Torus", "[", 
    RowBox[{"R_", ",", "r_", ",", "\[Theta]_", ",", "\[Phi]_"}], "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"R", "+", 
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ")"}], " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{"r", " ", 
      RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"TorusVolume", "[", 
    RowBox[{"R_", ",", "r_"}], "]"}], "=", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Pi]", "2"], "R", " ", 
    SuperscriptBox["r", "2"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TorusVolume", "[", 
   RowBox[{"2", ",", "1"}], "]"}], "//", "N"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TorusSample", "[", 
   RowBox[{"N_", ",", "M_", ",", "i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"Torus", "[", 
   RowBox[{"2", ",", "1", ",", 
    FractionBox[
     RowBox[{"2", "\[Pi]", " ", "i"}], "N"], ",", 
    FractionBox[
     RowBox[{"2", "\[Pi]", " ", "j"}], "M"]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ParametricPlot3D", "[", 
    RowBox[{
     RowBox[{"Torus", "[", 
      RowBox[{"2", ",", "1", ",", "\[Theta]", ",", "\[Phi]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"\[Theta]", ",", "0", ",", 
       RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\[Phi]", ",", "0", ",", 
       RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
     RowBox[{"AxesLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z"}], "}"}]}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"ListPointPlot3D", "[", 
    RowBox[{"Flatten", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"TorusSample", "[", 
         RowBox[{"60", ",", "20", ",", "i", ",", "j"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "20"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "60"}], "}"}]}], "]"}], ",", "1"}], "]"}], "]"}]}],
   "}"}]}], "Input",
 CellChangeTimes->{{3.60530326388444*^9, 3.60530327035681*^9}, {
   3.6053033048367825`*^9, 3.6053033626270876`*^9}, {3.6053034043744755`*^9, 
   3.6053034172622128`*^9}, {3.605303486944198*^9, 3.6053034974617996`*^9}, {
   3.6053035845447807`*^9, 3.6053037029515533`*^9}, {3.605304029281218*^9, 
   3.605304039642811*^9}, {3.605305062622322*^9, 3.6053050743329916`*^9}, 
   3.605307382028984*^9, {3.6053176021624265`*^9, 3.6053176023304358`*^9}, {
   3.605318836373019*^9, 3.6053188521489215`*^9}, {3.6056755722345853`*^9, 
   3.6056755763198185`*^9}, {3.6056763335421295`*^9, 3.605676336260285*^9}, {
   3.6056776414139357`*^9, 3.605677663031172*^9}, {3.6056777897974224`*^9, 
   3.60567779324962*^9}, {3.6056867128575044`*^9, 3.605686729531458*^9}}],

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", "=", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"Flatten", "[", 
          RowBox[{
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"TorusSample", "[", 
              RowBox[{"60", ",", "20", ",", 
               RowBox[{"i", "+", "k"}], ",", 
               RowBox[{"j", "+", "l"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"l", ",", 
               RowBox[{"-", "1"}], ",", "2"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"k", ",", 
               RowBox[{"-", "1"}], ",", "2"}], "}"}]}], "]"}], ",", "1"}], 
          "]"}], "]"}], "//", "N"}]}], "}"}], ",", 
     RowBox[{"With", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"pX", "=", 
          RowBox[{"p", "\[LeftDoubleBracket]", 
           RowBox[{"1", ",", "All"}], "\[RightDoubleBracket]"}]}], ",", 
         RowBox[{"pY", "=", 
          RowBox[{"p", "\[LeftDoubleBracket]", 
           RowBox[{"2", ",", "All"}], "\[RightDoubleBracket]"}]}], ",", 
         RowBox[{"pZ", "=", 
          RowBox[{"p", "\[LeftDoubleBracket]", 
           RowBox[{"3", ",", "All"}], "\[RightDoubleBracket]"}]}]}], "}"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"CatmullClarkForm4", ".", "pX", ".", "pY", ".", "pZ"}], "-", 
        RowBox[{"CatmullClarkForm4", ".", "pX", ".", "pZ", ".", "pY"}]}]}], 
      "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "60"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "20"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6056792453706765`*^9, 3.6056792750893764`*^9}, {
  3.605680808448079*^9, 3.605680809894162*^9}, {3.6056810337459655`*^9, 
  3.605681097601618*^9}, {3.60573621673385*^9, 3.6057362281175013`*^9}}]
}, Closed]]
}, Open  ]]
},
WindowSize->{890, 934},
WindowMargins->{{99, Automatic}, {Automatic, 22}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 425, 5, 79, "Section"],
Cell[1007, 29, 414, 8, 49, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1458, 42, 502, 7, 79, "Section"],
Cell[CellGroupData[{
Cell[1985, 53, 180, 2, 43, "Subsection"],
Cell[2168, 57, 3288, 86, 172, "Input"],
Cell[5459, 145, 216, 7, 31, "Input"],
Cell[5678, 154, 3114, 75, 92, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[8829, 234, 154, 2, 35, "Subsection"],
Cell[8986, 238, 4175, 104, 272, "Input"],
Cell[13164, 344, 193, 6, 31, "Input"],
Cell[13360, 352, 15250, 343, 1145, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[28647, 700, 213, 3, 35, "Subsection"],
Cell[28863, 705, 1842, 56, 92, "Input"],
Cell[30708, 763, 1176, 32, 72, "Input"],
Cell[31887, 797, 660, 17, 31, "Input"],
Cell[32550, 816, 1124, 30, 92, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[33711, 851, 147, 2, 35, "Subsection"],
Cell[33861, 855, 5060, 133, 231, "Input"],
Cell[38924, 990, 3649, 93, 212, "Input"],
Cell[42576, 1085, 5037, 117, 192, "Input"],
Cell[47616, 1204, 1825, 50, 72, "Input"],
Cell[49444, 1256, 10636, 233, 692, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[60117, 1494, 105, 1, 35, "Subsection"],
Cell[60225, 1497, 12510, 356, 2365, "Input"],
Cell[72738, 1855, 5566, 165, 1262, "Input"],
Cell[78307, 2022, 1609, 44, 82, "Input"],
Cell[79919, 2068, 3085, 77, 139, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[83041, 2150, 157, 2, 35, "Subsection"],
Cell[83201, 2154, 405, 9, 48, "Input"],
Cell[83609, 2165, 4591, 123, 212, "Input"],
Cell[88203, 2290, 4517, 127, 248, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[92757, 2422, 153, 2, 35, "Subsection"],
Cell[92913, 2426, 15400, 414, 893, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[108350, 2845, 185, 2, 35, "Subsection"],
Cell[108538, 2849, 10336, 243, 539, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[118923, 3098, 659, 9, 79, "Section"],
Cell[CellGroupData[{
Cell[119607, 3111, 152, 2, 43, "Subsection"],
Cell[119762, 3115, 226, 5, 31, "Input"],
Cell[119991, 3122, 403, 9, 52, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[120431, 3136, 94, 1, 35, "Subsection"],
Cell[120528, 3139, 215, 4, 31, "Input"],
Cell[120746, 3145, 171, 3, 31, "Input"],
Cell[120920, 3150, 225, 5, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[121182, 3160, 148, 2, 35, "Subsection"],
Cell[121333, 3164, 139, 2, 29, "Input"],
Cell[121475, 3168, 904, 25, 66, "Input"],
Cell[122382, 3195, 462, 8, 29, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[122893, 3209, 547, 7, 79, "Section"],
Cell[CellGroupData[{
Cell[123465, 3220, 223, 3, 43, "Subsection"],
Cell[CellGroupData[{
Cell[123713, 3227, 104, 1, 34, "Subsubsection"],
Cell[123820, 3230, 141, 2, 146, "Input"],
Cell[123964, 3234, 4718, 96, 2886, "Input"],
Cell[128685, 3332, 358, 7, 167, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[129080, 3344, 177, 2, 28, "Subsubsection"],
Cell[129260, 3348, 385, 11, 146, "Input"],
Cell[129648, 3361, 308, 8, 146, "Input"],
Cell[129959, 3371, 884, 22, 252, "Input"],
Cell[130846, 3395, 1122, 30, 763, "Input"],
Cell[131971, 3427, 1048, 31, 355, "Input"],
Cell[133022, 3460, 663, 17, 355, "Input"],
Cell[133688, 3479, 133, 2, 146, "Input"],
Cell[133824, 3483, 249, 5, 146, "Input"],
Cell[134076, 3490, 262, 7, 146, "Input"],
Cell[134341, 3499, 262, 7, 146, "Input"],
Cell[134606, 3508, 212, 4, 146, "Input"],
Cell[134821, 3514, 227, 6, 252, "Input"],
Cell[135051, 3522, 148, 2, 146, "Input"],
Cell[135202, 3526, 190, 4, 146, "Input"],
Cell[135395, 3532, 153, 2, 146, "Input"],
Cell[135551, 3536, 190, 4, 146, "Input"],
Cell[135744, 3542, 140, 2, 146, "Input"],
Cell[135887, 3546, 256, 5, 167, "Input"],
Cell[136146, 3553, 172, 3, 167, "Input"],
Cell[136321, 3558, 470, 9, 408, "Input"],
Cell[136794, 3569, 591, 15, 167, "Input"],
Cell[137388, 3586, 281, 5, 289, "Input"],
Cell[137672, 3593, 370, 10, 167, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[138091, 3609, 163, 2, 43, "Subsection"],
Cell[138257, 3613, 1563, 49, 41251, "Input"],
Cell[139823, 3664, 997, 25, 26324, "Input"],
Cell[140823, 3691, 828, 18, 19014, "Input"],
Cell[141654, 3711, 473, 13, 19014, "Input"],
Cell[142130, 3726, 114, 3, 11281, "Input"],
Cell[142247, 3731, 252, 6, 11281, "Input"],
Cell[142502, 3739, 505, 13, 19014, "Input"],
Cell[143010, 3754, 401, 10, 11281, "Input"],
Cell[143414, 3766, 160, 2, 11281, "Input"],
Cell[143577, 3770, 503, 13, 11281, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[144117, 3788, 249, 3, 35, "Subsection"],
Cell[CellGroupData[{
Cell[144391, 3795, 104, 1, 34, "Subsubsection"],
Cell[144498, 3798, 117, 2, 85, "Input"],
Cell[144618, 3802, 3792, 65, 1557, "Input"],
Cell[148413, 3869, 286, 6, 85, "Input"],
Cell[148702, 3877, 387, 8, 146, "Input"],
Cell[149092, 3887, 165, 3, 85, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[149294, 3895, 207, 3, 28, "Subsubsection"],
Cell[149504, 3900, 880, 17, 85, "Input"],
Cell[150387, 3919, 937, 23, 204, "Input"],
Cell[151327, 3944, 597, 18, 146, "Input"],
Cell[151927, 3964, 218, 5, 85, "Input"],
Cell[152148, 3971, 1598, 42, 321, "Input"],
Cell[153749, 4015, 2885, 72, 906, "Input"],
Cell[156637, 4089, 576, 15, 146, "Input"],
Cell[157216, 4106, 242, 7, 85, "Input"],
Cell[157461, 4115, 556, 15, 146, "Input"],
Cell[158020, 4132, 361, 10, 85, "Input"],
Cell[158384, 4144, 212, 3, 85, "Input"],
Cell[158599, 4149, 772, 20, 263, "Input"],
Cell[159374, 4171, 148, 2, 85, "Input"],
Cell[159525, 4175, 544, 13, 85, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[160118, 4194, 274, 4, 43, "Subsection"],
Cell[CellGroupData[{
Cell[160417, 4202, 104, 1, 34, "Subsubsection"],
Cell[160524, 4205, 113, 2, 127, "Input"],
Cell[160640, 4209, 4553, 91, 2654, "Input"],
Cell[165196, 4302, 335, 7, 127, "Input"],
Cell[165534, 4311, 508, 13, 220, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[166079, 4329, 209, 3, 28, "Subsubsection"],
Cell[166291, 4334, 929, 18, 84229, "Input"],
Cell[167223, 4354, 983, 23, 196315, "Input"],
Cell[168209, 4379, 218, 5, 84229, "Input"],
Cell[168430, 4386, 1788, 43, 307497, "Input"],
Cell[170221, 4431, 2933, 73, 862174, "Input"],
Cell[173157, 4506, 630, 16, 141928, "Input"],
Cell[173790, 4524, 154, 3, 84229, "Input"],
Cell[173947, 4529, 308, 8, 84229, "Input"],
Cell[174258, 4539, 579, 15, 141928, "Input"],
Cell[174840, 4556, 412, 11, 84229, "Input"],
Cell[175255, 4569, 264, 4, 84229, "Input"],
Cell[175522, 4575, 572, 14, 84229, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[176143, 4595, 472, 6, 43, "Subsection"],
Cell[CellGroupData[{
Cell[176640, 4605, 104, 1, 34, "Subsubsection"],
Cell[176747, 4608, 117, 2, 146, "Input"],
Cell[176867, 4612, 5119, 97, 3221, "Input"],
Cell[181989, 4711, 231, 5, 220, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[182257, 4721, 207, 3, 28, "Subsubsection"],
Cell[182467, 4726, 983, 19, 84229, "Input"],
Cell[183453, 4747, 985, 23, 196315, "Input"],
Cell[184441, 4772, 218, 5, 84229, "Input"],
Cell[184662, 4779, 2037, 48, 307497, "Input"],
Cell[186702, 4829, 2984, 74, 862174, "Input"],
Cell[189689, 4905, 763, 18, 141928, "Input"],
Cell[190455, 4925, 417, 10, 84229, "Input"],
Cell[190875, 4937, 607, 16, 141928, "Input"],
Cell[191485, 4955, 465, 12, 84229, "Input"],
Cell[191953, 4969, 312, 4, 84229, "Input"],
Cell[192268, 4975, 329, 8, 84229, "Input"],
Cell[192600, 4985, 681, 16, 84229, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[193330, 5007, 552, 7, 43, "Subsection"],
Cell[CellGroupData[{
Cell[193907, 5018, 104, 1, 34, "Subsubsection"],
Cell[194014, 5021, 141, 2, 84229, "Input"],
Cell[194158, 5025, 5919, 105, 1930966, "Input"],
Cell[200080, 5132, 282, 6, 84229, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[200399, 5143, 207, 3, 28, "Subsubsection"],
Cell[200609, 5148, 274, 5, 141928, "Input"],
Cell[200886, 5155, 1033, 19, 84229, "Input"],
Cell[201922, 5176, 1037, 24, 196315, "Input"],
Cell[202962, 5202, 218, 5, 84229, "Input"],
Cell[203183, 5209, 10258, 228, 1136739, "Input"],
Cell[213444, 5439, 2278, 50, 530199, "Input"],
Cell[215725, 5491, 3032, 74, 862174, "Input"],
Cell[218760, 5567, 815, 19, 141928, "Input"],
Cell[219578, 5588, 445, 11, 84229, "Input"],
Cell[220026, 5601, 631, 16, 141928, "Input"],
Cell[220660, 5619, 511, 12, 84229, "Input"],
Cell[221174, 5633, 360, 5, 84229, "Input"],
Cell[221537, 5640, 380, 9, 84229, "Input"],
Cell[221920, 5651, 703, 16, 84229, "Input"]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[222672, 5673, 618, 8, 43, "Subsection"],
Cell[CellGroupData[{
Cell[223315, 5685, 104, 1, 34, "Subsubsection"],
Cell[223422, 5688, 166, 3, 85, "Input"],
Cell[223591, 5693, 6631, 110, 2035, "Input"],
Cell[230225, 5805, 331, 7, 85, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[230593, 5817, 269, 3, 28, "Subsubsection"],
Cell[230865, 5822, 206, 4, 141928, "Input"],
Cell[231074, 5828, 1083, 20, 84229, "Input"],
Cell[232160, 5850, 1087, 25, 196315, "Input"],
Cell[233250, 5877, 142, 2, 84229, "Input"],
Cell[233395, 5881, 218, 5, 84229, "Input"],
Cell[233616, 5888, 13302, 286, 1136739, "Input"],
Cell[246921, 6176, 2450, 53, 585806, "Input"],
Cell[249374, 6231, 3083, 75, 862174, "Input"],
Cell[252460, 6308, 867, 20, 141928, "Input"],
Cell[253330, 6330, 467, 11, 84229, "Input"],
Cell[253800, 6343, 293, 8, 84229, "Input"],
Cell[254096, 6353, 241, 7, 84229, "Input"],
Cell[254340, 6362, 193, 5, 84229, "Input"],
Cell[254536, 6369, 130, 2, 84229, "Input"],
Cell[254669, 6373, 123, 3, 87664, "Text"],
Cell[254795, 6378, 259, 7, 84229, "Input"],
Cell[255057, 6387, 134, 2, 84229, "Input"],
Cell[255194, 6391, 453, 9, 141928, "Input"],
Cell[255650, 6402, 137, 2, 84229, "Input"],
Cell[255790, 6406, 268, 5, 141928, "Input"],
Cell[256061, 6413, 639, 15, 84229, "Input"],
Cell[256703, 6430, 414, 6, 84229, "Input"],
Cell[257120, 6438, 464, 11, 141928, "Input"],
Cell[257587, 6451, 725, 16, 84229, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[258373, 6474, 493, 7, 79, "Section"],
Cell[CellGroupData[{
Cell[258891, 6485, 248, 3, 43, "Subsection"],
Cell[259142, 6490, 2816, 72, 116, "Input"],
Cell[261961, 6564, 1558, 43, 166, "Input"],
Cell[263522, 6609, 2249, 58, 116, "Input"],
Cell[265774, 6669, 879, 26, 65, "Input"],
Cell[266656, 6697, 1928, 48, 192, "Input"],
Cell[268587, 6747, 51, 1, 57, "Input"],
Cell[268641, 6750, 142, 2, 39, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[268820, 6757, 197, 3, 35, "Subsection"],
Cell[269020, 6762, 5080, 107, 419, "Input"],
Cell[274103, 6871, 4747, 112, 470, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[278887, 6988, 165, 5, 36, "Subsection"],
Cell[279055, 6995, 244, 4, 39, "Input"],
Cell[279302, 7001, 2025, 42, 192, "Input"],
Cell[281330, 7045, 936, 24, 65, "Input"],
Cell[282269, 7071, 291, 6, 39, "Input"],
Cell[282563, 7079, 2586, 63, 242, "Input"],
Cell[285152, 7144, 353, 7, 39, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[285542, 7156, 101, 1, 35, "Subsection"],
Cell[285646, 7159, 71, 1, 39, "Input"],
Cell[285720, 7162, 297, 9, 65, "Input"],
Cell[286020, 7173, 224, 5, 39, "Input"],
Cell[286247, 7180, 1556, 43, 141, "Input"],
Cell[287806, 7225, 508, 12, 65, "Input"],
Cell[288317, 7239, 783, 18, 90, "Input"],
Cell[289103, 7259, 168, 2, 39, "Input"],
Cell[289274, 7263, 197, 4, 39, "Input"],
Cell[289474, 7269, 438, 11, 90, "Input"],
Cell[289915, 7282, 331, 8, 65, "Input"],
Cell[290249, 7292, 122, 2, 39, "Input"],
Cell[290374, 7296, 121, 2, 39, "Input"],
Cell[290498, 7300, 197, 4, 39, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[290732, 7309, 258, 3, 35, "Subsection"],
Cell[290993, 7314, 1062, 34, 68, "Input"],
Cell[292058, 7350, 1256, 38, 103, "Input"],
Cell[293317, 7390, 310, 10, 44, "Input"],
Cell[293630, 7402, 1487, 42, 101, "Input"],
Cell[295120, 7446, 161, 2, 44, "Input"],
Cell[295284, 7450, 187, 4, 44, "Text"],
Cell[295474, 7456, 3292, 72, 180, "Input"],
Cell[298769, 7530, 98, 1, 44, "Text"],
Cell[298870, 7533, 285, 8, 44, "Input"],
Cell[299158, 7543, 185, 2, 44, "Text"],
Cell[299346, 7547, 3078, 78, 215, "Input"],
Cell[302427, 7627, 1902, 48, 103, "Input"]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
